#!/bin/bash
# bash tools for calendars and months display + key change in a file + put/get

#-----------------------------------------------------------------------------
# Syntax:
#   * ndays $DATE $CALENDAR returns the number of days for current month
#   * last_d $DATE $CALENDAR returns the last date (current one - 1 day)
#   * next_d $DATE $CALENDAR returns the next date (current one + 1 day)
#      - $DATE:     YYYYMMDD or YYYYMM date format
#      - $CALENDAR: 365d, 3605d, 366d or greg (gregorian)
#-----------------------------------------------------------------------------

#--- Regular year
days_regl[1]=31; days_regl[4]=30; days_regl[7]=31; days_regl[10]=31
days_regl[2]=28; days_regl[5]=31; days_regl[8]=31; days_regl[11]=30
days_regl[3]=31; days_regl[6]=30; days_regl[9]=30; days_regl[12]=31

#--- Leap year
days_leap[1]=31; days_leap[4]=30; days_leap[7]=31; days_leap[10]=31
days_leap[2]=29; days_leap[5]=31; days_leap[8]=31; days_leap[11]=30
days_leap[3]=31; days_leap[6]=30; days_leap[9]=30; days_leap[12]=31

#--- months names (for display)
months[1]="january"  ; months[2]="february"  ; months[3]="march"
months[4]="april"    ; months[5]="may"       ; months[6]="june"
months[7]="july"     ; months[8]="august"    ; months[9]="september" 
months[10]="october" ; months[11]="november" ; months[12]="december" 


#--- PUT sed COMMAND IN A FILE TO CHANGE KEYS k* VALUES TO v* IN f* DEF FILE
#--- ARGUMENTS: 1. FILE  2. LIST OF KEYS (filename,key1=value1,...,keyN=valueN)
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function change_keys {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  outfile=$1; f=$2; file=${f%%,*}; f=${f#*,}
  while :; do g=${f%%,*}; f=${f#*,}; 
    echo '    sed "s/'${g%%=*}'=.*/'${g%%=*}'='${g#*=}'/" < '$file' > tmp.tmp; mv tmp.tmp '$file >> $outfile
    if [ $f == $g ]; then break; fi
  done
}

#--- PUT mfget/cp COMMAND IN A FILE DEPENDING ON THE SOURCE FOLDERS
#--- ARGUMENTS:  1. FILE  2. ARCHIVE FOLDER  3. LIST OF FULL FILES NAMES
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function get {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  fil=$1; shift; arc=$1; shift; cpf=""; mff=""
  while [ $# -ge 1 ]; do
    if [ $1 == $arc${1#*$arc} ]; then mff="$mff $1"; else cpf="$cpf $1"; fi ; shift
  done
  if [ "$cpf" != "" ]; then echo "    cp$cpf ."    >> $fil ; fi
  if [ "$mff" != "" ]; then echo "    mfget$mff ." >> $fil ; fi
}

#--- PUT mfput/cp COMMAND IN A FILE DEPENDING ON THE DESTINATION FOLDER
#--- ARGUMENTS:  1. FILE  2. ARCHIVE FOLDER  3. DESTINATION FOLDER 3. LIST OF FILES NAMES
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function put {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  fil=$1; shift; arc=$1; shift; des=$1; shift; ff=""
  while [ $# -ge 1 ]; do ff="$ff $1"; shift; done
  if [ $des == $arc${des#*$arc} ]; then
    echo "    mfput$ff $des" >> $fil
  else
    echo "    cp$ff $des"    >> $fil
  fi
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function remz {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  OU=$1
  while [ "`echo $OU|cut -c1-1`" = '0' ]; do OU=`echo $OU|cut -c2-${#OU}`; done
  local OU=$OU ; echo $OU
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function y {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local Y=`echo $1|cut -c1-4`; echo $Y
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function m {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local M=`echo $1|cut -c5-6`; echo $M
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function d {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local D=`echo $1|cut -c7-8`; echo $D
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function year {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local Y=`remz $(y $1)`; echo $Y
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function month {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local M=`remz $(m $1)`; echo $M
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function day {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local D=`remz $(d $1)`; echo $D
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function ym {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  local YM=`y $1``m $1`; echo $YM
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function addz {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  if [ $1 -lt 10 ]; then OU=0$1; else OU=$1; fi
  local OU=$OU; echo $OU
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function date {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  DATE=$1
  if [ $# -ge 2 ]; then DATE=$DATE`addz $(remz $2)` ; fi
  if [ $# -ge 3 ]; then DATE=$DATE`addz $(remz $3)` ; fi
  local DATE=$DATE; echo $DATE
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function isleap {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Y=`year $1`
  case $2 in
    365d) leap=0 ;;
    366d) leap=1 ;;
    greg) leap=0
      Y400=$((400*(Y/400)));  Y100=$((100*(Y/100)));  Y4=$((4*(Y/4)))
      if [[ $Y100 -ne $Y && $Y4 -eq $Y ]]; then leap=1;  fi
      if [  $Y400 -eq $Y ]; then leap=1; fi
  esac
  local leap=$leap; echo $leap
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function ndays_m {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  case $2 in
    360d) D=30 ;;
    365d|366d|greg) M=`month $1`
      if [ `isleap $1 $2` -eq 0 ]; then D=${days_regl[$M]}
      else                              D=${days_leap[$M]} ; fi ;;
  esac
  local D=$D; echo $D
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function ndays_y {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  case $2 in
    360d) D=360 ;;
    365d) D=365 ;;
    366d) D=366 ;;
    greg) D=365 ; if [ `isleap $1 $2` -eq 1 ]; then D=366 ; fi
  esac
  local D=$D; echo $D
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function last_m {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Y=`year $1`; M=`month $1`
  if [ $M -eq 1  ]; then DATEm=$((Y-1))12; else DATEm=$Y`addz $((M-1))`; fi
  local DATEm=$DATEm ; echo $DATEm
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function next_m {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Y=`year $1`; M=`month $1`
  if [ $M -eq 12 ]; then DATEp=$((Y+1))01; else DATEp=$Y`addz $((M+1))`; fi
  local DATEp=$DATEp ; echo $DATEp
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function last_d {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  DAT=`ym $1`; D=`day $1`; Dm=`addz $((D-1))`
  if [ $D -eq 1 ]; then DAT=`last_m $DAT` ; Dm=`ndays_m $DAT $2`; fi
  local DATE=$DAT$Dm ; echo $DATE
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function next_d {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  DAT=`ym $1`; D=`day $1`; Dp=`addz $((D+1))`
  if [ $D -eq `ndays_m $DAT $2` ]; then DAT=`next_m $DAT` ; Dp=01 ; fi
  local DATE=$DAT$Dp ; echo $DATE
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function elapsed_m {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  if [ $1 -gt $2 ]; then echo 'Erreur dans elapsed_m.'
    echo "La seconde date ($2) doit etre posterieure a la premiere ($1)."; exit
  fi
  local M=$((12*(`year $2`-`year $1`)+`month $2`-`month $1`)) ; echo $M
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function elapsed_d {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  if [ $1 -gt $2 ]; then echo 'Erreur dans elapsed_d.'
    echo "La seconde date ($2) doit etre posterieure a la premiere ($1)."; exit
  fi
  Yb=`year $1`; Mb=`month $1`; Db=`day $1`
  Ye=`year $2`; Me=`month $2`; De=`day $2`

#--- nombre de jours jusqu'en fin de premier mois
  D=$((`ndays_m $Yb$Mb $3`-$Db+1))

#--- nombre de jours jusqu'en fin de premiere annee
  M=$((Mb+1))
  while [ $M -le  12 ]; do D=$((D+$(ndays_m $Yb`addz $M` $3))) ; M=$((M+1)); done

#--- nombre de jours pour les annees completes
  Y=$((Yb+1))
  while [ $Y -lt $Ye ]; do D=$((D+`ndays_y $Y $3`)) ; Y=$((Y+1)); done

#--- nombre de jours des mois complets de la derniere annee
  M=1
  while [ $M -lt $Me ]; do D=$((D+$(ndays_m $Ye`addz $M` $3))) ; M=$((M+1)); done

#--- nombre de jours du dernier mois
  D=$((D+De))

  local D=$D ; echo $D
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function datedisp {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  DATEDISP="${months[`month $1`]} `year $1`"
  if [ ${#1} -eq 8 ]; then
    DATEDISP="$DATEDISP `day $1`"
    case `echo $1|cut -c8-8` in
      1) DATEDISP="${DATEDISP}st" ;;
      2) DATEDISP="${DATEDISP}nd" ;;
      3) DATEDISP="${DATEDISP}rd" ;;
      *) DATEDISP="${DATEDISP}th" ;;
    esac
  fi
  local DATEDISP=$DATEDISP ; echo $DATEDISP
}

