      SUBROUTINE PVtheta(ilon,ilev,pucov,pvcov,pteta,
     $           ztfi,zplay,zplev,
     $           nbteta,theta,PVteta)
      IMPLICIT none

c=======================================================================
c
c   Auteur:  I. Musat
c   -------
c
c   Objet:
c   ------
c
c    *******************************************************************
c    Calcul de la vorticite potentielle PVteta sur des iso-theta selon
c    la methodologie du NCEP/NCAR :
c    1) on calcule la stabilite statique N**2=g/T*(dT/dz+g/cp) sur les
c       niveaux du modele => N2
c    2) on interpole les vents, la temperature et le N**2 sur des isentropes
c       (en fait sur des iso-theta) lineairement en log(theta) =>
c       ucovteta, vcovteta, N2teta
c    3) on calcule la vorticite absolue sur des iso-theta => vorateta
c    4) on calcule la densite rho sur des iso-theta => rhoteta 
c
c       rhoteta = (T/theta)**(cp/R)*p0/(R*T)
c
c    5) on calcule la vorticite potentielle sur des iso-theta => PVteta
c
c       PVteta = (vorateta * N2 * theta)/(g * rhoteta) ! en PVU
c
c       NB: 1PVU=10**(-6) K*m**2/(s * kg)
c
c       PVteta =  vorateta * N2/(g**2 * rhoteta) ! en 1/(Pa*s)
c
c
c    *******************************************************************
c
c
c     Variables d'entree : ilon,ilev,pucov,pvcov,pteta,ztfi,zplay,zplev,nbteta,theta
c                       -> sur la grille dynamique
c     Variable de sortie : PVteta
c                       -> sur la grille physique 
c=======================================================================

!-----------------------------------------------------------------------
!   INCLUDE 'dimensions.h'
!
!   dimensions.h contient les dimensions du modele
!   ndm est tel que iim=2**ndm
!-----------------------------------------------------------------------

      INTEGER iim,jjm,llm,ndm

      PARAMETER (iim= 96,jjm=71,llm=39,ndm=1)

!-----------------------------------------------------------------------
!
! $Header$
!
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
!-----------------------------------------------------------------------
!   INCLUDE 'paramet.h'

      INTEGER  iip1,iip2,iip3,jjp1,llmp1,llmp2,llmm1
      INTEGER  kftd,ip1jm,ip1jmp1,ip1jmi1,ijp1llm
      INTEGER  ijmllm,mvar
      INTEGER jcfil,jcfllm

      PARAMETER( iip1= iim+1,iip2=iim+2,iip3=iim+3                      &
     &    ,jjp1=jjm+1-1/jjm)
      PARAMETER( llmp1 = llm+1,  llmp2 = llm+2, llmm1 = llm-1 )
      PARAMETER( kftd  = iim/2 -ndm )
      PARAMETER( ip1jm  = iip1*jjm,  ip1jmp1= iip1*jjp1 )
      PARAMETER( ip1jmi1= ip1jm - iip1 )
      PARAMETER( ijp1llm= ip1jmp1 * llm, ijmllm= ip1jm * llm )
      PARAMETER( mvar= ip1jmp1*( 2*llm+1) + ijmllm )
      PARAMETER( jcfil=jjm/2+5, jcfllm=jcfil*llm )

!-----------------------------------------------------------------------
c
c variables Input
c
      INTEGER ilon, ilev
      REAL pvcov(iip1,jjm,ilev)
      REAL pucov(iip1,jjp1,ilev)
      REAL pteta(iip1,jjp1,ilev)
      REAL ztfi(ilon,ilev)
      REAL zplay(ilon,ilev), zplev(ilon,ilev+1)
      INTEGER nbteta
      REAL theta(nbteta)
c
c variable Output
c
      REAL PVteta(ilon,nbteta)
c
c variables locales
c
      INTEGER i, j, l, ig0
      REAL SSUM
      REAL teta(ilon, ilev)
      REAL ptetau(ip1jmp1, ilev), ptetav(ip1jm, ilev)
      REAL ucovteta(ip1jmp1,ilev), vcovteta(ip1jm,ilev)
      REAL N2(ilon,ilev-1), N2teta(ilon,nbteta)
      REAL ztfiteta(ilon,nbteta)
      REAL rhoteta(ilon,nbteta)
      REAL vorateta(iip1,jjm,nbteta)
      REAL voratetafi(ilon,nbteta), vorpol(iim)
c
!
! $Header$
!
!CDK comgeom2
      COMMON/comgeom/                                                   &
     & cu(iip1,jjp1),cv(iip1,jjm),unscu2(iip1,jjp1),unscv2(iip1,jjm)  , &
     & aire(iip1,jjp1),airesurg(iip1,jjp1),aireu(iip1,jjp1)           , &
     & airev(iip1,jjm),unsaire(iip1,jjp1),apoln,apols                 , &
     & unsairez(iip1,jjm),airuscv2(iip1,jjm),airvscu2(iip1,jjm)       , &
     & aireij1(iip1,jjp1),aireij2(iip1,jjp1),aireij3(iip1,jjp1)       , &
     & aireij4(iip1,jjp1),alpha1(iip1,jjp1),alpha2(iip1,jjp1)         , &
     & alpha3(iip1,jjp1),alpha4(iip1,jjp1),alpha1p2(iip1,jjp1)        , &
     & alpha1p4(iip1,jjp1),alpha2p3(iip1,jjp1),alpha3p4(iip1,jjp1)    , &
     & fext(iip1,jjm),constang(iip1,jjp1), rlatu(jjp1),rlatv(jjm),      &
     & rlonu(iip1),rlonv(iip1),cuvsurcv(iip1,jjm),cvsurcuv(iip1,jjm)  , &
     & cvusurcu(iip1,jjp1),cusurcvu(iip1,jjp1)                        , &
     & cuvscvgam1(iip1,jjm),cuvscvgam2(iip1,jjm),cvuscugam1(iip1,jjp1), &
     & cvuscugam2(iip1,jjp1),cvscuvgam(iip1,jjm),cuscvugam(iip1,jjp1) , &
     & unsapolnga1,unsapolnga2,unsapolsga1,unsapolsga2                , &
     & unsair_gam1(iip1,jjp1),unsair_gam2(iip1,jjp1)                  , &
     & unsairz_gam(iip1,jjm),aivscu2gam(iip1,jjm),aiuscv2gam(iip1,jjm)  &
     & , xprimu(iip1),xprimv(iip1)


      REAL                                                               &
     & cu,cv,unscu2,unscv2,aire,airesurg,aireu,airev,apoln,apols,unsaire &
     & ,unsairez,airuscv2,airvscu2,aireij1,aireij2,aireij3,aireij4     , &
     & alpha1,alpha2,alpha3,alpha4,alpha1p2,alpha1p4,alpha2p3,alpha3p4 , &
     & fext,constang,rlatu,rlatv,rlonu,rlonv,cuvscvgam1,cuvscvgam2     , &
     & cvuscugam1,cvuscugam2,cvscuvgam,cuscvugam,unsapolnga1           , &
     & unsapolnga2,unsapolsga1,unsapolsga2,unsair_gam1,unsair_gam2     , &
     & unsairz_gam,aivscu2gam,aiuscv2gam,cuvsurcv,cvsurcuv,cvusurcu    , &
     & cusurcvu,xprimu,xprimv
!
! $Id: comconst.h 1707 2013-01-11 09:19:19Z fairhead $
!
!-----------------------------------------------------------------------
! INCLUDE comconst.h

      COMMON/comconsti/im,jm,lllm,imp1,jmp1,lllmm1,lllmp1,lcl,          &
     &                 iflag_top_bound
      COMMON/comconstr/dtvr,daysec,                                     &
     & pi,dtphys,dtdiss,rad,r,cpp,kappa,cotot,unsim,g,omeg              &
     &                   ,dissip_factz,dissip_deltaz,dissip_zref        &
     &                   ,tau_top_bound,                                &
     & daylen,year_day,molmass, ihf


      INTEGER im,jm,lllm,imp1,jmp1,lllmm1,lllmp1,lcl
      REAL dtvr ! dynamical time step (in s)
      REAL daysec !length (in s) of a standard day
      REAL pi    ! something like 3.14159....
      REAL dtphys ! (s) time step for the physics
      REAL dtdiss ! (s) time step for the dissipation
      REAL rad ! (m) radius of the planet
      REAL r ! Reduced Gas constant r=R/mu
             ! with R=8.31.. J.K-1.mol-1, mu: mol mass of atmosphere (kg/mol)
      REAL cpp   ! Specific heat Cp (J.kg-1.K-1)
      REAL kappa ! kappa=R/Cp 
      REAL cotot
      REAL unsim ! = 1./iim
      REAL g ! (m/s2) gravity
      REAL omeg ! (rad/s) rotation rate of the planet
      REAL dissip_factz,dissip_deltaz,dissip_zref
      INTEGER iflag_top_bound
      REAL tau_top_bound
      REAL daylen ! length of solar day, in 'standard' day length
      REAL year_day ! Number of standard days in a year
      REAL molmass ! (g/mol) molar mass of the atmosphere

      REAL ihf ! (W/m2) Intrinsic heat flux (for giant planets)

!-----------------------------------------------------------------------
!
! $Id: comvert.h 1669 2012-10-16 12:41:50Z fairhead $
!
!-----------------------------------------------------------------------
!   INCLUDE 'comvert.h'

      COMMON/comvertr/ap(llm+1),bp(llm+1),presnivs(llm),dpres(llm),     &
     &               pa,preff,nivsigs(llm),nivsig(llm+1),               &
     &               aps(llm),bps(llm),scaleheight,pseudoalt(llm)

      common/comverti/disvert_type, pressure_exner

      real ap     ! hybrid pressure contribution at interlayers
      real bp     ! hybrid sigma contribution at interlayer
      real presnivs ! (reference) pressure at mid-layers
      real dpres
      real pa     ! reference pressure (Pa) at which hybrid coordinates
                  ! become purely pressure
      real preff  ! reference surface pressure (Pa)
      real nivsigs
      real nivsig
      real aps    ! hybrid pressure contribution at mid-layers
      real bps    ! hybrid sigma contribution at mid-layers
      real scaleheight ! atmospheric (reference) scale height (km)
      real pseudoalt ! for planets

      integer disvert_type ! type of vertical discretization:
                           ! 1: Earth (default for planet_type==earth),
                           !     automatic generation
                           ! 2: Planets (default for planet_type!=earth),
                           !     using 'z2sig.def' (or 'esasig.def) file

      logical pressure_exner
!     compute pressure inside layers using Exner function, else use mean
!     of pressure values at interfaces

 !-----------------------------------------------------------------------
c
c projection teta sur la grille physique
c
      DO l=1,llm
       teta(1,l)   =  pteta(1,1,l)
       ig0         = 2
       DO j = 2, jjm
        DO i = 1, iim
         teta(ig0,l)    = pteta(i,j,l)
         ig0            = ig0 + 1
        ENDDO
       ENDDO
       teta(ig0,l)    = pteta(1,jjp1,l)
      ENDDO
c
c calcul pteta sur les grilles U et V
c
      DO l=1, llm
       DO j=1, jjp1
        DO i=1, iip1
         ig0=i+(j-1)*iip1
         ptetau(ig0,l)=pteta(i,j,l)
        ENDDO !i
       ENDDO !j
       DO j=1, jjm
        DO i=1, iip1
         ig0=i+(j-1)*iip1
         ptetav(ig0,l)=0.5*(pteta(i,j,l)+pteta(i,j+1,l))
        ENDDO !i
       ENDDO !j
      ENDDO !l
c
c projection pucov, pvcov sur une surface de theta constante
c
      DO l=1, nbteta
cIM 1rout CALL tetaleveli1j1(ip1jmp1,llm,.true.,ptetau,theta(l),
       CALL tetalevel(ip1jmp1,llm,.true.,ptetau,theta(l),
     .                pucov,ucovteta(:,l))
cIM 1rout CALL tetaleveli1j(ip1jm,llm,.true.,ptetav,theta(l),
       CALL tetalevel(ip1jm,llm,.true.,ptetav,theta(l),
     .                pvcov,vcovteta(:,l))
      ENDDO !l
c
c calcul vorticite absolue sur une iso-theta : vorateta
c
      CALL tourabs(nbteta,vcovteta,ucovteta,vorateta)
c
c projection vorateta sur la grille physique => voratetafi
c
      DO l=1,nbteta
       DO j=2,jjm
        ig0=1+(j-2)*iim
        DO i=1,iim
         voratetafi(ig0+i+1,l) = vorateta( i ,j-1,l) * alpha4(i+1,j) +
     $                           vorateta(i+1,j-1,l) * alpha1(i+1,j) +
     $                           vorateta(i  ,j  ,l) * alpha3(i+1,j) +
     $                           vorateta(i+1,j  ,l) * alpha2(i+1,j)
        ENDDO
        voratetafi(ig0 +1,l) = voratetafi(ig0 +1+ iim,l)
       ENDDO
      ENDDO
c
      DO l=1,nbteta
       DO i=1,iim
        vorpol(i)  = vorateta(i,1,l)*aire(i,1)
       ENDDO
       voratetafi(1,l)= SSUM(iim,vorpol,1)/apoln
      ENDDO
c
      DO l=1,nbteta
       DO i=1,iim
        vorpol(i)  = vorateta(i,jjm,l)* aire(i,jjm +1)
       ENDDO
       voratetafi(ilon,l)= SSUM(iim,vorpol,1)/apols
      ENDDO
c 
c calcul N**2 sur la grille physique => N2
c
      DO l=1, llm-1 
       DO i=1, ilon
        N2(i,l) = (g**2 * zplay(i,l) * 
     $            (ztfi(i,l+1)-ztfi(i,l)) )/
     $            (R*ztfi(i,l)*ztfi(i,l)*
     $            (zplev(i,l)-zplev(i,l+1)) )+
     $            (g**2)/(ztfi(i,l)*CPP)
       ENDDO !i
      ENDDO !l
c
c calcul N2 sur une iso-theta => N2teta 
c
      DO l=1, nbteta
       CALL tetalevel(ilon,llm-1,.true.,teta,theta(l),
     $                N2,N2teta(:,l))
       CALL tetalevel(ilon,llm,.true.,teta,theta(l),
     $                ztfi,ztfiteta(:,l))
      ENDDO !l=1, nbteta
c
c calcul rho et PV sur une iso-theta : rhoteta, PVteta
c
      DO l=1, nbteta
       DO i=1, ilon
        rhoteta(i,l)=(ztfiteta(i,l)/theta(l))**(CPP/R)*
     $  (preff/(R*ztfiteta(i,l)))
c
c PVteta en PVU
c
        PVteta(i,l)=(theta(l)*g*voratetafi(i,l)*N2teta(i,l))/
     $              (g**2*rhoteta(i,l))
c
c PVteta en 1/(Pa*s)
c
        PVteta(i,l)=(voratetafi(i,l)*N2teta(i,l))/
     $              (g**2*rhoteta(i,l))
       ENDDO !i
      ENDDO !l
c
      RETURN
      END 
