!
! $Id: calfis_p.F 1407 2010-07-07 10:31:52Z fairhead $
!
C
C
      SUBROUTINE calfis_p(lafin,
     $                  jD_cur, jH_cur,
     $                  pucov,
     $                  pvcov,
     $                  pteta,
     $                  pq,
     $                  pmasse,
     $                  pps,
     $                  pp,
     $                  ppk,
     $                  pphis,
     $                  pphi,
     $                  pducov,
     $                  pdvcov,
     $                  pdteta,
     $                  pdq,
     $                  flxw,
     $                  clesphy0,
     $                  pdufi,
     $                  pdvfi,
     $                  pdhfi,
     $                  pdqfi,
     $                  pdpsfi

     :                  ,pxt ! <-> pq
     :                  ,pdxt !<-> pdq
     :                  ,pdxtfi ! <-> pdqfi
     :                  ,prlat





     :                  )

! If using physics
! Ehouarn: For now, calfis_p needs Earth physics
c
c    Auteur :  P. Le Van, F. Hourdin 
c   .........
      USE dimphy
      USE mod_phys_lmdz_para, mpi_root_xx=>mpi_root 
      USE mod_interface_dyn_phys
      USE IOPHY

      USE parallel, ONLY : omp_chunk, using_mpi
      USE Write_Field
      Use Write_field_p
      USE Times
      USE infotrac
      USE control_mod

      IMPLICIT NONE
c=======================================================================
c
c   1. rearrangement des tableaux et transformation
c      variables dynamiques  >  variables physiques
c   2. calcul des termes physiques
c   3. retransformation des tendances physiques en tendances dynamiques
c
c   remarques:
c   ----------
c
c    - les vents sont donnes dans la physique par leurs composantes 
c      naturelles.
c    - la variable thermodynamique de la physique est une variable
c      intensive :   T 
c      pour la dynamique on prend    T * ( preff / p(l) ) **kappa
c    - les deux seules variables dependant de la geometrie necessaires
c      pour la physique sont la latitude pour le rayonnement et 
c      l'aire de la maille quand on veut integrer une grandeur 
c      horizontalement.
c    - les points de la physique sont les points scalaires de la 
c      la dynamique; numerotation:
c          1 pour le pole nord
c          (jjm-1)*iim pour l'interieur du domaine
c          ngridmx pour le pole sud
c      ---> ngridmx=2+(jjm-1)*iim
c
c     Input :
c     -------
c       ecritphy        frequence d'ecriture (en jours)de histphy
c       pucov           covariant zonal velocity
c       pvcov           covariant meridional velocity 
c       pteta           potential temperature
c       pps             surface pressure
c       pmasse          masse d'air dans chaque maille
c       pts             surface temperature  (K)
c       callrad         clef d'appel au rayonnement
c
c    Output :
c    --------
c        pdufi          tendency for the natural zonal velocity (ms-1)
c        pdvfi          tendency for the natural meridional velocity 
c        pdhfi          tendency for the potential temperature
c        pdtsfi         tendency for the surface temperature
c
c        pdtrad         radiative tendencies  \  both input
c        pfluxrad       radiative fluxes      /  and output
c
c=======================================================================
c
c-----------------------------------------------------------------------
c
c    0.  Declarations :
c    ------------------

!-----------------------------------------------------------------------
!   INCLUDE 'dimensions.h'
!
!   dimensions.h contient les dimensions du modele
!   ndm est tel que iim=2**ndm
!-----------------------------------------------------------------------

      INTEGER iim,jjm,llm,ndm

      PARAMETER (iim= 96,jjm=71,llm=39,ndm=1)

!-----------------------------------------------------------------------
!
! $Header$
!
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
!-----------------------------------------------------------------------
!   INCLUDE 'paramet.h'

      INTEGER  iip1,iip2,iip3,jjp1,llmp1,llmp2,llmm1
      INTEGER  kftd,ip1jm,ip1jmp1,ip1jmi1,ijp1llm
      INTEGER  ijmllm,mvar
      INTEGER jcfil,jcfllm

      PARAMETER( iip1= iim+1,iip2=iim+2,iip3=iim+3                      &
     &    ,jjp1=jjm+1-1/jjm)
      PARAMETER( llmp1 = llm+1,  llmp2 = llm+2, llmm1 = llm-1 )
      PARAMETER( kftd  = iim/2 -ndm )
      PARAMETER( ip1jm  = iip1*jjm,  ip1jmp1= iip1*jjp1 )
      PARAMETER( ip1jmi1= ip1jm - iip1 )
      PARAMETER( ijp1llm= ip1jmp1 * llm, ijmllm= ip1jm * llm )
      PARAMETER( mvar= ip1jmp1*( 2*llm+1) + ijmllm )
      PARAMETER( jcfil=jjm/2+5, jcfllm=jcfil*llm )

!-----------------------------------------------------------------------
!
! $Id: temps.h 1665 2012-10-09 13:35:26Z fairhead $
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
! jD_ref = jour julien de la date de reference (lancement de l'experience)
! hD_ref = "heure" julienne de la date de reference
!-----------------------------------------------------------------------
! INCLUDE 'temps.h'

      COMMON/temps/itaufin, dt, day_ini, day_end, annee_ref, day_ref,   &
     &             itau_dyn, itau_phy, jD_ref, jH_ref, calend,          &
     &             start_time


      INTEGER   itaufin
      INTEGER itau_dyn, itau_phy
      INTEGER day_ini, day_end, annee_ref, day_ref
      REAL      dt, jD_ref, jH_ref, start_time
      CHARACTER (len=10) :: calend

!$OMP THREADPRIVATE(/temps/)
!-----------------------------------------------------------------------

      INTEGER ngridmx
      PARAMETER( ngridmx = 2+(jjm-1)*iim - 1/jjm   )

!
! $Id: comconst.h 1707 2013-01-11 09:19:19Z fairhead $
!
!-----------------------------------------------------------------------
! INCLUDE comconst.h

      COMMON/comconsti/im,jm,lllm,imp1,jmp1,lllmm1,lllmp1,lcl,          &
     &                 iflag_top_bound
      COMMON/comconstr/dtvr,daysec,                                     &
     & pi,dtphys,dtdiss,rad,r,cpp,kappa,cotot,unsim,g,omeg              &
     &                   ,dissip_factz,dissip_deltaz,dissip_zref        &
     &                   ,tau_top_bound,                                &
     & daylen,year_day,molmass, ihf


      INTEGER im,jm,lllm,imp1,jmp1,lllmm1,lllmp1,lcl
      REAL dtvr ! dynamical time step (in s)
      REAL daysec !length (in s) of a standard day
      REAL pi    ! something like 3.14159....
      REAL dtphys ! (s) time step for the physics
      REAL dtdiss ! (s) time step for the dissipation
      REAL rad ! (m) radius of the planet
      REAL r ! Reduced Gas constant r=R/mu
             ! with R=8.31.. J.K-1.mol-1, mu: mol mass of atmosphere (kg/mol)
      REAL cpp   ! Specific heat Cp (J.kg-1.K-1)
      REAL kappa ! kappa=R/Cp 
      REAL cotot
      REAL unsim ! = 1./iim
      REAL g ! (m/s2) gravity
      REAL omeg ! (rad/s) rotation rate of the planet
      REAL dissip_factz,dissip_deltaz,dissip_zref
      INTEGER iflag_top_bound
      REAL tau_top_bound
      REAL daylen ! length of solar day, in 'standard' day length
      REAL year_day ! Number of standard days in a year
      REAL molmass ! (g/mol) molar mass of the atmosphere

      REAL ihf ! (W/m2) Intrinsic heat flux (for giant planets)

!-----------------------------------------------------------------------
!
! $Id: comvert.h 1669 2012-10-16 12:41:50Z fairhead $
!
!-----------------------------------------------------------------------
!   INCLUDE 'comvert.h'

      COMMON/comvertr/ap(llm+1),bp(llm+1),presnivs(llm),dpres(llm),     &
     &               pa,preff,nivsigs(llm),nivsig(llm+1),               &
     &               aps(llm),bps(llm),scaleheight,pseudoalt(llm)

      common/comverti/disvert_type, pressure_exner

      real ap     ! hybrid pressure contribution at interlayers
      real bp     ! hybrid sigma contribution at interlayer
      real presnivs ! (reference) pressure at mid-layers
      real dpres
      real pa     ! reference pressure (Pa) at which hybrid coordinates
                  ! become purely pressure
      real preff  ! reference surface pressure (Pa)
      real nivsigs
      real nivsig
      real aps    ! hybrid pressure contribution at mid-layers
      real bps    ! hybrid sigma contribution at mid-layers
      real scaleheight ! atmospheric (reference) scale height (km)
      real pseudoalt ! for planets

      integer disvert_type ! type of vertical discretization:
                           ! 1: Earth (default for planet_type==earth),
                           !     automatic generation
                           ! 2: Planets (default for planet_type!=earth),
                           !     using 'z2sig.def' (or 'esasig.def) file

      logical pressure_exner
!     compute pressure inside layers using Exner function, else use mean
!     of pressure values at interfaces

 !-----------------------------------------------------------------------
!
! $Header$
!
!CDK comgeom2
      COMMON/comgeom/                                                   &
     & cu(iip1,jjp1),cv(iip1,jjm),unscu2(iip1,jjp1),unscv2(iip1,jjm)  , &
     & aire(iip1,jjp1),airesurg(iip1,jjp1),aireu(iip1,jjp1)           , &
     & airev(iip1,jjm),unsaire(iip1,jjp1),apoln,apols                 , &
     & unsairez(iip1,jjm),airuscv2(iip1,jjm),airvscu2(iip1,jjm)       , &
     & aireij1(iip1,jjp1),aireij2(iip1,jjp1),aireij3(iip1,jjp1)       , &
     & aireij4(iip1,jjp1),alpha1(iip1,jjp1),alpha2(iip1,jjp1)         , &
     & alpha3(iip1,jjp1),alpha4(iip1,jjp1),alpha1p2(iip1,jjp1)        , &
     & alpha1p4(iip1,jjp1),alpha2p3(iip1,jjp1),alpha3p4(iip1,jjp1)    , &
     & fext(iip1,jjm),constang(iip1,jjp1), rlatu(jjp1),rlatv(jjm),      &
     & rlonu(iip1),rlonv(iip1),cuvsurcv(iip1,jjm),cvsurcuv(iip1,jjm)  , &
     & cvusurcu(iip1,jjp1),cusurcvu(iip1,jjp1)                        , &
     & cuvscvgam1(iip1,jjm),cuvscvgam2(iip1,jjm),cvuscugam1(iip1,jjp1), &
     & cvuscugam2(iip1,jjp1),cvscuvgam(iip1,jjm),cuscvugam(iip1,jjp1) , &
     & unsapolnga1,unsapolnga2,unsapolsga1,unsapolsga2                , &
     & unsair_gam1(iip1,jjp1),unsair_gam2(iip1,jjp1)                  , &
     & unsairz_gam(iip1,jjm),aivscu2gam(iip1,jjm),aiuscv2gam(iip1,jjm)  &
     & , xprimu(iip1),xprimv(iip1)


      REAL                                                               &
     & cu,cv,unscu2,unscv2,aire,airesurg,aireu,airev,apoln,apols,unsaire &
     & ,unsairez,airuscv2,airvscu2,aireij1,aireij2,aireij3,aireij4     , &
     & alpha1,alpha2,alpha3,alpha4,alpha1p2,alpha1p4,alpha2p3,alpha3p4 , &
     & fext,constang,rlatu,rlatv,rlonu,rlonv,cuvscvgam1,cuvscvgam2     , &
     & cvuscugam1,cvuscugam2,cvscuvgam,cuscvugam,unsapolnga1           , &
     & unsapolnga2,unsapolsga1,unsapolsga2,unsair_gam1,unsair_gam2     , &
     & unsairz_gam,aivscu2gam,aiuscv2gam,cuvsurcv,cvsurcuv,cvusurcu    , &
     & cusurcvu,xprimu,xprimv
!
! $Header$
!
!
! gestion des impressions de sorties et de dbogage
! lunout:    unit du fichier dans lequel se font les sorties 
!                           (par defaut 6, la sortie standard)
! prt_level: niveau d'impression souhait (0 = minimum)
!
      INTEGER lunout, prt_level
      COMMON /comprint/ lunout, prt_level
      include 'mpif.h'
c    Arguments :
c    -----------
      LOGICAL  lafin
!      REAL heure
      REAL, intent(in):: jD_cur, jH_cur
      REAL pvcov(iip1,jjm,llm)
      REAL pucov(iip1,jjp1,llm)
      REAL pteta(iip1,jjp1,llm)
      REAL pmasse(iip1,jjp1,llm)
      REAL pq(iip1,jjp1,llm,nqtot)
      REAL pphis(iip1,jjp1)
      REAL pphi(iip1,jjp1,llm)
c
      REAL pdvcov(iip1,jjm,llm)
      REAL pducov(iip1,jjp1,llm)
      REAL pdteta(iip1,jjp1,llm)
      REAL pdq(iip1,jjp1,llm,nqtot)
      REAL flxw(iip1,jjp1,llm)  ! Flux de masse verticale sur la grille dynamique
c
      REAL pps(iip1,jjp1)
      REAL pp(iip1,jjp1,llmp1)
      REAL ppk(iip1,jjp1,llm)
c
      REAL pdvfi(iip1,jjm,llm)
      REAL pdufi(iip1,jjp1,llm)
      REAL pdhfi(iip1,jjp1,llm)
      REAL pdqfi(iip1,jjp1,llm,nqtot)
      REAL pdpsfi(iip1,jjp1)
!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env ! si oui, la temperature de cond est celle de l'environnement: permet d'eviter temperature de cond aberrantes
        logical ok_prod_nucl_tritium ! si oui, production de tritium par essais nucleaires
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)

        integer niso ! nombre d'isotopes utiliss
        parameter (niso=use_iso_eau+use_iso_HDO+use_iso_O18
     &           +use_iso_O17+use_iso_HTO)

        character*3 striso(niso)
      
        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso)
     &    , tnat(niso), toce(niso), tcorr(niso)
     &    , tdifrel(niso)
     &    , talph1(niso), talph2(niso), talph3(niso)
     &    , talps1(niso), talps2(niso)
     &    , tkcin0(niso)
     &    , tkcin1(niso),tkcin2(niso)
     &    , alpha_liq_sol(niso) 
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg
     &    , pxtmelt, pxtice, pxtmin, pxtmax
     &    , bidouille_anti_divergence,essai_convergence
     &    , initialisation_iso, initialisation_isotrac
     &    , modif_sst,deltaTtest
     &    , modif_sic,deltasic
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice 
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot
     &    , rh_cste_surf_cond, T_cste_surf_cond
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie
     &    , alphak_stewart, cste_surf_cond
     &    , mixlen
     &    , tnat, toce, tcorr
     &    , tdifrel
     &    , calendrier_guide
     &    , talph1, talph2, talph3
     &    , talps1, talps2
     &    , tkcin0, tkcin1, tkcin2
     &    , alpha_liq_sol, Rdefault
     &    , striso
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce
     &    , ratqsbasnew,Pcrit_ratqs
     &    , ok_bidouille_wake,cond_temp_env
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl
     &    , year_nucl, lat_nucl, lon_nucl
     &    , zmin_nucl, zmax_nucl, HTO_nucl
      ! variables dynamiques en input:
      !integer niso
      real pxt(iip1,jjp1,llm,2,ntraciso)  ! pq(iip1,jjp1,llm,nqmx)
!      real pxt2(iip1,jjp1,llm,2,ntraciso)
      real pdxt(iip1,jjp1,llm,2,ntraciso) ! pdq(iip1,jjp1,llm,nqmx)
! variables dynamiques en output: 
      real pdxtfi(iip1,jjp1,llm,2,ntraciso)  !pdqfi(iip1,jjp1,llm,nqmx)      
      ! variables intermediaires
      integer ixt
      ! variables pour vrifier calculs
      real rlat(ngridmx),prlat(iip1,jjp1)
       logical sorties_gdat ! si true, sorties gdat intermdiaires internes aux subroutines
        parameter (sorties_gdat=.false.)      
!
!        logical essai_convergence ! si false, on peut rtroagir sur
!        l'eau normale, si true, on ne peut pas, et a permet de tester
!        la convergence avec la verison d'LMDZ sans isotopes.
!        Attention: si on  fait un essai de convergence, mieux vaut
!        enlever iso_debug, sinon, risuqes de plantage si non
!        conservation de l'eau. 
!        parameter (essai_convergence=.false.)
!        parameter (essai_convergence=.false.)


    ! essai convergence a t dplac dans wateriso2.h et est lue  l'execution

      real errmax ! erreur maximale en absolu.
      parameter (errmax=1e-8)
!      parameter (errmax=1e10)

      real errmax_sol ! erreur maximale en absolu.
      parameter (errmax_sol=5e-7)

      real errmaxrel ! erreur maximale en relatif autorise
!      parameter (errmaxrel=1e10)
      parameter (errmaxrel=1e-3)

      real borne ! valeur maximale que n'importe quelle variable peut
                 ! atteindre, en val abs; utile pour vrif que pas NAN
      parameter (borne=1e19)

      real ridicule ! valeur maximale pour qu'une variable de type
                    ! rapoport de mlange puisse tre considre comme ngligeable. Si
                    ! ngligeable, alors on ne verifie pas si sa compo iso esta brrante.
      parameter (ridicule=1e-12)      
!      parameter (ridicule=1)
!
      real ridicule_rain ! valeur limite de ridicule pour les flux de pluies (rain, zrfl...)
      parameter (ridicule_rain=1e-8) ! en kg/s <-> 1e-3mm/day

      real ridicule_evap ! valeur limite de ridicule pour les evap
      parameter (ridicule_evap=ridicule_rain*1e-2) ! en kg/s <-> 1e-3mm/day

      real ridicule_qsol ! valeur limite de ridicule pour les qsol
      parameter (ridicule_qsol=ridicule_rain) ! en kg <-> 1e-8kg

      real ridicule_snow ! valeur limite de ridicule pour les snow
      parameter (ridicule_snow=ridicule_qsol) ! en kg/s <-> 1e-8kg

      real ridicule_trac
      parameter (ridicule_trac=ridicule*1e4)
      
      real deltalim ! deltalim est le maximum de deltaD qu'on
                             ! autorise dans la vapeur, au-dela, en suppose que c'est abrrant.
                             ! dans le liquide, on autorise deltalim*faccond.
!      parameter (deltalim=1e10)
!      parameter (deltalim=300.0)
       ! maintenant dfini dans iso.def

       real deltalimtrac ! max de deltaD dans les traceurs, dfini dans iso.def

      real deltalim_snow ! deltalim est le maximum de deltaD qu'on
                             ! autorise dans la neige, au-dela, en suppose que c'est abrrant.
!      parameter (deltalim_snow=500.0) ! initalis dans iso_init

	real deltaDmin
!	parameter (deltaDmin=-900.0)
    ! maintentant, dfini dans iso.def

      real o17excess_bas,o17excess_haut ! bornes inf et sup de l'O17-excess
!      parameter(o17excess_bas=-200.0,o17excess_haut=120)
      integer nlevmaxO17

      logical O17_verif
!      parameter (O17_verif=.true.)

      real faccond  ! dans le liquide, on autorise R(deltalim)*faccond.
      parameter (faccond=1.1)
      
!      logical bidouille_anti_divergence ! si true, alors on fait un
!                                        ! rappel rgulier des xt4 vers les q pour 
!                                        !viter accumulation d'rreurs et  divergences
!      parameter (bidouille_anti_divergence=.true.)
!      parameter (bidouille_anti_divergence=.false.)
    ! bidouille_anti_divergence a t dplac dans wateriso2.h et est lue  l'execution

        real expb_max
        parameter (expb_max=30.0)

	real deltaDfaible ! deltaD en dessous duquel la vapeur est tellement faible 
        !que on peut accepter que la remise  l'quilibre du sol avec 
        ! cette vapeur donne des deltaDevap aberrants.
        parameter (deltaDfaible=-300)
        real deltaDfaible_lax ! deltaD en dessous duquel la vapeur est tellement faible 
        !que on peut accepter que la remise  l'quilibre du sol avec 
        ! cette vapeur donne des deltaDevap aberrants.
        parameter (deltaDfaible_lax=-180)

    	real faible_evap ! faible vaporation: on est plus laxiste 
        !pour les deltaD aberrant dans le cas de l'vap venant d'orchidee
        parameter (faible_evap=3.0)

    	real Tmin_verif
	parameter (Tmin_verif=5.0) ! en K
    	real Tmax_verif
	parameter (Tmax_verif=1000.0) ! en K

        COMMON /isoverif/ deltalim,deltalimtrac,deltalim_snow,deltaDmin
	COMMON /isoverifO17/ O17_verif,o17excess_bas,o17excess_haut
	COMMON /isoverifO17bis/ nlevmaxO17      

      INTEGER        longcles
      PARAMETER    ( longcles = 20 )
      REAL clesphy0( longcles )

! Ehouarn: for now calfis_p needs some informations from physics to compile
c    Local variables :
c    -----------------

      INTEGER i,j,l,ig0,ig,iq,iiq
      REAL,ALLOCATABLE,SAVE :: zpsrf(:)
      REAL,ALLOCATABLE,SAVE :: zplev(:,:),zplay(:,:)
      REAL,ALLOCATABLE,SAVE :: zphi(:,:),zphis(:)
c
      REAL,ALLOCATABLE,SAVE :: zufi(:,:), zvfi(:,:)
      REAL,ALLOCATABLE,SAVE :: ztfi(:,:),zqfi(:,:,:)
c
      REAL,ALLOCATABLE,SAVE :: pcvgu(:,:), pcvgv(:,:)
      REAL,ALLOCATABLE,SAVE :: pcvgt(:,:), pcvgq(:,:,:)
c
      REAL,ALLOCATABLE,SAVE :: zdufi(:,:),zdvfi(:,:)
      REAL,ALLOCATABLE,SAVE :: zdtfi(:,:),zdqfi(:,:,:)
      REAL,ALLOCATABLE,SAVE :: zdpsrf(:)
      REAL,SAVE,ALLOCATABLE ::  flxwfi(:,:)     ! Flux de masse verticale sur la grille physiq
      REAL,ALLOCATABLE,SAVE :: zxtfi(:,:,:,:)
      REAL,ALLOCATABLE,SAVE :: zdxtfi(:,:,:,:) 
c
      REAL,ALLOCATABLE,SAVE :: zplev_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zplay_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zphi_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zphis_omp(:)
      REAL,ALLOCATABLE,SAVE :: presnivs_omp(:)
      REAL,ALLOCATABLE,SAVE :: zufi_omp(:,:) 
      REAL,ALLOCATABLE,SAVE :: zvfi_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: ztfi_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zqfi_omp(:,:,:)
      REAL,ALLOCATABLE,SAVE :: zdufi_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zdvfi_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zdtfi_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zdqfi_omp(:,:,:)
      REAL,ALLOCATABLE,SAVE :: zdpsrf_omp(:)
      REAL,SAVE,ALLOCATABLE ::  flxwfi_omp(:,:)     ! Flux de masse verticale sur la grille physiq
      REAL,ALLOCATABLE,SAVE :: zxtfi_omp(:,:,:,:)
      REAL,ALLOCATABLE,SAVE :: zdxtfi_omp(:,:,:,:)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Introduction du splitting (FH)
! Question pour Yann :
! J'ai t surpris au dbut que les tableaux zufi_omp, zdufi_omp n'co soitent
! en SAVE. Je crois comprendre que c'est parce que tu voulais qu'il
! soit allocatable (plutot par exemple que de passer une dimension
! dpendant du process en argument des routines) et que, du coup,
! le SAVE vite d'avoir  refaire l'allocation  chaque appel.
! Tu confirmes ?
! J'ai suivi le mme principe pour les zdufic_omp
! Mais c'est surement bien que tu controles.
! 

      REAL,ALLOCATABLE,SAVE :: zdufic_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zdvfic_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zdtfic_omp(:,:)
      REAL,ALLOCATABLE,SAVE :: zdqfic_omp(:,:,:)
      REAL jH_cur_split,zdt_split
      LOGICAL debut_split,lafin_split
      INTEGER isplit
      REAL,ALLOCATABLE,SAVE :: zdxtfic_omp(:,:,:,:)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

c$OMP THREADPRIVATE(zplev_omp,zplay_omp,zphi_omp,zphis_omp,
c$OMP+                 presnivs_omp,zufi_omp,zvfi_omp,ztfi_omp,
c$OMP+                 zqfi_omp,zdufi_omp,zdvfi_omp,
c$OMP+                 zdtfi_omp,zdqfi_omp,zdpsrf_omp,flxwfi_omp,
c$OMP+                 zdufic_omp,zdvfic_omp,zdtfic_omp,zdqfic_omp)     

c$OMP THREADPRIVATE(zxtfi_omp,zdxtfi_omp,zdxtfic_omp) 

      LOGICAL,SAVE :: first_omp=.true.
c$OMP THREADPRIVATE(first_omp)
      
      REAL zsin(iim),zcos(iim),z1(iim)
      REAL zsinbis(iim),zcosbis(iim),z1bis(iim)
      REAL unskap, pksurcp
c
cIM diagnostique PVteta, Amip2
      INTEGER ntetaSTD
      PARAMETER(ntetaSTD=3)
      REAL rtetaSTD(ntetaSTD)
      DATA rtetaSTD/350., 380., 405./ ! Earth-specific values, beware !!
      REAL PVteta(klon,ntetaSTD)
c

      
      REAL SSUM

      LOGICAL firstcal, debut
      DATA firstcal/.true./
      SAVE firstcal,debut
c$OMP THREADPRIVATE(firstcal,debut)
      
      REAL,SAVE,dimension(1:iim,1:llm):: du_send,du_recv,dv_send,dv_recv
      INTEGER :: ierr
      INTEGER,dimension(MPI_STATUS_SIZE,4) :: Status
      INTEGER, dimension(4) :: Req
      REAL,ALLOCATABLE,SAVE:: zdufi2(:,:),zdvfi2(:,:)
      integer :: k,kstart,kend
      INTEGER :: offset  

      LOGICAL tracerdyn
c
c-----------------------------------------------------------------------
c
c    1. Initialisations :
c    --------------------
c

      klon=klon_mpi
      
      PVteta(:,:)=0.
            
c
      IF ( firstcal )  THEN
        debut = .TRUE.
        IF (ngridmx.NE.2+(jjm-1)*iim) THEN
         write(lunout,*) 'STOP dans calfis'
         write(lunout,*)
     &   'La dimension ngridmx doit etre egale a 2 + (jjm-1)*iim'
         write(lunout,*) '  ngridmx  jjm   iim   '
         write(lunout,*) ngridmx,jjm,iim
         STOP
        ENDIF
c$OMP MASTER
      ALLOCATE(zpsrf(klon))
      ALLOCATE(zplev(klon,llm+1),zplay(klon,llm))
      ALLOCATE(zphi(klon,llm),zphis(klon))
      ALLOCATE(zufi(klon,llm), zvfi(klon,llm))
      ALLOCATE(ztfi(klon,llm),zqfi(klon,llm,nqtot))
      ALLOCATE(pcvgu(klon,llm), pcvgv(klon,llm))
      ALLOCATE(pcvgt(klon,llm), pcvgq(klon,llm,2))
      ALLOCATE(zdufi(klon,llm),zdvfi(klon,llm))
      ALLOCATE(zdtfi(klon,llm),zdqfi(klon,llm,nqtot))
      ALLOCATE(zdpsrf(klon))
      ALLOCATE(zdufi2(klon+iim,llm),zdvfi2(klon+iim,llm))
      ALLOCATE(flxwfi(klon,llm))
      ALLOCATE(zxtfi(klon,llm,2,ntraciso))
      ALLOCATE(zdxtfi(klon,llm,2,ntraciso))
c$OMP END MASTER
c$OMP BARRIER	  
      ELSE
        debut = .FALSE.
      ENDIF

c
c
c-----------------------------------------------------------------------
c   40. transformation des variables dynamiques en variables physiques:
c   ---------------------------------------------------------------

c   41. pressions au sol (en Pascals)
c   ----------------------------------

c$OMP MASTER
      call start_timer(timer_physic)
c$OMP END MASTER

c$OMP MASTER             
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
      do ig0=1,klon
        i=index_i(ig0)
        j=index_j(ig0)
        zpsrf(ig0)=pps(i,j)
      enddo
c$OMP END MASTER


c   42. pression intercouches :
c
c   -----------------------------------------------------------------
c     .... zplev  definis aux (llm +1) interfaces des couches  ....
c     .... zplay  definis aux (  llm )    milieux des couches  .... 
c   -----------------------------------------------------------------

c    ...    Exner = cp * ( p(l) / preff ) ** kappa     ....
c
       unskap   = 1./ kappa
c
c      print *,omp_rank,'klon--->',klon
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l = 1, llmp1
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
        do ig0=1,klon
          i=index_i(ig0)
          j=index_j(ig0)
          zplev( ig0,l ) = pp(i,j,l)
        enddo
      ENDDO
c$OMP END DO NOWAIT
c
c

c   43. temperature naturelle (en K) et pressions milieux couches .
c   ---------------------------------------------------------------
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
        do ig0=1,klon
          i=index_i(ig0)
          j=index_j(ig0)
          pksurcp        = ppk(i,j,l) / cpp
          zplay(ig0,l)   = preff * pksurcp ** unskap
          ztfi(ig0,l)    = pteta(i,j,l)  * pksurcp
        enddo

      ENDDO
c$OMP END DO NOWAIT

c   43.bis traceurs
c   ---------------
c

      DO iq=1,nqtot
         iiq=niadv(iq)
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
           if ((l.eq.1).and.(iq.eq.1)) then
!             write(*,*) 'calfis 451 tmp: index_i,index_j=',
!     :       index_i(1),index_i(klon),index_j(1),index_j(klon)
           endif
           do ig0=1,klon
             i=index_i(ig0)
             j=index_j(ig0)
             zqfi(ig0,l,iq)  = pq(i,j,l,iiq)
           enddo !do ig0=1,klon    
         ENDDO !DO l=1,llm
c$OMP END DO NOWAIT	 
      ENDDO !DO iq=1,nqtot


      DO iq=1,2
        iiq=niadv(iq)           
        do ixt=1,ntraciso
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
           if ((l.eq.1).and.(iq.eq.1)) then
!             write(*,*) 'calfis 451 tmp: index_i,index_j=',
!     :       index_i(1),index_i(klon),index_j(1),index_j(klon)
           endif
           do ig0=1,klon
             i=index_i(ig0)
             j=index_j(ig0) 
             zxtfi(ig0,l,iq,ixt) = pxt(i,j,l,iiq,ixt)   
         enddo !do ig0=1,klon    
        ENDDO !DO l=1,llm
c$OMP END DO NOWAIT	            
        enddo ! do ixt=1,ntraciso  
      ENDDO !DO iq=1,2    
     


c   Geopotentiel calcule par rapport a la surface locale:
c   -----------------------------------------------------

      CALL gr_dyn_fi_p(llm,iip1,jjp1,klon,pphi,zphi)

      CALL gr_dyn_fi_p(1,iip1,jjp1,klon,pphis,zphis)

c$OMP BARRIER

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm
	 DO ig=1,klon
	   zphi(ig,l)=zphi(ig,l)-zphis(ig)
	 ENDDO
      ENDDO
c$OMP END DO NOWAIT
      

c
c   45. champ u:
c   ------------

      kstart=1
      kend=klon
      
      if (is_north_pole) kstart=2
      if (is_south_pole) kend=klon-1
      
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
      DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
!CDIR SPARSE
        do ig0=kstart,kend
          i=index_i(ig0)
          j=index_j(ig0)
          if (i==1) then
            zufi(ig0,l)= 0.5 *(  pucov(iim,j,l)/cu(iim,j)
     $                         + pucov(1,j,l)/cu(1,j) )
          else
            zufi(ig0,l)= 0.5*(  pucov(i-1,j,l)/cu(i-1,j) 
     $                       + pucov(i,j,l)/cu(i,j) )
          endif
        enddo
      ENDDO
c$OMP END DO NOWAIT

c   46.champ v:
c   -----------

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
        DO ig0=kstart,kend
          i=index_i(ig0)
          j=index_j(ig0)
          zvfi(ig0,l)= 0.5 *(  pvcov(i,j-1,l)/cv(i,j-1) 
     $                       + pvcov(i,j,l)/cv(i,j) )
    
         ENDDO
      ENDDO
c$OMP END DO NOWAIT

c   47. champs de vents aux pole nord   
c   ------------------------------
c        U = 1 / pi  *  integrale [ v * cos(long) * d long ]
c        V = 1 / pi  *  integrale [ v * sin(long) * d long ]

      if (is_north_pole) then
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
        DO l=1,llm

           z1(1)   =(rlonu(1)-rlonu(iim)+2.*pi)*pvcov(1,1,l)/cv(1,1)
           DO i=2,iim
              z1(i)   =(rlonu(i)-rlonu(i-1))*pvcov(i,1,l)/cv(i,1)
           ENDDO
  
           DO i=1,iim
              zcos(i)   = COS(rlonv(i))*z1(i)
              zsin(i)   = SIN(rlonv(i))*z1(i)
           ENDDO
  
           zufi(1,l)  = SSUM(iim,zcos,1)/pi
           zvfi(1,l)  = SSUM(iim,zsin,1)/pi
  
        ENDDO
c$OMP END DO NOWAIT      
      endif


c   48. champs de vents aux pole sud:
c   ---------------------------------
c        U = 1 / pi  *  integrale [ v * cos(long) * d long ]
c        V = 1 / pi  *  integrale [ v * sin(long) * d long ]

      if (is_south_pole) then
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
        DO l=1,llm
  
         z1(1)   =(rlonu(1)-rlonu(iim)+2.*pi)*pvcov(1,jjm,l)/cv(1,jjm)
           DO i=2,iim
             z1(i)   =(rlonu(i)-rlonu(i-1))*pvcov(i,jjm,l)/cv(i,jjm)
	   ENDDO
  
           DO i=1,iim
              zcos(i)    = COS(rlonv(i))*z1(i)
              zsin(i)    = SIN(rlonv(i))*z1(i)
	   ENDDO
  
           zufi(klon,l)  = SSUM(iim,zcos,1)/pi
           zvfi(klon,l)  = SSUM(iim,zsin,1)/pi
        ENDDO
c$OMP END DO NOWAIT       
      endif


      IF (is_sequential.and.(planet_type=="earth")) THEN
! PVtheta calls tetalevel, which is in the physics
cIM calcul PV a teta=350, 380, 405K
        CALL PVtheta(ngridmx,llm,pucov,pvcov,pteta,
     $           ztfi,zplay,zplev,
     $           ntetaSTD,rtetaSTD,PVteta)
c
      ENDIF

c On change de grille, dynamique vers physiq, pour le flux de masse verticale
      CALL gr_dyn_fi_p(llm,iip1,jjp1,klon,flxw,flxwfi)

c-----------------------------------------------------------------------
c   Appel de la physique:
c   ---------------------


c$OMP BARRIER
      if (first_omp) then
        klon=klon_omp

        allocate(zplev_omp(klon,llm+1))
        allocate(zplay_omp(klon,llm))
        allocate(zphi_omp(klon,llm))
        allocate(zphis_omp(klon))
        allocate(presnivs_omp(llm))
        allocate(zufi_omp(klon,llm))
        allocate(zvfi_omp(klon,llm))
        allocate(ztfi_omp(klon,llm))
        allocate(zqfi_omp(klon,llm,nqtot))
        allocate(zdufi_omp(klon,llm))
        allocate(zdvfi_omp(klon,llm))
        allocate(zdtfi_omp(klon,llm))
        allocate(zdqfi_omp(klon,llm,nqtot))
        allocate(zdufic_omp(klon,llm))
        allocate(zdvfic_omp(klon,llm))
        allocate(zdtfic_omp(klon,llm))
        allocate(zdqfic_omp(klon,llm,nqtot))
        allocate(zdpsrf_omp(klon))
        allocate(flxwfi_omp(klon,llm))
        allocate(zxtfi_omp(ntraciso,klon,llm,2))
        allocate(zdxtfi_omp(ntraciso,klon,llm,2))
        allocate(zdxtfic_omp(ntraciso,klon,llm,2))
	first_omp=.false.
      endif
       
	   
      klon=klon_omp
      offset=klon_omp_begin-1
      
      do l=1,llm+1
        do i=1,klon
          zplev_omp(i,l)=zplev(offset+i,l)
	enddo 
      enddo
	  
       do l=1,llm
        do i=1,klon  
	  zplay_omp(i,l)=zplay(offset+i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  zphi_omp(i,l)=zphi(offset+i,l)
	enddo 
      enddo
	
      do i=1,klon
	zphis_omp(i)=zphis(offset+i)
      enddo 
     
	
      do l=1,llm
        presnivs_omp(l)=presnivs(l)
      enddo 
	
      do l=1,llm
        do i=1,klon
	  zufi_omp(i,l)=zufi(offset+i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  zvfi_omp(i,l)=zvfi(offset+i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  ztfi_omp(i,l)=ztfi(offset+i,l)
	enddo 
      enddo
	
      do iq=1,nqtot
        do l=1,llm
          do i=1,klon
            zqfi_omp(i,l,iq)=zqfi(offset+i,l,iq)
	  enddo
	enddo 
      enddo
      do ixt=1,ntraciso
       do iq=1,2
        do l=1,llm
          do i=1,klon
            zxtfi_omp(ixt,i,l,iq)=zxtfi(offset+i,l,iq,ixt)
           enddo !do ixt=1,ntraciso
	  enddo !do l=1,llm
	enddo  !do iq=1,2
      enddo !do ixt=1,ntraciso
	
      do l=1,llm
        do i=1,klon
	  zdufi_omp(i,l)=zdufi(offset+i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  zdvfi_omp(i,l)=zdvfi(offset+i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
          zdtfi_omp(i,l)=zdtfi(offset+i,l)
	enddo 
      enddo
	
      do iq=1,nqtot
        do l=1,llm
          do i=1,klon
	    zdqfi_omp(i,l,iq)=zdqfi(offset+i,l,iq)
	  enddo 
        enddo
      enddo
      do ixt=1,ntraciso
      do iq=1,2
        do l=1,llm
          do i=1,klon
            zdxtfi_omp(ixt,i,l,iq)=zdxtfi(offset+i,l,iq,ixt)
           enddo
	  enddo
	enddo 
      enddo !do ixt=1,ntraciso
      	
      do i=1,klon
	zdpsrf_omp(i)=zdpsrf(offset+i)
      enddo 

      do l=1,llm
        do i=1,klon
          flxwfi_omp(i,l)=flxwfi(offset+i,l)
	enddo 
      enddo
      
c$OMP BARRIER
      

!$OMP MASTER
!      write(lunout,*) 'PHYSIQUE AVEC NSPLIT_PHYS=',nsplit_phys
!$OMP END MASTER
      zdt_split=dtphys/nsplit_phys
      zdufic_omp(:,:)=0.
      zdvfic_omp(:,:)=0.
      zdtfic_omp(:,:)=0.
      zdqfic_omp(:,:,:)=0.
      zdxtfic_omp(:,:,:,:)=0.

      do isplit=1,nsplit_phys

         jH_cur_split=jH_cur+(isplit-1) * dtvr / (daysec *nsplit_phys)
         debut_split=debut.and.isplit==1
         lafin_split=lafin.and.isplit==nsplit_phys


      if (planet_type=="earth") then

      CALL physiq (klon,
     .             llm,
     .             debut_split,
     .             lafin_split,
     .             jD_cur,
     .             jH_cur_split,
     .             zdt_split,
     .             zplev_omp,
     .             zplay_omp,
     .             zphi_omp,
     .             zphis_omp,
     .             presnivs_omp,
     .             clesphy0,
     .             zufi_omp,
     .             zvfi_omp,
     .             ztfi_omp,
     .             zqfi_omp,
c#ifdef INCA
     .             flxwfi_omp,
c#endif
     .             zdufi_omp,
     .             zdvfi_omp,
     .             zdtfi_omp,
     .             zdqfi_omp,
     .             zdpsrf_omp,
cIM diagnostique PVteta, Amip2          
     .             pducov,
     .             PVteta
     :            , zxtfi_omp ! xtx
     :            , zdxtfi_omp ! d_xtx     
     :            , rlat 
     :                  )

      else if ( planet_type=="generic" ) then

      CALL physiq (klon,     !! ngrid
     .             llm,            !! nlayer
     .             nqtot,          !! nq
     .             tname,          !! tracer names from dynamical core (given in infotrac)
     .             debut_split,    !! firstcall 
     .             lafin_split,    !! lastcall
     .             jD_cur,         !! pday. see leapfrog_p 
     .             jH_cur_split,   !! ptime "fraction of day"
     .             zdt_split,      !! ptimestep
     .             zplev_omp,  !! pplev
     .             zplay_omp,  !! pplay
     .             zphi_omp,   !! pphi
     .             zufi_omp,   !! pu
     .             zvfi_omp,   !! pv
     .             ztfi_omp,   !! pt
     .             zqfi_omp,   !! pq
     .             flxwfi_omp, !! pw !! or 0. anyway this is for diagnostic. not used in physiq.
     .             zdufi_omp,  !! pdu
     .             zdvfi_omp,  !! pdv
     .             zdtfi_omp,  !! pdt
     .             zdqfi_omp,  !! pdq
     .             zdpsrf_omp, !! pdpsrf
     .             tracerdyn)      !! tracerdyn <-- utilite ???

      endif ! of if (planet_type=="earth")


         zufi_omp(:,:)=zufi_omp(:,:)+zdufi_omp(:,:)*zdt_split
         zvfi_omp(:,:)=zvfi_omp(:,:)+zdvfi_omp(:,:)*zdt_split
         ztfi_omp(:,:)=ztfi_omp(:,:)+zdtfi_omp(:,:)*zdt_split
         zqfi_omp(:,:,:)=zqfi_omp(:,:,:)+zdqfi_omp(:,:,:)*zdt_split
         zxtfi_omp(:,:,:,:)=zxtfi_omp(:,:,:,:)+zdxtfi_omp(:,:,:,:)
     :          *zdt_split     

         zdufic_omp(:,:)=zdufic_omp(:,:)+zdufi_omp(:,:)
         zdvfic_omp(:,:)=zdvfic_omp(:,:)+zdvfi_omp(:,:)
         zdtfic_omp(:,:)=zdtfic_omp(:,:)+zdtfi_omp(:,:)
         zdqfic_omp(:,:,:)=zdqfic_omp(:,:,:)+zdqfi_omp(:,:,:)
         zdxtfic_omp(:,:,:,:)=zdxtfic_omp(:,:,:,:)+zdxtfi_omp(:,:,:,:)
      enddo

! of #ifdef 1

      zdufi_omp(:,:)=zdufic_omp(:,:)/nsplit_phys
      zdvfi_omp(:,:)=zdvfic_omp(:,:)/nsplit_phys
      zdtfi_omp(:,:)=zdtfic_omp(:,:)/nsplit_phys
      zdqfi_omp(:,:,:)=zdqfic_omp(:,:,:)/nsplit_phys
      zdxtfi_omp(:,:,:,:)=zdxtfic_omp(:,:,:,:)/nsplit_phys

c$OMP BARRIER

      do l=1,llm+1
        do i=1,klon
          zplev(offset+i,l)=zplev_omp(i,l)
	enddo 
      enddo
	  
       do l=1,llm
        do i=1,klon  
	  zplay(offset+i,l)=zplay_omp(i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  zphi(offset+i,l)=zphi_omp(i,l)
	enddo 
      enddo
	

      do i=1,klon
	zphis(offset+i)=zphis_omp(i)
      enddo 
     
	
      do l=1,llm
        presnivs(l)=presnivs_omp(l)
      enddo 
	
      do l=1,llm
        do i=1,klon
	  zufi(offset+i,l)=zufi_omp(i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  zvfi(offset+i,l)=zvfi_omp(i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  ztfi(offset+i,l)=ztfi_omp(i,l)
	enddo 
      enddo
	
      do iq=1,nqtot
        do l=1,llm
          do i=1,klon
            zqfi(offset+i,l,iq)=zqfi_omp(i,l,iq)
	  enddo
	enddo 
      enddo
      do ixt=1,ntraciso
       do iq=1,2
        do l=1,llm
          do i=1,klon
            zxtfi(offset+i,l,iq,ixt)=zxtfi_omp(ixt,i,l,iq)
	  enddo !do i=1,klon
	enddo  !do l=1,llm
       enddo !do iq=1,2
      enddo !do ixt=1,ntraciso

      do l=1,llm
        do i=1,klon
	  zdufi(offset+i,l)=zdufi_omp(i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
	  zdvfi(offset+i,l)=zdvfi_omp(i,l)
	enddo 
      enddo
	
      do l=1,llm
        do i=1,klon
          zdtfi(offset+i,l)=zdtfi_omp(i,l)
	enddo 
      enddo
	
      do iq=1,nqtot
        do l=1,llm
          do i=1,klon
	    zdqfi(offset+i,l,iq)=zdqfi_omp(i,l,iq)
	  enddo 
        enddo
      enddo
      do ixt=1,ntraciso
       do iq=1,2
        do l=1,llm
          do i=1,klon
            zdxtfi(offset+i,l,iq,ixt)=zdxtfi_omp(ixt,i,l,iq)
          enddo  !do i=1,klon
        enddo !do l=1,llm
       enddo !do iq=1,2
      enddo !do ixt=1,ntraciso
     
      do i=1,klon
	zdpsrf(offset+i)=zdpsrf_omp(i)
      enddo 
      

      klon=klon_mpi
500   CONTINUE
c$OMP BARRIER

c$OMP MASTER
      call stop_timer(timer_physic)
c$OMP END MASTER

      IF (using_mpi) THEN
            
      if (MPI_rank>0) then

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)       
       DO l=1,llm      
        du_send(1:iim,l)=zdufi(1:iim,l)
        dv_send(1:iim,l)=zdvfi(1:iim,l)
       ENDDO
c$OMP END DO NOWAIT       

c$OMP BARRIER
c$OMP MASTER
!$OMP CRITICAL (MPI)
        call MPI_ISSEND(du_send,iim*llm,MPI_REAL8,MPI_Rank-1,401,
     &                   COMM_LMDZ,Req(1),ierr)
        call MPI_ISSEND(dv_send,iim*llm,MPI_REAL8,MPI_Rank-1,402,
     &                  COMM_LMDZ,Req(2),ierr)
!$OMP END CRITICAL (MPI)
c$OMP END MASTER
c$OMP BARRIER
     
      endif
   
      if (MPI_rank<MPI_Size-1) then
c$OMP BARRIER
c$OMP MASTER      
!$OMP CRITICAL (MPI)
        call MPI_IRECV(du_recv,iim*llm,MPI_REAL8,MPI_Rank+1,401,
     &                 COMM_LMDZ,Req(3),ierr)
        call MPI_IRECV(dv_recv,iim*llm,MPI_REAL8,MPI_Rank+1,402,
     &                 COMM_LMDZ,Req(4),ierr)
!$OMP END CRITICAL (MPI)
c$OMP END MASTER
      endif

c$OMP BARRIER


c$OMP MASTER    
!$OMP CRITICAL (MPI)
      if (MPI_rank>0 .and. MPI_rank< MPI_Size-1) then
        call MPI_WAITALL(4,Req(1),Status,ierr)
      else if (MPI_rank>0) then
        call MPI_WAITALL(2,Req(1),Status,ierr)
      else if (MPI_rank <MPI_Size-1) then
        call MPI_WAITALL(2,Req(3),Status,ierr)
      endif
!$OMP END CRITICAL (MPI)
c$OMP END MASTER

c$OMP BARRIER     

      ENDIF ! using_mpi
      
      
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm
            
        zdufi2(1:klon,l)=zdufi(1:klon,l)
        zdufi2(klon+1:klon+iim,l)=du_recv(1:iim,l)
            
        zdvfi2(1:klon,l)=zdvfi(1:klon,l)
        zdvfi2(klon+1:klon+iim,l)=dv_recv(1:iim,l) 
  
         pdhfi(:,jj_begin,l)=0
         pdqfi(:,jj_begin,l,:)=0
         pdufi(:,jj_begin,l)=0
         pdvfi(:,jj_begin,l)=0
         pdxtfi(:,jj_begin,l,:,:)=0
         
         if (.not. is_south_pole) then
           pdhfi(:,jj_end,l)=0
           pdqfi(:,jj_end,l,:)=0
           pdufi(:,jj_end,l)=0
           pdvfi(:,jj_end,l)=0
         endif
         pdxtfi(:,jj_end,l,:,:)=0
      
       ENDDO 
c$OMP END DO NOWAIT

c$OMP MASTER
       pdpsfi(:,jj_begin)=0    
       if (.not. is_south_pole) then
	 pdpsfi(:,jj_end)=0
       endif
c$OMP END MASTER
c-----------------------------------------------------------------------
c   transformation des tendances physiques en tendances dynamiques:
c   ---------------------------------------------------------------

c  tendance sur la pression :
c  -----------------------------------
      CALL gr_fi_dyn_p(1,klon,iip1,jjp1,zdpsrf,pdpsfi)
c
c   62. enthalpie potentielle
c   ---------------------
      
      kstart=1
      kend=klon

      if (is_north_pole) kstart=2
      if (is_south_pole)  kend=klon-1

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm

!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
!cdir NODEP
        do ig0=kstart,kend
          i=index_i(ig0)
          j=index_j(ig0)
          pdhfi(i,j,l) = cpp * zdtfi(ig0,l) / ppk(i,j,l)
          if (i==1) pdhfi(iip1,j,l) =  cpp * zdtfi(ig0,l) / ppk(i,j,l)
         enddo          

        if (is_north_pole) then
            DO i=1,iip1
              pdhfi(i,1,l)    = cpp *  zdtfi(1,l)      / ppk(i, 1  ,l)
            enddo
        endif
        
        if (is_south_pole) then
            DO i=1,iip1
              pdhfi(i,jjp1,l) = cpp *  zdtfi(klon,l)/ ppk(i,jjp1,l)
            ENDDO
        endif
      ENDDO
c$OMP END DO NOWAIT
      
c   62. humidite specifique
c   ---------------------
! Ehouarn: removed this useless bit: was overwritten at step 63 anyways
!      DO iq=1,nqtot
!c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
!         DO l=1,llm
!!!cdir NODEP 
!           do ig0=kstart,kend
!             i=index_i(ig0)
!             j=index_j(ig0)
!             pdqfi(i,j,l,iq) = zdqfi(ig0,l,iq) 
!             if (i==1) pdqfi(iip1,j,l,iq) = zdqfi(ig0,l,iq) 
!           enddo
!           
!           if (is_north_pole) then
!             do i=1,iip1
!               pdqfi(i,1,l,iq)    = zdqfi(1,l,iq)             
!             enddo
!           endif
!           
!           if (is_south_pole) then
!             do i=1,iip1
!               pdqfi(i,jjp1,l,iq) = zdqfi(klon,l,iq) 
!             enddo
!           endif
!         ENDDO
!c$OMP END DO NOWAIT
!      ENDDO

c   63. traceurs
c   ------------
C     initialisation des tendances

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm
        pdqfi(:,:,l,:)=0.
        pdxtfi(:,:,l,:,:)=0.
      ENDDO
c$OMP END DO NOWAIT	 

C
!cdir NODEP
      DO iq=1,nqtot
         iiq=niadv(iq)
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
!cdir NODEP           
	     DO ig0=kstart,kend
              i=index_i(ig0)
              j=index_j(ig0)
              pdqfi(i,j,l,iiq) = zdqfi(ig0,l,iq)
              if (i==1) pdqfi(iip1,j,l,iiq) = zdqfi(ig0,l,iq)
            ENDDO !DO ig0=kstart,kend
	    
	    IF (is_north_pole) then
	      DO i=1,iip1
                pdqfi(i,1,l,iiq)    = zdqfi(1,l,iq)
	      ENDDO
	    ENDIF
	    
	    IF (is_south_pole) then
	      DO i=1,iip1
                pdqfi(i,jjp1,l,iiq) = zdqfi(klon,l,iq)
	      ENDDO
	    ENDIF
	    
         ENDDO
c$OMP END DO NOWAIT	 
      ENDDO
      
!cdir NODEP
      DO iq=1,2
         iiq=niadv(iq)
         do ixt=1,ntraciso
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
!cdir NODEP           
	     DO ig0=kstart,kend
              i=index_i(ig0)
              j=index_j(ig0)
                pdxtfi(i,j,l,iiq,ixt) = zdxtfi(ig0,l,iq,ixt)
                if (i==1) pdxtfi(iip1,j,l,iiq,ixt) = 
     :                 zdxtfi(ig0,l,iq,ixt)
            ENDDO !DO ig0=kstart,kend
	    
	    IF (is_north_pole) then
	      DO i=1,iip1
                  pdxtfi(i,1,l,iiq,ixt)    = zdxtfi(1,l,iq,ixt)
	      ENDDO
	    ENDIF
	    
	    IF (is_south_pole) then
	      DO i=1,iip1
                  pdxtfi(i,jjp1,l,iiq,ixt) = zdxtfi(klon,l,iq,ixt)  
	      ENDDO
	    ENDIF
         ENDDO
c$OMP END DO NOWAIT
         enddo ! do ixt=1,ntraciso	 
      ENDDO !DO iq=1,2


c   65. champ u:
c   ------------
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
!cdir NODEP
         do ig0=kstart,kend
           i=index_i(ig0)
           j=index_j(ig0)
           
           if (i/=iim) then
             pdufi(i,j,l)=0.5*(zdufi2(ig0,l)+zdufi2(ig0+1,l))*cu(i,j)
           endif
           
           if (i==1) then
              pdufi(iim,j,l)=0.5*(  zdufi2(ig0,l)
     $                            + zdufi2(ig0+iim-1,l))*cu(iim,j)
             pdufi(iip1,j,l)=0.5*(zdufi2(ig0,l)+zdufi2(ig0+1,l))*cu(i,j)
           endif
         
         enddo
         
         if (is_north_pole) then
           DO i=1,iip1
            pdufi(i,1,l)    = 0.
           ENDDO
         endif
         
         if (is_south_pole) then
           DO i=1,iip1
            pdufi(i,jjp1,l) = 0.
           ENDDO
         endif
         
      ENDDO
c$OMP END DO NOWAIT

c   67. champ v:
c   ------------

      kstart=1
      kend=klon

      if (is_north_pole) kstart=2
      if (is_south_pole)  kend=klon-1-iim
      
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
      DO l=1,llm
!CDIR ON_ADB(index_i)
!CDIR ON_ADB(index_j) 
!cdir NODEP
        do ig0=kstart,kend
           i=index_i(ig0)
           j=index_j(ig0)
           pdvfi(i,j,l)=0.5*(zdvfi2(ig0,l)+zdvfi2(ig0+iim,l))*cv(i,j)
           if (i==1) pdvfi(iip1,j,l) = 0.5*(zdvfi2(ig0,l)+
     $	                                    zdvfi2(ig0+iim,l))
     $				          *cv(i,j)
        enddo
         
      ENDDO
c$OMP END DO NOWAIT


c   68. champ v pres des poles:
c   ---------------------------
c      v = U * cos(long) + V * SIN(long)

      if (is_north_pole) then

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
        DO l=1,llm

          DO i=1,iim
            pdvfi(i,1,l)=
     $      zdufi(1,l)*COS(rlonv(i))+zdvfi(1,l)*SIN(rlonv(i))
       
            pdvfi(i,1,l)=
     $      0.5*(pdvfi(i,1,l)+zdvfi(i+1,l))*cv(i,1)
          ENDDO

          pdvfi(iip1,1,l)  = pdvfi(1,1,l)

        ENDDO
c$OMP END DO NOWAIT

      endif    
      
      if (is_south_pole) then

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
         DO l=1,llm
  
           DO i=1,iim
              pdvfi(i,jjm,l)=zdufi(klon,l)*COS(rlonv(i))
     $        +zdvfi(klon,l)*SIN(rlonv(i))

              pdvfi(i,jjm,l)=
     $        0.5*(pdvfi(i,jjm,l)+zdvfi(klon-iip1+i,l))*cv(i,jjm)
           ENDDO

           pdvfi(iip1,jjm,l)= pdvfi(1,jjm,l)

        ENDDO
c$OMP END DO NOWAIT
     
      endif
c-----------------------------------------------------------------------

700   CONTINUE
 
      firstcal = .FALSE.

! of #ifdef 1
      RETURN
      END
