!
! $Id: albedo.F 1403 2010-07-01 09:02:53Z fairhead $
!
c
c
      SUBROUTINE alboc(rjour,rlat,albedo)
      USE dimphy
      IMPLICIT none
c======================================================================
c Auteur(s): Z.X. Li (LMD/CNRS) (adaptation du GCM du LMD)
c Date: le 16 mars 1995
c Objet: Calculer l'albedo sur l'ocean
c Methode: Integrer numeriquement l'albedo pendant une journee
c
c Arguments;
c rjour (in,R)  : jour dans l'annee (a compter du 1 janvier)
c rlat (in,R)   : latitude en degre
c albedo (out,R): albedo obtenu (de 0 a 1)
c======================================================================
cym#include "dimensions.h"
cym#include "dimphy.h"
!
! $Header$
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
! A1.0 Fundamental constants
      REAL RPI,RCLUM,RHPLA,RKBOL,RNAVO
! A1.1 Astronomical constants
      REAL RDAY,REA,REPSM,RSIYEA,RSIDAY,ROMEGA
! A1.1.bis Constantes concernant l'orbite de la Terre:
      REAL R_ecc, R_peri, R_incl
! A1.2 Geoide
      REAL RA,RG,R1SA
! A1.3 Radiation
!     REAL RSIGMA,RI0
      REAL RSIGMA
! A1.4 Thermodynamic gas phase
      REAL R,RMD,RMO3,RMV,RD,RV,RCPD,RCPV,RCVD,RCVV
      REAL RKAPPA,RETV
! A1.5,6 Thermodynamic liquid,solid phases
      REAL RCW,RCS
! A1.7 Thermodynamic transition of phase
      REAL RLVTT,RLSTT,RLMLT,RTT,RATM
! A1.8 Curve of saturation
      REAL RESTT,RALPW,RBETW,RGAMW,RALPS,RBETS,RGAMS
      REAL RALPD,RBETD,RGAMD
!
      COMMON/YOMCST/RPI   ,RCLUM ,RHPLA ,RKBOL ,RNAVO                   &
     &      ,RDAY  ,REA   ,REPSM ,RSIYEA,RSIDAY,ROMEGA                  &
     &      ,R_ecc, R_peri, R_incl                                      &
     &      ,RA    ,RG    ,R1SA                                         &
     &      ,RSIGMA                                                     &
     &      ,R     ,RMD   ,RMO3  ,RMV   ,RD    ,RV    ,RCPD             &
     &      ,RCPV  ,RCVD  ,RCVV  ,RKAPPA,RETV                           &
     &      ,RCW   ,RCS                                                 &
     &      ,RLVTT ,RLSTT ,RLMLT ,RTT   ,RATM                           &
     &      ,RESTT ,RALPW ,RBETW ,RGAMW ,RALPS ,RBETS ,RGAMS            &
     &      ,RALPD ,RBETD ,RGAMD
!    ------------------------------------------------------------------
!$OMP THREADPRIVATE(/YOMCST/)

!
! $Id: clesphys.h 1750 2013-04-25 15:27:27Z fairhead $
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation 
!                 (les placer en colonne 6 et en colonne 73)
!
!..include cles_phys.h
!
       LOGICAL cycle_diurne,soil_model,new_oliq,ok_orodr,ok_orolf 
       LOGICAL ok_limitvrai
       INTEGER nbapp_rad, iflag_con
       REAL co2_ppm, co2_ppm0, solaire
       REAL(kind=8) RCO2, RCH4, RN2O, RCFC11, RCFC12  
       REAL(kind=8) RCO2_act, RCH4_act, RN2O_act, RCFC11_act, RCFC12_act  
       REAL(kind=8) CH4_ppb, N2O_ppb, CFC11_ppt, CFC12_ppt
!IM ajout CFMIP2/CMIP5
       REAL co2_ppm_per
       LOGICAL ok_4xCO2atm
       REAL(kind=8) RCO2_per,RCH4_per,RN2O_per,RCFC11_per,RCFC12_per
       REAL(kind=8) CH4_ppb_per,N2O_ppb_per,CFC11_ppt_per,CFC12_ppt_per

!OM ---> correction du bilan d'eau global
!OM Correction sur precip KE
       REAL cvl_corr
!OM Fonte calotte dans bilan eau
       LOGICAL ok_lic_melt

!IM simulateur ISCCP 
       INTEGER top_height, overlap
!IM seuils cdrm, cdrh
       REAL cdmmax, cdhmax
!IM param. stabilite s/ terres et en dehors
       REAL ksta, ksta_ter
!IM ok_kzmin : clef calcul Kzmin dans la CL de surface cf FH
       LOGICAL ok_kzmin
!IM, MAFo fmagic, pmagic : parametres - additionnel et multiplicatif - 
!                          pour regler l albedo sur ocean
       REAL fmagic, pmagic
! Hauteur (imposee) du contenu en eau du sol
           REAL qsol0
! Frottement au sol (Cdrag)
       Real f_cdrag_ter,f_cdrag_oce
! Rugoro
       Real f_rugoro

!IM lev_histhf  : niveau sorties 6h
!IM lev_histday : niveau sorties journalieres
!IM lev_histmth : niveau sorties mensuelles
!IM lev_histdayNMC : on peut sortir soit sur 8 (comme AR5) ou bien 
!                    sur 17 niveaux de pression
       INTEGER lev_histhf, lev_histday, lev_histmth
       INTEGER lev_histdayNMC
       Integer lev_histins, lev_histLES  
!IM ok_histNMC  : sortie fichiers niveaux de pression (histmthNMC, histdayNMC, histhfNMC)
!IM freq_outNMC : frequences de sortie fichiers niveaux de pression (histmthNMC, histdayNMC, histhfNMC)
!IM freq_calNMC : frequences de calcul fis. hist*NMC.nc
!IM pasphys : pas de temps de physique (secondes)
       REAL pasphys
       LOGICAL ok_histNMC(3)
       REAL freq_outNMC(3) , freq_calNMC(3)
       CHARACTER(len=4) type_run
! aer_type: pour utiliser un fichier constant dans readaerosol 
       CHARACTER*8 :: aer_type 
       LOGICAL ok_isccp, ok_regdyn
       REAL lonmin_ins, lonmax_ins, latmin_ins, latmax_ins
       REAL ecrit_ins, ecrit_hf, ecrit_day
       REAL ecrit_mth, ecrit_tra, ecrit_reg 
       REAL ecrit_LES
       REAL freq_ISCCP, ecrit_ISCCP
       REAL freq_COSP
       LOGICAL :: ok_cosp,ok_mensuelCOSP,ok_journeCOSP,ok_hfCOSP
       INTEGER :: ip_ebil_phy, iflag_rrtm
       LOGICAL :: ok_strato
       LOGICAL :: ok_hines
       LOGICAL :: ok_qch4

       COMMON/clesphys/cycle_diurne, soil_model, new_oliq,              &
     &     ok_orodr, ok_orolf, ok_limitvrai, nbapp_rad, iflag_con       &
     &     , co2_ppm, solaire                                           &
     &     , RCO2, RCH4, RN2O, RCFC11, RCFC12                           &
     &     , RCO2_act, RCH4_act, RN2O_act, RCFC11_act, RCFC12_act       &
     &     , ok_4xCO2atm                                                & 
     &     , RCO2_per, RCH4_per, RN2O_per, RCFC11_per, RCFC12_per       &
     &     , CH4_ppb, N2O_ppb, CFC11_ppt, CFC12_ppt                     &
     &     , CH4_ppb_per, N2O_ppb_per, CFC11_ppt_per, CFC12_ppt_per     &
     &     , top_height, overlap, cdmmax, cdhmax, ksta, ksta_ter        &
     &     , ok_kzmin, fmagic, pmagic                                   &
     &     , f_cdrag_ter,f_cdrag_oce,f_rugoro                           &
     &     , lev_histhf, lev_histday, lev_histmth                       &
     &     , lev_histins, lev_histLES, lev_histdayNMC                   &
     &     , pasphys, ok_histNMC, freq_outNMC, freq_calNMC              &
     &     , type_run, ok_isccp, ok_regdyn, ok_cosp                     &
     &     , ok_mensuelCOSP,ok_journeCOSP,ok_hfCOSP                     &
     &     , lonmin_ins, lonmax_ins, latmin_ins, latmax_ins             &
     &     , ecrit_ins, ecrit_hf, ecrit_day                             &
     &     , ecrit_mth, ecrit_tra, ecrit_reg                            &
     &     , freq_ISCCP, ecrit_ISCCP, freq_COSP, ip_ebil_phy            &
     &     , ok_lic_melt, cvl_corr, aer_type                            &
     &     , qsol0, iflag_rrtm, ok_strato,ok_hines,ecrit_LES            &
     &     , co2_ppm0, ok_qch4
     
       save /clesphys/
!$OMP THREADPRIVATE(/clesphys/)
 
c
c fmagic -> clesphys.h/.inc
c     REAL fmagic ! un facteur magique pour regler l'albedo
ccc      PARAMETER (fmagic=0.7)
cccIM => a remplacer  
c       PARAMETER (fmagic=1.32)
c       PARAMETER (fmagic=1.0)
c       PARAMETER (fmagic=0.7)
      INTEGER npts ! il controle la precision de l'integration
      PARAMETER (npts=120) ! 120 correspond a l'interval 6 minutes
c
      REAL rlat(klon), rjour, albedo(klon)
      REAL zdist, zlonsun, zpi, zdeclin
      REAL rmu,alb, srmu, salb, fauxo, aa, bb
      INTEGER i, k
cccIM
      LOGICAL ancien_albedo
      PARAMETER(ancien_albedo=.FALSE.) 
c     SAVE albedo
c
      IF ( ancien_albedo ) THEN
c
      zpi = 4. * ATAN(1.)
c
c Calculer la longitude vraie de l'orbite terrestre:
      CALL orbite(rjour,zlonsun,zdist)
c
c Calculer la declinaison du soleil (qui varie entre + et - R_incl):
      zdeclin = ASIN(SIN(zlonsun*zpi/180.0)*SIN(R_incl*zpi/180.0))
c
      DO 999 i=1,klon
      aa = SIN(rlat(i)*zpi/180.0) * SIN(zdeclin)
      bb = COS(rlat(i)*zpi/180.0) * COS(zdeclin)
c
c Midi local (angle du temps = 0.0):
      rmu = aa + bb * COS(0.0)
      rmu = MAX(0.0, rmu)
      fauxo = (1.47-ACOS(rmu))/.15
      alb = 0.03+0.630/(1.+fauxo*fauxo)
      srmu = rmu
      salb = alb * rmu
c
c Faire l'integration numerique de midi a minuit (le facteur 2
c prend en compte l'autre moitie de la journee):
      DO k = 1, npts
         rmu = aa + bb * COS(REAL(k)/REAL(npts)*zpi)
         rmu = MAX(0.0, rmu)
         fauxo = (1.47-ACOS(rmu))/.15
         alb = 0.03+0.630/(1.+fauxo*fauxo)
         srmu = srmu + rmu * 2.0
         salb = salb + alb*rmu * 2.0
      ENDDO
      IF (srmu .NE. 0.0) THEN
         albedo(i) = salb / srmu * fmagic+pmagic
      ELSE ! nuit polaire (on peut prendre une valeur quelconque)
         albedo(i) = fmagic
      ENDIF
  999 CONTINUE
c
c nouvel albedo 
c
      ELSE
c
      zpi = 4. * ATAN(1.)
c
c Calculer la longitude vraie de l'orbite terrestre:
      CALL orbite(rjour,zlonsun,zdist)
c
c Calculer la declinaison du soleil (qui varie entre + et - R_incl):
      zdeclin = ASIN(SIN(zlonsun*zpi/180.0)*SIN(R_incl*zpi/180.0))
c
      DO 1999 i=1,klon
      aa = SIN(rlat(i)*zpi/180.0) * SIN(zdeclin)
      bb = COS(rlat(i)*zpi/180.0) * COS(zdeclin)
c
c Midi local (angle du temps = 0.0):
      rmu = aa + bb * COS(0.0)
      rmu = MAX(0.0, rmu)
cIM cf. PB  alb = 0.058/(rmu + 0.30)
c     alb = 0.058/(rmu + 0.30) * 1.5
      alb = 0.058/(rmu + 0.30) * 1.2
c     alb = 0.058/(rmu + 0.30) * 1.3
      srmu = rmu
      salb = alb * rmu
c
c Faire l'integration numerique de midi a minuit (le facteur 2
c prend en compte l'autre moitie de la journee):
      DO k = 1, npts
         rmu = aa + bb * COS(REAL(k)/REAL(npts)*zpi)
         rmu = MAX(0.0, rmu)
cIM cf. PB      alb = 0.058/(rmu + 0.30)
c        alb = 0.058/(rmu + 0.30) * 1.5
         alb = 0.058/(rmu + 0.30) * 1.2
c        alb = 0.058/(rmu + 0.30) * 1.3
         srmu = srmu + rmu * 2.0
         salb = salb + alb*rmu * 2.0
      ENDDO
      IF (srmu .NE. 0.0) THEN
         albedo(i) = salb / srmu * fmagic+pmagic
      ELSE ! nuit polaire (on peut prendre une valeur quelconque)
         albedo(i) = fmagic
      ENDIF
1999  CONTINUE
      ENDIF
      RETURN
      END
c=====================================================================
      SUBROUTINE alboc_cd(rmu0,albedo)
      USE dimphy
      IMPLICIT none
c======================================================================
c Auteur(s): Z.X. Li (LMD/CNRS)
c date: 19940624
c Calculer l'albedo sur l'ocean en fonction de l'angle zenithal moyen
c Formule due a Larson and Barkstrom (1977) Proc. of the symposium
C on radiation in the atmosphere, 19-28 August 1976, science Press,
C 1977 pp 451-453, ou These de 3eme cycle de Sylvie Joussaume.
c
c Arguments
c rmu0    (in): cosinus de l'angle solaire zenithal
c albedo (out): albedo de surface de l'ocean
c======================================================================
cym#include "dimensions.h"
cym#include "dimphy.h"

!
! $Id: clesphys.h 1750 2013-04-25 15:27:27Z fairhead $
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation 
!                 (les placer en colonne 6 et en colonne 73)
!
!..include cles_phys.h
!
       LOGICAL cycle_diurne,soil_model,new_oliq,ok_orodr,ok_orolf 
       LOGICAL ok_limitvrai
       INTEGER nbapp_rad, iflag_con
       REAL co2_ppm, co2_ppm0, solaire
       REAL(kind=8) RCO2, RCH4, RN2O, RCFC11, RCFC12  
       REAL(kind=8) RCO2_act, RCH4_act, RN2O_act, RCFC11_act, RCFC12_act  
       REAL(kind=8) CH4_ppb, N2O_ppb, CFC11_ppt, CFC12_ppt
!IM ajout CFMIP2/CMIP5
       REAL co2_ppm_per
       LOGICAL ok_4xCO2atm
       REAL(kind=8) RCO2_per,RCH4_per,RN2O_per,RCFC11_per,RCFC12_per
       REAL(kind=8) CH4_ppb_per,N2O_ppb_per,CFC11_ppt_per,CFC12_ppt_per

!OM ---> correction du bilan d'eau global
!OM Correction sur precip KE
       REAL cvl_corr
!OM Fonte calotte dans bilan eau
       LOGICAL ok_lic_melt

!IM simulateur ISCCP 
       INTEGER top_height, overlap
!IM seuils cdrm, cdrh
       REAL cdmmax, cdhmax
!IM param. stabilite s/ terres et en dehors
       REAL ksta, ksta_ter
!IM ok_kzmin : clef calcul Kzmin dans la CL de surface cf FH
       LOGICAL ok_kzmin
!IM, MAFo fmagic, pmagic : parametres - additionnel et multiplicatif - 
!                          pour regler l albedo sur ocean
       REAL fmagic, pmagic
! Hauteur (imposee) du contenu en eau du sol
           REAL qsol0
! Frottement au sol (Cdrag)
       Real f_cdrag_ter,f_cdrag_oce
! Rugoro
       Real f_rugoro

!IM lev_histhf  : niveau sorties 6h
!IM lev_histday : niveau sorties journalieres
!IM lev_histmth : niveau sorties mensuelles
!IM lev_histdayNMC : on peut sortir soit sur 8 (comme AR5) ou bien 
!                    sur 17 niveaux de pression
       INTEGER lev_histhf, lev_histday, lev_histmth
       INTEGER lev_histdayNMC
       Integer lev_histins, lev_histLES  
!IM ok_histNMC  : sortie fichiers niveaux de pression (histmthNMC, histdayNMC, histhfNMC)
!IM freq_outNMC : frequences de sortie fichiers niveaux de pression (histmthNMC, histdayNMC, histhfNMC)
!IM freq_calNMC : frequences de calcul fis. hist*NMC.nc
!IM pasphys : pas de temps de physique (secondes)
       REAL pasphys
       LOGICAL ok_histNMC(3)
       REAL freq_outNMC(3) , freq_calNMC(3)
       CHARACTER(len=4) type_run
! aer_type: pour utiliser un fichier constant dans readaerosol 
       CHARACTER*8 :: aer_type 
       LOGICAL ok_isccp, ok_regdyn
       REAL lonmin_ins, lonmax_ins, latmin_ins, latmax_ins
       REAL ecrit_ins, ecrit_hf, ecrit_day
       REAL ecrit_mth, ecrit_tra, ecrit_reg 
       REAL ecrit_LES
       REAL freq_ISCCP, ecrit_ISCCP
       REAL freq_COSP
       LOGICAL :: ok_cosp,ok_mensuelCOSP,ok_journeCOSP,ok_hfCOSP
       INTEGER :: ip_ebil_phy, iflag_rrtm
       LOGICAL :: ok_strato
       LOGICAL :: ok_hines
       LOGICAL :: ok_qch4

       COMMON/clesphys/cycle_diurne, soil_model, new_oliq,              &
     &     ok_orodr, ok_orolf, ok_limitvrai, nbapp_rad, iflag_con       &
     &     , co2_ppm, solaire                                           &
     &     , RCO2, RCH4, RN2O, RCFC11, RCFC12                           &
     &     , RCO2_act, RCH4_act, RN2O_act, RCFC11_act, RCFC12_act       &
     &     , ok_4xCO2atm                                                & 
     &     , RCO2_per, RCH4_per, RN2O_per, RCFC11_per, RCFC12_per       &
     &     , CH4_ppb, N2O_ppb, CFC11_ppt, CFC12_ppt                     &
     &     , CH4_ppb_per, N2O_ppb_per, CFC11_ppt_per, CFC12_ppt_per     &
     &     , top_height, overlap, cdmmax, cdhmax, ksta, ksta_ter        &
     &     , ok_kzmin, fmagic, pmagic                                   &
     &     , f_cdrag_ter,f_cdrag_oce,f_rugoro                           &
     &     , lev_histhf, lev_histday, lev_histmth                       &
     &     , lev_histins, lev_histLES, lev_histdayNMC                   &
     &     , pasphys, ok_histNMC, freq_outNMC, freq_calNMC              &
     &     , type_run, ok_isccp, ok_regdyn, ok_cosp                     &
     &     , ok_mensuelCOSP,ok_journeCOSP,ok_hfCOSP                     &
     &     , lonmin_ins, lonmax_ins, latmin_ins, latmax_ins             &
     &     , ecrit_ins, ecrit_hf, ecrit_day                             &
     &     , ecrit_mth, ecrit_tra, ecrit_reg                            &
     &     , freq_ISCCP, ecrit_ISCCP, freq_COSP, ip_ebil_phy            &
     &     , ok_lic_melt, cvl_corr, aer_type                            &
     &     , qsol0, iflag_rrtm, ok_strato,ok_hines,ecrit_LES            &
     &     , co2_ppm0, ok_qch4
     
       save /clesphys/
!$OMP THREADPRIVATE(/clesphys/)
 
      REAL rmu0(klon), albedo(klon)
c
c     REAL fmagic ! un facteur magique pour regler l'albedo
ccc      PARAMETER (fmagic=0.7)
cccIM => a remplacer  
c       PARAMETER (fmagic=1.32)
c       PARAMETER (fmagic=1.0)
c       PARAMETER (fmagic=0.7) 
c
      REAL fauxo
      INTEGER i
cccIM
      LOGICAL ancien_albedo
      PARAMETER(ancien_albedo=.FALSE.) 
c     SAVE albedo
c
      IF ( ancien_albedo ) THEN
c
      DO i = 1, klon
c
         rmu0(i) = MAX(rmu0(i),0.0)
c
         fauxo = ( 1.47 - ACOS( rmu0(i) ) )/0.15
         albedo(i) = fmagic*( .03 + .630/( 1. + fauxo*fauxo))+pmagic
         albedo(i) = MAX(MIN(albedo(i),0.60),0.04)
      ENDDO
c
c nouvel albedo 
c
      ELSE
c
      DO i = 1, klon
         rmu0(i) = MAX(rmu0(i),0.0)
cIM:orig albedo(i) = 0.058/(rmu0(i) + 0.30)
         albedo(i) = fmagic * 0.058/(rmu0(i) + 0.30)+pmagic
         albedo(i) = MAX(MIN(albedo(i),0.60),0.04)
      ENDDO
c
      ENDIF
c
      RETURN
      END
c========================================================================
