      SUBROUTINE concvl (iflag_con,iflag_clos,
     .             dtime,paprs,pplay,
     .             t,q,t_wake,q_wake,s_wake,u,v,tra,ntra,
     .             ALE,ALP,work1,work2,
     .             d_t,d_q,d_u,d_v,d_tra,
     .             rain, snow, kbas, ktop, sigd,
     .             cbmf,plcl,plfc,wbeff,upwd,dnwd,dnwdbis,
     .             Ma,mip,Vprecip,
     .             cape,cin,tvp,Tconv,iflag,
     .             pbase,bbase,dtvpdt1,dtvpdq1,dplcldt,dplcldr,
     .             qcondc,wd,pmflxr,pmflxs,
! RomP >>>
!!     .             da,phi,mp,dd_t,dd_q,lalim_conv,wght_th)
     .             da,phi,mp,phi2,d1a,dam,sij,clw,elij,        ! RomP
     .             dd_t,dd_q,lalim_conv,wght_th,               ! RomP
     .             evap, ep, epmlmMm,eplaMm,                   ! RomP
     .             wdtrainA,wdtrainM                       ! RomP
! RomP <<<

     :       ,xt,xt_wake,d_xt,xtrain,xtsnow,dd_xt
     :       ,xtVprecip  
     :        )
***************************************************************
*                                                             *
* CONCVL                                                      *
*                                                             *
*                                                             *
* written by   : Sandrine Bony-Lena, 17/05/2003, 11.16.04    *
* modified by :                                               *
***************************************************************
*
c
      USE dimphy
      USE infotrac, ONLY : nbtr
      IMPLICIT none
c======================================================================
c Auteur(s): S. Bony-Lena (LMD/CNRS) date: ???
c Objet: schema de convection de Emanuel (1991) interface
c======================================================================
c Arguments:
c dtime--input-R-pas d'integration (s)
c s-------input-R-la valeur "s" pour chaque couche
c sigs----input-R-la valeur "sigma" de chaque couche
c sig-----input-R-la valeur de "sigma" pour chaque niveau
c psolpa--input-R-la pression au sol (en Pa)
C pskapa--input-R-exponentiel kappa de psolpa
c h-------input-R-enthalpie potentielle (Cp*T/P**kappa)
c q-------input-R-vapeur d'eau (en kg/kg)
c
c work*: input et output: deux variables de travail,
c                            on peut les mettre a 0 au debut
c ALE-----input-R-energie disponible pour soulevement
c ALP-----input-R-puissance disponible pour soulevement
c
C d_h-----output-R-increment de l'enthalpie potentielle (h)
c d_q-----output-R-increment de la vapeur d'eau
c rain----output-R-la pluie (mm/s)
c snow----output-R-la neige (mm/s)
c upwd----output-R-saturated updraft mass flux (kg/m**2/s)
c dnwd----output-R-saturated downdraft mass flux (kg/m**2/s)
c dnwd0---output-R-unsaturated downdraft mass flux (kg/m**2/s)
c Ma------output-R-adiabatic ascent mass flux (kg/m2/s)
c mip-----output-R-mass flux shed by adiabatic ascent (kg/m2/s)
c Vprecip-output-R-vertical profile of precipitations (kg/m2/s)
c Tconv---output-R-environment temperature seen by convective scheme (K)
c Cape----output-R-CAPE (J/kg)
c Cin ----output-R-CIN  (J/kg)
c Tvp-----output-R-Temperature virtuelle d'une parcelle soulevee
c                  adiabatiquement a partir du niveau 1 (K)
c deltapb-output-R-distance entre LCL et base de la colonne (<0 ; Pa)
c Ice_flag-input-L-TRUE->prise en compte de la thermodynamique de la glace
c dd_t-----output-R-increment de la temperature du aux descentes precipitantes
c dd_q-----output-R-increment de la vapeur d'eau du aux desc precip
c======================================================================
c
!-----------------------------------------------------------------------
!   INCLUDE 'dimensions.h'
!
!   dimensions.h contient les dimensions du modele
!   ndm est tel que iim=2**ndm
!-----------------------------------------------------------------------

      INTEGER iim,jjm,llm,ndm

      PARAMETER (iim= 96,jjm=71,llm=39,ndm=1)

!-----------------------------------------------------------------------
c
       INTEGER iflag_con,iflag_clos
c
       REAL dtime, paprs(klon,klev+1),pplay(klon,klev)
       REAL t(klon,klev),q(klon,klev),u(klon,klev),v(klon,klev)
       REAL t_wake(klon,klev),q_wake(klon,klev)
       Real s_wake(klon)
       REAL tra(klon,klev,nbtr)
       INTEGER ntra
       REAL work1(klon,klev),work2(klon,klev),ptop2(klon)
       REAL pmflxr(klon,klev+1),pmflxs(klon,klev+1)
       REAL ALE(klon),ALP(klon)
c
       REAL d_t(klon,klev),d_q(klon,klev),d_u(klon,klev),d_v(klon,klev)
       REAL dd_t(klon,klev),dd_q(klon,klev)
       REAL d_tra(klon,klev,nbtr)
       REAL rain(klon),snow(klon)
c
!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env ! si oui, la temperature de cond est celle de l'environnement: permet d'eviter temperature de cond aberrantes
        logical ok_prod_nucl_tritium ! si oui, production de tritium par essais nucleaires
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)

        integer niso ! nombre d'isotopes utiliss
        parameter (niso=use_iso_eau+use_iso_HDO+use_iso_O18
     &           +use_iso_O17+use_iso_HTO)

        character*3 striso(niso)
      
        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso)
     &    , tnat(niso), toce(niso), tcorr(niso)
     &    , tdifrel(niso)
     &    , talph1(niso), talph2(niso), talph3(niso)
     &    , talps1(niso), talps2(niso)
     &    , tkcin0(niso)
     &    , tkcin1(niso),tkcin2(niso)
     &    , alpha_liq_sol(niso) 
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg
     &    , pxtmelt, pxtice, pxtmin, pxtmax
     &    , bidouille_anti_divergence,essai_convergence
     &    , initialisation_iso, initialisation_isotrac
     &    , modif_sst,deltaTtest
     &    , modif_sic,deltasic
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice 
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot
     &    , rh_cste_surf_cond, T_cste_surf_cond
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie
     &    , alphak_stewart, cste_surf_cond
     &    , mixlen
     &    , tnat, toce, tcorr
     &    , tdifrel
     &    , calendrier_guide
     &    , talph1, talph2, talph3
     &    , talps1, talps2
     &    , tkcin0, tkcin1, tkcin2
     &    , alpha_liq_sol, Rdefault
     &    , striso
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce
     &    , ratqsbasnew,Pcrit_ratqs
     &    , ok_bidouille_wake,cond_temp_env
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl
     &    , year_nucl, lat_nucl, lon_nucl
     &    , zmin_nucl, zmax_nucl, HTO_nucl
       REAL xt(ntraciso,klon,klev)
       REAL xtl(ntraciso,klon,klev)
       REAL xt_undi(ntraciso,klon,klev)
       REAL d_xt(ntraciso,klon,klev)
       real dd_xt(ntraciso,klon,klev)
       REAL xtrain(ntraciso,klon)
       real xtsnow(ntraciso,klon)
       real fxtd(ntraciso,klon)
       REAL xt1(ntraciso,klon,klev)
       integer ixt
       ! juste diagnostique
        REAL xt_wake(ntraciso,klon,klev)    
       INTEGER kbas(klon),ktop(klon)
       REAL em_ph(klon,klev+1),em_p(klon,klev)
       REAL upwd(klon,klev),dnwd(klon,klev),dnwdbis(klon,klev)

!!       REAL Ma(klon,klev), mip(klon,klev),Vprecip(klon,klev)     !jyg
       REAL Ma(klon,klev), mip(klon,klev),Vprecip(klon,klev+1)     !jyg
       real xtVprecip(ntraciso,klon,klev+1)
       real da(klon,klev),phi(klon,klev,klev),mp(klon,klev)
! RomP >>>
       real phi2(klon,klev,klev)                                    
       real d1a(klon,klev),dam(klon,klev)
       real sij(klon,klev,klev),clw(klon,klev),elij(klon,klev,klev)
       REAL wdtrainA(klon,klev),wdtrainM(klon,klev)
       REAL evap(klon,klev),ep(klon,klev)
       REAL epmlmMm(klon,klev,klev),eplaMm(klon,klev)
! RomP <<<
       REAL cape(klon),cin(klon),tvp(klon,klev)
       REAL Tconv(klon,klev)
c
cCR:test: on passe lentr et alim_star des thermiques
       INTEGER lalim_conv(klon)
       REAL wght_th(klon,klev)
       REAL em_sig1feed ! sigma at lower bound of feeding layer
       REAL em_sig2feed ! sigma at upper bound of feeding layer
       REAL em_wght(klev) ! weight density determining the feeding mixture
con enleve le save
c       SAVE em_sig1feed,em_sig2feed,em_wght
c
       INTEGER iflag(klon)
       REAL rflag(klon)
       REAL pbase(klon),bbase(klon)
       REAL dtvpdt1(klon,klev),dtvpdq1(klon,klev)
       REAL dplcldt(klon),dplcldr(klon)
       REAL qcondc(klon,klev)
       REAL wd(klon)
       REAL Plim1(klon),Plim2(klon),asupmax(klon,klev)
       REAL supmax0(klon),asupmaxmin(klon)
c
       REAL sigd(klon)
       REAL zx_t,zdelta,zx_qs,zcor
c
!       INTEGER iflag_mix
!       SAVE iflag_mix
       INTEGER noff, minorig
       INTEGER i,k,itra
       REAL qs(klon,klev),qs_wake(klon,klev)
       REAL cbmf(klon),plcl(klon),plfc(klon),wbeff(klon)
cLF       SAVE cbmf
cIM/JYG      REAL, SAVE, ALLOCATABLE :: cbmf(:)
ccc$OMP THREADPRIVATE(cbmf)!       
       REAL cbmflast(klon)
       INTEGER ifrst
       SAVE ifrst
       DATA ifrst /0/
c$OMP THREADPRIVATE(ifrst)

c
C     Variables supplementaires liees au bilan d'energie
c      Real paire(klon)
cLF      Real ql(klon,klev)
c      Save paire
cLF      Save ql
cLF      Real t1(klon,klev),q1(klon,klev)
cLF      Save t1,q1
c      Data paire /1./
       REAL, SAVE, ALLOCATABLE :: ql(:,:), q1(:,:), t1(:,:)
c$OMP THREADPRIVATE(ql, q1, t1)
c
C     Variables liees au bilan d'energie et d'enthalpi
      REAL ztsol(klon)
      REAL      h_vcol_tot, h_dair_tot, h_qw_tot, h_ql_tot
     $        , h_qs_tot, qw_tot, ql_tot, qs_tot , ec_tot
      SAVE      h_vcol_tot, h_dair_tot, h_qw_tot, h_ql_tot
     $        , h_qs_tot, qw_tot, ql_tot, qs_tot , ec_tot
c$OMP THREADPRIVATE(h_vcol_tot, h_dair_tot, h_qw_tot, h_ql_tot)
c$OMP THREADPRIVATE(h_qs_tot, qw_tot, ql_tot, qs_tot , ec_tot)
      REAL      d_h_vcol, d_h_dair, d_qt, d_qw, d_ql, d_qs, d_ec
      REAL      d_h_vcol_phy
      REAL      fs_bound, fq_bound
      SAVE      d_h_vcol_phy
c$OMP THREADPRIVATE(d_h_vcol_phy)
      REAL      zero_v(klon)
      CHARACTER*15 ztit
      INTEGER   ip_ebil  ! PRINT level for energy conserv. diag.
      SAVE      ip_ebil
      DATA      ip_ebil/2/
c$OMP THREADPRIVATE(ip_ebil)
      INTEGER   if_ebil ! level for energy conserv. dignostics
      SAVE      if_ebil
      DATA      if_ebil/2/
c$OMP THREADPRIVATE(if_ebil)
c+jld ec_conser
      REAL d_t_ec(klon,klev)    ! tendance du a la conersion Ec -> E thermique
      REAL ZRCPD
c-jld ec_conser
cLF
      INTEGER nloc
      logical, save :: first=.true.
c$OMP THREADPRIVATE(first)
      INTEGER, SAVE :: itap, igout
c$OMP THREADPRIVATE(itap, igout)
c
!
! $Header$
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
! A1.0 Fundamental constants
      REAL RPI,RCLUM,RHPLA,RKBOL,RNAVO
! A1.1 Astronomical constants
      REAL RDAY,REA,REPSM,RSIYEA,RSIDAY,ROMEGA
! A1.1.bis Constantes concernant l'orbite de la Terre:
      REAL R_ecc, R_peri, R_incl
! A1.2 Geoide
      REAL RA,RG,R1SA
! A1.3 Radiation
!     REAL RSIGMA,RI0
      REAL RSIGMA
! A1.4 Thermodynamic gas phase
      REAL R,RMD,RMO3,RMV,RD,RV,RCPD,RCPV,RCVD,RCVV
      REAL RKAPPA,RETV
! A1.5,6 Thermodynamic liquid,solid phases
      REAL RCW,RCS
! A1.7 Thermodynamic transition of phase
      REAL RLVTT,RLSTT,RLMLT,RTT,RATM
! A1.8 Curve of saturation
      REAL RESTT,RALPW,RBETW,RGAMW,RALPS,RBETS,RGAMS
      REAL RALPD,RBETD,RGAMD
!
      COMMON/YOMCST/RPI   ,RCLUM ,RHPLA ,RKBOL ,RNAVO                   &
     &      ,RDAY  ,REA   ,REPSM ,RSIYEA,RSIDAY,ROMEGA                  &
     &      ,R_ecc, R_peri, R_incl                                      &
     &      ,RA    ,RG    ,R1SA                                         &
     &      ,RSIGMA                                                     &
     &      ,R     ,RMD   ,RMO3  ,RMV   ,RD    ,RV    ,RCPD             &
     &      ,RCPV  ,RCVD  ,RCVV  ,RKAPPA,RETV                           &
     &      ,RCW   ,RCS                                                 &
     &      ,RLVTT ,RLSTT ,RLMLT ,RTT   ,RATM                           &
     &      ,RESTT ,RALPW ,RBETW ,RGAMW ,RALPS ,RBETS ,RGAMS            &
     &      ,RALPD ,RBETD ,RGAMD
!    ------------------------------------------------------------------
!$OMP THREADPRIVATE(/YOMCST/)

      INTEGER choice, iflag_mix
      REAL  gammas, alphas, betas, Fmax, qqa1, qqa2, qqa3, scut
      REAL  Qcoef1max,Qcoef2max,Supcrit1,Supcrit2
!
      COMMON/YOMCST2/gammas,    alphas, betas, Fmax, scut,              &
     &               qqa1, qqa2, qqa3,                                  &
     &               Qcoef1max,Qcoef2max,                               &
     &               Supcrit1, Supcrit2,                                &
     &               choice,iflag_mix
!$OMP THREADPRIVATE(/YOMCST2/)
!    --------------------------------------------------------------------

!
! $Header$
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!*    COMMON *YOETHF* DERIVED CONSTANTS SPECIFIC TO ECMWF THERMODYNAMICS
!
!     *R__ES*   *CONSTANTS USED FOR COMPUTATION OF SATURATION
!                MIXING RATIO OVER LIQUID WATER(*R_LES*) OR
!                ICE(*R_IES*).
!     *RVTMP2*  *RVTMP2=RCPV/RCPD-1.
!     *RHOH2O*  *DENSITY OF LIQUID WATER.   (RATM/100.)
!
      REAL R2ES, R3LES, R3IES, R4LES, R4IES, R5LES, R5IES
      REAL RVTMP2, RHOH2O
      COMMON /YOETHF/R2ES, R3LES, R3IES, R4LES, R4IES, R5LES, R5IES,    &
     &               RVTMP2, RHOH2O
!$OMP THREADPRIVATE(/YOETHF/)
!
! $Header$
!
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!     ------------------------------------------------------------------
!     This COMDECK includes the Thermodynamical functions for the cy39
!       ECMWF Physics package.
!       Consistent with YOMCST Basic physics constants, assuming the
!       partial pressure of water vapour is given by a first order
!       Taylor expansion of Qs(T) w.r.t. to Temperature, using constants
!       in YOETHF
!     ------------------------------------------------------------------
      REAL PTARG, PDELARG, P5ARG, PQSARG, PCOARG
      REAL FOEEW, FOEDE, qsats, qsatl, dqsats, dqsatl
      LOGICAL thermcep
      PARAMETER (thermcep=.TRUE.)
!
      FOEEW ( PTARG,PDELARG ) = EXP (                                   &
     &          (R3LES*(1.-PDELARG)+R3IES*PDELARG) * (PTARG-RTT)        &
     & / (PTARG-(R4LES*(1.-PDELARG)+R4IES*PDELARG)) )
!
      FOEDE ( PTARG,PDELARG,P5ARG,PQSARG,PCOARG ) = PQSARG*PCOARG*P5ARG &
     & / (PTARG-(R4LES*(1.-PDELARG)+R4IES*PDELARG))**2
!
      qsats(ptarg) = 100.0 * 0.622 * 10.0                               &
     &           ** (2.07023 - 0.00320991 * ptarg                       &
     &           - 2484.896 / ptarg + 3.56654 * LOG10(ptarg))
      qsatl(ptarg) = 100.0 * 0.622 * 10.0                               &
     &           ** (23.8319 - 2948.964 / ptarg                         &
     &           - 5.028 * LOG10(ptarg)                                 &
     &           - 29810.16 * EXP( - 0.0699382 * ptarg)                 &
     &           + 25.21935 * EXP( - 2999.924 / ptarg))
!
      dqsats(ptarg,pqsarg) = RLVTT/RCPD*pqsarg * (3.56654/ptarg         &
     &                     +2484.896*LOG(10.)/ptarg**2                  &
     &                     -0.00320991*LOG(10.))
      dqsatl(ptarg,pqsarg) = RLVTT/RCPD*pqsarg*LOG(10.)*                &
     &                (2948.964/ptarg**2-5.028/LOG(10.)/ptarg           &
     &                +25.21935*2999.924/ptarg**2*EXP(-2999.924/ptarg)  &
     &                +29810.16*0.0699382*EXP(-0.0699382*ptarg))
!
! $Header$
!
!
! gestion des impressions de sorties et de dbogage
! lunout:    unit du fichier dans lequel se font les sorties 
!                           (par defaut 6, la sortie standard)
! prt_level: niveau d'impression souhait (0 = minimum)
!
      INTEGER lunout, prt_level
      COMMON /comprint/ lunout, prt_level

c
      if (first) then
c Allocate some variables LF 04/2008
c
cIM/JYG allocate(cbmf(klon))
        allocate(ql(klon,klev))
        allocate(t1(klon,klev))
        allocate(q1(klon,klev))
        itap=0
        igout=klon/2+1/klon
      endif
c Incrementer le compteur de la physique
      itap   = itap + 1

c    Copy T into Tconv
      DO k = 1,klev
        DO i = 1,klon
          Tconv(i,k) = T(i,k)
        ENDDO
      ENDDO
c
      IF (if_ebil.ge.1) THEN
        DO i=1,klon
          ztsol(i) = t(i,1)
          zero_v(i)=0.
          Do k = 1,klev
            ql(i,k) = 0.
            do ixt=1,ntraciso
              xtl(ixt,i,k) = 0.
            enddo
          ENDDO
        END DO
      END IF
c
cym
      snow(:)=0
      xtsnow(:,:)=0
c      IF (ifrst .EQ. 0) THEN
c         ifrst = 1
       if (first) then
         first=.false.
c
C===========================================================================
C    READ IN PARAMETERS FOR THE CLOSURE AND THE MIXING DISTRIBUTION
C===========================================================================
C
      if (iflag_con.eq.3) then
c     CALL cv3_inicp()
      CALL cv3_inip()
      endif
c
C===========================================================================
C    READ IN PARAMETERS FOR CONVECTIVE INHIBITION BY TROPOS. DRYNESS
C===========================================================================
C
cc$$$         open (56,file='supcrit.data')
cc$$$         read (56,*) Supcrit1, Supcrit2
cc$$$         close (56)
c
        IF (prt_level .ge. 10) 
     &                print*, 'supcrit1, supcrit2' ,supcrit1, supcrit2
C
C===========================================================================
C      Initialisation pour les bilans d'eau et d'energie
C===========================================================================
         IF (if_ebil.ge.1) d_h_vcol_phy=0.
c
         DO i = 1, klon
          cbmf(i) = 0.
!!          plcl(i) = 0.
          sigd(i) = 0.
         ENDDO
      ENDIF   !(ifrst .EQ. 0)

c Initialisation a chaque pas de temps
      plfc(:)  = 0.
      wbeff(:) = 100.
      plcl(:) = 0.

      DO k = 1, klev+1
         DO i=1,klon
         em_ph(i,k) = paprs(i,k) / 100.0
         pmflxr(i,k)=0.
         pmflxs(i,k)=0.
      ENDDO
      ENDDO
c
      DO k = 1, klev
         DO i=1,klon
         em_p(i,k) = pplay(i,k) / 100.0
      ENDDO
      ENDDO
c
!
!  Feeding layer
!
      em_sig1feed = 1.
      em_sig2feed = 0.97
c      em_sig2feed = 0.8
! Relative Weight densities
       do k=1,klev
         em_wght(k)=1.
       end do
cCRtest: couche alim des tehrmiques ponderee par a*
c       DO i = 1, klon
c         do k=1,lalim_conv(i)
c         em_wght(k)=wght_th(i,k)
c         print*,'em_wght=',em_wght(k),wght_th(i,k)
c       end do
c      END DO

!      write(*,*) 'concvl 345 tmp'

      if (iflag_con .eq. 4) then
      DO k = 1, klev
        DO i = 1, klon
         zx_t = t(i,k)
         zdelta=MAX(0.,SIGN(1.,rtt-zx_t))
         zx_qs= MIN(0.5 , r2es * FOEEW(zx_t,zdelta)/em_p(i,k)/100.0)
         zcor=1./(1.-retv*zx_qs)
         qs(i,k)=zx_qs*zcor
        ENDDO
        DO i = 1, klon
         zx_t = t_wake(i,k)
         zdelta=MAX(0.,SIGN(1.,rtt-zx_t))
         zx_qs= MIN(0.5 , r2es * FOEEW(zx_t,zdelta)/em_p(i,k)/100.0)
         zcor=1./(1.-retv*zx_qs)
         qs_wake(i,k)=zx_qs*zcor
        ENDDO
      ENDDO
      else ! iflag_con=3 (modif de puristes qui fait la diffce pour la convergence numerique)
      DO k = 1, klev
        DO i = 1, klon
         zx_t = t(i,k)
         zdelta=MAX(0.,SIGN(1.,rtt-zx_t))
         zx_qs= r2es * FOEEW(zx_t,zdelta)/em_p(i,k)/100.0
         zx_qs= MIN(0.5,zx_qs)
         zcor=1./(1.-retv*zx_qs)
         zx_qs=zx_qs*zcor
         qs(i,k)=zx_qs
        ENDDO
        DO i = 1, klon
         zx_t = t_wake(i,k)
         zdelta=MAX(0.,SIGN(1.,rtt-zx_t))
         zx_qs= r2es * FOEEW(zx_t,zdelta)/em_p(i,k)/100.0
         zx_qs= MIN(0.5,zx_qs)
         zcor=1./(1.-retv*zx_qs)
         zx_qs=zx_qs*zcor
         qs_wake(i,k)=zx_qs
        ENDDO
      ENDDO
      endif ! iflag_con

!      write(*,*) 'concvl 386 tmp: iflag_con=',iflag_con
c
C------------------------------------------------------------------

C Main driver for convection:
C               iflag_con=3 -> nvlle version de KE (JYG)
C		iflag_con = 30  -> equivalent to convect3
C		iflag_con = 4  -> equivalent to convect1/2
c
c
      if (iflag_con.eq.30) then
     
        ! ISOVERIF ! end verif       

      !print *, '-> cv_driver'      !jyg
      CALL cv_driver(klon,klev,klev+1,ntra,iflag_con,
     :              t,q,qs,u,v,tra,
     $              em_p,em_ph,iflag,
     $              d_t,d_q,d_u,d_v,d_tra,rain,
     $              Vprecip,cbmf,work1,work2,    
     $              kbas,ktop,
     $              dtime,Ma,upwd,dnwd,dnwdbis,qcondc,wd,cape,
     $              da,phi,mp,phi2,d1a,dam,sij,clw,elij,       !RomP
     $              evap,ep,epmlmMm,eplaMm,                    !RomP
     $              wdtrainA,wdtrainM                      !RomP
     :             ,xt,d_xt,xtrain,xtVprecip
     :              )
      !print *, 'cv_driver ->'      !jyg

      ! verif
       ! end verif       
c
      DO i = 1,klon
        cbmf(i) = Ma(i,kbas(i))
      ENDDO
c
      else !if (iflag_con.eq.30) then

cLF   necessary for gathered fields
      nloc=klon
      CALL cva_driver(klon,klev,klev+1,ntra,nloc,
     $              iflag_con,iflag_mix,iflag_clos,dtime,
     :              t,q,qs,t_wake,q_wake,qs_wake,s_wake,u,v,tra,
     $              em_p,em_ph,
     .              ALE,ALP,
     .              em_sig1feed,em_sig2feed,em_wght,
     .              iflag,d_t,d_q,d_u,d_v,d_tra,rain,kbas,ktop,
     $              cbmf,plcl,plfc,wbeff,work1,work2,ptop2,sigd,
     $              Ma,mip,Vprecip,upwd,dnwd,dnwdbis,qcondc,wd,
     $              cape,cin,tvp,
     $              dd_t,dd_q,Plim1,Plim2,asupmax,supmax0,
     $              asupmaxmin,lalim_conv,
!AC!+!RomP
     $              da,phi,mp,phi2,d1a,dam,sij,clw,  ! RomP
     $              elij,evap,ep,wdtrainA,wdtrainM  ! RomP
!AC!+!RomP
     :             ,xt,xt_wake,d_xt, xtrain,dd_xt
     :             ,xtVprecip
     :              )
      
      endif   !if (iflag_con.eq.30) then
C------------------------------------------------------------------
      IF (prt_level .ge. 10)
     .   WRITE(lunout,*) ' cva_driver -> cbmf,plcl,plfc,wbeff ',
     .                     cbmf(1),plcl(1),plfc(1),wbeff(1)

!      write(*,*) 'concvl 507: apres cva_driver'
      DO i = 1,klon
        rain(i) = rain(i)/86400.
        rflag(i)=iflag(i)
       do ixt = 1, ntraciso
        xtrain(ixt,i) = xtrain(ixt,i)/86400.
       enddo
      ENDDO

      DO k = 1, klev
        DO i = 1, klon
           d_t(i,k) = dtime*d_t(i,k)
           d_q(i,k) = dtime*d_q(i,k)
           d_u(i,k) = dtime*d_u(i,k)
           d_v(i,k) = dtime*d_v(i,k)
           do ixt = 1, ntraciso
            d_xt(ixt,i,k) = dtime*d_xt(ixt,i,k)
           enddo
        ENDDO ! DO i = 1, klon
      ENDDO !DO k = 1, klev

      if ((use_iso_eau.eq.1).and.(bidouille_anti_divergence)) then
        do k=1,klev   
        do i=1,klon
            d_xt(iso_eau,i,k)=d_q(i,k)
        enddo !do i=1,klon
        enddo !do k=1,klev               
      endif !if ((use_iso_eau.eq.1).and.(bidouille_anti_divergence)) then  
c
       if (iflag_con.eq.30) then
       DO itra = 1,ntra
        DO k = 1, klev
         DO i = 1, klon
            d_tra(i,k,itra) =dtime*d_tra(i,k,itra) 
         ENDDO
        ENDDO
       ENDDO
       endif

c!AC!
       if (iflag_con.eq.3) then
       DO itra = 1,ntra
        DO k = 1, klev
         DO i = 1, klon
            d_tra(i,k,itra) =dtime*d_tra(i,k,itra) 
         ENDDO
        ENDDO
       ENDDO 
       endif
c!AC!

      DO k = 1, klev
        DO i = 1, klon
          t1(i,k) = t(i,k)+ d_t(i,k)
          q1(i,k) = q(i,k)+ d_q(i,k)
            do ixt=1,ntraciso
               xt1(ixt,i,k) = xt(ixt,i,k)+ d_xt(ixt,i,k)
            enddo
        ENDDO !DO i = 1, klon
      ENDDO !DO k = 1, klev

c                                                  !jyg
c--Separation neige/pluie (pour diagnostics)       !jyg
      DO k = 1, klev                               !jyg
      DO i = 1, klon                               !jyg
       IF (t1(i,k).LT.RTT) THEN                    !jyg
         pmflxs(i,k)=Vprecip(i,k)                  !jyg
       ELSE                                        !jyg
         pmflxr(i,k)=Vprecip(i,k)                  !jyg
       ENDIF                                       !jyg
      ENDDO                                        !jyg
      ENDDO                                        !jyg
c
cc      IF (if_ebil.ge.2) THEN
cc        ztit='after convect'
cc        CALL diagetpq(paire,ztit,ip_ebil,2,2,dtime
cc     e      , t1,q1,ql,qs,u,v,paprs,pplay
cc     s      , d_h_vcol, d_qt, d_qw, d_ql, d_qs, d_ec)
cc         call diagphy(paire,ztit,ip_ebil
cc     e      , zero_v, zero_v, zero_v, zero_v, zero_v
cc     e      , zero_v, rain, zero_v, ztsol
cc     e      , d_h_vcol, d_qt, d_ec
cc     s      , fs_bound, fq_bound )
cc      END IF
C
c
c les traceurs ne sont pas mis dans cette version de convect4:
      if (iflag_con.eq.4) then
       DO itra = 1,ntra
        DO k = 1, klev
         DO i = 1, klon
            d_tra(i,k,itra) = 0.
         ENDDO
        ENDDO
       ENDDO
      endif
c     print*, 'concvl->: dd_t,dd_q ',dd_t(1,1),dd_q(1,1)

        DO k = 1, klev
         DO i = 1, klon
            dtvpdt1(i,k) = 0.
            dtvpdq1(i,k) = 0.
         ENDDO
        ENDDO
        DO i = 1, klon
           dplcldt(i) = 0.
           dplcldr(i) = 0.
        ENDDO
c
       if(prt_level.GE.20) THEN
       DO k=1,klev
!       print*,'physiq apres_add_con i k it d_u d_v d_t d_q qdl0',igout
!    .,k,itap,d_u_con(igout,k) ,d_v_con(igout,k), d_t_con(igout,k),
!    .d_q_con(igout,k),dql0(igout,k)
!      print*,'phys apres_add_con itap Ma cin ALE ALP wak t q undi t q'
!    .,itap,Ma(igout,k),cin(igout),ALE(igout), ALP(igout),
!    . t_wake(igout,k),q_wake(igout,k),t_undi(igout,k),q_undi(igout,k)
!      print*,'phy apres_add_con itap CON rain snow EMA wk1 wk2 Vpp mip'
!    .,itap,rain_con(igout),snow_con(igout),ema_work1(igout,k),
!    .ema_work2(igout,k),Vprecip(igout,k), mip(igout,k)
!      print*,'phy apres_add_con itap upwd dnwd dnwd0 cape tvp Tconv '
!    .,itap,upwd(igout,k),dnwd(igout,k),dnwd0(igout,k),cape(igout),
!    .tvp(igout,k),Tconv(igout,k)
!      print*,'phy apres_add_con itap dtvpdt dtvdq dplcl dplcldr qcondc'
!    .,itap,dtvpdt1(igout,k),dtvpdq1(igout,k),dplcldt(igout),
!    .dplcldr(igout),qcondc(igout,k)
!      print*,'phy apres_add_con itap wd pmflxr Kpmflxr Kp1 Kpmflxs Kp1'
!    .,itap,wd(igout),pmflxr(igout,k),pmflxr(igout,k+1),pmflxs(igout,k)
!    .,pmflxs(igout,k+1)
!      print*,'phy apres_add_con itap da phi mp ftd fqd lalim wgth',
!    .itap,da(igout,k),phi(igout,k,k),mp(igout,k),ftd(igout,k),
!    . fqd(igout,k),lalim_conv(igout),wght_th(igout,k)
      ENDDO
      endif !(prt_level.EQ.20) THEN
c
      RETURN
      END
 
