!
! $Id: fisrtilp.F90 1575 2011-09-21 13:57:48Z jghattas $
!
!
SUBROUTINE fisrtilp(dtime,paprs,pplay,t,q,ptconv,ratqs, &
     d_t, d_q, d_ql, rneb, radliq, rain, snow,          &
     pfrac_impa, pfrac_nucl, pfrac_1nucl,               &
     frac_impa, frac_nucl, beta,                        &
     prfl, psfl, rhcl, zqta, fraca,                     &
     ztv, zpspsk, ztla, zthl, iflag_cldcon,             &
     flag_ss, flagb_ss, rneb_ss, clsk_ss                &

     &           ,xt,xtrain,xtsnow                      &
     &           ,d_xt,d_xtl,pxtrainfl,pxtsnowfl        &




     &  )

  !
  USE dimphy
  IMPLICIT none
  !======================================================================
  ! Auteur(s): Z.X. Li (LMD/CNRS)
  ! Date: le 20 mars 1995
  ! Objet: condensation et precipitation stratiforme.
  !        schema de nuage
  !======================================================================
  !======================================================================
  !ym include "dimensions.h"
  !ym include "dimphy.h"
  include "YOMCST.h"
  include "tracstoke.h"
  include "fisrtilp.h"
  include "iniprint.h"

  !
  ! Arguments:
  !
  REAL dtime ! intervalle du temps (s)
  REAL paprs(klon,klev+1) ! pression a inter-couche
  REAL pplay(klon,klev) ! pression au milieu de couche
  REAL t(klon,klev) ! temperature (K)
  REAL q(klon,klev) ! humidite specifique (kg/kg)
  REAL d_t(klon,klev) ! incrementation de la temperature (K)
  REAL d_q(klon,klev) ! incrementation de la vapeur d'eau
  REAL d_ql(klon,klev) ! incrementation de l'eau liquide
  REAL rneb(klon,klev) ! fraction nuageuse
  REAL radliq(klon,klev) ! eau liquide utilisee dans rayonnements
  REAL rhcl(klon,klev) ! humidite relative en ciel clair
  REAL rain(klon) ! pluies (mm/s)
  REAL snow(klon) ! neige (mm/s)
  REAL prfl(klon,klev+1) ! flux d'eau precipitante aux interfaces (kg/m2/s)
  REAL psfl(klon,klev+1) ! flux d'eau precipitante aux interfaces (kg/m2/s) 
  REAL ztv(klon,klev)
  REAL zqta(klon,klev),fraca(klon,klev) 
  REAL sigma1(klon,klev),sigma2(klon,klev)
  REAL qltot(klon,klev),ctot(klon,klev)
  REAL zpspsk(klon,klev),ztla(klon,klev)
  REAL zthl(klon,klev)

  logical lognormale(klon)

!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env
        logical ok_prod_nucl_tritium
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)
        

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)
        
        integer niso ! nombre d'isotopes utiliss   
        parameter (niso=use_iso_HTO+use_iso_O17 &
     &               +use_iso_O18+use_iso_HDO+use_iso_eau) 

        character*3 striso(niso)    

        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso) &
     &    , tnat(niso), toce(niso), tcorr(niso) &
     &    , tdifrel(niso) &
     &    , talph1(niso), talph2(niso), talph3(niso) &
     &    , talps1(niso), talps2(niso) &
     &    , tkcin0(niso) &
     &    , tkcin1(niso),tkcin2(niso) &
     &    , alpha_liq_sol(niso) &
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg &
     &    , pxtmelt, pxtice, pxtmin, pxtmax &
     &    , bidouille_anti_divergence,essai_convergence &
     &    , initialisation_iso, initialisation_isotrac &
     &    , modif_sst,deltaTtest &
     &    , modif_sic,deltasic &
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit &
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo &
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice   &
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot &
     &    , rh_cste_surf_cond, T_cste_surf_cond  &
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie &
     &    , alphak_stewart, cste_surf_cond &
     &    , mixlen &
     &    , tnat, toce, tcorr &
     &    , tdifrel &
     &    , calendrier_guide &
     &    , talph1, talph2, talph3 &
     &    , talps1, talps2 &
     &    , tkcin0, tkcin1, tkcin2 &
     &    , alpha_liq_sol, Rdefault &
     &    , striso  &
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO &
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont &
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim &
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce &
     &    , ratqsbasnew,Pcrit_ratqs &
     &    , ok_bidouille_wake,cond_temp_env &
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl &
     &    , year_nucl, lat_nucl, lon_nucl &
     &    , zmin_nucl, zmax_nucl, HTO_nucl 
      real double_to_real
      double precision real_to_double
      !integer niso
      real xt(ntraciso,klon,klev)
      real d_xt(ntraciso,klon,klev)
      real d_xtl(ntraciso,klon,klev)
      real xtrain(ntraciso,klon)
      real xtsnow(ntraciso,klon)
      real pxtrainfl(ntraciso,klon,klev+1)
      real pxtsnowfl(ntraciso,klon,klev+1)      
      ! xt <-> q
      ! xtrain et xtsonw <-> rain et snow
      ! d_xt et d_xtl <-> d_q et d_ql
      ! pxtrainfl et pxtsnowflux <-> prfl et psfl

  !AA
  ! Coeffients de fraction lessivee : pour OFF-LINE
  !
  REAL pfrac_nucl(klon,klev)
  REAL pfrac_1nucl(klon,klev)
  REAL pfrac_impa(klon,klev)
  !
  ! Fraction d'aerosols lessivee par impaction et par nucleation
  ! POur ON-LINE
  !
  REAL frac_impa(klon,klev)
  REAL frac_nucl(klon,klev)
  real zct      ,zcl
  !AA
  !
  ! Options du programme:
  !
  REAL seuil_neb ! un nuage existe vraiment au-dela
  PARAMETER (seuil_neb=0.001)

  INTEGER ninter ! sous-intervals pour la precipitation
  INTEGER ncoreczq  
  INTEGER iflag_cldcon
  PARAMETER (ninter=5)
  LOGICAL evap_prec ! evaporation de la pluie
  PARAMETER (evap_prec=.TRUE.)
  REAL ratqs(klon,klev) ! determine la largeur de distribution de vapeur
  logical ptconv(klon,klev) ! determine la largeur de distribution de vapeur

  real zpdf_sig(klon),zpdf_k(klon),zpdf_delta(klon)
  real Zpdf_a(klon),zpdf_b(klon),zpdf_e1(klon),zpdf_e2(klon)
  real erf   
  REAL qcloud(klon)
  !
  LOGICAL cpartiel ! condensation partielle
  PARAMETER (cpartiel=.TRUE.)
  REAL t_coup
  PARAMETER (t_coup=234.0)

  ! pour la sursat de Marine/Bernard
  real zpdf_beta(klon)
  real RH_crit, pi
  REAL rhcl_p0, rhcl_p1, rhcl_p2, rhcl_p
  real zpdf_delta_o(klon), zpdf_a_o(klon), zpdf_b_o(klon)
  real zpdf_e1_o(klon)
  real rneb_o(klon,klev)
  real flag_ss(klon,klev),clsk_ss(klon,klev),rneb_ss(klon,klev)
  real flagb_ss(klon,klev)

  !
  ! Variables locales:
  !
  INTEGER i, k, n, kk
  REAL zqs(klon), zdqs(klon), zdelta, zcor, zcvm5   
  REAL zrfl(klon), zrfln(klon), zqev, zqevt
  REAL zoliq(klon), zcond(klon), zq(klon), zqn(klon), zdelq
  REAL ztglace, zt(klon)
  INTEGER nexpo ! exponentiel pour glace/eau
  REAL zdz(klon),zrho(klon),ztot      , zrhol(klon)
  REAL zchau      ,zfroi      ,zfice(klon),zneb(klon)

  ! sursat de Marine/Bernard   
  REAL zqs_star(klon), ys(klon), yss(klon)

      real zxtrfl(ntraciso,klon)
      real zxtrfln(ntraciso,klon)
      real zxt(ntraciso,klon) ! equivalent local de xt
      real zxtn(ntraciso,klon) ! isotpes nuageux
      real zxtn_reduit(niso)
      real zxtcond(ntraciso,klon)
      real zxtoliq(ntraciso,klon)
      real zq_ancien(klon)
      integer ixt
       logical sorties_gdat ! si true, sorties gdat intermdiaires internes aux subroutines
        parameter (sorties_gdat=.false.)      
!
!        logical essai_convergence ! si false, on peut rtroagir sur
!        l'eau normale, si true, on ne peut pas, et a permet de tester
!        la convergence avec la verison d'LMDZ sans isotopes.
!        Attention: si on  fait un essai de convergence, mieux vaut
!        enlever iso_debug, sinon, risuqes de plantage si non
!        conservation de l'eau. 
!        parameter (essai_convergence=.false.)
!        parameter (essai_convergence=.false.)


    ! essai convergence a t dplac dans wateriso2.h et est lue  l'execution

      real errmax ! erreur maximale en absolu.
      parameter (errmax=1e-8)
!      parameter (errmax=1e10)

      real errmax_sol ! erreur maximale en absolu.
      parameter (errmax_sol=5e-7)

      real errmaxrel ! erreur maximale en relatif autorise
!      parameter (errmaxrel=1e10)
      parameter (errmaxrel=1e-3)

      real borne ! valeur maximale que n'importe quelle variable peut
                 ! atteindre, en val abs; utile pour vrif que pas NAN
      parameter (borne=1e19)

      real ridicule ! valeur maximale pour qu'une variable de type
                    ! rapoport de mlange puisse tre considre comme ngligeable. Si
                    ! ngligeable, alors on ne verifie pas si sa compo iso esta brrante.
      parameter (ridicule=1e-12)      
!      parameter (ridicule=1)
!
      real ridicule_rain ! valeur limite de ridicule pour les flux de pluies (rain, zrfl...)
      parameter (ridicule_rain=1e-8) ! en kg/s <-> 1e-3mm/day

      real ridicule_evap ! valeur limite de ridicule pour les evap
      parameter (ridicule_evap=ridicule_rain*1e-2) ! en kg/s <-> 1e-3mm/day

      real ridicule_qsol ! valeur limite de ridicule pour les qsol
      parameter (ridicule_qsol=ridicule_rain) ! en kg <-> 1e-8kg

      real ridicule_snow ! valeur limite de ridicule pour les snow
      parameter (ridicule_snow=ridicule_qsol) ! en kg/s <-> 1e-8kg

      real ridicule_trac
      parameter (ridicule_trac=ridicule*1e4)
      
      real deltalim ! deltalim est le maximum de deltaD qu'on
                             ! autorise dans la vapeur, au-dela, en suppose que c'est abrrant.
                             ! dans le liquide, on autorise deltalim*faccond.
!      parameter (deltalim=1e10)
!      parameter (deltalim=300.0)
       ! maintenant dfini dans iso.def

       real deltalimtrac ! max de deltaD dans les traceurs, dfini dans iso.def

      real deltalim_snow ! deltalim est le maximum de deltaD qu'on
                             ! autorise dans la neige, au-dela, en suppose que c'est abrrant.
!      parameter (deltalim_snow=500.0) ! initalis dans iso_init

	real deltaDmin
!	parameter (deltaDmin=-900.0)
    ! maintentant, dfini dans iso.def

      real o17excess_bas,o17excess_haut ! bornes inf et sup de l'O17-excess
!      parameter(o17excess_bas=-200.0,o17excess_haut=120)
      integer nlevmaxO17

      logical O17_verif
!      parameter (O17_verif=.true.)

      real faccond  ! dans le liquide, on autorise R(deltalim)*faccond.
      parameter (faccond=1.1)
      
!      logical bidouille_anti_divergence ! si true, alors on fait un
!                                        ! rappel rgulier des xt4 vers les q pour 
!                                        !viter accumulation d'rreurs et  divergences
!      parameter (bidouille_anti_divergence=.true.)
!      parameter (bidouille_anti_divergence=.false.)
    ! bidouille_anti_divergence a t dplac dans wateriso2.h et est lue  l'execution

        real expb_max
        parameter (expb_max=30.0)

	real deltaDfaible ! deltaD en dessous duquel la vapeur est tellement faible 
        !que on peut accepter que la remise  l'quilibre du sol avec 
        ! cette vapeur donne des deltaDevap aberrants.
        parameter (deltaDfaible=-300)
        real deltaDfaible_lax ! deltaD en dessous duquel la vapeur est tellement faible 
        !que on peut accepter que la remise  l'quilibre du sol avec 
        ! cette vapeur donne des deltaDevap aberrants.
        parameter (deltaDfaible_lax=-180)

    	real faible_evap ! faible vaporation: on est plus laxiste 
        !pour les deltaD aberrant dans le cas de l'vap venant d'orchidee
        parameter (faible_evap=3.0)

    	real Tmin_verif
	parameter (Tmin_verif=5.0) ! en K
    	real Tmax_verif
	parameter (Tmax_verif=1000.0) ! en K

        COMMON /isoverif/ deltalim,deltalimtrac,deltalim_snow,deltaDmin
	COMMON /isoverifO17/ O17_verif,o17excess_bas,o17excess_haut
	COMMON /isoverifO17bis/ nlevmaxO17      
      integer iso_verif_nonan_nostop
      real zxtice(ntraciso,klon),zxtliq(ntraciso,klon)
      real zrfl_ancien(klon)
      real zqev_diag(klon)
      real zxtrfl_ancien(ntraciso,klon)
      real zxt_ancien(ntraciso,klon)
!      logical negation ! pour nier conditions logiques
  !
  LOGICAL appel1er
  SAVE appel1er
  !$OMP THREADPRIVATE(appel1er)
  !
  !---------------------------------------------------------------
  !
  !AA Variables traceurs:
  !AA  Provisoire !!! Parametres alpha du lessivage
  !AA  A priori on a 4 scavenging # possibles
  !
  REAL a_tr_sca(4)
  save a_tr_sca
  !$OMP THREADPRIVATE(a_tr_sca)
  !
  ! Variables intermediaires
  !
  REAL zalpha_tr
  REAL zfrac_lessi
  REAL zprec_cond(klon)
  !AA
! RomP >>> 15 nov 2012
  REAL   beta(klon,klev) ! taux de conversion de l'eau cond
! RomP <<<
  REAL zmair, zcpair, zcpeau
  !     Pour la conversion eau-neige
  REAL zlh_solid(klon), zm_solid
  !IM 
  !ym      INTEGER klevm1
  !---------------------------------------------------------------
  !
  ! Fonctions en ligne:
  !
  REAL fallvs,fallvc ! vitesse de chute pour crystaux de glace
  REAL zzz
  include "YOETHF.h"
  include "FCTTRE.h"
  fallvc (zzz) = 3.29/2.0 * ((zzz)**0.16) * ffallv_con
  fallvs (zzz) = 3.29/2.0 * ((zzz)**0.16) * ffallv_lsc
  !
  DATA appel1er /.TRUE./
  !ym
  zdelq=0.0

  if (prt_level>9)write(lunout,*)'NUAGES4 A. JAM'
  IF (appel1er) THEN
     !
     WRITE(lunout,*) 'fisrtilp, ninter:', ninter
     WRITE(lunout,*) 'fisrtilp, evap_prec:', evap_prec
     WRITE(lunout,*) 'fisrtilp, cpartiel:', cpartiel
     WRITE(lunout,*) 'fisrtilp, dtime:',dtime
     IF (ABS(dtime/REAL(ninter)-360.0).GT.0.001) THEN
        WRITE(lunout,*) 'fisrtilp: Ce n est pas prevu, voir Z.X.Li', dtime
        WRITE(lunout,*) 'Je prefere un sous-intervalle de 6 minutes'
        !         CALL abort
     ENDIF
     appel1er = .FALSE.

        ! cam verif 
         ! end cam verif
     !
     !AA initialiation provisoire
     a_tr_sca(1) = -0.5
     a_tr_sca(2) = -0.5
     a_tr_sca(3) = -0.5
     a_tr_sca(4) = -0.5
     !
     !AA Initialisation a 1 des coefs des fractions lessivees 
     !
     !cdir collapse
     DO k = 1, klev
        DO i = 1, klon
           pfrac_nucl(i,k)=1.
           pfrac_1nucl(i,k)=1.
           pfrac_impa(i,k)=1.
           beta(i,k)=0.  !RomP initialisation

        flag_ss(i,k) = 0.
        clsk_ss(i,k) = 0.
        rneb_ss(i,k) = 0.
        flagb_ss(i,k) = 0.
        ENDDO
     ENDDO

  ENDIF          !  test sur appel1er
  !
  !MAf Initialisation a 0 de zoliq
  !      DO i = 1, klon
  !         zoliq(i)=0.
  !      ENDDO 
  ! Determiner les nuages froids par leur temperature
  !  nexpo regle la raideur de la transition eau liquide / eau glace.
  !
  ztglace = RTT - 15.0
  nexpo = 6
  !cc      nexpo = 1
  !
  ! Initialiser les sorties:
  !
  !cdir collapse
  DO k = 1, klev+1
     DO i = 1, klon
        prfl(i,k) = 0.0
        psfl(i,k) = 0.0
          do ixt=1,ntraciso
            pxtrainfl(ixt,i,k)=0.0
            pxtsnowfl(ixt,i,k)=0.0
          enddo !do ixt=1,niso
     ENDDO !DO i = 1, klon
  ENDDO !DO k = 1, klev+1     

  !cdir collapse
  DO k = 1, klev
     DO i = 1, klon
        d_t(i,k) = 0.0
        d_q(i,k) = 0.0
        d_ql(i,k) = 0.0
          do ixt=1,ntraciso
            d_xt(ixt,i,k) = 0.0
            d_xtl(ixt,i,k) = 0.0
          enddo !do ixt=1,niso
        rneb(i,k) = 0.0
        radliq(i,k) = 0.0
        frac_nucl(i,k) = 1. 
        frac_impa(i,k) = 1. 
     ENDDO !DO i = 1, klon
  ENDDO !DO k = 1, klev
  DO i = 1, klon
     rain(i) = 0.0
     snow(i) = 0.0
     zoliq(i)=0.
         do ixt=1,ntraciso
            xtrain(ixt,i) = 0.0
            xtsnow(ixt,i) = 0.0
            zxtoliq(ixt,i)=0.
         enddo !do ixt=1,niso
        ! temporaire:
!        write(*,*) 'ilp tmp 345: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)
     !     ENDDO
     !
     ! Initialiser le flux de precipitation a zero
     !
     !     DO i = 1, klon
     zrfl(i) = 0.0
          if (essai_convergence) then
          else
            zrfln(i)=0.0
          endif
          do ixt=1,ntraciso
            zxtrfl(ixt,i)=0.0
            zxtrfln(ixt,i)=0.0
          enddo
     zneb(i) = seuil_neb
  ENDDO !DO i = 1, klon
  !
  !
  !AA Pour plus de securite 

  zalpha_tr   = 0.
  zfrac_lessi = 0.

  !AA----------------------------------------------------------
  !
  ncoreczq=0
  ! Boucle verticale (du haut vers le bas)
  !
  !IM : klevm1
  !ym      klevm1=klev-1
  DO k = klev, 1, -1
     !
      
      ! cam debug!      
       ! verif en debut de boucle
      ! end verif en debut de boucle
      ! end cam debug
     !AA----------------------------------------------------------
     !
     DO i = 1, klon
        zt(i)=t(i,k)
        zq(i)=q(i,k)
         if (essai_convergence) then
         else            
           q(i,k)=max(0.0,q(i,k))
           zq(i)=max(0.0,zq(i))
         endif
         do ixt=1,ntraciso
           xt(ixt,i,k)=max(xt(ixt,i,k),0.0) 
         enddo !do ixt=1,niso
         do ixt=1,ntraciso
           zxt(ixt,i)=xt(ixt,i,k)
         enddo !do ixt=1,niso  

         ! cam verif
         ! end cam verif
     ENDDO
     !
     ! Calculer la varition de temp. de l'air du a la chaleur sensible
     ! transporter par la pluie.
     ! Il resterait a rajouter cet effet de la chaleur sensible sur les
     ! flux de surface, du a la diff. de temp. entre le 1er niveau et la
     ! surface.
     !
     IF(k.LE.klevm1) THEN         
        DO i = 1, klon
           !IM
           zmair=(paprs(i,k)-paprs(i,k+1))/RG
           zcpair=RCPD*(1.0+RVTMP2*zq(i))
           zcpeau=RCPD*RVTMP2
           zt(i) = ( (t(i,k+1)+d_t(i,k+1))*zrfl(i)*dtime*zcpeau &
                + zmair*zcpair*zt(i) ) &
                / (zmair*zcpair + zrfl(i)*dtime*zcpeau)
           !     C        WRITE (6,*) 'cppluie ', zt(i)-(t(i,k+1)+d_t(i,k+1))
        ENDDO
     ENDIF
     !
     !
     ! Calculer l'evaporation de la precipitation
     !



!        write(*,*) 'ilp tmp 507: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)     
!
!
! Calculer l''evaporation de la precipitation
!
      DO i = 1, klon
         zq_ancien(i)=zq(i)
         zqev_diag(i)=0.0
         zrfl_ancien(i)=zrfl(i)
         do ixt=1,ntraciso
           zxtrfl_ancien(ixt,i)=zxtrfl(ixt,i)
           zxt_ancien(ixt,i)=zxt(ixt,i)
         enddo 
      enddo   ! DO i = 1, klon  

     IF (evap_prec) THEN
        DO i = 1, klon
           IF (zrfl(i) .GT.0.) THEN
              IF (thermcep) THEN
                 zdelta=MAX(0.,SIGN(1.,RTT-zt(i)))
                 zqs(i)= R2ES*FOEEW(zt(i),zdelta)/pplay(i,k)
                 zqs(i)=MIN(0.5,zqs(i))
                 zcor=1./(1.-RETV*zqs(i))
                 zqs(i)=zqs(i)*zcor
              ELSE
                 IF (zt(i) .LT. t_coup) THEN
                    zqs(i) = qsats(zt(i)) / pplay(i,k)
                 ELSE
                    zqs(i) = qsatl(zt(i)) / pplay(i,k)
                 ENDIF
              ENDIF
              zqev = MAX (0.0, (zqs(i)-zq(i))*zneb(i) )
              zqevt = coef_eva * (1.0-zq(i)/zqs(i)) * SQRT(zrfl(i)) &
                   * (paprs(i,k)-paprs(i,k+1))/pplay(i,k)*zt(i)*RD/RG
              zqevt = MAX(0.0,MIN(zqevt,zrfl(i))) &
                   * RG*dtime/(paprs(i,k)-paprs(i,k+1))
              zqev = MIN (zqev, zqevt)
              zrfln(i) = zrfl(i) - zqev*(paprs(i,k)-paprs(i,k+1)) &
                   /RG/dtime

         zqev_diag(i)=zqev

         if (essai_convergence) then
         else
            zrfln(i)=max(0.0,zrfln(i))
         endif
              ! pour la glace, on r-vapore toute la prcip dans la
              ! couche du dessous
              ! la glace venant de la couche du dessus est simplement
              ! dans la couche du dessous.

              IF (zt(i) .LT. t_coup.and.reevap_ice) zrfln(i)=0.

              zq(i) = zq(i) - (zrfln(i)-zrfl(i)) &
                   * (RG/(paprs(i,k)-paprs(i,k+1)))*dtime
              zt(i) = zt(i) + (zrfln(i)-zrfl(i)) &
                   * (RG/(paprs(i,k)-paprs(i,k+1)))*dtime &
                   * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
              zrfl(i) = zrfln(i)
           ENDIF
        ENDDO
      ! * traitement de l'vaporation
      call iso_revap_fisrtilp(klon,klev,k, &
     &            zrfl_ancien,zrfl,zrfln,zt,zxt_ancien, &
     &            zxtrfl,zxtrfl_ancien,zxtrfln,zxt, &
     &            paprs,dtime, &
     &            zqs,zq_ancien,zqev_diag,zq)      

      ENDIF ! if evap_prec
     !
     ! Calculer Qs et L/Cp*dQs/dT:
     !
     IF (thermcep) THEN
        DO i = 1, klon
           zdelta = MAX(0.,SIGN(1.,RTT-zt(i)))
           zcvm5 = R5LES*RLVTT*(1.-zdelta) + R5IES*RLSTT*zdelta
           zcvm5 = zcvm5 /RCPD/(1.0+RVTMP2*zq(i))
           zqs(i) = R2ES*FOEEW(zt(i),zdelta)/pplay(i,k)
           zqs(i) = MIN(0.5,zqs(i))
           zcor = 1./(1.-RETV*zqs(i))
           zqs(i) = zqs(i)*zcor
           zdqs(i) = FOEDE(zt(i),zdelta,zcvm5,zqs(i),zcor)
        ENDDO !DO i = 1, klon
     ELSE !IF (thermcep) THEN
        DO i = 1, klon
           IF (zt(i).LT.t_coup) THEN
              zqs(i) = qsats(zt(i))/pplay(i,k)
              zdqs(i) = dqsats(zt(i),zqs(i))
           ELSE
              zqs(i) = qsatl(zt(i))/pplay(i,k)
              zdqs(i) = dqsatl(zt(i),zqs(i))
           ENDIF
        ENDDO !DO i = 1, klon
     ENDIF !IF (thermcep) THEN
     !
     ! Determiner la condensation partielle et calculer la quantite
     ! de l'eau condensee:
     !

     IF (cpartiel) THEN

        !        print*,'Dans partiel k=',k
        !
        !   Calcul de l'eau condensee et de la fraction nuageuse et de l'eau
        !   nuageuse a partir des PDF de Sandrine Bony.
        !   rneb  : fraction nuageuse
        !   zqn   : eau totale dans le nuage
        !   zcond : eau condensee moyenne dans la maille.
        !  on prend en compte le rchauffement qui diminue la partie
        ! condensee
        !
        !   Version avec les raqts

        if (ok_sursat_marine.eq.1) then
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Marine: c'est ici qu'intervient la sursaturation
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Calcul de zqs_star, seuil critique de sursaturation

        
        DO i = 1, klon

        IF (zt(i) .LT. t_coup) THEN
        CALL thresh_supersat(zt(i),RH_crit)
        zqs_star(i) = zqs(i)*RH_crit
        zpdf_beta(i)  = RH_crit
        ELSE
        RH_crit = 1.
        zqs_star(i) = zqs(i)
        zpdf_beta(i)  = RH_crit
        ENDIF

        if (zqs(i) .gt. zqs_star(i)) then
        write(*,*) 'ATTENTION zqs > zqs_star!!'
        write(*,*) 'zqs zqs_star RH_crit=', &
     & zqs(i), zqs_star(i), RH_crit
        STOP
        endif !if (zqs(i) .gt. zqs_star(i)) then

        ENDDO !DO i = 1, klon
        endif !if (sursat_marine.eq.1) then


        if (iflag_pdf.eq.0) then

           do i=1,klon
              zdelq = min(ratqs(i,k),0.99) * zq(i)
              rneb(i,k) = (zq(i)+zdelq-zqs(i)) / (2.0*zdelq)
              zqn(i) = (zq(i)+zdelq+zqs(i))/2.0
           enddo !do i=1,klon

        if (ok_sursat_marine.eq.1) then
         DO i=1,klon
           IF (rneb(i,k) .LE. 0.0) THEN
              rhcl(i,k)=zq(i)/zqs(i)
           ELSE IF (rneb(i,k) .GE. 1.0) THEN
              rhcl(i,k)=1.0
           ELSE
              rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
           ENDIF
         ENDDO
        endif !if (ok_sursat_marine.eq.1) then
           
            ! Hypothse simplificatrice mais peut-tre fausse: 
            ! eau nuageuse a mme composition que eau de la maille
            ! verif 
            ! end verif
            do i=1,klon
            do ixt=1,ntraciso
              if (zq(i).gt.0) then
               zxtn(ixt,i)=zqn(i)*(zxt(ixt,i)/zq(i))
               zxtn(ixt,i)=max(0.0,zxtn(ixt,i))
              else
               zxtn(ixt,i)=0.
              endif                  
            enddo !do ixt=1,niso
            enddo !do i=1,klon

            ! verif 
            ! end verif
            
!           write(*,*) 'il pleut 641: fin des calculs de zqn et zxtn'           
!           write(*,*) 'iflag_pdf=',iflag_pdf

        else !if (iflag_pdf.eq.0) then

             ! verif
             ! end verif

           !   Version avec les nouvelles PDFs.
           do i=1,klon
              if(zq(i).lt.1.e-15) then
                 ncoreczq=ncoreczq+1
                zq_ancien(i)=zq(i)
                 zq(i)=1.e-15

                ! on conserve le mme rapport pour les isotopes
                if (zq_ancien(i).gt.0.0) then
                   do ixt=1,ntraciso
                     zxt(ixt,i)=zxt(ixt,i)/zq_ancien(i)*zq(i)
                     zxt(ixt,i)=max(0.0,zxt(ixt,i))
                   enddo   
                else !if (zq_ancien(i).gt.0.0) then
                    do ixt=1,ntraciso
                     zxt(ixt,i)=0.0
                   enddo 
                endif !if (zq_ancien(i).gt.0.0) then

                ! verif





                ! end verif

              endif !if(zq(i).lt.1.e-15) then
           enddo ! do i=1,klon

           if (iflag_cldcon>=5) then

              call cloudth(klon,klev,k,ztv, &
                   zq,zqta,fraca, &
                   qcloud,ctot,zpspsk,paprs,ztla,zthl, &
                   ratqs,zqs,t)

              do i=1,klon
                 rneb(i,k)=ctot(i,k)
                 zqn(i)=qcloud(i)

                if (ok_sursat_marine.eq.1) then
                ! on prend le rhcl du cas creneau 
                 
                  IF (rneb(i,k) .LE. 0.0) THEN
                        rhcl(i,k)=zq(i)/zqs(i)
                  ELSE IF (rneb(i,k) .GE. 1.0) THEN
                        rhcl(i,k)=1.0 
                  ELSE
                        rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
                  ENDIF
                endif !if (ok_sursat_marine.eq.1) then
           ! if (ok_sursat_marine.eq.1) then

              enddo !do i=1,klon

           endif !if (iflag_cldcon>=5) then

           if (iflag_cldcon <= 4) then
              lognormale = .true.
           elseif (iflag_cldcon >= 6) then
              ! lognormale en l'absence des thermiques
              lognormale = fraca(:,k) < 1e-10
           else
              ! Dans le cas iflag_cldcon=5, on prend systmatiquement la
              ! bi-gaussienne
              lognormale = .false.
           end if     

           do i=1,klon
              if (lognormale(i)) then
                 zpdf_sig(i)=ratqs(i,k)*zq(i)
                 zpdf_k(i)=-sqrt(log(1.+(zpdf_sig(i)/zq(i))**2))
                 
                 if (ok_sursat_marine.eq.1) then 
                   ys(i) = - (log(zqs(i)/zq(i)))/zpdf_k(i) - zpdf_k(i)/2.
                   yss(i) = ys(i) - log(zpdf_beta(i))/zpdf_k(i)

                   if (yss(i) .lt. ys(i)) then
                     write(*,*) 'attention yss < ys !!'
                     STOP
                   endif
                 endif !if (ok_sursat_marine.eq.1) then 

                 if ((ok_sursat_marine.eq.0).or. &
     &                 (zqs(i) .eq. zqs_star(i))) then
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Cas ou il n'y a pas sursaturation
! On conserve les formules d'origine 
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc    
          
                 zpdf_delta(i)=log(zq(i)/zqs(i))
                 zpdf_a(i)=zpdf_delta(i)/(zpdf_k(i)*sqrt(2.))
                 zpdf_b(i)=zpdf_k(i)/(2.*sqrt(2.))
                 zpdf_e1(i)=zpdf_a(i)-zpdf_b(i)
                 zpdf_e1(i)=sign(min(abs(zpdf_e1(i)),5.),zpdf_e1(i))
                 zpdf_e1(i)=1.-erf(zpdf_e1(i))
                 zpdf_e2(i)=zpdf_a(i)+zpdf_b(i)
                 zpdf_e2(i)=sign(min(abs(zpdf_e2(i)),5.),zpdf_e2(i))
                 zpdf_e2(i)=1.-erf(zpdf_e2(i))

               if (zpdf_e1(i).lt.1.e-10) then
                    rneb(i,k)=0.
                    zqn(i)=zqs(i)
               else !if (zpdf_e1(i).lt.1.e-10) then
                    rneb(i,k)=0.5*zpdf_e1(i)
                    zqn(i)=zq(i)*zpdf_e2(i)/zpdf_e1(i)
               endif !if (zpdf_e1(i).lt.1.e-10) then

               
!               if (ok_sursat_marine.eq.1) then
!                 rhcl(i,k) = &
!     & erf((zpdf_k(i)+yss(i))/sqrt(2.))-erf(zpdf_k(i)/sqrt(2.))
!                 rhcl(i,k) = 0.5*zq(i)*rhcl(i,k)
!
!                 rhcl(i,k) = rhcl(i,k)/zqs(i)
!
!                IF (rneb(i,k) .LE. 0.0) THEN
!                   rhcl(i,k)=zq(i)/zqs(i)
!                ELSE IF (rneb(i,k) .GE. 1.0) THEN
!                   rhcl(i,k)=1.0
!                ELSE
!                   rhcl(i,k)=rhcl(i,k)/(1.-rneb(i,k))
!                ENDIF
!              endif !if (ok_sursat_marine.eq.1) then

              else       ! sursaturation
                  ! on est dans ce cas soit si ok_sursat_marine.eq.1 et si on a
                  ! sursat

                  !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Cas ou il y a de la sursaturation
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

              pi = ACOS(-1.)

! Calcul de rneb orig
!cccccccccccccccccccc

          zpdf_delta_o(i) = log(zq(i)/zqs(i))
          zpdf_a_o(i) = zpdf_delta_o(i)/(zpdf_k(i)*sqrt(2.))
          zpdf_b_o(i) = zpdf_k(i)/(2.*sqrt(2.))
          zpdf_e1_o(i) = zpdf_a_o(i)-zpdf_b_o(i)
          zpdf_e1_o(i) = sign(min(abs(zpdf_e1_o(i)),5.),zpdf_e1_o(i))
          zpdf_e1_o(i)=1.-erf(zpdf_e1_o(i))

          if (zpdf_e1_o(i) .lt. 1.e-10) then
          rneb_o(i,k) = 0.
          else
          rneb_o(i,k) = 0.5*zpdf_e1_o(i)
          endif

! Calcul de rneb
!ccccccccccccccc

          if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          zpdf_e1(i) = 1.-erf(yss(i)/sqrt(2.))

          if (zpdf_e1(i) .lt. 1.e-10) then
          rneb(i,k) = exp(-yss(i)**2/2.)*(yss(i)-ys(i))/(2.*sqrt(2.*pi))
          else
          rneb(i,k) = 0.5*zpdf_e1(i) + &
     & exp(-yss(i)**2/2.)*(yss(i)-ys(i))/(2.*sqrt(2.*pi))
          endif

         if (rneb(i,k) .lt. 0.) then
         write(*,*) 'ys, yss, yss-ys, 0.5*erf(yss(i)/sqrt(2.))',&
     & ys(i), yss(i), yss(i)-ys(i), 0.5*erf(yss(i)/sqrt(2.))
         endif


          else !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

! DL de rneb pour beta proche de 1

          zpdf_e1(i) = 1.-erf(ys(i)/sqrt(2.))

          if (zpdf_e1(i) .lt. 1.e-10) then
          rneb(i,k) = exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1.)/(2.*zpdf_k(i)*sqrt(2.*pi))- &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**2/(4.*zpdf_k(i)*sqrt(2.*pi))+ &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**3* &
     & (1.+2*zpdf_k(i)**2-ys(i)**2)/(12.*zpdf_k(i)**3*sqrt(2.*pi))
          else
          rneb(i,k) = 0.5*zpdf_e1(i) + &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1.)/(2.*zpdf_k(i)*sqrt(2.*pi))- &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**2/(4.*zpdf_k(i)*sqrt(2.*pi))+ &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**3* &
     & (1.+2*zpdf_k(i)**2-ys(i)**2)/(12.*zpdf_k(i)**3*sqrt(2.*pi))
          endif

          if (rneb(i,k) .lt. 0.) then
          write(*,*) 'rneb< 0 ds le cas DL'
          endif


          endif !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          if ( rneb(i,k) .lt. 0. .or.  rneb(i,k) .gt. 1.) then
          write(*,*) 'SURSAT:  rneb(i,k)< 0 or rneb(i,k)>1', &
     & 'rneb(i,k),ys(i),zpdf_beta(i),zpdf_k(i),pi', &
     & rneb(i,k),ys(i),zpdf_beta(i),zpdf_k(i),pi
          endif



! Flag presence de sursaturation
!ccccccccccccccccccccccccccccccc

         if (abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i)) .and. &
     & (rneb_o(i,k) - rneb(i,k) .gt. 1.e-5 .or. &
     & (rneb_o(i,k) .gt. 0.98 .and. rneb(i,k) .gt. 0.98))) then

         flag_ss(i,k) = 1.

         else

         flag_ss(i,k) = 0.

         if (rneb_o(i,k) - rneb(i,k) .lt. -1.e-5) then
         write(*,*) 'rneb_o < rneb, rneb_o, rneb, yss, ys,& 
     & 0.5*zpdf_e1_o, 0.5*zpdf_e1, exp',&
     & rneb_o(i,k),rneb(i,k),yss(i), ys(i), &
     & 0.5*zpdf_e1_o(i),0.5*zpdf_e1(i), &
     & exp(-yss(i)**2/2.)*(yss(i)-ys(i))/(2.*sqrt(2.*pi))
         endif

         endif

! Fraction de ciel clair sursature
!ccccccccccccccccccccccccccccccccc

        if (abs(zpdf_beta(i) -1.) .gt. epsilon(zpdf_beta(i)) .and. &
     & rneb_o(i,k) - rneb(i,k) .gt. 1.e-5) then
        clsk_ss(i,k) = rneb_o(i,k) - rneb(i,k)
        else
        clsk_ss(i,k) = 0.
        endif


! Nuage dans un environnement sursature
!ccccccccccccccccccccccccccccccccccccccc

         if (rneb(i,k) .gt. 0 .and. & 
     & abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then
         rneb_ss(i,k) = rneb(i,k)
         endif

! Autre detection de sursaturation
!ccccccccccccccccccccccccccccccccc

         if (rneb_o(i,k) .gt. 0 .and. &
     & abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then
         flagb_ss(i,k) = 1.
         else
         flagb_ss(i,k) = 0. 
         endif

! Calcul de zqn
!cccccccccccccc          

! Calcul de zqn
!cccccccccccccc

          if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          zpdf_e1(i) = 0.5*zq(i)*(1.-erf((yss(i)+zpdf_k(i))/sqrt(2.)))

          zqn(i) = zpdf_e1(i) + &
     & exp(-zpdf_k(i)**2/2.-yss(i)**2/2.-zpdf_k(i)*(ys(i)+yss(i)))* &
     & zq(i)* &
     & (exp(zpdf_k(i)*yss(i))+ &
     & exp(zpdf_k(i)*ys(i))*(zpdf_k(i)*ys(i)-zpdf_k(i)*yss(i)-1))/ &
     & (zpdf_k(i)**2*sqrt(2.*pi)*(yss(i)-ys(i)))

          else !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

! DL de zqn pour beta proche de 1

          zpdf_e1(i) = 0.5*zq(i)*(1.-erf((zpdf_k(i)+ys(i))/sqrt(2.)))

          zqn(i) = zpdf_e1(i)+ &
     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)/ &
     & (2.*zpdf_k(i)*sqrt(2.*pi))- &
     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**2/ &
     & (12.*zpdf_k(i)*sqrt(2.*pi))+ &
     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**3* &
     & (2.+zpdf_k(i)**2-2.*ys(i)**2)/(24.*zpdf_k(i)**3*sqrt(2.*pi))

          endif !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          if (rneb(i,k) .gt. 0. .and. rneb(i,k) .lt. 1.) then
          zqn(i) = zqn(i)/rneb(i,k)    ! a verifier  
          endif

! Calcul de rhcl
!ccccccccccccccc

!          if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

!          rhcl_p0 = &
!     & erf((zpdf_k(i)+yss(i))/sqrt(2.))-erf(zpdf_k(i)/sqrt(2.))

!          rhcl_p1 = &
!     & exp(-0.5*zpdf_k(i)**2-0.5*yss(i)**2-zpdf_k(i)*(ys(i)+yss(i)))

!          rhcl_p2 = exp(zpdf_k(i)*yss(i))+exp(zpdf_k(i)*ys(i))* &
!     & (-1.+zpdf_k(i)*ys(i)-zpdf_k(i)*yss(i))

!          rhcl_p = rhcl_p1*rhcl_p2*sqrt(2./pi)/ &
!     & ((ys(i)-yss(i))*zpdf_k(i)**2)

!          rhcl(i,k) = 0.5*zq(i)*(rhcl_p0+rhcl_p)

!          rhcl(i,k) = rhcl(i,k)/zqs(i)

!          else !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then!

! DL de rhcl pour beta proche de 1

!          rhcl(i,k) = &
!     & 0.5*zq(i)* &
!     & (erf((zpdf_k(i)+ys(i))/sqrt(2.))-erf(zpdf_k(i)/sqrt(2.)))- &
!     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)/ &
!     & (2.*zpdf_k(i)*sqrt(2.*pi))+ &
!     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**2/ &
!     & (12.*zpdf_k(i)*sqrt(2.*pi))- &
!     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**3* &
!     & (2.+zpdf_k(i)**2-2.*ys(i)**2)/ &
!     & (24.*zpdf_k(i)**3*sqrt(2.*pi))

!          rhcl(i,k) = rhcl(i,k)/zqs(i)

!          endif !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          
!           IF (rneb(i,k) .LE. 0.0) THEN
!              rhcl(i,k)=zq(i)/zqs(i)
!           ELSE IF (rneb(i,k) .GE. 1.0) THEN
!              rhcl(i,k)=1.0
!           ELSE
!              rhcl(i,k)=rhcl(i,k)/(1.-rneb(i,k)) 
!           ENDIF

          endif  ! sursaturation

          endif ! if (lognormale(i)) then
           enddo !do i=1,klon


        endif ! iflag_pdf

!        write(*,*) 'ilp tmp 1090: zzoliq,xtoliq(1,363)=',zoliq(363),zxtoliq(1,363)
        DO i=1,klon
           IF (rneb(i,k) .LE. 0.0) THEN
              zqn(i) = 0.0
              rneb(i,k) = 0.0
              zcond(i) = 0.0
           ELSE IF (rneb(i,k) .GE. 1.0) THEN
              zqn(i) = zq(i)
              rneb(i,k) = 1.0                  
              zcond(i) = MAX(0.0,zqn(i)-zqs(i))
           ELSE
              zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)
           ENDIF
        ENDDO !DO i=1,klon

        ! si pas sursat Marine, on calcule rchl  la fin.
        ! Sinon, a a dj t calcul avant, il ne faut pas l'craser
!        if (ok_sursat_marine.eq.0) then   
        ! 22 dec 2012: Marine envoie nouvelle version, maintenant on calcule
        ! rhcl tout  la fin
         DO i=1,klon
           IF (rneb(i,k) .LE. 0.0) THEN
                rhcl(i,k)=zq(i)/zqs(i)
           ELSE IF (rneb(i,k) .GE. 1.0) THEN
                rhcl(i,k)=1.0
           ELSE
                rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
           ENDIF
         ENDDO !DO i=1,klon
!        endif !if (ok_sursat_marine.eq.0) then

        !         do i=1,klon
        !            IF (rneb(i,k) .LE. 0.0) zqn(i) = 0.0
        !            IF (rneb(i,k) .GE. 1.0) zqn(i) = zq(i)
        !            rneb(i,k) = MAX(0.0,MIN(1.0,rneb(i,k)))
        !c           zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)/(1.+zdqs(i))
        !c  On ne divise pas par 1+zdqs pour forcer a avoir l'eau predite par
        !c  la convection.
        !c  ATTENTION !!! Il va falloir verifier tout ca.
        !            zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)
        !c           print*,'ZDQS ',zdqs(i)
        !c--Olivier
        !            rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
        !            IF (rneb(i,k) .LE. 0.0) rhcl(i,k)=zq(i)/zqs(i)
        !            IF (rneb(i,k) .GE. 1.0) rhcl(i,k)=1.0
        !c--fin

        if (ok_sursat_marine.eq.1) then
        DO i=1,klon
        if (zqn(i) .lt. 0) then
        write(*,*) 'ATTENTION zqn(i) < 0 '
        STOP
        endif
        if (rhcl(i,k) .lt. 0) then
        write(*,*) 'ATTENTION rhcl(i,k) < 0 '
        STOP
        endif
        ENDDO
        endif ! if (ok_sursat_marine.eq.1) then

        !           ENDDO
     ELSE  ! cpartiel
        DO i = 1, klon
           IF (zq(i).GT.zqs(i)) THEN
              rneb(i,k) = 1.0
           ELSE
              rneb(i,k) = 0.0
           ENDIF
           zcond(i) = MAX(0.0,zq(i)-zqs(i))/(1.+zdqs(i))
        ENDDO
     ENDIF





        ! on calcule la compo iso de l'eau dans le nuage et en ciel
        ! clair: zxtn/zqn et zxtcs/zqcs

              ! Hypothsesimplificatrice mais peut-tre fausse: 
              ! eau nuageuse a mme composition que eau de la maille 

              ! ce calcul n'eat pas sens dpendre du type de PDF utilis     
         do i=1,klon  
              if ((bidouille_anti_divergence).and. &
     &           (use_iso_eau.eq.1)) then
                zxt(iso_eau,i)= max(zq(i),0.0)  
!                if (zxt(iso_eau,i).gt.0.0) then
!                    do ixt=1,niso
!                        zxt(ixt,i)=zxt(ixt,i)*zq(i)/zxt(iso_eau,i)
!                    enddo
!                else !if (zxt(iso_eau,i).gt.0.0) then
!                    do ixt=1,niso
!                        zxt(ixt,i)=0.0
!                    enddo    
!                endif !if (zxt(iso_eau,i).gt.0.0) then
              endif !if ((bidouille_anti_divergence).and. 

              if (zq(i).gt.1e-15) then   
                ! 2 fev 2010: on modifie seuil de zq  
                ! avant, c'tait 0. Vrifier que a n'a pas de
                ! consquences trop graves pour les isotopes.
                do ixt=1,ntraciso             
                   zxtn(ixt,i)=zqn(i)*(zxt(ixt,i)/zq(i))  
                enddo !do ixt=1,niso         
              else !if (zq(i).gt.0.0) then 
                  ! problme: zq~0 et sans compo, mais zqn >0
                  ! on fait ce qu'on peut dans ce cas pathologique
                do ixt=1,ntraciso           
                   zxtn(ixt,i)=0.0              
                enddo !do ixt=1,niso 
                if ((bidouille_anti_divergence) &
     &                   .and.(use_iso_eau.eq.1)) then
!                   write(*,*) 'ilp tmp 848: warning: bidouille'
                   zxtn(iso_eau,i)=zqn(i)
                endif !if ((bidouille_anti_divergence)
              endif  !if (zq(i).gt.0.0) then 

!                 write(*,*) 'il pleut 695: calcul de zxtn, i=',i
!                 write(*,*) 'zxtn(4,i)=',zxtn(4,i)
!                 write(*,*) 'zqn(i)=',zqn(i)
              do ixt=1,ntraciso    
               zxtn(ixt,i)=max(0.0,zxtn(ixt,i))          
              enddo !do ixt=1,niso

         enddo ! do i=1,klon
         
!        write(*,*) 'ilp tmp 1311: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)

     !
     DO i = 1, klon
        zq(i) = zq(i) - zcond(i)
        !         zt(i) = zt(i) + zcond(i) * RLVTT/RCPD
        zt(i) = zt(i) + zcond(i) * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
     ENDDO
     !
      ! isotopes lors de la condensation
      
      do i=1,klon  
!        call calcul_zfice(zt(i),zfice(i)) ! inliner pour optimisation:
        zfice(i)=1.0-(zt(i)-pxtice)/(pxtmelt-pxtice)
        zfice(i) = MIN(MAX(zfice(i),0.0),1.0)
       enddo  
        
        ! calcul de la composition du condensat
        ! hypothse simplificatrice (mais  garder en mmoire!): l'eau
        ! nuageuse a la mme composition que l'eau totale de la maille:
        ! c'est  dire zxtn=(zxt/zq)*zqn
        
        call condiso_liq_ice_vectall(zxtn,zqn,zcond,zt,zfice, &
     &            zxtice,zxtliq,klon)
        do i=1,klon
        do ixt=1,ntraciso                 
          zxtcond(ixt,i)=zxtice(ixt,i)+zxtliq(ixt,i)
          zxtcond(ixt,i)=max(zxtcond(ixt,i),0.0)
          zxtcond(ixt,i)=min(zxtcond(ixt,i),zxt(ixt,i))
        enddo !do ixt=1,niso 
        enddo !do i=1,klon

       do i=1,klon   
       enddo  !do i=1,klon       
        
!#ifdef ISOVERIFIDRIS

       ! retranchage  zxt 
      do i=1,klon 
       do ixt=1,ntraciso
          zxt(ixt,i)=zxt(ixt,i)-zxtcond(ixt,i)
       enddo !do ixt=1,niso   
      enddo !do i=1,klon 


     
      
     ! Partager l'eau condensee en precipitation et eau liquide nuageuse
     !
      DO i = 1, klon
        IF (rneb(i,k).GT.0.0) THEN
           zoliq(i) = zcond(i)
           zrho(i) = pplay(i,k) / zt(i) / RD
           zdz(i) = (paprs(i,k)-paprs(i,k+1)) / (zrho(i)*RG)
           zfice(i) = 1.0 - (zt(i)-ztglace) / (273.13-ztglace)
           zfice(i) = MIN(MAX(zfice(i),0.0),1.0)
           zfice(i) = zfice(i)**nexpo
           zneb(i) = MAX(rneb(i,k), seuil_neb)
           radliq(i,k) = zoliq(i)/REAL(ninter+1)
        ENDIF !IF (rneb(i,k).GT.0.0) THEN
      ENDDO !DO i = 1, klon
      ! initilsation des quantits totales d'eau liquide et solide qui
      ! tombent
      do i=1,klon
         if (rneb(i,k).gt.0.0) then
            do ixt=1,ntraciso
               zxtoliq(ixt,i)=zxtcond(ixt,i)
            enddo
         endif ! if (rneb(i,k).gt.0.0)
       enddo
          
     !
     DO n = 1, ninter
        DO i = 1, klon
           IF (rneb(i,k).GT.0.0) THEN
              zrhol(i) = zrho(i) * zoliq(i) / zneb(i)

              IF (zneb(i).EQ.seuil_neb) THEN
                 ztot = 0.0
              ELSE
                 !  quantite d'eau a eliminer: zchau
                 !  meme chose pour la glace: zfroi
                 if (ptconv(i,k)) then
                    zcl   =cld_lc_con
                    zct   =1./cld_tau_con
                    zfroi    = dtime/REAL(ninter)/zdz(i)*zoliq(i) &
                         *fallvc(zrhol(i)) * zfice(i)
                 else
                    zcl   =cld_lc_lsc
                    zct   =1./cld_tau_lsc
                    zfroi    = dtime/REAL(ninter)/zdz(i)*zoliq(i) &
                         *fallvs(zrhol(i)) * zfice(i)
                 endif
                 zchau    = zct   *dtime/REAL(ninter) * zoliq(i) &
                      *(1.0-EXP(-(zoliq(i)/zneb(i)/zcl   )**2)) *(1.-zfice(i))
                 ztot    = zchau    + zfroi
                 ztot    = MAX(ztot   ,0.0)
              ENDIF
              ztot    = MIN(ztot,zoliq(i))
              zoliq(i) = MAX(zoliq(i)-ztot   , 0.0)
              radliq(i,k) = radliq(i,k) + zoliq(i)/REAL(ninter+1)
           ENDIF
        ENDDO
     ENDDO
     !


      do i=1,klon
       IF (rneb(i,k).GT.0.0) THEN
        if (zcond(i).gt.ridicule**2) then
            ! le 21 dec: on change .gt.0 en .gt.ridicule**2 pour viter valeur
            ! ridiculement petites
           do ixt=1,ntraciso
             zxtoliq(ixt,i)=zoliq(i)*(zxtcond(ixt,i)/zcond(i))
           enddo !do ixt=1,niso           
        else !if (zcond(i).gt.0.0) then            
           if (zoliq(i).lt.ridicule) then
               do ixt=1,ntraciso
                zxtoliq(ixt,i)=0.0
               enddo
           else !if (zoliq(i).lt.ridicule) then
               ! modif 28 oct 2008
              do ixt=1,niso
                zxtoliq(ixt,i)=zoliq(i)*Rdefault(ixt)
              enddo !do ixt=1,niso
            endif   !if (zoliq(i).lt.ridicule) then         
        endif !if (zcond(i).gt.0.0) then
       endif !IF (rneb(i,k).GT.0.0) THEN
      enddo ! do i=1,klon 
           
         ! cam verif
                
     DO i = 1, klon
        IF (rneb(i,k).GT.0.0) THEN
           d_ql(i,k) = zoliq(i)
           zrfl(i) = zrfl(i)+ MAX(zcond(i)-zoliq(i),0.0) &
                * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)

         
         do ixt=1,ntraciso
            d_xtl(ixt,i,k)=zxtoliq(ixt,i)
         enddo ! do ixt=1,niso   
            ! Rq: 28 oct 2008: si zoliq a la mme compo que zcond, alors
            ! on vite pb numriques si on prend directement zrfl avec
            ! la compo de zcond?
!          do ixt=1,niso  
!            zxtrfl(ixt,i)=zxtrfl(ixt,i) &
!     &           +max(zxtcond(ixt,i)-zxtoliq(ixt,i),0.0) &
!     &           * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)
!          enddo  
            if (zcond(i).gt.ridicule**2) then
                ! modif le 19 dec 2012 pour viter zcond pathologiquement petits
                do ixt=1,ntraciso
                  zxtrfl(ixt,i)=zxtrfl(ixt,i) &
     &                  +(zxtcond(ixt,i)/zcond(i)) &
     &                  * MAX(zcond(i)-zoliq(i),0.0) &
     &                  * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)
                enddo !do ixt=1,niso
            else !if (zcond(i).gt.0.0) then
                if (zcond(i)-zoliq(i).gt.0.0) then
                    do ixt=1,niso
                        zxtrfl(ixt,i)=zxtrfl(ixt,i) &
     &                  +Rdefault(ixt) &
     &                  * MAX(zcond(i)-zoliq(i),0.0) &
     &                  * (paprs(i,k)-paprs(i,k+1))/(RG*dtime) 
                    enddo !do ixt=1,niso  
                else !if (zcond(i)-zoliq(i).gt.0.0) then
                    ! zrfl non modifi
                endif !if (zcond(i)-zoliq(i).gt.0.0) then
            endif !if (zcond(i).gt.0.0) then

         ! cam verif         
!#ifdef ISOVERIFIDRIS
         ! bidouille
         if ((bidouille_anti_divergence).and.(use_iso_eau.eq.1)) then 
               zxtrfl(iso_eau,i)= zrfl(i)   
!             if (zxtrfl(iso_eau,i).gt.0.0) then
!             do ixt=1,niso
!               zxtrfl(ixt,i)=zxtrfl(ixt,i)*zrfl(i)/zxtrfl(iso_eau,i)
!             enddo
!           endif !if (zxtrfl(iso_eau,i).gt.0.0) then
         endif ! if (bidouille_anti_divergence) then
         ! end bidouille
         ! end cam verif
        ENDIF !IF (rneb(i,k).GT.0.0) THEN

        IF (zt(i).LT.RTT) THEN
           psfl(i,k)=zrfl(i)
         do ixt=1,ntraciso
           pxtsnowfl(ixt,i,k)=zxtrfl(ixt,i)
         enddo
        ELSE !IF (zt(i).LT.RTT) THEN
           prfl(i,k)=zrfl(i)
         do ixt=1,ntraciso
           pxtrainfl(ixt,i,k)=zxtrfl(ixt,i)
         enddo         
        ENDIF !IF (zt(i).LT.RTT) THEN
     ENDDO !DO i = 1, klon 
     !
     ! Calculer les tendances de q et de t:
     !
     DO i = 1, klon
        d_q(i,k) = zq(i) - q(i,k)
         do ixt=1,ntraciso
             d_xt(ixt,i,k)=zxt(ixt,i)-xt(ixt,i,k)   
         enddo
         ! cam verif
         ! end cam verif
        d_t(i,k) = zt(i) - t(i,k)
     ENDDO
     !
     !AA--------------- Calcul du lessivage stratiforme  -------------

     DO i = 1,klon
        !
        if(zcond(i).gt.zoliq(i)+1.e-10) then
         beta(i,k) = (zcond(i)-zoliq(i))/zcond(i)/dtime
        else
         beta(i,k) = 0.
        endif
        zprec_cond(i) = MAX(zcond(i)-zoliq(i),0.0) &
             * (paprs(i,k)-paprs(i,k+1))/RG
        IF (rneb(i,k).GT.0.0.and.zprec_cond(i).gt.0.) THEN
           !AA lessivage nucleation LMD5 dans la couche elle-meme
           if (t(i,k) .GE. ztglace) THEN
              zalpha_tr = a_tr_sca(3)
           else
              zalpha_tr = a_tr_sca(4)
           endif
           zfrac_lessi = 1. - EXP(zalpha_tr*zprec_cond(i)/zneb(i))
           pfrac_nucl(i,k)=pfrac_nucl(i,k)*(1.-zneb(i)*zfrac_lessi)
           frac_nucl(i,k)= 1.-zneb(i)*zfrac_lessi 
           !
           ! nucleation avec un facteur -1 au lieu de -0.5
           zfrac_lessi = 1. - EXP(-zprec_cond(i)/zneb(i))
           pfrac_1nucl(i,k)=pfrac_1nucl(i,k)*(1.-zneb(i)*zfrac_lessi)
        ENDIF
        !
     ENDDO      ! boucle sur i
     !
     !AA Lessivage par impaction dans les couches en-dessous
     DO kk = k-1, 1, -1
        DO i = 1, klon
           IF (rneb(i,k).GT.0.0.and.zprec_cond(i).gt.0.) THEN
              if (t(i,kk) .GE. ztglace) THEN
                 zalpha_tr = a_tr_sca(1)
              else
                 zalpha_tr = a_tr_sca(2)
              endif
              zfrac_lessi = 1. - EXP(zalpha_tr*zprec_cond(i)/zneb(i))
              pfrac_impa(i,kk)=pfrac_impa(i,kk)*(1.-zneb(i)*zfrac_lessi)
              frac_impa(i,kk)= 1.-zneb(i)*zfrac_lessi
           ENDIF
        ENDDO
     ENDDO
      ! verif en fin de boucle
      ! end verif en fin de boucle

     !
     !AA----------------------------------------------------------
     !                     FIN DE BOUCLE SUR K   
  end DO
  !
  !AA-----------------------------------------------------------
  !
  ! Pluie ou neige au sol selon la temperature de la 1ere couche
  !
  DO i = 1, klon
     IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
        snow(i) = zrfl(i)
        zlh_solid(i) = RLSTT-RLVTT
        do ixt=1,ntraciso
          xtsnow(ixt,i) = zxtrfl(ixt,i)
        enddo !do ixt=1,ntraciso
     ELSE !IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
        rain(i) = zrfl(i)
        zlh_solid(i) = 0.
        do ixt=1,ntraciso
          xtrain(ixt,i) = zxtrfl(ixt,i)
        enddo !do ixt=1,ntraciso
     ENDIF !IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
      ! cam verif 
       ! end cam verif   
  ENDDO
  !
  ! For energy conservation : when snow is present, the solification
  ! latent heat is considered.
  DO k = 1, klev
     DO i = 1, klon
        zcpair=RCPD*(1.0+RVTMP2*(q(i,k)+d_q(i,k)))
        zmair=(paprs(i,k)-paprs(i,k+1))/RG
        zm_solid = (prfl(i,k)-prfl(i,k+1)+psfl(i,k)-psfl(i,k+1))*dtime
        d_t(i,k) = d_t(i,k) + zlh_solid(i) *zm_solid / (zcpair*zmair)
     END DO
  END DO
  !

  if (ncoreczq>0) then
     WRITE(lunout,*)'WARNING : ZQ dans fisrtilp ',ncoreczq,' val < 1.e-15.'
  endif

      ! verif en sortie de il pleut 
      ! end verif

END SUBROUTINE fisrtilp
