!
! $Header$
!
c
c
      SUBROUTINE hydrol(dtime,pctsrf,rain_fall,snow_fall,evap,
     .                  agesno, tsol,qsol,snow,runoff)
      USE dimphy
      IMPLICIT none
c======================================================================
c Auteur(s): Z.X. Li (LMD/CNRS)
c date: 19940414
c======================================================================
c
c Traitement de l'hydrologie du sol
c ---------------------------------
c rain_fall: taux de pluie
c snow_fall: taux de neige
c agesno: age de la neige
c evap: taux d'evaporation
c tsol: temperature du sol
c qsol: humidite du sol
c snow: couverture neigeuse
C
cym#include "dimensions.h"
cym#include "dimphy.h"
!
! $Header$
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
! A1.0 Fundamental constants
      REAL RPI,RCLUM,RHPLA,RKBOL,RNAVO
! A1.1 Astronomical constants
      REAL RDAY,REA,REPSM,RSIYEA,RSIDAY,ROMEGA
! A1.1.bis Constantes concernant l'orbite de la Terre:
      REAL R_ecc, R_peri, R_incl
! A1.2 Geoide
      REAL RA,RG,R1SA
! A1.3 Radiation
!     REAL RSIGMA,RI0
      REAL RSIGMA
! A1.4 Thermodynamic gas phase
      REAL R,RMD,RMO3,RMV,RD,RV,RCPD,RCPV,RCVD,RCVV
      REAL RKAPPA,RETV
! A1.5,6 Thermodynamic liquid,solid phases
      REAL RCW,RCS
! A1.7 Thermodynamic transition of phase
      REAL RLVTT,RLSTT,RLMLT,RTT,RATM
! A1.8 Curve of saturation
      REAL RESTT,RALPW,RBETW,RGAMW,RALPS,RBETS,RGAMS
      REAL RALPD,RBETD,RGAMD
!
      COMMON/YOMCST/RPI   ,RCLUM ,RHPLA ,RKBOL ,RNAVO                   &
     &      ,RDAY  ,REA   ,REPSM ,RSIYEA,RSIDAY,ROMEGA                  &
     &      ,R_ecc, R_peri, R_incl                                      &
     &      ,RA    ,RG    ,R1SA                                         &
     &      ,RSIGMA                                                     &
     &      ,R     ,RMD   ,RMO3  ,RMV   ,RD    ,RV    ,RCPD             &
     &      ,RCPV  ,RCVD  ,RCVV  ,RKAPPA,RETV                           &
     &      ,RCW   ,RCS                                                 &
     &      ,RLVTT ,RLSTT ,RLMLT ,RTT   ,RATM                           &
     &      ,RESTT ,RALPW ,RBETW ,RGAMW ,RALPS ,RBETS ,RGAMS            &
     &      ,RALPD ,RBETD ,RGAMD
!    ------------------------------------------------------------------
!$OMP THREADPRIVATE(/YOMCST/)
!
! $Header$
!
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
      INTEGER nbsrf
      PARAMETER (nbsrf=4) ! nombre de sous-fractions pour une maille
!
      INTEGER is_oce
      PARAMETER (is_oce=3) ! ocean
      INTEGER is_sic
      PARAMETER (is_sic=4) ! glace de mer
      INTEGER is_ter
      PARAMETER (is_ter=1) ! terre
      INTEGER is_lic
      PARAMETER (is_lic=2) ! glacier continental
!
      REAL epsfra
      PARAMETER (epsfra=1.0E-05)
!
      CHARACTER(len=3) clnsurf(nbsrf)
      DATA clnsurf/'ter', 'lic', 'oce', 'sic'/
      SAVE clnsurf
!$OMP THREADPRIVATE(clnsurf)
c
      REAL chasno ! epaisseur du sol: 0.15 m
      PARAMETER (chasno=3.334E+05/(2.3867E+06*0.15))
      REAL mx_eau_sol
      PARAMETER (mx_eau_sol=150.0)
c
      REAL dtime
      REAL pctsrf(klon,nbsrf)
      REAL snow(klon,nbsrf), tsol(klon,nbsrf), qsol(klon,nbsrf)
      REAL snow_fall(klon), rain_fall(klon), evap(klon)
      REAL runoff(klon), agesno(klon)
C
      INTEGER i, is
      REAL subli, fsno
C-----------------------------------------------------------------------
      DO 99999 i = 1, klon
c
         runoff(i) = 0.0
c
         is = is_ter
         snow(i,is) = snow(i,is) + snow_fall(i) * dtime * pctsrf(i,is)
         IF (pctsrf(i,is) .GT. epsfra) THEN
            subli = MIN(evap(i)*dtime,snow(i,is))
            snow(i,is) = snow(i,is) - subli
            fsno = MIN(MAX((tsol(i,is)-RTT)/chasno,0.0),snow(i,is))
            snow(i,is) = snow(i,is) - fsno
            tsol(i,is) = tsol(i,is) - fsno*chasno
            qsol(i,is) = qsol(i,is) + (rain_fall(i)-evap(i))*dtime
     .                              + subli + fsno
            qsol(i,is) = MAX(qsol(i,is),0.0)
            runoff(i) = runoff(i) + MAX(qsol(i,is)-mx_eau_sol, 0.0)
     .                            * pctsrf(i,is)
            qsol(i,is) = MIN(qsol(i,is),mx_eau_sol)
ccc         ELSE
ccc            snow(i,is) = 0.0
ccc            qsol(i,is) = 0.0
ccc            tsol(i,is) = 0.0
         ENDIF
c
         is = is_lic
         snow(i,is) = snow(i,is) + snow_fall(i) * dtime * pctsrf(i,is)
         IF (pctsrf(i,is) .GT. epsfra) THEN
            subli = MIN(evap(i)*dtime,snow(i,is))
            snow(i,is) = snow(i,is) - subli
            fsno = MIN(MAX((tsol(i,is)-RTT)/chasno,0.0),snow(i,is))
            snow(i,is) = snow(i,is) - fsno
            tsol(i,is) = tsol(i,is) - fsno*chasno
            qsol(i,is) = qsol(i,is) + (rain_fall(i)-evap(i))*dtime
     .                              + subli + fsno
            qsol(i,is) = MAX(qsol(i,is),0.0)
            runoff(i) = runoff(i) + MAX(qsol(i,is)-mx_eau_sol, 0.0)
     .                            * pctsrf(i,is)
            qsol(i,is) = MIN(qsol(i,is),mx_eau_sol)
c je limite la temperature a RTT-1.8 (il faudrait aussi prendre l'eau de
c la fonte) (Laurent Li, le 14mars98):
cIM cf GK   tsol(i,is) = MIN(tsol(i,is),RTT-1.8)
cIM cf GK : la glace fond a 0C, non pas a -1.8
            tsol(i,is) = MIN(tsol(i,is),RTT)
c
ccc         ELSE
ccc            snow(i,is) = 0.0
ccc            qsol(i,is) = 0.0
ccc            tsol(i,is) = 0.0
         ENDIF
c
         is = is_sic
         qsol(i,is) = 0.0
         snow(i,is) = snow(i,is) + snow_fall(i) * dtime * pctsrf(i,is)
         IF (pctsrf(i,is) .GT. epsfra) THEN
            subli = MIN(evap(i)*dtime,snow(i,is))
            snow(i,is) = snow(i,is) - subli
            fsno = MIN(MAX((tsol(i,is)-RTT)/chasno,0.0),snow(i,is))
            snow(i,is) = snow(i,is) - fsno
            tsol(i,is) = tsol(i,is) - fsno*chasno
c je limite la temperature a RTT-1.8 (il faudrait aussi prendre l'eau de
c la fonte) (Laurent Li, le 14mars98):
cIM cf GK   tsol(i,is) = MIN(tsol(i,is),RTT-1.8)
cIM cf GK : la glace fond a 0C, non pas a -1.8
            tsol(i,is) = MIN(tsol(i,is),RTT)
c
ccc         ELSE
ccc            snow(i,is) = 0.0
ccc            tsol(i,is) = 0.0
         ENDIF
c
         agesno(i) = (agesno(i)+ (1.-agesno(i)/50.)*dtime/86400.)
     .             * EXP(-1.*MAX(0.0,snow_fall(i))*dtime/0.3)
         agesno(i) = MAX(agesno(i),0.0)
c
99999 CONTINUE
c
      RETURN
      END
