!
MODULE ocean_forced_mod
!
! This module is used for both the sub-surfaces ocean and sea-ice for the case of a 
! forced ocean,  "ocean=force".
!
  IMPLICIT NONE

CONTAINS
!
!****************************************************************************************
!
  SUBROUTINE ocean_forced_noice( &
       itime, dtime, jour, knon, knindex, &
       p1lay, cdragh, cdragm, precip_rain, precip_snow, &
       temp_air, spechum, &
       AcoefH, AcoefQ, BcoefH, BcoefQ, &
       AcoefU, AcoefV, BcoefU, BcoefV, &
       ps, u1, v1, &
       radsol, snow, agesno, & 
       qsurf, evap, fluxsens, fluxlat, flux_u1, flux_v1, &
       tsurf_new, dflux_s, dflux_l &

            ,xtprecip_rain, xtprecip_snow, xtspechum,Roce,rlat, &
            xtsnow,xtevap,h1 &  




            )
!
! This subroutine treats the "open ocean", all grid points that are not entierly covered
! by ice.
! The routine receives data from climatologie file limit.nc and does some calculations at the 
! surface. 
!
    USE dimphy
    USE calcul_fluxs_mod
    USE limit_read_mod
    INCLUDE "indicesol.h"
    INCLUDE "YOMCST.h"

! Input arguments
!****************************************************************************************
    INTEGER, INTENT(IN)                      :: itime, jour, knon
    INTEGER, DIMENSION(klon), INTENT(IN)     :: knindex
    REAL, INTENT(IN)                         :: dtime
    REAL, DIMENSION(klon), INTENT(IN)        :: p1lay
    REAL, DIMENSION(klon), INTENT(IN)        :: cdragh, cdragm
    REAL, DIMENSION(klon), INTENT(IN)        :: precip_rain, precip_snow
    REAL, DIMENSION(klon), INTENT(IN)        :: temp_air, spechum
    REAL, DIMENSION(klon), INTENT(IN)        :: AcoefH, AcoefQ, BcoefH, BcoefQ
    REAL, DIMENSION(klon), INTENT(IN)        :: AcoefU, AcoefV, BcoefU, BcoefV
    REAL, DIMENSION(klon), INTENT(IN)        :: ps
    REAL, DIMENSION(klon), INTENT(IN)        :: u1, v1
!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env
        logical ok_prod_nucl_tritium
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)
        

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)
        
        integer niso ! nombre d'isotopes utiliss   
        parameter (niso=use_iso_HTO+use_iso_O17 &
     &               +use_iso_O18+use_iso_HDO+use_iso_eau) 

        character*3 striso(niso)    

        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso) &
     &    , tnat(niso), toce(niso), tcorr(niso) &
     &    , tdifrel(niso) &
     &    , talph1(niso), talph2(niso), talph3(niso) &
     &    , talps1(niso), talps2(niso) &
     &    , tkcin0(niso) &
     &    , tkcin1(niso),tkcin2(niso) &
     &    , alpha_liq_sol(niso) &
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg &
     &    , pxtmelt, pxtice, pxtmin, pxtmax &
     &    , bidouille_anti_divergence,essai_convergence &
     &    , initialisation_iso, initialisation_isotrac &
     &    , modif_sst,deltaTtest &
     &    , modif_sic,deltasic &
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit &
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo &
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice   &
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot &
     &    , rh_cste_surf_cond, T_cste_surf_cond  &
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie &
     &    , alphak_stewart, cste_surf_cond &
     &    , mixlen &
     &    , tnat, toce, tcorr &
     &    , tdifrel &
     &    , calendrier_guide &
     &    , talph1, talph2, talph3 &
     &    , talps1, talps2 &
     &    , tkcin0, tkcin1, tkcin2 &
     &    , alpha_liq_sol, Rdefault &
     &    , striso  &
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO &
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont &
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim &
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce &
     &    , ratqsbasnew,Pcrit_ratqs &
     &    , ok_bidouille_wake,cond_temp_env &
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl &
     &    , year_nucl, lat_nucl, lon_nucl &
     &    , zmin_nucl, zmax_nucl, HTO_nucl 
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)    :: xtprecip_rain, xtprecip_snow
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)    :: xtspechum
    real, dimension(klon), intent(IN) :: rlat

! In/Output arguments
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(INOUT)     :: radsol
    REAL, DIMENSION(klon), INTENT(INOUT)     :: snow
    REAL, DIMENSION(klon), INTENT(INOUT)     :: agesno !? put to 0 in ocean
    REAL, DIMENSION(niso,klon), INTENT(IN)    :: xtsnow
    REAL, DIMENSION(niso,klon), INTENT(INOUT)    :: Roce

! Output arguments
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(OUT)       :: qsurf
    REAL, DIMENSION(klon), INTENT(OUT)       :: evap, fluxsens, fluxlat
    REAL, DIMENSION(klon), INTENT(OUT)       :: flux_u1, flux_v1
    REAL, DIMENSION(klon), INTENT(OUT)       :: tsurf_new
    REAL, DIMENSION(klon), INTENT(OUT)       :: dflux_s, dflux_l
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)    :: xtevap
    REAL, DIMENSION(klon), INTENT(IN)    :: h1

! Local variables
!****************************************************************************************
    INTEGER                     :: i
    REAL, DIMENSION(klon)       :: cal, beta, dif_grnd
    REAL, DIMENSION(klon)       :: alb_neig, tsurf_lim, zx_sl
    REAL, DIMENSION(klon)       :: u0, v0
    REAL, DIMENSION(klon)       :: u1_lay, v1_lay
    LOGICAL                     :: check=.FALSE.

      real, parameter :: t_coup = 273.15      

!****************************************************************************************
! Start calculation
!****************************************************************************************
    IF (check) WRITE(*,*)' Entering ocean_forced_noice'
    
    
!****************************************************************************************
! 1)    
! Read sea-surface temperature from file limit.nc
!
!****************************************************************************************
!        write(*,*) 'ocean_forced_mod tmp 129: klon,knon=', &
!        &       klon,knon

    CALL limit_read_sst(knon,knindex,tsurf_lim &
     &  ,Roce,rlat   &
     &   )

!****************************************************************************************
! 2)
! Flux calculation
!
!****************************************************************************************
! Set some variables for calcul_fluxs
    cal = 0.
    beta = 1.
    dif_grnd = 0.
    alb_neig(:) = 0.
    agesno(:) = 0.
! Suppose zero surface speed
    u0(:)=0.0
    v0(:)=0.0
    u1_lay(:) = u1(:) - u0(:)
    v1_lay(:) = v1(:) - v0(:)

! Calcul de tsurf_new, evap, fluxlat, fluxsens, dflux_s, dflux_l and qsurf
    CALL calcul_fluxs(knon, is_oce, dtime, &
         tsurf_lim, p1lay, cal, beta, cdragh, ps, &
         precip_rain, precip_snow, snow, qsurf,  &
         radsol, dif_grnd, temp_air, spechum, u1_lay, v1_lay, &
         AcoefH, AcoefQ, BcoefH, BcoefQ, &
         tsurf_new, evap, fluxlat, fluxsens, dflux_s, dflux_l)

! - Flux calculation at first modele level for U and V
    CALL calcul_flux_wind(knon, dtime, &
         u0, v0, u1, v1, cdragm, &
         AcoefU, AcoefV, BcoefU, BcoefV, &
         p1lay, temp_air, &
         flux_u1, flux_v1)  

        call calcul_iso_surf_oce_vectall(klon, knon,t_coup, &
     &    ps,tsurf_new,spechum,u1_lay, v1_lay, xtspechum,dtime, &
     &    evap,tsurf_new, Roce,xtevap,h1 &
     &   )


  END SUBROUTINE ocean_forced_noice
!
!***************************************************************************************
!
  SUBROUTINE ocean_forced_ice( &
       itime, dtime, jour, knon, knindex, &
       tsurf_in, p1lay, cdragh, cdragm, precip_rain, precip_snow, temp_air, spechum, &
       AcoefH, AcoefQ, BcoefH, BcoefQ, &
       AcoefU, AcoefV, BcoefU, BcoefV, &
       ps, u1, v1, &
       radsol, snow, qsol, agesno, tsoil, &
       qsurf, alb1_new, alb2_new, evap, fluxsens, fluxlat, flux_u1, flux_v1, &
       tsurf_new, dflux_s, dflux_l &
            ,xtprecip_rain, xtprecip_snow, xtspechum,Roce, &
            xtsnow, xtsol,xtevap &  
            )
!
! This subroutine treats the ocean where there is ice. 
! The routine reads data from climatologie file and does flux calculations at the 
! surface.
!
    USE dimphy
    USE calcul_fluxs_mod
    USE surface_data,     ONLY : calice, calsno, tau_gl
    USE limit_read_mod
    USE fonte_neige_mod,  ONLY : fonte_neige

    INCLUDE "indicesol.h"
    INCLUDE "dimsoil.h"
    INCLUDE "YOMCST.h"
    INCLUDE "clesphys.h"

! Input arguments
!****************************************************************************************
    INTEGER, INTENT(IN)                  :: itime, jour, knon
    INTEGER, DIMENSION(klon), INTENT(IN) :: knindex
    REAL, INTENT(IN)                     :: dtime
    REAL, DIMENSION(klon), INTENT(IN)    :: tsurf_in
    REAL, DIMENSION(klon), INTENT(IN)    :: p1lay
    REAL, DIMENSION(klon), INTENT(IN)    :: cdragh, cdragm
    REAL, DIMENSION(klon), INTENT(IN)    :: precip_rain, precip_snow
    REAL, DIMENSION(klon), INTENT(IN)    :: temp_air, spechum
    REAL, DIMENSION(klon), INTENT(IN)    :: AcoefH, AcoefQ, BcoefH, BcoefQ
    REAL, DIMENSION(klon), INTENT(IN)    :: AcoefU, AcoefV, BcoefU, BcoefV
    REAL, DIMENSION(klon), INTENT(IN)    :: ps
    REAL, DIMENSION(klon), INTENT(IN)    :: u1, v1
!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env
        logical ok_prod_nucl_tritium
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)
        

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)
        
        integer niso ! nombre d'isotopes utiliss   
        parameter (niso=use_iso_HTO+use_iso_O17 &
     &               +use_iso_O18+use_iso_HDO+use_iso_eau) 

        character*3 striso(niso)    

        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso) &
     &    , tnat(niso), toce(niso), tcorr(niso) &
     &    , tdifrel(niso) &
     &    , talph1(niso), talph2(niso), talph3(niso) &
     &    , talps1(niso), talps2(niso) &
     &    , tkcin0(niso) &
     &    , tkcin1(niso),tkcin2(niso) &
     &    , alpha_liq_sol(niso) &
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg &
     &    , pxtmelt, pxtice, pxtmin, pxtmax &
     &    , bidouille_anti_divergence,essai_convergence &
     &    , initialisation_iso, initialisation_isotrac &
     &    , modif_sst,deltaTtest &
     &    , modif_sic,deltasic &
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit &
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo &
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice   &
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot &
     &    , rh_cste_surf_cond, T_cste_surf_cond  &
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie &
     &    , alphak_stewart, cste_surf_cond &
     &    , mixlen &
     &    , tnat, toce, tcorr &
     &    , tdifrel &
     &    , calendrier_guide &
     &    , talph1, talph2, talph3 &
     &    , talps1, talps2 &
     &    , tkcin0, tkcin1, tkcin2 &
     &    , alpha_liq_sol, Rdefault &
     &    , striso  &
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO &
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont &
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim &
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce &
     &    , ratqsbasnew,Pcrit_ratqs &
     &    , ok_bidouille_wake,cond_temp_env &
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl &
     &    , year_nucl, lat_nucl, lon_nucl &
     &    , zmin_nucl, zmax_nucl, HTO_nucl 
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)    :: xtprecip_rain, xtprecip_snow
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)    :: xtspechum
    REAL, DIMENSION(niso,klon), INTENT(IN)    :: Roce

! In/Output arguments
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(INOUT)          :: radsol
    REAL, DIMENSION(klon), INTENT(INOUT)          :: snow, qsol
    REAL, DIMENSION(klon), INTENT(INOUT)          :: agesno
    REAL, DIMENSION(klon, nsoilmx), INTENT(INOUT) :: tsoil
    REAL, DIMENSION(niso,klon), INTENT(IN)    :: xtsnow, xtsol

! Output arguments
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(OUT)            :: qsurf
    REAL, DIMENSION(klon), INTENT(OUT)            :: alb1_new  ! new albedo in visible SW interval
    REAL, DIMENSION(klon), INTENT(OUT)            :: alb2_new  ! new albedo in near IR interval
    REAL, DIMENSION(klon), INTENT(OUT)            :: evap, fluxsens, fluxlat
    REAL, DIMENSION(klon), INTENT(OUT)            :: flux_u1, flux_v1
    REAL, DIMENSION(klon), INTENT(OUT)            :: tsurf_new
    REAL, DIMENSION(klon), INTENT(OUT)            :: dflux_s, dflux_l      
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)    :: xtevap

! Local variables
!****************************************************************************************
    LOGICAL                     :: check=.FALSE.
    INTEGER                     :: i
    REAL                        :: zfra
    REAL, PARAMETER             :: t_grnd=271.35
    REAL, DIMENSION(klon)       :: cal, beta, dif_grnd, capsol
    REAL, DIMENSION(klon)       :: alb_neig, tsurf_tmp
    REAL, DIMENSION(klon)       :: soilcap, soilflux
    REAL, DIMENSION(klon)       :: u0, v0
    REAL, DIMENSION(klon)       :: u1_lay, v1_lay

      real, parameter :: t_coup = 273.15
      real, dimension(klon) :: fqfonte_diag
      real, dimension(klon) ::  snow_evap_diag 
      real, dimension(klon) ::  fqcalving_diag 
      real, DIMENSION(klon) :: run_off_lic_0_diag
      real  :: max_eau_sol_diag  
      real, dimension(klon)  ::  runoff_diag    
      integer ixt
      REAL, DIMENSION(niso,klon) :: xtsnow_prec,xtsol_prec
      REAL, DIMENSION(klon) :: snow_prec,qsol_prec  

!****************************************************************************************
! Start calculation
!****************************************************************************************
    IF (check) WRITE(*,*)'Entering surface_seaice, knon=',knon 
!    write(*,*) 'ocean_forced_mod 315: klon,knon=',klon,knon

!****************************************************************************************
! 1) 
! Flux calculation : tsurf_new, evap, fluxlat, fluxsens, flux_u1, flux_v1
!                    dflux_s, dflux_l and qsurf
!****************************************************************************************
    tsurf_tmp(:) = tsurf_in(:)

! calculate the parameters cal, beta, capsol and dif_grnd
    CALL calbeta(dtime, is_sic, knon, snow, qsol, beta, capsol, dif_grnd)

    
    IF (soil_model) THEN 
! update tsoil and calculate soilcap and soilflux
       CALL soil(dtime, is_sic, knon, snow, tsurf_tmp, tsoil,soilcap, soilflux)
       cal(1:knon) = RCPD / soilcap(1:knon)
       radsol(1:knon) = radsol(1:knon)  + soilflux(1:knon)
       dif_grnd = 1.0 / tau_gl
    ELSE 
       dif_grnd = 1.0 / tau_gl
       cal = RCPD * calice
       WHERE (snow > 0.0) cal = RCPD * calsno 
    ENDIF

    beta = 1.0
! Suppose zero surface speed
    u0(:)=0.0
    v0(:)=0.0
    u1_lay(:) = u1(:) - u0(:)
    v1_lay(:) = v1(:) - v0(:)
    CALL calcul_fluxs(knon, is_sic, dtime, &
         tsurf_tmp, p1lay, cal, beta, cdragh, ps, &
         precip_rain, precip_snow, snow, qsurf,  &
         radsol, dif_grnd, temp_air, spechum, u1_lay, v1_lay, &
         AcoefH, AcoefQ, BcoefH, BcoefQ, &
         tsurf_new, evap, fluxlat, fluxsens, dflux_s, dflux_l)

! - Flux calculation at first modele level for U and V
    CALL calcul_flux_wind(knon, dtime, &
         u0, v0, u1, v1, cdragm, &
         AcoefU, AcoefV, BcoefU, BcoefV, &
         p1lay, temp_air, &
         flux_u1, flux_v1)  

!****************************************************************************************
! 2)
! Calculations due to snow and runoff
!
!****************************************************************************************

   ! verif
   ! end verif

    do i=1,knon
      snow_prec(i)=snow(i)
      do ixt=1,niso
      xtsnow_prec(ixt,i)=xtsnow(ixt,i)
      enddo !do ixt=1,niso
      ! initialisation:
      fqfonte_diag(i)=0.0
      snow_evap_diag(i)=0.0
    enddo !do i=1,knon

     CALL fonte_neige( knon, is_sic, knindex, dtime, &
         tsurf_tmp, precip_rain, precip_snow, &
         snow, qsol, tsurf_new, evap &
     & ,fqfonte_diag,snow_evap_diag,fqcalving_diag   &
     & ,max_eau_sol_diag,runoff_diag,run_off_lic_0_diag   &
     &   )

! isotopes: tout est externalisé
         call calcul_iso_surf_sic_vectall(klon,knon, &
             &   evap,snow_evap_diag,Tsurf_new,Roce,snow, &
             &   fqfonte_diag,dtime,t_coup, &
             &   precip_snow,xtprecip_snow, snow_prec,xtsnow_prec, &
             &   xtspechum,spechum,ps, &
             &   xtevap,xtsnow,fqcalving_diag &
     &  )   
!#ifdef ISOVERIF
!#ifdef 1
    
! Calculation of albedo at snow (alb_neig) and update the age of snow (agesno)
! 
    CALL albsno(klon, knon, dtime, agesno(:), alb_neig(:), precip_snow(:))  

    WHERE (snow(1:knon) .LT. 0.0001) agesno(1:knon) = 0.

    alb1_new(:) = 0.0
    DO i=1, knon
       zfra = MAX(0.0,MIN(1.0,snow(i)/(snow(i)+10.0)))
       alb1_new(i) = alb_neig(i) * zfra +  0.6 * (1.0-zfra)
    ENDDO

    alb2_new(:) = alb1_new(:)

  END SUBROUTINE ocean_forced_ice
!
!****************************************************************************************
!
END MODULE ocean_forced_mod






