! Routines complementaires pour la physique planetaire.


      subroutine iniaqua(nlon,latfi,lonfi,iflag_phys)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  Creation d'un etat initial et de conditions aux limites 
!  (resp startphy.nc et limit.nc) pour des configurations idealisees 
! du modele LMDZ dans sa version terrestre.
!  iflag_phys est un parametre qui controle
!  iflag_phys = N  
!    de 100 a 199 : aqua planetes avec SST forcees
!                 N-100 determine le type de SSTs
!    de 200 a 299 : terra planetes avec Ts calcule
!        
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      use comgeomphy, only : rlatd,rlond
      use dimphy, only : klon
      use surface_data, only : type_ocean,ok_veget
      use pbl_surface_mod, only : pbl_surface_init
      USE fonte_neige_mod, only : fonte_neige_init
      use phys_state_var_mod
      use control_mod, only : dayref,nday,iphysiq


      USE IOIPSL 
      IMPLICIT NONE

!-----------------------------------------------------------------------
!   INCLUDE 'dimensions.h'
!
!   dimensions.h contient les dimensions du modele
!   ndm est tel que iim=2**ndm
!-----------------------------------------------------------------------

      INTEGER iim,jjm,llm,ndm

      PARAMETER (iim= 96,jjm=71,llm=39,ndm=1)

!-----------------------------------------------------------------------
!   #include "dimphy.h"
!   #include "YOMCST.h"
!
! $Id: comconst.h 1707 2013-01-11 09:19:19Z fairhead $
!
!-----------------------------------------------------------------------
! INCLUDE comconst.h

      COMMON/comconsti/im,jm,lllm,imp1,jmp1,lllmm1,lllmp1,lcl,          &
     &                 iflag_top_bound
      COMMON/comconstr/dtvr,daysec,                                     &
     & pi,dtphys,dtdiss,rad,r,cpp,kappa,cotot,unsim,g,omeg              &
     &                   ,dissip_factz,dissip_deltaz,dissip_zref        &
     &                   ,tau_top_bound,                                &
     & daylen,year_day,molmass, ihf


      INTEGER im,jm,lllm,imp1,jmp1,lllmm1,lllmp1,lcl
      REAL dtvr ! dynamical time step (in s)
      REAL daysec !length (in s) of a standard day
      REAL pi    ! something like 3.14159....
      REAL dtphys ! (s) time step for the physics
      REAL dtdiss ! (s) time step for the dissipation
      REAL rad ! (m) radius of the planet
      REAL r ! Reduced Gas constant r=R/mu
             ! with R=8.31.. J.K-1.mol-1, mu: mol mass of atmosphere (kg/mol)
      REAL cpp   ! Specific heat Cp (J.kg-1.K-1)
      REAL kappa ! kappa=R/Cp 
      REAL cotot
      REAL unsim ! = 1./iim
      REAL g ! (m/s2) gravity
      REAL omeg ! (rad/s) rotation rate of the planet
      REAL dissip_factz,dissip_deltaz,dissip_zref
      INTEGER iflag_top_bound
      REAL tau_top_bound
      REAL daylen ! length of solar day, in 'standard' day length
      REAL year_day ! Number of standard days in a year
      REAL molmass ! (g/mol) molar mass of the atmosphere

      REAL ihf ! (W/m2) Intrinsic heat flux (for giant planets)

!-----------------------------------------------------------------------

!
! $Id: clesphys.h 1750 2013-04-25 15:27:27Z fairhead $
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation 
!                 (les placer en colonne 6 et en colonne 73)
!
!..include cles_phys.h
!
       LOGICAL cycle_diurne,soil_model,new_oliq,ok_orodr,ok_orolf 
       LOGICAL ok_limitvrai
       INTEGER nbapp_rad, iflag_con
       REAL co2_ppm, co2_ppm0, solaire
       REAL(kind=8) RCO2, RCH4, RN2O, RCFC11, RCFC12  
       REAL(kind=8) RCO2_act, RCH4_act, RN2O_act, RCFC11_act, RCFC12_act  
       REAL(kind=8) CH4_ppb, N2O_ppb, CFC11_ppt, CFC12_ppt
!IM ajout CFMIP2/CMIP5
       REAL co2_ppm_per
       LOGICAL ok_4xCO2atm
       REAL(kind=8) RCO2_per,RCH4_per,RN2O_per,RCFC11_per,RCFC12_per
       REAL(kind=8) CH4_ppb_per,N2O_ppb_per,CFC11_ppt_per,CFC12_ppt_per

!OM ---> correction du bilan d'eau global
!OM Correction sur precip KE
       REAL cvl_corr
!OM Fonte calotte dans bilan eau
       LOGICAL ok_lic_melt

!IM simulateur ISCCP 
       INTEGER top_height, overlap
!IM seuils cdrm, cdrh
       REAL cdmmax, cdhmax
!IM param. stabilite s/ terres et en dehors
       REAL ksta, ksta_ter
!IM ok_kzmin : clef calcul Kzmin dans la CL de surface cf FH
       LOGICAL ok_kzmin
!IM, MAFo fmagic, pmagic : parametres - additionnel et multiplicatif - 
!                          pour regler l albedo sur ocean
       REAL fmagic, pmagic
! Hauteur (imposee) du contenu en eau du sol
           REAL qsol0
! Frottement au sol (Cdrag)
       Real f_cdrag_ter,f_cdrag_oce
! Rugoro
       Real f_rugoro

!IM lev_histhf  : niveau sorties 6h
!IM lev_histday : niveau sorties journalieres
!IM lev_histmth : niveau sorties mensuelles
!IM lev_histdayNMC : on peut sortir soit sur 8 (comme AR5) ou bien 
!                    sur 17 niveaux de pression
       INTEGER lev_histhf, lev_histday, lev_histmth
       INTEGER lev_histdayNMC
       Integer lev_histins, lev_histLES  
!IM ok_histNMC  : sortie fichiers niveaux de pression (histmthNMC, histdayNMC, histhfNMC)
!IM freq_outNMC : frequences de sortie fichiers niveaux de pression (histmthNMC, histdayNMC, histhfNMC)
!IM freq_calNMC : frequences de calcul fis. hist*NMC.nc
!IM pasphys : pas de temps de physique (secondes)
       REAL pasphys
       LOGICAL ok_histNMC(3)
       REAL freq_outNMC(3) , freq_calNMC(3)
       CHARACTER(len=4) type_run
! aer_type: pour utiliser un fichier constant dans readaerosol 
       CHARACTER*8 :: aer_type 
       LOGICAL ok_isccp, ok_regdyn
       REAL lonmin_ins, lonmax_ins, latmin_ins, latmax_ins
       REAL ecrit_ins, ecrit_hf, ecrit_day
       REAL ecrit_mth, ecrit_tra, ecrit_reg 
       REAL ecrit_LES
       REAL freq_ISCCP, ecrit_ISCCP
       REAL freq_COSP
       LOGICAL :: ok_cosp,ok_mensuelCOSP,ok_journeCOSP,ok_hfCOSP
       INTEGER :: ip_ebil_phy, iflag_rrtm
       LOGICAL :: ok_strato
       LOGICAL :: ok_hines
       LOGICAL :: ok_qch4

       COMMON/clesphys/cycle_diurne, soil_model, new_oliq,              &
     &     ok_orodr, ok_orolf, ok_limitvrai, nbapp_rad, iflag_con       &
     &     , co2_ppm, solaire                                           &
     &     , RCO2, RCH4, RN2O, RCFC11, RCFC12                           &
     &     , RCO2_act, RCH4_act, RN2O_act, RCFC11_act, RCFC12_act       &
     &     , ok_4xCO2atm                                                & 
     &     , RCO2_per, RCH4_per, RN2O_per, RCFC11_per, RCFC12_per       &
     &     , CH4_ppb, N2O_ppb, CFC11_ppt, CFC12_ppt                     &
     &     , CH4_ppb_per, N2O_ppb_per, CFC11_ppt_per, CFC12_ppt_per     &
     &     , top_height, overlap, cdmmax, cdhmax, ksta, ksta_ter        &
     &     , ok_kzmin, fmagic, pmagic                                   &
     &     , f_cdrag_ter,f_cdrag_oce,f_rugoro                           &
     &     , lev_histhf, lev_histday, lev_histmth                       &
     &     , lev_histins, lev_histLES, lev_histdayNMC                   &
     &     , pasphys, ok_histNMC, freq_outNMC, freq_calNMC              &
     &     , type_run, ok_isccp, ok_regdyn, ok_cosp                     &
     &     , ok_mensuelCOSP,ok_journeCOSP,ok_hfCOSP                     &
     &     , lonmin_ins, lonmax_ins, latmin_ins, latmax_ins             &
     &     , ecrit_ins, ecrit_hf, ecrit_day                             &
     &     , ecrit_mth, ecrit_tra, ecrit_reg                            &
     &     , freq_ISCCP, ecrit_ISCCP, freq_COSP, ip_ebil_phy            &
     &     , ok_lic_melt, cvl_corr, aer_type                            &
     &     , qsol0, iflag_rrtm, ok_strato,ok_hines,ecrit_LES            &
     &     , co2_ppm0, ok_qch4
     
       save /clesphys/
!$OMP THREADPRIVATE(/clesphys/)
 
!
! $Header$
!
      INTEGER nsoilmx
      PARAMETER (nsoilmx=11)
!
! $Header$
!
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
      INTEGER nbsrf
      PARAMETER (nbsrf=4) ! nombre de sous-fractions pour une maille
!
      INTEGER is_oce
      PARAMETER (is_oce=3) ! ocean
      INTEGER is_sic
      PARAMETER (is_sic=4) ! glace de mer
      INTEGER is_ter
      PARAMETER (is_ter=1) ! terre
      INTEGER is_lic
      PARAMETER (is_lic=2) ! glacier continental
!
      REAL epsfra
      PARAMETER (epsfra=1.0E-05)
!
      CHARACTER(len=3) clnsurf(nbsrf)
      DATA clnsurf/'ter', 'lic', 'oce', 'sic'/
      SAVE clnsurf
!$OMP THREADPRIVATE(clnsurf)
!
! $Id: temps.h 1665 2012-10-09 13:35:26Z fairhead $
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
! jD_ref = jour julien de la date de reference (lancement de l'experience)
! hD_ref = "heure" julienne de la date de reference
!-----------------------------------------------------------------------
! INCLUDE 'temps.h'

      COMMON/temps/itaufin, dt, day_ini, day_end, annee_ref, day_ref,   &
     &             itau_dyn, itau_phy, jD_ref, jH_ref, calend,          &
     &             start_time


      INTEGER   itaufin
      INTEGER itau_dyn, itau_phy
      INTEGER day_ini, day_end, annee_ref, day_ref
      REAL      dt, jD_ref, jH_ref, start_time
      CHARACTER (len=10) :: calend

!$OMP THREADPRIVATE(/temps/)
!-----------------------------------------------------------------------

      integer,intent(in) :: nlon,iflag_phys
cIM ajout latfi, lonfi      
      real,intent(in) :: lonfi(nlon),latfi(nlon)

      INTEGER type_profil,type_aqua

c  Ajouts initialisation des surfaces
      REAL :: run_off_lic_0(nlon)
      REAL :: qsolsrf(nlon,nbsrf),snsrf(nlon,nbsrf)
      REAL :: frugs(nlon,nbsrf)
      REAL :: agesno(nlon,nbsrf)
      REAL :: tsoil(nlon,nsoilmx,nbsrf)
      REAL :: tslab(nlon), seaice(nlon)
      REAL evap(nlon,nbsrf),fder(nlon)
!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env ! si oui, la temperature de cond est celle de l'environnement: permet d'eviter temperature de cond aberrantes
        logical ok_prod_nucl_tritium ! si oui, production de tritium par essais nucleaires
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)

        integer niso ! nombre d'isotopes utiliss
        parameter (niso=use_iso_eau+use_iso_HDO+use_iso_O18
     &           +use_iso_O17+use_iso_HTO)

        character*3 striso(niso)
      
        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso)
     &    , tnat(niso), toce(niso), tcorr(niso)
     &    , tdifrel(niso)
     &    , talph1(niso), talph2(niso), talph3(niso)
     &    , talps1(niso), talps2(niso)
     &    , tkcin0(niso)
     &    , tkcin1(niso),tkcin2(niso)
     &    , alpha_liq_sol(niso) 
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg
     &    , pxtmelt, pxtice, pxtmin, pxtmax
     &    , bidouille_anti_divergence,essai_convergence
     &    , initialisation_iso, initialisation_isotrac
     &    , modif_sst,deltaTtest
     &    , modif_sic,deltasic
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice 
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot
     &    , rh_cste_surf_cond, T_cste_surf_cond
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie
     &    , alphak_stewart, cste_surf_cond
     &    , mixlen
     &    , tnat, toce, tcorr
     &    , tdifrel
     &    , calendrier_guide
     &    , talph1, talph2, talph3
     &    , talps1, talps2
     &    , tkcin0, tkcin1, tkcin2
     &    , alpha_liq_sol, Rdefault
     &    , striso
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce
     &    , ratqsbasnew,Pcrit_ratqs
     &    , ok_bidouille_wake,cond_temp_env
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl
     &    , year_nucl, lat_nucl, lon_nucl
     &    , zmin_nucl, zmax_nucl, HTO_nucl
      REAL :: xtrun_off_lic_0(niso,nlon)
      REAL xtevap(ntraciso,nlon,nbsrf)
      REAL :: xtsnsrf(niso,nlon,nbsrf)



c    Arguments :
c    -----------

!      integer radpas  
      integer it,unit,i,k,itap

      real airefi,zcufi,zcvfi

      real rugos,albedo
      REAL tsurf
      REAL time,timestep,day,day0
      real qsol_f,qsol(nlon)
      real rugsrel(nlon)
!      real zmea(nlon),zstd(nlon),zsig(nlon)
!      real zgam(nlon),zthe(nlon),zpic(nlon),zval(nlon)
!      real rlon(nlon),rlat(nlon)
      logical alb_ocean
!      integer demih_pas
      real xtsol(niso,nlon)
      REAL Rland_ice(niso,klon)

      CHARACTER*80 ans,file_forctl, file_fordat, file_start
      character*100 file,var
      character*2 cnbl

      REAL phy_nat(nlon,360)
      REAL phy_alb(nlon,360)
      REAL phy_sst(nlon,360)
      REAL phy_bil(nlon,360)
      REAL phy_rug(nlon,360)
      REAL phy_ice(nlon,360)
      REAL phy_fter(nlon,360)
      REAL phy_foce(nlon,360)
      REAL phy_fsic(nlon,360)
      REAL phy_flic(nlon,360)

      integer, save::  read_climoz=0 ! read ozone climatology


!-------------------------------------------------------------------------
!  declaration pour l'appel a phyredem
!-------------------------------------------------------------------------

!      real pctsrf(nlon,nbsrf),ftsol(nlon,nbsrf)
      real falbe(nlon,nbsrf),falblw(nlon,nbsrf)
!      real pbl_tke(nlon,llm,nbsrf)
!      real rain_fall(nlon),snow_fall(nlon)
!      real solsw(nlon), sollw(nlon),radsol(nlon)
!      real t_ancien(nlon,llm),q_ancien(nlon,llm),rnebcon(nlon,llm)
!      real ratqs(nlon,llm)
!      real clwcon(nlon,llm)

      INTEGER        longcles
      PARAMETER    ( longcles = 20 )
      REAL clesphy0( longcles )


c-----------------------------------------------------------------------
c   dynamial tendencies :
c   ---------------------

      INTEGER l,ierr,aslun

      REAL longitude,latitude
      REAL paire 

      DATA latitude,longitude/48.,0./

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! INITIALISATIONS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!-----------------------------------------------------------------------
!    Initialisations  des constantes
!    -------------------------------


      type_aqua=iflag_phys/100
      type_profil=iflag_phys-type_aqua*100
      print*,'iniaqua:type_aqua, type_profil',type_aqua, type_profil

      if (klon.ne.nlon) then
        write(*,*)"iniaqua: klon=",klon," nlon=",nlon
        stop'probleme de dimensions dans iniaqua'
      endif
      call phys_state_var_init(read_climoz)


      read_climoz=0
      day0=217.
      day=day0
      it=0
      time=0.

cIM ajout latfi, lonfi
      rlatd=latfi
      rlond=lonfi
      rlat=rlatd*180./pi
      rlon=rlond*180./pi

!-----------------------------------------------------------------------
!  initialisations de la physique
!-----------------------------------------------------------------------

         day_ini=dayref
         day_end=day_ini+nday
         airefi=1.
         zcufi=1.
         zcvfi=1.
      nbapp_rad=24
      CALL getin('nbapp_rad',nbapp_rad)

!---------------------------------------------------------------------
c Creation des conditions aux limites:
c ------------------------------------
! Initialisations des constantes
! Ajouter les manquants dans planete.def... (albedo etc)
      co2_ppm=348.
      CALL getin('co2_ppm',co2_ppm)
      solaire=1365.
      CALL getin('solaire',solaire)
      radsol=0.
      qsol_f=10.
!      CALL getin('albedo',albedo) ! albedo is set below, depending on type_aqua
      alb_ocean=.true.
      CALL getin('alb_ocean',alb_ocean)

c  Conditions aux limites:
c  -----------------------

      qsol(:)    = qsol_f
      rugsrel = 0.0    ! (rugsrel = rugoro)
      rugoro = 0.0
      u_ancien = 0.0
      v_ancien = 0.0
      agesno  = 50.0
! Relief plat
      zmea = 0.
      zstd = 0.
      zsig = 0.
      zgam = 0.
      zthe = 0.
      zpic = 0.
      zval = 0.

! Une seule surface
      pctsrf=0.
      if (type_aqua==1) then
         rugos=1.e-4
         albedo=0.19
         pctsrf(:,is_oce)=1.
      else if (type_aqua==2) then
         rugos=0.03
         albedo=0.1
         pctsrf(:,is_ter)=1.
      endif

      CALL getin('rugos',rugos)
      zmasq(:)=pctsrf(:,is_oce)

! pctsrf_pot(:,is_oce) = 1. - zmasq(:)
! pctsrf_pot(:,is_sic) = 1. - zmasq(:)

! Si alb_ocean on calcule un albedo oceanique moyen
!  if (alb_ocean) then
! Voir pourquoi on avait ca.
!          CALL ini_alb_oce(phy_alb)
!      else 
      phy_alb(:,:) = albedo ! albedo land only (old value condsurf_jyg=0.3)
!      endif !alb_ocean
      
      do i=1,360
cIM Terraplanete   phy_sst(:,i) = 260.+50.*cos(rlatd(:))**2
cIM ajout calcul profil sst selon le cas considere (cf. FBr)

      phy_nat(:,i) = 1.0    ! 0=ocean libre, 1=land, 2=glacier, 3=banquise
      phy_bil(:,i) = 1.0    ! ne sert que pour les slab_ocean
      phy_rug(:,i) = rugos  ! longueur rugosite utilisee sur land only 
      phy_ice(:,i) = 0.0    ! fraction de glace (?)
      phy_fter(:,i) = pctsrf(:,is_ter)  ! fraction de glace (?)
      phy_foce(:,i) = pctsrf(:,is_oce)  ! fraction de glace (?)
      phy_fsic(:,i) = pctsrf(:,is_sic)  ! fraction de glace (?)
      phy_flic(:,i) = pctsrf(:,is_lic)  ! fraction de glace (?)
      enddo
cIM calcul profil sst
      call profil_sst(nlon, rlatd, type_profil, phy_sst)

      call writelim
     s   (klon,phy_nat,phy_alb,phy_sst,phy_bil,phy_rug,phy_ice,
     s    phy_fter,phy_foce,phy_flic,phy_fsic)


!---------------------------------------------------------------------
c Ecriture de l'etat initial:
c ---------------------------

C
C Ecriture etat initial physique
C
      timestep   = dtvr * FLOAT(iphysiq)
      radpas    = NINT (daysec/timestep/ FLOAT(nbapp_rad) )

      DO i = 1, longcles
       clesphy0(i) = 0.
      ENDDO
      clesphy0(1) = FLOAT( iflag_con )
      clesphy0(2) = FLOAT( nbapp_rad )
c     IF( cycle_diurne  ) clesphy0(3) =  1. 
      clesphy0(3)=1. ! cycle_diurne
      clesphy0(4)=1. ! soil_model
      clesphy0(5)=1. ! new_oliq
      clesphy0(6)=0. ! ok_orodr
      clesphy0(7)=0. ! ok_orolf
      clesphy0(8)=0. ! ok_limitvrai


c=======================================================================
c  Profils initiaux
c=======================================================================

! On initialise les temperatures de surfaces comme les sst
      do i=1,nlon
         ftsol(i,:)=phy_sst(i,1)
         tsoil(i,:,:)=phy_sst(i,1)
         tslab(i)=phy_sst(i,1)
      enddo

      falbe(:,:)=albedo
      falblw(:,:)=albedo
      rain_fall(:)=0.
      snow_fall(:)=0.
      solsw(:)=0.
      sollw(:)=0.
      radsol(:)=0.
      xtrain_fall(:,:)=0.
      xtsnow_fall(:,:)=0.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  intialisation bidon mais pas grave
      t_ancien(:,:)=0.
      q_ancien(:,:)=0.
      xt_ancien(:,:,:)=0.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      rnebcon=0.
      ratqs=0.
      clwcon=0.
      pbl_tke=1.e-8

! variables supplementaires pour appel a plb_surface_init
      fder(:)=0.
      seaice(:)=0.
      run_off_lic_0=0.
      evap=0.
      xtrun_off_lic_0=0.
      xtevap=0.

! Initialisations necessaires avant phyredem
      type_ocean = "force"
      call fonte_neige_init(run_off_lic_0
     ; ,xtrun_off_lic_0
     :   )
      qsolsrf(:,:)=qsol(1) ! humidite du sol des sous surface
      snsrf(:,:)=0.        ! couverture de neige des sous surface
      frugs(:,:)=rugos        ! couverture de neige des sous surface
      xtsnsrf(:,:,:)=0.     

      call pbl_surface_init(qsol, fder, snsrf, qsolsrf,
     .     evap, frugs, agesno, tsoil
     :   ,xtsol,xtsnsrf,xtevap,Rland_ice
     :  )

        print*,'iniaqua: before phyredem'

      falb1=albedo
      falb2=albedo
      zmax0=0.
      f0=0.
      ema_work1=0.
      ema_work2=0.
      wake_deltat=0.
      wake_deltaq=0.
      wake_s=0.
      wake_cstar=0.
      wake_pe=0.
      wake_fip=0.
      fm_therm=0.
      entr_therm=0.
      detr_therm=0.
      wake_deltaxt=0.

      CALL phyredem ("startphy.nc")
      ! call phyisoredem directement dans phyredem

        print*,'iniaqua: after phyredem'
      call phys_state_var_end

      return
      end


c====================================================================
c====================================================================
      SUBROUTINE zenang_an(cycle_diurne,gmtime,rlat,rlon,rmu0,fract)
      USE dimphy
      IMPLICIT none
c====================================================================
c=============================================================
c         CALL zenang(cycle_diurne,gmtime,rlat,rlon,rmu0,fract)
c Auteur : A. Campoy et F. Hourdin
c Objet  : calculer les valeurs moyennes du cos de l'angle zenithal
c          et l'ensoleillement moyen entre gmtime1 et gmtime2 
c          connaissant la declinaison, la latitude et la longitude.
c 
c   Dans cette version particuliere, on calcule le rayonnement
c  moyen sur l'année à chaque latitude.
c   angle zenithal calculé pour obtenir un 
c   Fit polynomial de  l'ensoleillement moyen au sommet de l'atmosphere
c   en moyenne annuelle.
c   Spécifique de la terre. Utilisé pour les aqua planetes.
c
c Rque   : Different de la routine angle en ce sens que zenang 
c          fournit des moyennes de pmu0 et non des valeurs 
c          instantanees, du coup frac prend toutes les valeurs 
c          entre 0 et 1.
c Date   : premiere version le 13 decembre 1994
c          revu pour  GCM  le 30 septembre 1996
c===============================================================
c longi----INPUT : la longitude vraie de la terre dans son plan
c                  solaire a partir de l'equinoxe de printemps (degre)
c gmtime---INPUT : temps universel en fraction de jour
c pdtrad---INPUT : pas de temps du rayonnement (secondes)
c lat------INPUT : latitude en degres
c long-----INPUT : longitude en degres
c pmu0-----OUTPUT: angle zenithal moyen entre gmtime et gmtime+pdtrad
c frac-----OUTPUT: ensoleillement moyen entre gmtime et gmtime+pdtrad
c================================================================
!
! $Header$
!
!  ATTENTION!!!!: ce fichier include est compatible format fixe/format libre
!                 veillez  n'utiliser que des ! pour les commentaires
!                 et  bien positionner les & des lignes de continuation
!                 (les placer en colonne 6 et en colonne 73)
!
!
! A1.0 Fundamental constants
      REAL RPI,RCLUM,RHPLA,RKBOL,RNAVO
! A1.1 Astronomical constants
      REAL RDAY,REA,REPSM,RSIYEA,RSIDAY,ROMEGA
! A1.1.bis Constantes concernant l'orbite de la Terre:
      REAL R_ecc, R_peri, R_incl
! A1.2 Geoide
      REAL RA,RG,R1SA
! A1.3 Radiation
!     REAL RSIGMA,RI0
      REAL RSIGMA
! A1.4 Thermodynamic gas phase
      REAL R,RMD,RMO3,RMV,RD,RV,RCPD,RCPV,RCVD,RCVV
      REAL RKAPPA,RETV
! A1.5,6 Thermodynamic liquid,solid phases
      REAL RCW,RCS
! A1.7 Thermodynamic transition of phase
      REAL RLVTT,RLSTT,RLMLT,RTT,RATM
! A1.8 Curve of saturation
      REAL RESTT,RALPW,RBETW,RGAMW,RALPS,RBETS,RGAMS
      REAL RALPD,RBETD,RGAMD
!
      COMMON/YOMCST/RPI   ,RCLUM ,RHPLA ,RKBOL ,RNAVO                   &
     &      ,RDAY  ,REA   ,REPSM ,RSIYEA,RSIDAY,ROMEGA                  &
     &      ,R_ecc, R_peri, R_incl                                      &
     &      ,RA    ,RG    ,R1SA                                         &
     &      ,RSIGMA                                                     &
     &      ,R     ,RMD   ,RMO3  ,RMV   ,RD    ,RV    ,RCPD             &
     &      ,RCPV  ,RCVD  ,RCVV  ,RKAPPA,RETV                           &
     &      ,RCW   ,RCS                                                 &
     &      ,RLVTT ,RLSTT ,RLMLT ,RTT   ,RATM                           &
     &      ,RESTT ,RALPW ,RBETW ,RGAMW ,RALPS ,RBETS ,RGAMS            &
     &      ,RALPD ,RBETD ,RGAMD
!    ------------------------------------------------------------------
!$OMP THREADPRIVATE(/YOMCST/)
c================================================================
      logical cycle_diurne
      real  gmtime
      real rlat(klon), rlon(klon), rmu0(klon), fract(klon)
c================================================================
      integer i
      real gmtime1, gmtime2
      real pi_local


      real rmu0m(klon),rmu0a(klon)
c

      pi_local = 4.0 * ATAN(1.0)

c================================================================
c  Calcul de l'angle zenithal moyen sur la journee
c================================================================

      DO i=1,klon
        fract(i)=1. 
!  Calcule du flux moyen
        IF (abs(rlat(i)).LE.28.75) THEN
           rmu0m(i)=(210.1924+206.6059*cos(0.0174533*rlat(i))**2)/1365.
        ELSEIF (abs(rlat(i)).LE.43.75) THEN
          rmu0m(i)=(187.4562+236.1853*cos(0.0174533*rlat(i))**2)/1365.
        ELSEIF (abs(rlat(i)).LE.71.25) THEN
          rmu0m(i)=(162.4439+284.1192*cos(0.0174533*rlat(i))**2)/1365.
        ELSE
          rmu0m(i)=(172.8125+183.7673*cos(0.0174533*rlat(i))**2)/1365.
        ENDIF
      ENDDO

c================================================================
!  Avec ou sans cycle diurne
c================================================================

      IF (cycle_diurne) THEN

!  On redecompose flux  au sommet suivant un cycle diurne idealise
!  identique a toutes les latitudes.

         DO i=1,klon
           rmu0a(i)=2.*rmu0m(i)*sqrt(2.)*pi_local/(4.-pi_local)
           rmu0(i)=rmu0a(i)*abs(sin(pi_local*gmtime+pi_local*
     &      rlon(i)/360.))-rmu0a(i)/sqrt(2.)
         ENDDO

         DO i=1,klon
           IF (rmu0(i).LE.0.) THEN
              rmu0(i)=0.
              fract(i)=0.
           ELSE
              fract(i)=1.
           ENDIF
         ENDDO

!  Affichage de l'angel zenitale
!     print*,'************************************'
!     print*,'************************************'
!     print*,'************************************'
!     print*,'latitude=',rlat(i),'longitude=',rlon(i)
!     print*,'rmu0m=',rmu0m(i)
!     print*,'rmu0a=',rmu0a(i)
!     print*,'rmu0=',rmu0(i)
                                                               
      ELSE

        DO i=1,klon
           fract(i)=0.5
           rmu0(i)=rmu0m(i)*2.
        ENDDO

      ENDIF

      RETURN
      END

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine writelim
     s   (klon,phy_nat,phy_alb,phy_sst,phy_bil,phy_rug,phy_ice,
     s    phy_fter,phy_foce,phy_flic,phy_fsic)
c
      use mod_phys_lmdz_para, only: is_mpi_root,is_omp_root
      use mod_grid_phy_lmdz, only : klon_glo
      use mod_phys_lmdz_transfert_para, only : gather
!#include "dimensions.h"
!#include "dimphy.h"
!     NetCDF-3.
!
! netcdf version 3 fortran interface:
!

!
! external netcdf data types:
!
      integer nf_byte
      integer nf_int1
      integer nf_char
      integer nf_short
      integer nf_int2
      integer nf_int
      integer nf_float
      integer nf_real
      integer nf_double

      parameter (nf_byte = 1)
      parameter (nf_int1 = nf_byte)
      parameter (nf_char = 2)
      parameter (nf_short = 3)
      parameter (nf_int2 = nf_short)
      parameter (nf_int = 4)
      parameter (nf_float = 5)
      parameter (nf_real = nf_float)
      parameter (nf_double = 6)

!
! default fill values:
!
      integer           nf_fill_byte
      integer           nf_fill_int1
      integer           nf_fill_char
      integer           nf_fill_short
      integer           nf_fill_int2
      integer           nf_fill_int
      real              nf_fill_float
      real              nf_fill_real
      doubleprecision   nf_fill_double

      parameter (nf_fill_byte = -127)
      parameter (nf_fill_int1 = nf_fill_byte)
      parameter (nf_fill_char = 0)
      parameter (nf_fill_short = -32767)
      parameter (nf_fill_int2 = nf_fill_short)
      parameter (nf_fill_int = -2147483647)
      parameter (nf_fill_float = 9.9692099683868690e+36)
      parameter (nf_fill_real = nf_fill_float)
      parameter (nf_fill_double = 9.9692099683868690d+36)

!
! mode flags for opening and creating a netcdf dataset:
!
      integer nf_nowrite
      integer nf_write
      integer nf_clobber
      integer nf_noclobber
      integer nf_fill
      integer nf_nofill
      integer nf_lock
      integer nf_share
      integer nf_64bit_offset
      integer nf_sizehint_default
      integer nf_align_chunk
      integer nf_format_classic
      integer nf_format_64bit

      parameter (nf_nowrite = 0)
      parameter (nf_write = 1)
      parameter (nf_clobber = 0)
      parameter (nf_noclobber = 4)
      parameter (nf_fill = 0)
      parameter (nf_nofill = 256)
      parameter (nf_lock = 1024)
      parameter (nf_share = 2048)
      parameter (nf_64bit_offset = 512)
      parameter (nf_sizehint_default = 0)
      parameter (nf_align_chunk = -1)
      parameter (nf_format_classic = 1)
      parameter (nf_format_64bit = 2)

!
! size argument for defining an unlimited dimension:
!
      integer nf_unlimited
      parameter (nf_unlimited = 0)

!
! global attribute id:
!
      integer nf_global
      parameter (nf_global = 0)

!
! implementation limits:
!
      integer nf_max_dims
      integer nf_max_attrs
      integer nf_max_vars
      integer nf_max_name
      integer nf_max_var_dims

      parameter (nf_max_dims = 1024)
      parameter (nf_max_attrs = 8192)
      parameter (nf_max_vars = 8192)
      parameter (nf_max_name = 256)
      parameter (nf_max_var_dims = nf_max_dims)

!
! error codes:
!
      integer nf_noerr
      integer nf_ebadid
      integer nf_eexist
      integer nf_einval
      integer nf_eperm
      integer nf_enotindefine
      integer nf_eindefine
      integer nf_einvalcoords
      integer nf_emaxdims
      integer nf_enameinuse
      integer nf_enotatt
      integer nf_emaxatts
      integer nf_ebadtype
      integer nf_ebaddim
      integer nf_eunlimpos
      integer nf_emaxvars
      integer nf_enotvar
      integer nf_eglobal
      integer nf_enotnc
      integer nf_ests
      integer nf_emaxname
      integer nf_eunlimit
      integer nf_enorecvars
      integer nf_echar
      integer nf_eedge
      integer nf_estride
      integer nf_ebadname
      integer nf_erange
      integer nf_enomem
      integer nf_evarsize
      integer nf_edimsize
      integer nf_etrunc

      parameter (nf_noerr = 0)
      parameter (nf_ebadid = -33)
      parameter (nf_eexist = -35)
      parameter (nf_einval = -36)
      parameter (nf_eperm = -37)
      parameter (nf_enotindefine = -38)
      parameter (nf_eindefine = -39)
      parameter (nf_einvalcoords = -40)
      parameter (nf_emaxdims = -41)
      parameter (nf_enameinuse = -42)
      parameter (nf_enotatt = -43)
      parameter (nf_emaxatts = -44)
      parameter (nf_ebadtype = -45)
      parameter (nf_ebaddim = -46)
      parameter (nf_eunlimpos = -47)
      parameter (nf_emaxvars = -48)
      parameter (nf_enotvar = -49)
      parameter (nf_eglobal = -50)
      parameter (nf_enotnc = -51)
      parameter (nf_ests = -52)
      parameter (nf_emaxname = -53)
      parameter (nf_eunlimit = -54)
      parameter (nf_enorecvars = -55)
      parameter (nf_echar = -56)
      parameter (nf_eedge = -57)
      parameter (nf_estride = -58)
      parameter (nf_ebadname = -59)
      parameter (nf_erange = -60)
      parameter (nf_enomem = -61)
      parameter (nf_evarsize = -62)
      parameter (nf_edimsize = -63)
      parameter (nf_etrunc = -64)
!
! error handling modes:
!
      integer  nf_fatal
      integer nf_verbose

      parameter (nf_fatal = 1)
      parameter (nf_verbose = 2)

!
! miscellaneous routines:
!
      character*80   nf_inq_libvers
      external       nf_inq_libvers

      character*80   nf_strerror
!                         (integer             ncerr)
      external       nf_strerror

      logical        nf_issyserr
!                         (integer             ncerr)
      external       nf_issyserr

!
! control routines:
!
      integer         nf_inq_base_pe
!                         (integer             ncid,
!                          integer             pe)
      external        nf_inq_base_pe

      integer         nf_set_base_pe
!                         (integer             ncid,
!                          integer             pe)
      external        nf_set_base_pe

      integer         nf_create
!                         (character*(*)       path,
!                          integer             cmode,
!                          integer             ncid)
      external        nf_create

      integer         nf__create
!                         (character*(*)       path,
!                          integer             cmode,
!                          integer             initialsz,
!                          integer             chunksizehint,
!                          integer             ncid)
      external        nf__create

      integer         nf__create_mp
!                         (character*(*)       path,
!                          integer             cmode,
!                          integer             initialsz,
!                          integer             basepe,
!                          integer             chunksizehint,
!                          integer             ncid)
      external        nf__create_mp

      integer         nf_open
!                         (character*(*)       path,
!                          integer             mode,
!                          integer             ncid)
      external        nf_open

      integer         nf__open
!                         (character*(*)       path,
!                          integer             mode,
!                          integer             chunksizehint,
!                          integer             ncid)
      external        nf__open

      integer         nf__open_mp
!                         (character*(*)       path,
!                          integer             mode,
!                          integer             basepe,
!                          integer             chunksizehint,
!                          integer             ncid)
      external        nf__open_mp

      integer         nf_set_fill
!                         (integer             ncid,
!                          integer             fillmode,
!                          integer             old_mode)
      external        nf_set_fill

      integer         nf_set_default_format
!                          (integer             format,
!                          integer             old_format)
      external        nf_set_default_format

      integer         nf_redef
!                         (integer             ncid)
      external        nf_redef

      integer         nf_enddef
!                         (integer             ncid)
      external        nf_enddef

      integer         nf__enddef
!                         (integer             ncid,
!                          integer             h_minfree,
!                          integer             v_align,
!                          integer             v_minfree,
!                          integer             r_align)
      external        nf__enddef

      integer         nf_sync
!                         (integer             ncid)
      external        nf_sync

      integer         nf_abort
!                         (integer             ncid)
      external        nf_abort

      integer         nf_close
!                         (integer             ncid)
      external        nf_close

      integer         nf_delete
!                         (character*(*)       ncid)
      external        nf_delete

!
! general inquiry routines:
!

      integer         nf_inq
!                         (integer             ncid,
!                          integer             ndims,
!                          integer             nvars,
!                          integer             ngatts,
!                          integer             unlimdimid)
      external        nf_inq

      integer         nf_inq_ndims
!                         (integer             ncid,
!                          integer             ndims)
      external        nf_inq_ndims

      integer         nf_inq_nvars
!                         (integer             ncid,
!                          integer             nvars)
      external        nf_inq_nvars

      integer         nf_inq_natts
!                         (integer             ncid,
!                          integer             ngatts)
      external        nf_inq_natts

      integer         nf_inq_unlimdim
!                         (integer             ncid,
!                          integer             unlimdimid)
      external        nf_inq_unlimdim

      integer         nf_inq_format
!                         (integer             ncid,
!                          integer             format)
      external        nf_inq_format

!
! dimension routines:
!

      integer         nf_def_dim
!                         (integer             ncid,
!                          character(*)        name,
!                          integer             len,
!                          integer             dimid)
      external        nf_def_dim

      integer         nf_inq_dimid
!                         (integer             ncid,
!                          character(*)        name,
!                          integer             dimid)
      external        nf_inq_dimid

      integer         nf_inq_dim
!                         (integer             ncid,
!                          integer             dimid,
!                          character(*)        name,
!                          integer             len)
      external        nf_inq_dim

      integer         nf_inq_dimname
!                         (integer             ncid,
!                          integer             dimid,
!                          character(*)        name)
      external        nf_inq_dimname

      integer         nf_inq_dimlen
!                         (integer             ncid,
!                          integer             dimid,
!                          integer             len)
      external        nf_inq_dimlen

      integer         nf_rename_dim
!                         (integer             ncid,
!                          integer             dimid,
!                          character(*)        name)
      external        nf_rename_dim

!
! general attribute routines:
!

      integer         nf_inq_att
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype,
!                          integer             len)
      external        nf_inq_att

      integer         nf_inq_attid
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             attnum)
      external        nf_inq_attid

      integer         nf_inq_atttype
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype)
      external        nf_inq_atttype

      integer         nf_inq_attlen
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             len)
      external        nf_inq_attlen

      integer         nf_inq_attname
!                         (integer             ncid,
!                          integer             varid,
!                          integer             attnum,
!                          character(*)        name)
      external        nf_inq_attname

      integer         nf_copy_att
!                         (integer             ncid_in,
!                          integer             varid_in,
!                          character(*)        name,
!                          integer             ncid_out,
!                          integer             varid_out)
      external        nf_copy_att

      integer         nf_rename_att
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        curname,
!                          character(*)        newname)
      external        nf_rename_att

      integer         nf_del_att
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name)
      external        nf_del_att

!
! attribute put/get routines:
!

      integer         nf_put_att_text
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             len,
!                          character(*)        text)
      external        nf_put_att_text

      integer         nf_get_att_text
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          character(*)        text)
      external        nf_get_att_text

      integer         nf_put_att_int1
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype,
!                          integer             len,
!                          nf_int1_t           i1vals(1))
      external        nf_put_att_int1

      integer         nf_get_att_int1
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          nf_int1_t           i1vals(1))
      external        nf_get_att_int1

      integer         nf_put_att_int2
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype,
!                          integer             len,
!                          nf_int2_t           i2vals(1))
      external        nf_put_att_int2

      integer         nf_get_att_int2
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          nf_int2_t           i2vals(1))
      external        nf_get_att_int2

      integer         nf_put_att_int
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype,
!                          integer             len,
!                          integer             ivals(1))
      external        nf_put_att_int

      integer         nf_get_att_int
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             ivals(1))
      external        nf_get_att_int

      integer         nf_put_att_real
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype,
!                          integer             len,
!                          real                rvals(1))
      external        nf_put_att_real

      integer         nf_get_att_real
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          real                rvals(1))
      external        nf_get_att_real

      integer         nf_put_att_double
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             xtype,
!                          integer             len,
!                          double              dvals(1))
      external        nf_put_att_double

      integer         nf_get_att_double
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          double              dvals(1))
      external        nf_get_att_double

!
! general variable routines:
!

      integer         nf_def_var
!                         (integer             ncid,
!                          character(*)        name,
!                          integer             datatype,
!                          integer             ndims,
!                          integer             dimids(1),
!                          integer             varid)
      external        nf_def_var

      integer         nf_inq_var
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name,
!                          integer             datatype,
!                          integer             ndims,
!                          integer             dimids(1),
!                          integer             natts)
      external        nf_inq_var

      integer         nf_inq_varid
!                         (integer             ncid,
!                          character(*)        name,
!                          integer             varid)
      external        nf_inq_varid

      integer         nf_inq_varname
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name)
      external        nf_inq_varname

      integer         nf_inq_vartype
!                         (integer             ncid,
!                          integer             varid,
!                          integer             xtype)
      external        nf_inq_vartype

      integer         nf_inq_varndims
!                         (integer             ncid,
!                          integer             varid,
!                          integer             ndims)
      external        nf_inq_varndims

      integer         nf_inq_vardimid
!                         (integer             ncid,
!                          integer             varid,
!                          integer             dimids(1))
      external        nf_inq_vardimid

      integer         nf_inq_varnatts
!                         (integer             ncid,
!                          integer             varid,
!                          integer             natts)
      external        nf_inq_varnatts

      integer         nf_rename_var
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        name)
      external        nf_rename_var

      integer         nf_copy_var
!                         (integer             ncid_in,
!                          integer             varid,
!                          integer             ncid_out)
      external        nf_copy_var

!
! entire variable put/get routines:
!

      integer         nf_put_var_text
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        text)
      external        nf_put_var_text

      integer         nf_get_var_text
!                         (integer             ncid,
!                          integer             varid,
!                          character(*)        text)
      external        nf_get_var_text

      integer         nf_put_var_int1
!                         (integer             ncid,
!                          integer             varid,
!                          nf_int1_t           i1vals(1))
      external        nf_put_var_int1

      integer         nf_get_var_int1
!                         (integer             ncid,
!                          integer             varid,
!                          nf_int1_t           i1vals(1))
      external        nf_get_var_int1

      integer         nf_put_var_int2
!                         (integer             ncid,
!                          integer             varid,
!                          nf_int2_t           i2vals(1))
      external        nf_put_var_int2

      integer         nf_get_var_int2
!                         (integer             ncid,
!                          integer             varid,
!                          nf_int2_t           i2vals(1))
      external        nf_get_var_int2

      integer         nf_put_var_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             ivals(1))
      external        nf_put_var_int

      integer         nf_get_var_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             ivals(1))
      external        nf_get_var_int

      integer         nf_put_var_real
!                         (integer             ncid,
!                          integer             varid,
!                          real                rvals(1))
      external        nf_put_var_real

      integer         nf_get_var_real
!                         (integer             ncid,
!                          integer             varid,
!                          real                rvals(1))
      external        nf_get_var_real

      integer         nf_put_var_double
!                         (integer             ncid,
!                          integer             varid,
!                          doubleprecision     dvals(1))
      external        nf_put_var_double

      integer         nf_get_var_double
!                         (integer             ncid,
!                          integer             varid,
!                          doubleprecision     dvals(1))
      external        nf_get_var_double

!
! single variable put/get routines:
!

      integer         nf_put_var1_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          character*1         text)
      external        nf_put_var1_text

      integer         nf_get_var1_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          character*1         text)
      external        nf_get_var1_text

      integer         nf_put_var1_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          nf_int1_t           i1val)
      external        nf_put_var1_int1

      integer         nf_get_var1_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          nf_int1_t           i1val)
      external        nf_get_var1_int1

      integer         nf_put_var1_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          nf_int2_t           i2val)
      external        nf_put_var1_int2

      integer         nf_get_var1_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          nf_int2_t           i2val)
      external        nf_get_var1_int2

      integer         nf_put_var1_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          integer             ival)
      external        nf_put_var1_int

      integer         nf_get_var1_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          integer             ival)
      external        nf_get_var1_int

      integer         nf_put_var1_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          real                rval)
      external        nf_put_var1_real

      integer         nf_get_var1_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          real                rval)
      external        nf_get_var1_real

      integer         nf_put_var1_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          doubleprecision     dval)
      external        nf_put_var1_double

      integer         nf_get_var1_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             index(1),
!                          doubleprecision     dval)
      external        nf_get_var1_double

!
! variable array put/get routines:
!

      integer         nf_put_vara_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          character(*)        text)
      external        nf_put_vara_text

      integer         nf_get_vara_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          character(*)        text)
      external        nf_get_vara_text

      integer         nf_put_vara_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          nf_int1_t           i1vals(1))
      external        nf_put_vara_int1

      integer         nf_get_vara_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          nf_int1_t           i1vals(1))
      external        nf_get_vara_int1

      integer         nf_put_vara_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          nf_int2_t           i2vals(1))
      external        nf_put_vara_int2

      integer         nf_get_vara_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          nf_int2_t           i2vals(1))
      external        nf_get_vara_int2

      integer         nf_put_vara_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             ivals(1))
      external        nf_put_vara_int

      integer         nf_get_vara_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             ivals(1))
      external        nf_get_vara_int

      integer         nf_put_vara_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          real                rvals(1))
      external        nf_put_vara_real

      integer         nf_get_vara_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          real                rvals(1))
      external        nf_get_vara_real

      integer         nf_put_vara_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          doubleprecision     dvals(1))
      external        nf_put_vara_double

      integer         nf_get_vara_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          doubleprecision     dvals(1))
      external        nf_get_vara_double

!
! strided variable put/get routines:
!

      integer         nf_put_vars_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          character(*)        text)
      external        nf_put_vars_text

      integer         nf_get_vars_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          character(*)        text)
      external        nf_get_vars_text

      integer         nf_put_vars_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          nf_int1_t           i1vals(1))
      external        nf_put_vars_int1

      integer         nf_get_vars_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          nf_int1_t           i1vals(1))
      external        nf_get_vars_int1

      integer         nf_put_vars_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          nf_int2_t           i2vals(1))
      external        nf_put_vars_int2

      integer         nf_get_vars_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          nf_int2_t           i2vals(1))
      external        nf_get_vars_int2

      integer         nf_put_vars_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             ivals(1))
      external        nf_put_vars_int

      integer         nf_get_vars_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             ivals(1))
      external        nf_get_vars_int

      integer         nf_put_vars_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          real                rvals(1))
      external        nf_put_vars_real

      integer         nf_get_vars_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          real                rvals(1))
      external        nf_get_vars_real

      integer         nf_put_vars_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          doubleprecision     dvals(1))
      external        nf_put_vars_double

      integer         nf_get_vars_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          doubleprecision     dvals(1))
      external        nf_get_vars_double

!
! mapped variable put/get routines:
!

      integer         nf_put_varm_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          character(*)        text)
      external        nf_put_varm_text

      integer         nf_get_varm_text
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          character(*)        text)
      external        nf_get_varm_text

      integer         nf_put_varm_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          nf_int1_t           i1vals(1))
      external        nf_put_varm_int1

      integer         nf_get_varm_int1
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          nf_int1_t           i1vals(1))
      external        nf_get_varm_int1

      integer         nf_put_varm_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          nf_int2_t           i2vals(1))
      external        nf_put_varm_int2

      integer         nf_get_varm_int2
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          nf_int2_t           i2vals(1))
      external        nf_get_varm_int2

      integer         nf_put_varm_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          integer             ivals(1))
      external        nf_put_varm_int

      integer         nf_get_varm_int
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          integer             ivals(1))
      external        nf_get_varm_int

      integer         nf_put_varm_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          real                rvals(1))
      external        nf_put_varm_real

      integer         nf_get_varm_real
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          real                rvals(1))
      external        nf_get_varm_real

      integer         nf_put_varm_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          doubleprecision     dvals(1))
      external        nf_put_varm_double

      integer         nf_get_varm_double
!                         (integer             ncid,
!                          integer             varid,
!                          integer             start(1),
!                          integer             count(1),
!                          integer             stride(1),
!                          integer             imap(1),
!                          doubleprecision     dvals(1))
      external        nf_get_varm_double


!     NetCDF-2.
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! begin netcdf 2.4 backward compatibility:
!

!      
! functions in the fortran interface
!
      integer nccre
      integer ncopn
      integer ncddef
      integer ncdid
      integer ncvdef
      integer ncvid
      integer nctlen
      integer ncsfil

      external nccre
      external ncopn
      external ncddef
      external ncdid
      external ncvdef
      external ncvid
      external nctlen
      external ncsfil


      integer ncrdwr
      integer nccreat
      integer ncexcl
      integer ncindef
      integer ncnsync
      integer nchsync
      integer ncndirty
      integer nchdirty
      integer nclink
      integer ncnowrit
      integer ncwrite
      integer ncclob
      integer ncnoclob
      integer ncglobal
      integer ncfill
      integer ncnofill
      integer maxncop
      integer maxncdim
      integer maxncatt
      integer maxncvar
      integer maxncnam
      integer maxvdims
      integer ncnoerr
      integer ncebadid
      integer ncenfile
      integer nceexist
      integer nceinval
      integer nceperm
      integer ncenotin
      integer nceindef
      integer ncecoord
      integer ncemaxds
      integer ncename
      integer ncenoatt
      integer ncemaxat
      integer ncebadty
      integer ncebadd
      integer ncests
      integer nceunlim
      integer ncemaxvs
      integer ncenotvr
      integer nceglob
      integer ncenotnc
      integer ncfoobar
      integer ncsyserr
      integer ncfatal
      integer ncverbos
      integer ncentool


!
! netcdf data types:
!
      integer ncbyte
      integer ncchar
      integer ncshort
      integer nclong
      integer ncfloat
      integer ncdouble

      parameter(ncbyte = 1)
      parameter(ncchar = 2)
      parameter(ncshort = 3)
      parameter(nclong = 4)
      parameter(ncfloat = 5)
      parameter(ncdouble = 6)

!     
!     masks for the struct nc flag field; passed in as 'mode' arg to
!     nccreate and ncopen.
!     

!     read/write, 0 => readonly 
      parameter(ncrdwr = 1)
!     in create phase, cleared by ncendef 
      parameter(nccreat = 2)
!     on create destroy existing file 
      parameter(ncexcl = 4)
!     in define mode, cleared by ncendef 
      parameter(ncindef = 8)
!     synchronise numrecs on change (x'10')
      parameter(ncnsync = 16)
!     synchronise whole header on change (x'20')
      parameter(nchsync = 32)
!     numrecs has changed (x'40')
      parameter(ncndirty = 64)  
!     header info has changed (x'80')
      parameter(nchdirty = 128)
!     prefill vars on endef and increase of record, the default behavior
      parameter(ncfill = 0)
!     do not fill vars on endef and increase of record (x'100')
      parameter(ncnofill = 256)
!     isa link (x'8000')
      parameter(nclink = 32768)

!     
!     'mode' arguments for nccreate and ncopen
!     
      parameter(ncnowrit = 0)
      parameter(ncwrite = ncrdwr)
      parameter(ncclob = nf_clobber)
      parameter(ncnoclob = nf_noclobber)

!     
!     'size' argument to ncdimdef for an unlimited dimension
!     
      integer ncunlim
      parameter(ncunlim = 0)

!     
!     attribute id to put/get a global attribute
!     
      parameter(ncglobal  = 0)

!     
!     advisory maximums:
!     
      parameter(maxncop = 64)
      parameter(maxncdim = 1024)
      parameter(maxncatt = 8192)
      parameter(maxncvar = 8192)
!     not enforced 
      parameter(maxncnam = 256)
      parameter(maxvdims = maxncdim)

!     
!     global netcdf error status variable
!     initialized in error.c
!     

!     no error 
      parameter(ncnoerr = nf_noerr)
!     not a netcdf id 
      parameter(ncebadid = nf_ebadid)
!     too many netcdfs open 
      parameter(ncenfile = -31)   ! nc_syserr
!     netcdf file exists && ncnoclob
      parameter(nceexist = nf_eexist)
!     invalid argument 
      parameter(nceinval = nf_einval)
!     write to read only 
      parameter(nceperm = nf_eperm)
!     operation not allowed in data mode 
      parameter(ncenotin = nf_enotindefine )   
!     operation not allowed in define mode 
      parameter(nceindef = nf_eindefine)   
!     coordinates out of domain 
      parameter(ncecoord = nf_einvalcoords)
!     maxncdims exceeded 
      parameter(ncemaxds = nf_emaxdims)
!     string match to name in use 
      parameter(ncename = nf_enameinuse)   
!     attribute not found 
      parameter(ncenoatt = nf_enotatt)
!     maxncattrs exceeded 
      parameter(ncemaxat = nf_emaxatts)
!     not a netcdf data type 
      parameter(ncebadty = nf_ebadtype)
!     invalid dimension id 
      parameter(ncebadd = nf_ebaddim)
!     ncunlimited in the wrong index 
      parameter(nceunlim = nf_eunlimpos)
!     maxncvars exceeded 
      parameter(ncemaxvs = nf_emaxvars)
!     variable not found 
      parameter(ncenotvr = nf_enotvar)
!     action prohibited on ncglobal varid 
      parameter(nceglob = nf_eglobal)
!     not a netcdf file 
      parameter(ncenotnc = nf_enotnc)
      parameter(ncests = nf_ests)
      parameter (ncentool = nf_emaxname) 
      parameter(ncfoobar = 32)
      parameter(ncsyserr = -31)

!     
!     global options variable. used to determine behavior of error handler.
!     initialized in lerror.c
!     
      parameter(ncfatal = 1)
      parameter(ncverbos = 2)

!
!     default fill values.  these must be the same as in the c interface.
!
      integer filbyte
      integer filchar
      integer filshort
      integer fillong
      real filfloat
      doubleprecision fildoub

      parameter (filbyte = -127)
      parameter (filchar = 0)
      parameter (filshort = -32767)
      parameter (fillong = -2147483647)
      parameter (filfloat = 9.9692099683868690e+36)
      parameter (fildoub = 9.9692099683868690d+36)

!     NetCDF-4.
!     This is part of netCDF-4. Copyright 2006, UCAR, See COPYRIGHT
!     file for distribution information.

!     Netcdf version 4 fortran interface.

!     $Id: netcdf4.inc,v 1.28 2010/05/25 13:53:02 ed Exp $

!     New netCDF-4 types.
      integer nf_ubyte
      integer nf_ushort
      integer nf_uint
      integer nf_int64
      integer nf_uint64
      integer nf_string
      integer nf_vlen
      integer nf_opaque
      integer nf_enum
      integer nf_compound

      parameter (nf_ubyte = 7)
      parameter (nf_ushort = 8)
      parameter (nf_uint = 9)
      parameter (nf_int64 = 10)
      parameter (nf_uint64 = 11)
      parameter (nf_string = 12)
      parameter (nf_vlen = 13)
      parameter (nf_opaque = 14)
      parameter (nf_enum = 15)
      parameter (nf_compound = 16)

!     New netCDF-4 fill values.
      integer           nf_fill_ubyte
      integer           nf_fill_ushort
!      real              nf_fill_uint
!      real              nf_fill_int64
!      real              nf_fill_uint64
      parameter (nf_fill_ubyte = 255)
      parameter (nf_fill_ushort = 65535)

!     New constants.
      integer nf_format_netcdf4
      parameter (nf_format_netcdf4 = 3)

      integer nf_format_netcdf4_classic
      parameter (nf_format_netcdf4_classic = 4)

      integer nf_netcdf4
      parameter (nf_netcdf4 = 4096)

      integer nf_classic_model
      parameter (nf_classic_model = 256)

      integer nf_chunk_seq
      parameter (nf_chunk_seq = 0)
      integer nf_chunk_sub
      parameter (nf_chunk_sub = 1)
      integer nf_chunk_sizes
      parameter (nf_chunk_sizes = 2)

      integer nf_endian_native
      parameter (nf_endian_native = 0)
      integer nf_endian_little
      parameter (nf_endian_little = 1)
      integer nf_endian_big
      parameter (nf_endian_big = 2)

!     For NF_DEF_VAR_CHUNKING
      integer nf_chunked
      parameter (nf_chunked = 0)
      integer nf_contiguous
      parameter (nf_contiguous = 1)

!     For NF_DEF_VAR_FLETCHER32
      integer nf_nochecksum
      parameter (nf_nochecksum = 0)
      integer nf_fletcher32
      parameter (nf_fletcher32 = 1)

!     For NF_DEF_VAR_DEFLATE
      integer nf_noshuffle
      parameter (nf_noshuffle = 0)
      integer nf_shuffle
      parameter (nf_shuffle = 1)

!     For NF_DEF_VAR_SZIP
      integer nf_szip_ec_option_mask
      parameter (nf_szip_ec_option_mask = 4)
      integer nf_szip_nn_option_mask
      parameter (nf_szip_nn_option_mask = 32)

!     For parallel I/O.
      integer nf_mpiio      
      parameter (nf_mpiio = 8192)
      integer nf_mpiposix
      parameter (nf_mpiposix = 16384)
      integer nf_pnetcdf
      parameter (nf_pnetcdf = 32768)

!     For NF_VAR_PAR_ACCESS.
      integer nf_independent
      parameter (nf_independent = 0)
      integer nf_collective
      parameter (nf_collective = 1)

!     New error codes.
      integer nf_ehdferr        ! Error at HDF5 layer. 
      parameter (nf_ehdferr = -101)
      integer nf_ecantread      ! Can't read. 
      parameter (nf_ecantread = -102)
      integer nf_ecantwrite     ! Can't write. 
      parameter (nf_ecantwrite = -103)
      integer nf_ecantcreate    ! Can't create. 
      parameter (nf_ecantcreate = -104)
      integer nf_efilemeta      ! Problem with file metadata. 
      parameter (nf_efilemeta = -105)
      integer nf_edimmeta       ! Problem with dimension metadata. 
      parameter (nf_edimmeta = -106)
      integer nf_eattmeta       ! Problem with attribute metadata. 
      parameter (nf_eattmeta = -107)
      integer nf_evarmeta       ! Problem with variable metadata. 
      parameter (nf_evarmeta = -108)
      integer nf_enocompound    ! Not a compound type. 
      parameter (nf_enocompound = -109)
      integer nf_eattexists     ! Attribute already exists. 
      parameter (nf_eattexists = -110)
      integer nf_enotnc4        ! Attempting netcdf-4 operation on netcdf-3 file.   
      parameter (nf_enotnc4 = -111)
      integer nf_estrictnc3     ! Attempting netcdf-4 operation on strict nc3 netcdf-4 file.   
      parameter (nf_estrictnc3 = -112)
      integer nf_enotnc3        ! Attempting netcdf-3 operation on netcdf-4 file.   
      parameter (nf_enotnc3 = -113)
      integer nf_enopar         ! Parallel operation on file opened for non-parallel access.   
      parameter (nf_enopar = -114)
      integer nf_eparinit       ! Error initializing for parallel access.   
      parameter (nf_eparinit = -115)
      integer nf_ebadgrpid      ! Bad group ID.   
      parameter (nf_ebadgrpid = -116)
      integer nf_ebadtypid      ! Bad type ID.   
      parameter (nf_ebadtypid = -117)
      integer nf_etypdefined    ! Type has already been defined and may not be edited. 
      parameter (nf_etypdefined = -118)
      integer nf_ebadfield      ! Bad field ID.   
      parameter (nf_ebadfield = -119)
      integer nf_ebadclass      ! Bad class.   
      parameter (nf_ebadclass = -120)
      integer nf_emaptype       ! Mapped access for atomic types only.   
      parameter (nf_emaptype = -121)
      integer nf_elatefill      ! Attempt to define fill value when data already exists. 
      parameter (nf_elatefill = -122)
      integer nf_elatedef       ! Attempt to define var properties, like deflate, after enddef. 
      parameter (nf_elatedef = -123)
      integer nf_edimscale      ! Probem with HDF5 dimscales. 
      parameter (nf_edimscale = -124)
      integer nf_enogrp       ! No group found.
      parameter (nf_enogrp = -125)


!     New functions.

!     Parallel I/O.
      integer nf_create_par
      external nf_create_par

      integer nf_open_par
      external nf_open_par

      integer nf_var_par_access
      external nf_var_par_access

!     Functions to handle groups.
      integer nf_inq_ncid
      external nf_inq_ncid

      integer nf_inq_grps
      external nf_inq_grps

      integer nf_inq_grpname
      external nf_inq_grpname

      integer nf_inq_grpname_full
      external nf_inq_grpname_full

      integer nf_inq_grpname_len
      external nf_inq_grpname_len

      integer nf_inq_grp_parent
      external nf_inq_grp_parent

      integer nf_inq_grp_ncid
      external nf_inq_grp_ncid

      integer nf_inq_grp_full_ncid
      external nf_inq_grp_full_ncid

      integer nf_inq_varids
      external nf_inq_varids

      integer nf_inq_dimids
      external nf_inq_dimids

      integer nf_def_grp
      external nf_def_grp

!     New options for netCDF variables.
      integer nf_def_var_deflate
      external nf_def_var_deflate

      integer nf_inq_var_deflate
      external nf_inq_var_deflate

      integer nf_def_var_fletcher32
      external nf_def_var_fletcher32

      integer nf_inq_var_fletcher32
      external nf_inq_var_fletcher32

      integer nf_def_var_chunking
      external nf_def_var_chunking

      integer nf_inq_var_chunking
      external nf_inq_var_chunking

      integer nf_def_var_fill
      external nf_def_var_fill

      integer nf_inq_var_fill
      external nf_inq_var_fill

      integer nf_def_var_endian
      external nf_def_var_endian

      integer nf_inq_var_endian
      external nf_inq_var_endian

!     User defined types.
      integer nf_inq_typeids
      external nf_inq_typeids

      integer nf_inq_typeid
      external nf_inq_typeid

      integer nf_inq_type
      external nf_inq_type

      integer nf_inq_user_type
      external nf_inq_user_type

!     User defined types - compound types.
      integer nf_def_compound
      external nf_def_compound

      integer nf_insert_compound
      external nf_insert_compound

      integer nf_insert_array_compound
      external nf_insert_array_compound

      integer nf_inq_compound
      external nf_inq_compound

      integer nf_inq_compound_name
      external nf_inq_compound_name

      integer nf_inq_compound_size
      external nf_inq_compound_size

      integer nf_inq_compound_nfields
      external nf_inq_compound_nfields

      integer nf_inq_compound_field
      external nf_inq_compound_field

      integer nf_inq_compound_fieldname
      external nf_inq_compound_fieldname

      integer nf_inq_compound_fieldindex
      external nf_inq_compound_fieldindex

      integer nf_inq_compound_fieldoffset
      external nf_inq_compound_fieldoffset

      integer nf_inq_compound_fieldtype
      external nf_inq_compound_fieldtype

      integer nf_inq_compound_fieldndims
      external nf_inq_compound_fieldndims

      integer nf_inq_compound_fielddim_sizes
      external nf_inq_compound_fielddim_sizes

!     User defined types - variable length arrays.
      integer nf_def_vlen
      external nf_def_vlen

      integer nf_inq_vlen
      external nf_inq_vlen

      integer nf_free_vlen
      external nf_free_vlen

!     User defined types - enums.
      integer nf_def_enum
      external nf_def_enum

      integer nf_insert_enum
      external nf_insert_enum

      integer nf_inq_enum
      external nf_inq_enum

      integer nf_inq_enum_member
      external nf_inq_enum_member

      integer nf_inq_enum_ident
      external nf_inq_enum_ident

!     User defined types - opaque.
      integer nf_def_opaque
      external nf_def_opaque

      integer nf_inq_opaque
      external nf_inq_opaque

!     Write and read attributes of any type, including user defined
!     types.
      integer nf_put_att
      external nf_put_att
      integer nf_get_att
      external nf_get_att

!     Write and read variables of any type, including user defined
!     types.
      integer nf_put_var
      external nf_put_var
      integer nf_put_var1
      external nf_put_var1
      integer nf_put_vara
      external nf_put_vara
      integer nf_put_vars
      external nf_put_vars
      integer nf_get_var
      external nf_get_var
      integer nf_get_var1
      external nf_get_var1
      integer nf_get_vara
      external nf_get_vara
      integer nf_get_vars
      external nf_get_vars

!     64-bit int functions.
      integer nf_put_var1_int64
      external nf_put_var1_int64
      integer nf_put_vara_int64
      external nf_put_vara_int64
      integer nf_put_vars_int64
      external nf_put_vars_int64
      integer nf_put_varm_int64
      external nf_put_varm_int64
      integer nf_put_var_int64
      external nf_put_var_int64
      integer nf_get_var1_int64
      external nf_get_var1_int64
      integer nf_get_vara_int64
      external nf_get_vara_int64
      integer nf_get_vars_int64
      external nf_get_vars_int64
      integer nf_get_varm_int64
      external nf_get_varm_int64
      integer nf_get_var_int64
      external nf_get_var_int64

!     For helping F77 users with VLENs.
      integer nf_get_vlen_element
      external nf_get_vlen_element
      integer nf_put_vlen_element
      external nf_put_vlen_element

!     For dealing with file level chunk cache.
      integer nf_set_chunk_cache
      external nf_set_chunk_cache
      integer nf_get_chunk_cache
      external nf_get_chunk_cache

!     For dealing with per variable chunk cache.
      integer nf_set_var_chunk_cache
      external nf_set_var_chunk_cache
      integer nf_get_var_chunk_cache
      external nf_get_var_chunk_cache
 
      integer,intent(in) :: klon
      real,intent(in) :: phy_nat(klon,360)
      real,intent(in) :: phy_alb(klon,360)
      real,intent(in) :: phy_sst(klon,360)
      real,intent(in) :: phy_bil(klon,360)
      real,intent(in) :: phy_rug(klon,360)
      real,intent(in) :: phy_ice(klon,360)
      real,intent(in) :: phy_fter(klon,360)
      real,intent(in) :: phy_foce(klon,360)
      real,intent(in) :: phy_flic(klon,360)
      real,intent(in) :: phy_fsic(klon,360)

      real :: phy_glo(klon_glo,360) ! temporary variable, to store phy_***(:)
                                    ! on the whole physics grid 
      INTEGER ierr
      INTEGER dimfirst(3)
      INTEGER dimlast(3)
c
      INTEGER nid, ndim, ntim
      INTEGER dims(2), debut(2), epais(2)
      INTEGER id_tim
      INTEGER id_NAT, id_SST, id_BILS, id_RUG, id_ALB
      INTEGER id_FTER,id_FOCE,id_FSIC,id_FLIC
 
      if (is_mpi_root.and.is_omp_root) then
      
        PRINT*, 'writelim: Ecriture du fichier limit'
c
        ierr = NF_CREATE ("limit.nc", NF_CLOBBER, nid)
c
        ierr = NF_PUT_ATT_TEXT (nid, NF_GLOBAL, "title", 30,
     .                       "Fichier conditions aux limites")
!!        ierr = NF_DEF_DIM (nid, "points_physiques", klon, ndim)
        ierr = NF_DEF_DIM (nid, "points_physiques", klon_glo, ndim)
        ierr = NF_DEF_DIM (nid, "time", NF_UNLIMITED, ntim)
c
        dims(1) = ndim
        dims(2) = ntim
c
ccc      ierr = NF_DEF_VAR (nid, "TEMPS", NF_DOUBLE, 1,ntim, id_tim)
        ierr = NF_DEF_VAR (nid, "TEMPS", NF_FLOAT, 1,ntim, id_tim)
        ierr = NF_PUT_ATT_TEXT (nid, id_tim, "title", 17,
     .                        "Jour dans l annee")
ccc      ierr = NF_DEF_VAR (nid, "NAT", NF_DOUBLE, 2,dims, id_NAT)
        ierr = NF_DEF_VAR (nid, "NAT", NF_FLOAT, 2,dims, id_NAT)
        ierr = NF_PUT_ATT_TEXT (nid, id_NAT, "title", 23,
     .                        "Nature du sol (0,1,2,3)")
ccc      ierr = NF_DEF_VAR (nid, "SST", NF_DOUBLE, 2,dims, id_SST)
        ierr = NF_DEF_VAR (nid, "SST", NF_FLOAT, 2,dims, id_SST)
        ierr = NF_PUT_ATT_TEXT (nid, id_SST, "title", 35,
     .                        "Temperature superficielle de la mer")
ccc      ierr = NF_DEF_VAR (nid, "BILS", NF_DOUBLE, 2,dims, id_BILS)
        ierr = NF_DEF_VAR (nid, "BILS", NF_FLOAT, 2,dims, id_BILS)
        ierr = NF_PUT_ATT_TEXT (nid, id_BILS, "title", 32,
     .                        "Reference flux de chaleur au sol")
ccc      ierr = NF_DEF_VAR (nid, "ALB", NF_DOUBLE, 2,dims, id_ALB)
        ierr = NF_DEF_VAR (nid, "ALB", NF_FLOAT, 2,dims, id_ALB)
        ierr = NF_PUT_ATT_TEXT (nid, id_ALB, "title", 19,
     .                        "Albedo a la surface")
ccc      ierr = NF_DEF_VAR (nid, "RUG", NF_DOUBLE, 2,dims, id_RUG)
        ierr = NF_DEF_VAR (nid, "RUG", NF_FLOAT, 2,dims, id_RUG)
        ierr = NF_PUT_ATT_TEXT (nid, id_RUG, "title", 8,
     .                        "Rugosite")

        ierr = NF_DEF_VAR (nid, "FTER", NF_FLOAT, 2,dims, id_FTER)
        ierr = NF_PUT_ATT_TEXT (nid, id_FTER, "title", 8,"Frac. Terre")
        ierr = NF_DEF_VAR (nid, "FOCE", NF_FLOAT, 2,dims, id_FOCE)
        ierr = NF_PUT_ATT_TEXT (nid, id_FOCE, "title", 8,"Frac. Terre")
        ierr = NF_DEF_VAR (nid, "FSIC", NF_FLOAT, 2,dims, id_FSIC)
        ierr = NF_PUT_ATT_TEXT (nid, id_FSIC, "title", 8,"Frac. Terre")
        ierr = NF_DEF_VAR (nid, "FLIC", NF_FLOAT, 2,dims, id_FLIC)
        ierr = NF_PUT_ATT_TEXT (nid, id_FLIC, "title", 8,"Frac. Terre")
c
        ierr = NF_ENDDEF(nid)
c

! write the 'times'
        do k=1,360
          ierr = NF_PUT_VAR1_DOUBLE (nid,id_tim,k,DBLE(k))
        enddo

      endif ! of if (is_mpi_root.and.is_omp_root)

! write the fields, after having collected them on master

      call gather(phy_nat,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_NAT,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_nat"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_sst,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_SST,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_sst"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_bil,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_BILS,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_bil"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_alb,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_ALB,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_alb"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_rug,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_RUG,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_rug"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_fter,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_FTER,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_fter"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_foce,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_FOCE,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_foce"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_fsic,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_FSIC,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_fsic"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

      call gather(phy_flic,phy_glo)
      if (is_mpi_root.and.is_omp_root) then
        ierr=NF_PUT_VAR_DOUBLE(nid,id_FLIC,phy_glo)
        if(ierr.ne.NF_NOERR) then
          write(*,*) "writelim error with phy_flic"
          write(*,*) NF_STRERROR(ierr)
        endif
      endif

!  close file:
      if (is_mpi_root.and.is_omp_root) then
        ierr = NF_CLOSE(nid)
      endif

      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      SUBROUTINE profil_sst(nlon, rlatd, type_profil, phy_sst)
      use dimphy
      IMPLICIT none
c
      INTEGER nlon, type_profil, i, k, j
      REAL :: rlatd(nlon), phy_sst(nlon, 360) 
      INTEGER imn, imx, amn, amx, kmn, kmx
      INTEGER p, pplus, nlat_max
      parameter (nlat_max=72)
      REAL x_anom_sst(nlat_max)
c
      if (klon.ne.nlon) stop'probleme de dimensions dans iniaqua'
      do i=1,360
c      phy_sst(:,i) = 260.+50.*cos(rlatd(:))**2

c Rajout fbrlmd

      if(type_profil.EQ.1)then
c     Méthode 1 "Control" faible plateau à l'Equateur
      do j=1,klon
       phy_sst(j,i)=273.+27.*(1-sin(1.5*rlatd(j))**2)
c        PI/3=1.047197551
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif
      if(type_profil.EQ.2)then
c     Méthode 2 "Flat" fort plateau à l'Equateur
      do j=1,klon
       phy_sst(j,i)=273.+27.*(1-sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif


      if (type_profil.EQ.3) then
c     Méthode 3 "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.+0.5*27.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif

      if (type_profil.EQ.4) then
c     Méthode 4 : Méthode 3 + SST+2 "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.+0.5*29.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif

      if (type_profil.EQ.5) then
c     Méthode 5 : Méthode 3 + +2K "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.+2.+0.5*27.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.+2.
        endif

      enddo
      endif

      if(type_profil.EQ.6)then
c     Méthode 6 "cst" valeur constante de SST
      do j=1,klon
       phy_sst(j,i)=288.
      enddo
      endif


        if(type_profil.EQ.7)then
c     Méthode 7 "cst" valeur constante de SST +2
      do j=1,klon
       phy_sst(j,i)=288.+2.
      enddo
      endif

        p=0
        if(type_profil.EQ.8)then
c     Méthode 8 profil anomalies SST du modèle couplé AR4
       do j=1,klon
         if (rlatd(j).EQ.rlatd(j-1)) then
       phy_sst(j,i)=273.+x_anom_sst(pplus)
     &     +0.5*27.*(2-sin(1.5*rlatd(j))**2-sin(1.5*rlatd(j))**4)
          if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
            phy_sst(j,i)=273.+x_anom_sst(pplus)
          endif
        else
          p=p+1
          pplus=73-p
        phy_sst(j,i)=273.+x_anom_sst(pplus)
     &     +0.5*27.*(2-sin(1.5*rlatd(j))**2-sin(1.5*rlatd(j))**4)
          if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
            phy_sst(j,i)=273.+x_anom_sst(pplus)
          endif
          write (*,*) rlatd(j),x_anom_sst(pplus),phy_sst(j,i)
        endif
      enddo
      endif

      if (type_profil.EQ.9) then
c     Méthode 5 : Méthode 3 + -2K "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.-2.+0.5*27.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.-2.
        endif
      enddo
      endif


      if (type_profil.EQ.10) then
c     Méthode 10 : Méthode 3 + +4K "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.+4.+0.5*27.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
        if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.+4.
        endif
      enddo
      endif

      if (type_profil.EQ.11) then
c     Méthode 11 : Méthode 3 + 4CO2 "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.+0.5*27.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
        if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif

      if (type_profil.EQ.12) then
c     Méthode 12 : Méthode 10 + 4CO2 "Qobs" plateau réel à l'Equateur
      do j=1,klon
        phy_sst(j,i)=273.+4.+0.5*27.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
        if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.+4.
        endif
      enddo
      endif

      if (type_profil.EQ.13) then
c     Méthode 13 "Qmax" plateau réel à l'Equateur augmenté !
      do j=1,klon
        phy_sst(j,i)=273.+0.5*29.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif

      if (type_profil.EQ.14) then
c     Méthode 13 "Qmax2K" plateau réel à l'Equateur augmenté +2K !
      do j=1,klon
        phy_sst(j,i)=273.+2.+0.5*29.*(2-sin(1.5*rlatd(j))**2
     &                 -sin(1.5*rlatd(j))**4)
      if((rlatd(j).GT.1.0471975).OR.(rlatd(j).LT.-1.0471975))then
         phy_sst(j,i)=273.
        endif
      enddo
      endif

      enddo

cIM beg : verif profil SST: phy_sst
       amn=MIN(phy_sst(1,1),1000.)
       amx=MAX(phy_sst(1,1),-1000.)
       DO k=1, 360
       DO i=2, nlon
        IF(phy_sst(i,k).LT.amn) THEN
         amn=phy_sst(i,k)
         imn=i
         kmn=k
        ENDIF
        IF(phy_sst(i,k).GT.amx) THEN
         amx=phy_sst(i,k)
         imx=i
         kmx=k
        ENDIF
       ENDDO
       ENDDO
c
       PRINT*,' debut avant writelim min max phy_sst',imn,kmn,amn,
     & imx,kmx,amx
cIM end : verif profil SST: phy_sst

       return 
       end
