!
MODULE surf_ocean_mod

  IMPLICIT NONE

CONTAINS
!
!****************************************************************************************
!
  SUBROUTINE surf_ocean(rlon, rlat, swnet, lwnet, alb1, &
       rugos, windsp, rmu0, fder, tsurf_in, &
       itime, dtime, jour, knon, knindex, &
       p1lay, cdragh, cdragm, precip_rain, precip_snow, temp_air, spechum, &
       AcoefH, AcoefQ, BcoefH, BcoefQ, &
       AcoefU, AcoefV, BcoefU, BcoefV, &
       ps, u1, v1, rugoro, pctsrf, &
       snow, qsurf, agesno, &
       z0_new, alb1_new, alb2_new, evap, fluxsens, fluxlat, &
       tsurf_new, dflux_s, dflux_l, lmt_bils, &
       flux_u1, flux_v1 &

        &       ,xtprecip_rain, xtprecip_snow,xtspechum,Roce, &
        &       xtsnow,xtevap,h1 &




        &       )

  USE dimphy
  USE surface_data, ONLY     : type_ocean
  USE ocean_forced_mod, ONLY : ocean_forced_noice
  USE ocean_slab_mod, ONLY   : ocean_slab_noice
  USE ocean_cpl_mod, ONLY    : ocean_cpl_noice
!
! This subroutine will make a call to ocean_XXX_noice according to the ocean mode (force, 
! slab or couple). The calculations of albedo and rugosity for the ocean surface are 
! done in here because they are identical for the different modes of ocean. 
!
    INCLUDE "indicesol.h"
    INCLUDE "YOMCST.h"

! Input variables
!****************************************************************************************
    INTEGER, INTENT(IN)                      :: itime, jour, knon
    INTEGER, DIMENSION(klon), INTENT(IN)     :: knindex
    REAL, INTENT(IN)                         :: dtime
    REAL, DIMENSION(klon), INTENT(IN)        :: rlon, rlat
    REAL, DIMENSION(klon), INTENT(IN)        :: swnet  ! net shortwave radiation at surface  
    REAL, DIMENSION(klon), INTENT(IN)        :: lwnet  ! net longwave radiation at surface  
    REAL, DIMENSION(klon), INTENT(IN)        :: alb1   ! albedo in visible SW interval
    REAL, DIMENSION(klon), INTENT(IN)        :: rugos
    REAL, DIMENSION(klon), INTENT(IN)        :: windsp
    REAL, DIMENSION(klon), INTENT(IN)        :: rmu0  
    REAL, DIMENSION(klon), INTENT(IN)        :: fder
    REAL, DIMENSION(klon), INTENT(IN)        :: tsurf_in
    REAL, DIMENSION(klon), INTENT(IN)        :: p1lay
    REAL, DIMENSION(klon), INTENT(IN)        :: cdragh
    REAL, DIMENSION(klon), INTENT(IN)        :: cdragm
    REAL, DIMENSION(klon), INTENT(IN)        :: precip_rain, precip_snow
    REAL, DIMENSION(klon), INTENT(IN)        :: temp_air, spechum
    REAL, DIMENSION(klon), INTENT(IN)        :: AcoefH, AcoefQ, BcoefH, BcoefQ
    REAL, DIMENSION(klon), INTENT(IN)        :: AcoefU, AcoefV, BcoefU, BcoefV
    REAL, DIMENSION(klon), INTENT(IN)        :: ps
    REAL, DIMENSION(klon), INTENT(IN)        :: u1, v1
    REAL, DIMENSION(klon), INTENT(IN)        :: rugoro
    REAL, DIMENSION(klon,nbsrf), INTENT(IN)  :: pctsrf
!C----------------------------------------------------------------------------------
!C COMMON "wateriso.h" including all parameters + some fields of water isotopes:
!C filled by iso_init.f
!C
!C Sandrine Bony (LMD/CNRS), March 2003
!C----------------------------------------------------------------------------------

!c -- general:

!#include "dimisodyn.h"
        ! c'est ici que l'on choisi quels isotopes on veut simuler
        integer use_iso_eau ! eau normale H2O18
        integer use_iso_HDO ! HDO (Deuterium)
        integer use_iso_O18 ! H2018
        integer use_iso_O17 ! H2O17
        integer use_iso_HTO ! HTO (Tritium)        

        ! mettre la valeur 1 si on veut utiliser, 0 sinon
        parameter (use_iso_eau=0)
        parameter (use_iso_HDO=1)
        parameter (use_iso_O18=1)
        parameter (use_iso_O17=0)
        parameter (use_iso_HTO=0)
    
        

!c -- parameters that do not depend on the type of water isotope:

       real pxtmelt,pxtice,pxtmin,pxtmax
       real tdifexp, tv0cin, thumxt1
       integer ntot
       real h_land_ice
       real P_veg
       real musi,lambda_sursat
       real Kd
       real rh_cste_surf_cond,T_cste_surf_cond

!c type de simulation
	logical bidouille_anti_divergence
                ! si true, rappel rgulier de xteau vers q, pour viter drives lentes
	logical essai_convergence 
                ! si false, on fait rigoureusement comme dans LMDZ sans isotopes, 
		! meme si c'est gnant pour les isotopes
        integer initialisation_iso
                ! 0: dans fichier
                ! 1: R=0
                ! 2: R selon distill rayleigh
                ! 3: R=Rsmow
        integer initialisation_isotrac
                ! 1 pour idalis
                ! 0 pour lecture dans fichier
        integer modif_SST ! 0 par defaut, 1 si on veut modifier la sst
                ! 2 et 3: profils de SST
        real deltaTtest ! modif de la SST, uniforme.
	integer modif_sic ! on met des trous dans glace de mer
	real deltasic ! fraction de trous minimale
	real deltaTtestpoles
	real sstlatcrit
	real dsstlatcrit
	real deltaO18_oce
	integer albedo_prescrit ! 0 par defaut
				 ! 1 si on veut garder albedo constant
	real lon_min_albedo,lon_max_albedo
	real lat_min_albedo,lat_max_albedo
	real deltaP_BL,tdifexp_sol
        integer ruissellement_pluie,alphak_stewart
        integer calendrier_guide
        integer cste_surf_cond
	real mixlen
        integer evap_cont_cste
	real deltaO18_evap_cont,d_evap_cont
	integer lev_histiso,lev_histiso_diag
	integer no_pce
!	real slope_limiterxy,slope_limiterz
	real A_satlim
	integer ok_restrict_A_satlim,modif_ratqs
	real Pcrit_ratqs,ratqsbasnew
	real fac_modif_evaoce
	integer ok_bidouille_wake
	logical cond_temp_env
        logical ok_prod_nucl_tritium
        integer nessai
        parameter (nessai=486)
        integer day_nucl(nessai)
        integer month_nucl(nessai)
        integer year_nucl(nessai)
        real lat_nucl(nessai)
        real lon_nucl(nessai)
        real zmin_nucl(nessai)
        real zmax_nucl(nessai)
        real HTO_nucl(nessai)
        

!c -- parameters to define the indexation of isotopes
        integer iso_eau ! eau normale H2O18
        integer iso_HDO ! HDO (Deuterium)
        integer iso_O18 ! H2018
        integer iso_O17 ! H2O17
        integer iso_HTO ! HTO (Tritium)
        
        integer niso ! nombre d'isotopes utiliss   
        parameter (niso=use_iso_HTO+use_iso_O17 &
     &               +use_iso_O18+use_iso_HDO+use_iso_eau) 

        character*3 striso(niso)    

        integer ntraciso
        parameter (ntraciso=niso)
    	integer ntracisoOR
        parameter (ntracisoOR=niso)

!c -- parameters that depend on the type of water isotope:
        real fac_coeff_eq17_liq,fac_coeff_eq17_ice

       real tsatfac(niso) &
     &    , tnat(niso), toce(niso), tcorr(niso) &
     &    , tdifrel(niso) &
     &    , talph1(niso), talph2(niso), talph3(niso) &
     &    , talps1(niso), talps2(niso) &
     &    , tkcin0(niso) &
     &    , tkcin1(niso),tkcin2(niso) &
     &    , alpha_liq_sol(niso) &
     &    , Rdefault(niso)

! Rdefault est la valeur de r quand on ne sait pas quoi mettre. 
! Normallement, trs peu utilis.

!c -- everything is there:

       COMMON /wateriso/ h_land_ice, P_veg &
     &    , pxtmelt, pxtice, pxtmin, pxtmax &
     &    , bidouille_anti_divergence,essai_convergence &
     &    , initialisation_iso, initialisation_isotrac &
     &    , modif_sst,deltaTtest &
     &    , modif_sic,deltasic &
     &    , deltaTtestpoles,sstlatcrit,albedo_prescrit &
     &    , lon_min_albedo,lon_max_albedo,lat_min_albedo,lat_max_albedo &
     &    , tdifexp,tv0cin,fac_coeff_eq17_liq,fac_coeff_eq17_ice   &
     &    , thumxt1, musi,lambda_sursat, Kd, deltaO18_oce, ntot &
     &    , rh_cste_surf_cond, T_cste_surf_cond  &
     &    , deltaP_BL, tdifexp_sol, ruissellement_pluie &
     &    , alphak_stewart, cste_surf_cond &
     &    , mixlen &
     &    , tnat, toce, tcorr &
     &    , tdifrel &
     &    , calendrier_guide &
     &    , talph1, talph2, talph3 &
     &    , talps1, talps2 &
     &    , tkcin0, tkcin1, tkcin2 &
     &    , alpha_liq_sol, Rdefault &
     &    , striso  &
     &    , iso_eau,iso_HDO,iso_O18,iso_O17,iso_HTO &
     &    , evap_cont_cste,deltaO18_evap_cont,d_evap_cont &
     &    , lev_histiso,lev_histiso_diag,no_pce,A_satlim &
     &    , ok_restrict_A_satlim,modif_ratqs,fac_modif_evaoce &
     &    , ratqsbasnew,Pcrit_ratqs &
     &    , ok_bidouille_wake,cond_temp_env &
     &    , ok_prod_nucl_tritium

       COMMON /tritium/ day_nucl, month_nucl &
     &    , year_nucl, lat_nucl, lon_nucl &
     &    , zmin_nucl, zmax_nucl, HTO_nucl 
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)        :: xtprecip_rain, xtprecip_snow 
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)        :: xtspechum

! In/Output variables
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(INOUT)     :: snow
    REAL, DIMENSION(klon), INTENT(INOUT)     :: qsurf
    REAL, DIMENSION(klon), INTENT(INOUT)     :: agesno
    REAL, DIMENSION(niso,klon), INTENT(IN)        :: xtsnow 
    REAL, DIMENSION(niso,klon), INTENT(INOUT)        :: Roce  

! Output variables
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(OUT)       :: z0_new
    REAL, DIMENSION(klon), INTENT(OUT)       :: alb1_new  ! new albedo in visible SW interval
    REAL, DIMENSION(klon), INTENT(OUT)       :: alb2_new  ! new albedo in near IR interval
    REAL, DIMENSION(klon), INTENT(OUT)       :: evap, fluxsens, fluxlat
    REAL, DIMENSION(klon), INTENT(OUT)       :: tsurf_new
    REAL, DIMENSION(klon), INTENT(OUT)       :: dflux_s, dflux_l      
    REAL, DIMENSION(klon), INTENT(OUT)       :: lmt_bils
    REAL, DIMENSION(klon), INTENT(OUT)       :: flux_u1, flux_v1
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)        :: xtevap  
    REAL, DIMENSION(klon), INTENT(IN)        :: h1   

! Local variables
!****************************************************************************************
    INTEGER               :: i
    REAL                  :: tmp
    REAL, PARAMETER       :: cepdu2=(0.1)**2
    REAL, DIMENSION(klon) :: alb_eau
    REAL, DIMENSION(klon) :: radsol

! End definition
!****************************************************************************************


!****************************************************************************************
! Calculate total net radiance at surface
!
!****************************************************************************************
    radsol(:) = 0.0
    radsol(1:knon) = swnet(1:knon) + lwnet(1:knon)

!****************************************************************************************
! Switch according to type of ocean (couple, slab or forced)
!****************************************************************************************
    SELECT CASE(type_ocean)
    CASE('couple')
       CALL ocean_cpl_noice( &
            swnet, lwnet, alb1, &
            windsp, fder, & 
            itime, dtime, knon, knindex, &
            p1lay, cdragh, cdragm, precip_rain, precip_snow,temp_air,spechum,& 
            AcoefH, AcoefQ, BcoefH, BcoefQ, &
            AcoefU, AcoefV, BcoefU, BcoefV, &
            ps, u1, v1, &
            radsol, snow, agesno, &
            qsurf, evap, fluxsens, fluxlat, flux_u1, flux_v1, &
            tsurf_new, dflux_s, dflux_l)

    CASE('slab')
       CALL ocean_slab_noice( &
            itime, dtime, jour, knon, knindex, &
            p1lay, cdragh, cdragm, precip_rain, precip_snow, temp_air, spechum,&
            AcoefH, AcoefQ, BcoefH, BcoefQ, &
            AcoefU, AcoefV, BcoefU, BcoefV, &
            ps, u1, v1, tsurf_in, &
            radsol, snow, agesno, &
            qsurf, evap, fluxsens, fluxlat, flux_u1, flux_v1, &
            tsurf_new, dflux_s, dflux_l, lmt_bils)
       
    CASE('force')
       CALL ocean_forced_noice( &
            itime, dtime, jour, knon, knindex, &
            p1lay, cdragh, cdragm, precip_rain, precip_snow, &
            temp_air, spechum, &
            AcoefH, AcoefQ, BcoefH, BcoefQ, &
            AcoefU, AcoefV, BcoefU, BcoefV, &
            ps, u1, v1, &
            radsol, snow, agesno, &
            qsurf, evap, fluxsens, fluxlat, flux_u1, flux_v1, &
            tsurf_new, dflux_s, dflux_l &
            ,xtprecip_rain, xtprecip_snow, xtspechum,Roce,rlat, &
            xtsnow,xtevap,h1 &  
            )
    END SELECT

!****************************************************************************************
! Calculate albedo
!
!****************************************************************************************
    IF ( MINVAL(rmu0) == MAXVAL(rmu0) .AND. MINVAL(rmu0) == -999.999 ) THEN
       CALL alboc(REAL(jour),rlat,alb_eau)
    ELSE  ! diurnal cycle
       CALL alboc_cd(rmu0,alb_eau)
    ENDIF

    DO i =1, knon
       alb1_new(i) = alb_eau(knindex(i))
    ENDDO
    alb2_new(1:knon) = alb1_new(1:knon)

!****************************************************************************************
! Calculate the rugosity
!
!****************************************************************************************
    DO i = 1, knon
       tmp = MAX(cepdu2,u1(i)**2+v1(i)**2)
       z0_new(i) = 0.018*cdragm(i) * (u1(i)**2+v1(i)**2)/RG  &
            +  0.11*14e-6 / SQRT(cdragm(i) * tmp)
       z0_new(i) = MAX(1.5e-05,z0_new(i))
    ENDDO   

!
!****************************************************************************************
!    
  END SUBROUTINE surf_ocean
!
!****************************************************************************************
!
END MODULE surf_ocean_mod
