!
! $Id: addfi_p.F 1454 2010-11-18 12:01:24Z fairhead $
!
      SUBROUTINE addfi_p(pdt, leapf, forward,
     S          pucov, pvcov, pteta, pq   , pps ,
     S          pdufi, pdvfi, pdhfi,pdqfi, pdpfi  
#ifdef ISO
     :          ,pxt,pdxtfi
#endif      
     :               )
      USE parallel
      USE infotrac, ONLY : nqtot
      USE control_mod, ONLY : planet_type
      IMPLICIT NONE
c
c=======================================================================
c
c    Addition of the physical tendencies
c
c    Interface :
c    -----------
c
c      Input :
c      -------
c      pdt                    time step of integration
c      leapf                  logical
c      forward                logical
c      pucov(ip1jmp1,llm)     first component of the covariant velocity
c      pvcov(ip1ip1jm,llm)    second component of the covariant velocity
c      pteta(ip1jmp1,llm)     potential temperature
c      pts(ip1jmp1,llm)       surface temperature
c      pdufi(ip1jmp1,llm)     |
c      pdvfi(ip1jm,llm)       |   respective
c      pdhfi(ip1jmp1)         |      tendencies
c      pdtsfi(ip1jmp1)        |
c
c      Output :
c      --------
c      pucov
c      pvcov
c      ph
c      pts
c
c
c=======================================================================
c
c-----------------------------------------------------------------------
c
c    0.  Declarations :
c    ------------------
c
#include "dimensions.h"
#include "paramet.h"
#include "comconst.h"
#include "comgeom.h"
#include "serre.h"
c
c    Arguments :
c    -----------
c
      REAL pdt
c
      REAL pvcov(ip1jm,llm),pucov(ip1jmp1,llm)
      REAL pteta(ip1jmp1,llm),pq(ip1jmp1,llm,nqtot),pps(ip1jmp1)
c
      REAL pdvfi(ip1jm,llm),pdufi(ip1jmp1,llm)
      REAL pdqfi(ip1jmp1,llm,nqtot),pdhfi(ip1jmp1,llm),pdpfi(ip1jmp1)
c
      LOGICAL leapf,forward
#ifdef ISO
      ! input-output
      !integer niso
#include "wateriso2.h"
      real pxt(ip1jmp1,llm,2,ntraciso) ! <-> pq(ip1jmp1,llm,nq)
      real pdxtfi(ip1jmp1,llm,2,ntraciso) ! <->  pdqfi(ip1jmp1,llm,nq)
      ! local
      integer ixt
      ! pour verif:
      real pqprec(ip1jmp1,llm,2),pxtprec(ip1jmp1,llm,2,ntraciso)
      real pqprec_interm(ip1jmp1,llm,2)
#include "iso_verif.h"
#ifdef ISOVERIF      
      integer iso_verif_aberrant_nostop ! juste debug
      real deltaD
#endif      
#endif
c
c
c    Local variables :
c    -----------------
c
      REAL xpn(iim),xps(iim),tpn,tps
      INTEGER j,k,iq,ij
      REAL qtestw, qtestt
      PARAMETER ( qtestw = 1.0e-15 )
      PARAMETER ( qtestt = 1.0e-40 )

      REAL SSUM
      EXTERNAL SSUM
      
      INTEGER :: ijb,ije
c
c-----------------------------------------------------------------------
      
      ijb=ij_begin
      ije=ij_end
      
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)      
      DO k = 1,llm
         DO j = ijb,ije
            pteta(j,k)= pteta(j,k) + pdhfi(j,k) * pdt
         ENDDO
      ENDDO
c$OMP END DO NOWAIT

      if (pole_nord) then
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
        DO  k    = 1, llm
         DO  ij   = 1, iim
           xpn(ij) = aire(   ij   ) * pteta(  ij    ,k)
         ENDDO
         tpn      = SSUM(iim,xpn,1)/ apoln

         DO ij   = 1, iip1
           pteta(   ij   ,k)  = tpn
         ENDDO
       ENDDO
c$OMP END DO NOWAIT
      endif

      if (pole_sud) then
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
        DO  k    = 1, llm
         DO  ij   = 1, iim
           xps(ij) = aire(ij+ip1jm) * pteta(ij+ip1jm,k)
         ENDDO
         tps      = SSUM(iim,xps,1)/ apols

         DO ij   = 1, iip1
           pteta(ij+ip1jm,k)  = tps
         ENDDO
       ENDDO
c$OMP END DO NOWAIT
      endif
c

      ijb=ij_begin
      ije=ij_end
      if (pole_nord) ijb=ij_begin+iip1
      if (pole_sud)  ije=ij_end-iip1

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO k = 1,llm
         DO j = ijb,ije
            pucov(j,k)= pucov(j,k) + pdufi(j,k) * pdt
         ENDDO
      ENDDO
c$OMP END DO NOWAIT

      if (pole_nord) ijb=ij_begin

c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
      DO k = 1,llm
         DO j = ijb,ije
            pvcov(j,k)= pvcov(j,k) + pdvfi(j,k) * pdt
         ENDDO
      ENDDO
c$OMP END DO NOWAIT

c
      if (pole_sud)  ije=ij_end
c$OMP MASTER
      DO j = ijb,ije
         pps(j) = pps(j) + pdpfi(j) * pdt
      ENDDO
c$OMP END MASTER
 
 
#ifdef ISO
#ifdef ISOVERIFNAN
      DO iq = 1, 2
       DO k = 1,llm
         DO j = ijb, ije
          do ixt=1,ntraciso
           call iso_verif_noNaN(pxt(j,k,iq,ixt),'addfi 184')
           call iso_verif_noNaN(pdxtfi(j,k,iq,ixt),'addfi 186')
          enddo   
         enddo !DO j = ijb, ije
        enddo !DO k = 1,llm
       enddo !DO iq = 1, 2             
#endif
#ifdef ISOVERIF
      DO iq = 1, 2
       DO k = 1,llm
         DO j = ijb, ije
          if (use_iso_eau.eq.1) then  
               call iso_verif_egalite_choix(pxt(j,k,iq,iso_eau),
     :                   pq(j,k,iq),
     :                   'addfi 144',errmax,errmaxrel)
          endif
#ifdef ISOTRAC      
               call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij,k,iq,'addfi 151')
#endif           
         enddo !DO j = ijb, ije
        enddo !DO k = 1,llm
       enddo !DO iq = 1, 2  
#endif               
      ! remplissage des valeurs avant modifs de pq et pxt
      DO iq = 1, 2
         DO k = 1,llm
            DO j = ijb, ije
              pqprec(j,k,iq)= pq(j,k,iq) 
              pqprec_interm(j,k,iq)=pq(j,k,iq)+pdqfi(j,k,iq)*pdt
             enddo !DO j = ijb, ije
         enddo !DO k = 1,llm
      enddo !DO iq = 1, 2 
      do ixt=1,ntraciso
       DO iq = 1, 2
         DO k = 1,llm
            DO j = ijb, ije
                  pxtprec(j,k,iq,ixt)=pxt(j,k,iq,ixt)
            enddo !DO j = ijb, ije
         enddo !DO k = 1,llm
       enddo !DO iq = 1, 2
      enddo  !do ixt=1,ntraciso
            
#ifdef ISOVERIF
      DO iq = 1, 2
       DO k = 1,llm
         DO j = ijb, ije
          if (use_iso_eau.eq.1) then  
               call iso_verif_egalite_choix(pxtprec(j,k,iq,iso_eau),
     :                   pqprec(j,k,iq),
     :                   'addfi 155',errmax,errmaxrel)
          endif
#ifdef ISOTRAC      
              call iso_verif_traceur_retourne3D(pxtprec,ip1jmp1,llm,2,
     :                   j,k,iq,'addfi 187')
#endif           
         enddo !DO j = ijb, ije
        enddo !DO k = 1,llm
       enddo !DO iq = 1, 2  
#endif               
#endif     

      if (planet_type=="earth") then
      ! earth case, special treatment for first 2 tracers (water)
       DO iq = 1, 2
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO k = 1,llm
            DO j = ijb,ije
               pq(j,k,iq)= pq(j,k,iq) + pdqfi(j,k,iq) * pdt
               pq(j,k,iq)= AMAX1( pq(j,k,iq), qtestw )
            ENDDO
         ENDDO
c$OMP END DO NOWAIT
       ENDDO

#ifdef ISO
       DO iq = 1, 2
         DO k = 1,llm
            DO j = ijb,ije        
               if ((pqprec(j,k,iq).gt.0.0).and.
     :                   (pqprec_interm(j,k,iq).lt.ridicule*110)  
     :              .and.(pqprec_interm(j,k,iq)/
     :              max(pqprec(j,k,iq),1e-18).lt.5e-7)) then
                    ! dans ce cas, il ne reste presque plus d'eau par
                    !rapport à avant
                    ! on suppose alors que deltaD après= deltaD avant
                    ! modif le 16 nov 2007: on remonte le seuil pour
                    ! pq/pqprec à 5e-7, car à 2e-7 ce n'tait pas
                    ! suffisant pour éviter des problèmes de deltad
                    ! abbérants.
                    ! 12 aout 2008: on remonte seuil de pqprec_interm à
                    ! ridicule*80 au lieu de ridicule*10
                    ! 30 juillet: on remonte à ridicule*110
                    do ixt=1,ntraciso
                      pxt(j,k,iq,ixt)= (pxtprec(j,k,iq,ixt)
     :                        /pqprec(j,k,iq))*pqprec_interm(j,k,iq)
                   enddo    
#ifdef ISOTRAC
                   ! en faisant
                   ! xt(iso,zone)=xt_prec(iso,zone)/q_prec*q
                   ! on assure que la somme des traceurs sera égale à
                   ! l'isotope total
#endif
                else !if ((pqprec.gt.0.0)
                   ! il reste encore plein d'eau dans la boite ->
                   ! formule générale
                   do ixt=1,ntraciso
                      pxt(j,k,iq,ixt)= pxtprec(j,k,iq,ixt) 
     :                      + pdxtfi(j,k,iq,ixt) * pdt
                   enddo
               endif !if ((pqprec.gt.0.0)
                             
#ifdef ISOVERIFNAN
               do ixt=1,ntraciso
                call iso_verif_noNaN(pxt(j,k,iq,ixt),'addfi 287')
               enddo              
#endif
#ifdef ISOVERIF
              if (use_iso_eau.eq.1) then  
               call iso_verif_egalite_choix(pxtprec(j,k,iq,iso_eau),
     :                   pqprec(j,k,iq),
     :                   'addfi_p 189',errmax,errmaxrel)
               call iso_verif_egalite_choix(pdxtfi(j,k,iq,iso_eau),
     :                   pdqfi(j,k,iq),
     :                   'addfi_p 193',errmax,errmaxrel)                
               call iso_verif_egalite_choix(pxt(j,k,iq,iso_eau),
     :                   pqprec_interm(j,k,iq),
     :                   'addfi_p 196',errmax,errmaxrel)
              endif !if (use_iso_eau.eq.1) then  
              
              if (pqprec_interm(j,k,iq).gt.ridicule) then
                 if (use_iso_HDO.eq.1) then 
                   if (iso_verif_aberrant_nostop(pxt(j,k,iq,iso_HDO)
     :                   /pqprec_interm(j,k,iq),'addfi 145').eq.1) then
                      write(*,*) ' addfi 138: j,k,iq=',j,k,iq
                      write(*,*) 'deltaDprec=',
     :                   deltaD(pxtprec(j,k,iq,iso_HDO)/pqprec(j,k,iq))
                      write(*,*) 'deltaDsuplement=',
     :                   (pdxtfi(j,k,iq,iso_HDO)
     :                           /pdqfi(j,k,iq)/tnat(iso_HDO)-1)*1000
                      write(*,*) 'pdqfi(j,k,iq)*pdt=',pdqfi(j,k,iq)*pdt
                      write(*,*) 'pqprec_interm(j,k,iq)=',
     :                   pqprec_interm(j,k,iq)
                      write(*,*) 'pqprec(j,k,iq)=',pqprec(j,k,iq)
                      write(*,*) 'ffi=',pdqfi(j,k,iq)*pdt
     :                   /(pqprec_interm(j,k,iq))
                      write(*,*) '1-ffi=',pqprec(j,k,iq)
     :                   /pqprec_interm(j,k,iq)
                      write(*,*) 'deltaDnew=',deltaD(
     :                  pxt(j,k,iq,iso_HDO)/pqprec_interm(j,k,iq))      
                      stop
                   endif !if (iso_verif_aberrant_nostop(pxt(2,j,k,iq)
                 endif ! if (use_iso_HDO) then 
              endif !if (pqprec_interm(j,k,iq).gt.ridicule) then
#ifdef ISOTRAC      
                    call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij,k,iq,'addfi 274')
#endif 

#endif  
! #ifdef ISOVERIF                    

           if (pqprec_interm(j,k,iq).lt.qtestw) then
               ! ici, on doit augmenter artificiellement le contenu en
               ! eau, c'est très génant. Pour éviter trop de probèmes
               ! numériques, lié à des deltaDs abbérants dans si peu
               ! d'eau, on prend la compo de l'eau qu'il y avait avant.
                 if (pqprec(j,k,iq).gt.ridicule) then 
                     ! c'est bon, il y avait de l'eau                         
                     do ixt=1,ntraciso
                       pxt(j,k,iq,ixt)=pxtprec(j,k,iq,ixt)
     :                      /pqprec(j,k,iq)*qtestw
                     enddo !do ixt=1,niso
#ifdef ISOTRAC      
                     ! idem pour isos pour assurer somme des
                     ! traceurs=isotope total
#endif 
#ifdef ISOVERIF
                     if (use_iso_eau.eq.1) then
                     call iso_verif_egalite_choix(pxt(j,k,iq,iso_eau),
     :                   pq(j,k,iq),'addfi 156',errmax,errmaxrel)
                     endif !if (use_iso_eau.eq.1) then
                     if (use_iso_HDO.eq.1) then
                     call iso_verif_aberrant(pxt(j,k,iq,iso_HDO)
     :                   /pq(j,k,iq),'addfi 158') 
                     endif ! if (use_iso_HDO.eq.1) then
#ifdef ISOTRAC      
                    call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij,k,iq,'addfi 320')
#endif 
#endif

                  else if (k.gt.1) then !if (pq(j,k,iq).gt.0.0) then 
                       if (pqprec(j,k-1,iq).gt.ridicule) then
                           ! c'est bon, de l'eau en dessous
                           do ixt=1,ntraciso 
                             pxt(j,k,iq,ixt)=qtestw 
     :                         * pxtprec(j,k-1,iq,ixt)/pqprec(j,k-1,iq) 
                           enddo ! do ixt=1,niso 

#ifdef ISOVERIF              
                           if (use_iso_eau.eq.1) then
                             call iso_verif_egalite_choix(
     :                          pxt(j,k,iq,iso_eau),pq(j,k,iq),
     :                          'addfi 169',errmax,errmaxrel)
                           endif !if (use_iso_eau.eq.1) then
                           if (use_iso_HDO.eq.1) then
                             call iso_verif_aberrant(pxt(j,k,iq,iso_HDO)
     :                          /pq(j,k,iq),'addfi 171')
                           endif ! if (use_iso_HDO.eq.1) then
#ifdef ISOTRAC      
                    call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij,k,iq, 'addfi 336')
#endif                           
#endif

                      else ! if (pq(j,k,iq).gt.ridicule) then 
!                           write(*,*) 'addfi 157: sans espoir: j=',
!     :                           j,'; k=',k
                           do ixt=1,niso
                             pxt(j,k,iq,ixt)=0.0
                           enddo                                
                           if (use_iso_eau.eq.1) then
                             pxt(j,k,iq,iso_eau)=qtestw 
                           endif !if (use_iso_eau.eq.1) then
#ifdef ISOTRAC      
                           if (use_iso_eau.eq.1) then
                              pxt(index_trac(izone_poubelle,iso_eau)
     :                                   ,j,k,iq)=qtestw 
                           endif !if (use_iso_eau.eq.1) then     
#endif                           
                           !stop    
                       endif !if (pq(j,k-1,iq).gt.1) then
                   else !if (pq(j,k,iq).lt.qtestw) then
!                         write(*,*) 'addfi 161: sans espoir: j=',
!     :                           j,'; k=',k
                         do ixt=1,niso
                             pxt(j,k,iq,ixt)=0.0
                         enddo  
                         if (use_iso_eau.eq.1) then   
                            pxt(j,k,iq,iso_eau)=qtestw
                         endif !if (use_iso_eau.eq.1) then
#ifdef ISOTRAC      
                           if (use_iso_eau.eq.1) then
                              pxt(index_trac(izone_poubelle,iso_eau)
     :                                   ,j,k,iq)=qtestw 
                           endif !if (use_iso_eau.eq.1) then     
#endif                             
                         !stop  
                   endif !if (pq(j,k,iq).gt.0.0) then 
                   
                   
               endif !if (pq(j,k,iq).lt.qtestw) then

              ! verif bilan:
#ifdef ISOVERIF
              if (use_iso_eau.eq.1) then
              call iso_verif_egalite_choix(pxt(j,k,iq,iso_eau),
     :              pq(j,k,iq),'addfi 198',errmax,errmaxrel)
              endif                
              if (pqprec(j,k,iq).gt.ridicule) then  
                if (use_iso_HDO.eq.1) then                
                  if (iso_verif_aberrant_nostop(pxt(j,k,iq,iso_HDO)
     :                   /pq(j,k,iq),'addfi 199').eq.1) then
                     write(*,*) 'pqprec_interm(j,k,iq)=',
     :                   pqprec_interm(j,k,iq)
                     write(*,*) 'pqprec(j,k,iq)=',pqprec(j,k,iq)
                     write(*,*) 'qtestw=',qtestw
                     write(*,*) 'deltaDprec=',deltaD(pxtprec(j,k,iq,ixt)
     :                      /pqprec(j,k,iq))
                     stop
                  endif
                endif ! if (use_iso_eau.eq.1) then
              endif !if (pq(j,k,iq).gt.ridicule) then 
#ifdef ISOTRAC      
              call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij,k,iq,'addfi 399')
#endif              
#endif
             ENDDO !DO j = ijb,ije
         ENDDO !DO k = 1,llm
      ENDDO !DO iq = 1, 2                
#endif                     
      


       DO iq = 3, nqtot
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO k = 1,llm
            DO j = ijb,ije
               pq(j,k,iq)= pq(j,k,iq) + pdqfi(j,k,iq) * pdt
               pq(j,k,iq)= AMAX1( pq(j,k,iq), qtestt )
            ENDDO
         ENDDO
c$OMP END DO NOWAIT
       ENDDO !DO iq = 3, nqtot
      else !if (planet_type=="earth") then
      ! general case, treat all tracers equally)
#ifdef ISO
      write(*,*) 'addfi 468: cas non prevu'
      stop
#endif      
       DO iq = 1, nqtot
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
         DO k = 1,llm
            DO j = ijb,ije
               pq(j,k,iq)= pq(j,k,iq) + pdqfi(j,k,iq) * pdt
               pq(j,k,iq)= AMAX1( pq(j,k,iq), qtestt )
            ENDDO
         ENDDO
c$OMP END DO NOWAIT
       ENDDO
      endif ! of if (planet_type=="earth")

c$OMP MASTER
      if (pole_nord) then
      
        DO  ij   = 1, iim
          xpn(ij) = aire(   ij   ) * pps(  ij     )
        ENDDO

        tpn      = SSUM(iim,xpn,1)/apoln

        DO ij   = 1, iip1
          pps (   ij     )  = tpn
        ENDDO
      
      endif

      if (pole_sud) then
      
        DO  ij   = 1, iim
          xps(ij) = aire(ij+ip1jm) * pps(ij+ip1jm )
        ENDDO

        tps      = SSUM(iim,xps,1)/apols

        DO ij   = 1, iip1
          pps ( ij+ip1jm )  = tps
        ENDDO
      
      endif
c$OMP END MASTER

      if (pole_nord) then
        DO iq = 1, nqtot
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
          DO  k    = 1, llm
            DO  ij   = 1, iim
              xpn(ij) = aire(   ij   ) * pq(  ij    ,k,iq)
            ENDDO
            tpn      = SSUM(iim,xpn,1)/apoln
  
            DO ij   = 1, iip1
              pq (   ij   ,k,iq)  = tpn
            ENDDO
          ENDDO
c$OMP END DO NOWAIT	  
        ENDDO
      endif
      
      if (pole_sud) then
        DO iq = 1, nqtot
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
          DO  k    = 1, llm
            DO  ij   = 1, iim
              xps(ij) = aire(ij+ip1jm) * pq(ij+ip1jm,k,iq)
            ENDDO
            tps      = SSUM(iim,xps,1)/apols
  
            DO ij   = 1, iip1
              pq (ij+ip1jm,k,iq)  = tps
            ENDDO
          ENDDO
c$OMP END DO NOWAIT	  
        ENDDO
      endif
      
#ifdef ISO
      if (pole_nord) then
        do ixt=1,ntraciso
        DO iq = 1, 2
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
          DO  k    = 1, llm
            DO  ij   = 1, iim
              xpn(ij) = aire(   ij   ) * pxt(  ij    ,k,iq,ixt)
            ENDDO
            tpn      = SSUM(iim,xpn,1)/apoln
  
            DO ij   = 1, iip1
              pxt (   ij   ,k,iq,ixt)  = tpn
            ENDDO
          ENDDO
c$OMP END DO NOWAIT	  
        ENDDO
        enddo !do ixt=1,ntraciso
      endif !if (pole_nord) then
      
      if (pole_sud) then
        do ixt=1,ntraciso
        DO iq = 1, 2
c$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
          DO  k    = 1, llm
            DO  ij   = 1, iim
              xps(ij) = aire(ij+ip1jm) * pxt(ij+ip1jm,k,iq,ixt)
            ENDDO
            tps      = SSUM(iim,xps,1)/apols
  
            DO ij   = 1, iip1
              pxt (ij+ip1jm,k,iq,ixt)  = tps
            ENDDO
          ENDDO
c$OMP END DO NOWAIT	  
        ENDDO
        enddo !do ixt=1,ntraciso
      endif !if (pole_sud) then
#ifdef ISOVERIF
      DO iq = 1, 2
        DO  k    = 1, llm
          DO  ij   = 1, iim
              if (use_iso_eau.eq.1) then
                if (pole_nord) then
                call iso_verif_egalite_choix(pxt(ij,k,iq,iso_eau),
     :                   pq(ij,k,iq),'addfi 200',errmax,errmaxrel)
                endif !if (pole_nord) then
                if (pole_sud) then
                call iso_verif_egalite_choix(pxt(ij+ip1jm,k,iq,iso_eau)
     :                  ,pq(ij+ip1jm,k,iq),
     :                   'addfi 201',errmax,errmaxrel)
                endif !if (pole_sud) then
              endif !if ((use_iso_eau.eq.1).and.(ixt.eq.iso_eau)) then
              if (use_iso_HDO.eq.1) then
                  if (pole_nord) then
                  if (pq(ij,k,iq).gt.ridicule) then
                    call iso_verif_aberrant(pxt(ij,k,iq,iso_HDO)
     :                  /pq(ij,k,iq), 'addfi 202')
                  endif
                  endif
                  if (pole_sud) then
                  if (pq(ij,k,iq).gt.ridicule) then
                    call iso_verif_aberrant(pxt(ij+ip1jm,k,iq,iso_HDO)
     :                  /pq(ij+ip1jm,k,iq), 'addfi 203')
                  endif
                  endif !if (pole_sud) then
              endif
#ifdef ISOTRAC      
              if (pole_nord) then
              call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij,k,iq,'addfi 617')
              endif !if (pole_sud) then   
              if (pole_sud) then
              call iso_verif_traceur_retourne3D(pxt,ip1jmp1,llm,2,
     :                   ij+ip1jm,k,iq,'addfi 495')
              endif !if (pole_sud) then
#endif              
           ENDDO !DO ij   = 1, iip1   
        ENDDO !DO  k    = 1, llm
      ENDDO !DO iq = 1, 2   
#endif
#endif      
      
      RETURN
      END
