!
! $Id: fisrtilp.F90 1575 2011-09-21 13:57:48Z jghattas $
!
!
SUBROUTINE fisrtilp(dtime,paprs,pplay,t,q,ptconv,ratqs, &
     d_t, d_q, d_ql, rneb, radliq, rain, snow,          &
     pfrac_impa, pfrac_nucl, pfrac_1nucl,               &
     frac_impa, frac_nucl, beta,                        &
     prfl, psfl, rhcl, zqta, fraca,                     &
     ztv, zpspsk, ztla, zthl, iflag_cldcon,             &
     flag_ss, flagb_ss, rneb_ss, clsk_ss                &
#ifdef ISO
     &           ,xt,xtrain,xtsnow                      &
     &           ,d_xt,d_xtl,pxtrainfl,pxtsnowfl        &
#ifdef DIAGISO      
     &           ,fcond_ls,taux_cond_ls,taux_precip_ls  & ! juste diagnostique
#endif      
#endif      
     &  )

  !
  USE dimphy
  IMPLICIT none
  !======================================================================
  ! Auteur(s): Z.X. Li (LMD/CNRS)
  ! Date: le 20 mars 1995
  ! Objet: condensation et precipitation stratiforme.
  !        schema de nuage
  !======================================================================
  !======================================================================
  !ym include "dimensions.h"
  !ym include "dimphy.h"
  include "YOMCST.h"
  include "tracstoke.h"
  include "fisrtilp.h"
  include "iniprint.h"

  !
  ! Arguments:
  !
  REAL dtime ! intervalle du temps (s)
  REAL paprs(klon,klev+1) ! pression a inter-couche
  REAL pplay(klon,klev) ! pression au milieu de couche
  REAL t(klon,klev) ! temperature (K)
  REAL q(klon,klev) ! humidite specifique (kg/kg)
  REAL d_t(klon,klev) ! incrementation de la temperature (K)
  REAL d_q(klon,klev) ! incrementation de la vapeur d'eau
  REAL d_ql(klon,klev) ! incrementation de l'eau liquide
  REAL rneb(klon,klev) ! fraction nuageuse
  REAL radliq(klon,klev) ! eau liquide utilisee dans rayonnements
  REAL rhcl(klon,klev) ! humidite relative en ciel clair
  REAL rain(klon) ! pluies (mm/s)
  REAL snow(klon) ! neige (mm/s)
  REAL prfl(klon,klev+1) ! flux d'eau precipitante aux interfaces (kg/m2/s)
  REAL psfl(klon,klev+1) ! flux d'eau precipitante aux interfaces (kg/m2/s) 
  REAL ztv(klon,klev)
  REAL zqta(klon,klev),fraca(klon,klev) 
  REAL sigma1(klon,klev),sigma2(klon,klev)
  REAL qltot(klon,klev),ctot(klon,klev)
  REAL zpspsk(klon,klev),ztla(klon,klev)
  REAL zthl(klon,klev)

  logical lognormale(klon)

#ifdef ISO
#include "wateriso2F90.h"
      real double_to_real
      double precision real_to_double
      !integer niso
      real xt(ntraciso,klon,klev)
      real d_xt(ntraciso,klon,klev)
      real d_xtl(ntraciso,klon,klev)
      real xtrain(ntraciso,klon)
      real xtsnow(ntraciso,klon)
      real pxtrainfl(ntraciso,klon,klev+1)
      real pxtsnowfl(ntraciso,klon,klev+1)      
      ! xt <-> q
      ! xtrain et xtsonw <-> rain et snow
      ! d_xt et d_xtl <-> d_q et d_ql
      ! pxtrainfl et pxtsnowflux <-> prfl et psfl
#ifdef DIAGISO
      real fcond_ls(klon,klev)
      real taux_cond_ls(klon,klev)
      real taux_precip_ls(klon,klev)
#endif
#endif

  !AA
  ! Coeffients de fraction lessivee : pour OFF-LINE
  !
  REAL pfrac_nucl(klon,klev)
  REAL pfrac_1nucl(klon,klev)
  REAL pfrac_impa(klon,klev)
  !
  ! Fraction d'aerosols lessivee par impaction et par nucleation
  ! POur ON-LINE
  !
  REAL frac_impa(klon,klev)
  REAL frac_nucl(klon,klev)
  real zct      ,zcl
  !AA
  !
  ! Options du programme:
  !
  REAL seuil_neb ! un nuage existe vraiment au-dela
  PARAMETER (seuil_neb=0.001)

  INTEGER ninter ! sous-intervals pour la precipitation
  INTEGER ncoreczq  
  INTEGER iflag_cldcon
  PARAMETER (ninter=5)
  LOGICAL evap_prec ! evaporation de la pluie
  PARAMETER (evap_prec=.TRUE.)
  REAL ratqs(klon,klev) ! determine la largeur de distribution de vapeur
  logical ptconv(klon,klev) ! determine la largeur de distribution de vapeur

  real zpdf_sig(klon),zpdf_k(klon),zpdf_delta(klon)
  real Zpdf_a(klon),zpdf_b(klon),zpdf_e1(klon),zpdf_e2(klon)
  real erf   
  REAL qcloud(klon)
  !
  LOGICAL cpartiel ! condensation partielle
  PARAMETER (cpartiel=.TRUE.)
  REAL t_coup
  PARAMETER (t_coup=234.0)

  ! pour la sursat de Marine/Bernard
  real zpdf_beta(klon)
  real RH_crit, pi
  REAL rhcl_p0, rhcl_p1, rhcl_p2, rhcl_p
  real zpdf_delta_o(klon), zpdf_a_o(klon), zpdf_b_o(klon)
  real zpdf_e1_o(klon)
  real rneb_o(klon,klev)
  real flag_ss(klon,klev),clsk_ss(klon,klev),rneb_ss(klon,klev)
  real flagb_ss(klon,klev)

  !
  ! Variables locales:
  !
  INTEGER i, k, n, kk
  REAL zqs(klon), zdqs(klon), zdelta, zcor, zcvm5   
  REAL zrfl(klon), zrfln(klon), zqev, zqevt
  REAL zoliq(klon), zcond(klon), zq(klon), zqn(klon), zdelq
  REAL ztglace, zt(klon)
  INTEGER nexpo ! exponentiel pour glace/eau
  REAL zdz(klon),zrho(klon),ztot      , zrhol(klon)
  REAL zchau      ,zfroi      ,zfice(klon),zneb(klon)

  ! sursat de Marine/Bernard   
  REAL zqs_star(klon), ys(klon), yss(klon)

#ifdef ISO
      real zxtrfl(ntraciso,klon)
      real zxtrfln(ntraciso,klon)
      real zxt(ntraciso,klon) ! equivalent local de xt
      real zxtn(ntraciso,klon) ! isotpes nuageux
      real zxtn_reduit(niso)
      real zxtcond(ntraciso,klon)
      real zxtoliq(ntraciso,klon)
      real zq_ancien(klon)
      integer ixt
#include "iso_verif.h"
#ifdef ISOVERIF   
      integer iso_verif_aberrant_nostop
      integer iso_verif_egalite_choix_nostop
      integer iso_verif_positif_nostop
      integer iso_verif_positif_choix_nostop
      integer trace_cas(klon)
      integer iso_verif_traceur_nostop
#endif      
      integer iso_verif_nonan_nostop
      real zxtice(ntraciso,klon),zxtliq(ntraciso,klon)
      real zrfl_ancien(klon)
      real zqev_diag(klon)
      real zxtrfl_ancien(ntraciso,klon)
      real zxt_ancien(ntraciso,klon)
#ifdef ISOTRAC
        real zxtres(ntraciso) ! humidite spec nuageuse aprs condensation et precip
        real zqcs(klon) ! humidite spec en ciel clair
        real zxtcs(ntraciso,klon)        
        real zcondn(klon) ! humidite spec de l'eau liquide dans le nuage
        real zxtcondn(ntraciso,klon)
        integer izone
#ifdef ISOVERIF           
        real zq_verif
#endif        
#endif 
!      logical negation ! pour nier conditions logiques
#endif
  !
  LOGICAL appel1er
  SAVE appel1er
  !$OMP THREADPRIVATE(appel1er)
  !
  !---------------------------------------------------------------
  !
  !AA Variables traceurs:
  !AA  Provisoire !!! Parametres alpha du lessivage
  !AA  A priori on a 4 scavenging # possibles
  !
  REAL a_tr_sca(4)
  save a_tr_sca
  !$OMP THREADPRIVATE(a_tr_sca)
  !
  ! Variables intermediaires
  !
  REAL zalpha_tr
  REAL zfrac_lessi
  REAL zprec_cond(klon)
  !AA
! RomP >>> 15 nov 2012
  REAL   beta(klon,klev) ! taux de conversion de l'eau cond
! RomP <<<
  REAL zmair, zcpair, zcpeau
  !     Pour la conversion eau-neige
  REAL zlh_solid(klon), zm_solid
  !IM 
  !ym      INTEGER klevm1
  !---------------------------------------------------------------
  !
  ! Fonctions en ligne:
  !
  REAL fallvs,fallvc ! vitesse de chute pour crystaux de glace
  REAL zzz
  include "YOETHF.h"
  include "FCTTRE.h"
  fallvc (zzz) = 3.29/2.0 * ((zzz)**0.16) * ffallv_con
  fallvs (zzz) = 3.29/2.0 * ((zzz)**0.16) * ffallv_lsc
  !
  DATA appel1er /.TRUE./
  !ym
  zdelq=0.0

  if (prt_level>9)write(lunout,*)'NUAGES4 A. JAM'
  IF (appel1er) THEN
     !
     WRITE(lunout,*) 'fisrtilp, ninter:', ninter
     WRITE(lunout,*) 'fisrtilp, evap_prec:', evap_prec
     WRITE(lunout,*) 'fisrtilp, cpartiel:', cpartiel
     WRITE(lunout,*) 'fisrtilp, dtime:',dtime
     IF (ABS(dtime/REAL(ninter)-360.0).GT.0.001) THEN
        WRITE(lunout,*) 'fisrtilp: Ce n est pas prevu, voir Z.X.Li', dtime
        WRITE(lunout,*) 'Je prefere un sous-intervalle de 6 minutes'
        !         CALL abort
     ENDIF
     appel1er = .FALSE.

#ifdef ISO
        ! cam verif 
#ifdef ISOVERIF
          do k=1,klev
           do i=1,klon             
            if (use_iso_eau.eq.1) then        
             call iso_verif_egalite_choix(xt(iso_eau,i,k),q(i,k), &   
     &                   'il pleut 205',errmax,errmaxrel)
            endif !if (use_iso_eau.eq.1) then
            if (use_iso_HDO.eq.1) then    
             if (q(i,k).gt.ridicule) then
               call iso_verif_aberrant(xt(iso_HDO,i,k)/q(i,k), &
     &           'il pleut 210')
             endif !if (zq(i).gt.ridicule) then
            endif  !if (use_iso_HDO.eq.1) then
           enddo !do k=1,klev
          enddo !do i=1,klon
#endif
#ifdef ISOVERIF
        do k=1,klev
           do i=1,klon
             call iso_verif_positif(370.0-t(i,k),'il pleut 250')
             call iso_verif_positif(t(i,k)-100.0,'il pleut 251')
           enddo
        enddo
#endif
         ! end cam verif
#endif 
     !
     !AA initialiation provisoire
     a_tr_sca(1) = -0.5
     a_tr_sca(2) = -0.5
     a_tr_sca(3) = -0.5
     a_tr_sca(4) = -0.5
     !
     !AA Initialisation a 1 des coefs des fractions lessivees 
     !
     !cdir collapse
     DO k = 1, klev
        DO i = 1, klon
           pfrac_nucl(i,k)=1.
           pfrac_1nucl(i,k)=1.
           pfrac_impa(i,k)=1.
           beta(i,k)=0.  !RomP initialisation

        flag_ss(i,k) = 0.
        clsk_ss(i,k) = 0.
        rneb_ss(i,k) = 0.
        flagb_ss(i,k) = 0.
        ENDDO
     ENDDO

  ENDIF          !  test sur appel1er
  !
  !MAf Initialisation a 0 de zoliq
  !      DO i = 1, klon
  !         zoliq(i)=0.
  !      ENDDO 
  ! Determiner les nuages froids par leur temperature
  !  nexpo regle la raideur de la transition eau liquide / eau glace.
  !
  ztglace = RTT - 15.0
  nexpo = 6
  !cc      nexpo = 1
  !
  ! Initialiser les sorties:
  !
  !cdir collapse
  DO k = 1, klev+1
     DO i = 1, klon
        prfl(i,k) = 0.0
        psfl(i,k) = 0.0
#ifdef ISO                 
          do ixt=1,ntraciso
            pxtrainfl(ixt,i,k)=0.0
            pxtsnowfl(ixt,i,k)=0.0
          enddo !do ixt=1,niso
#endif
     ENDDO !DO i = 1, klon
  ENDDO !DO k = 1, klev+1     

  !cdir collapse
  DO k = 1, klev
     DO i = 1, klon
        d_t(i,k) = 0.0
        d_q(i,k) = 0.0
        d_ql(i,k) = 0.0
#ifdef ISO
#ifdef DIAGISO         
          fcond_ls(i,k)=0.0
#endif  
          do ixt=1,ntraciso
            d_xt(ixt,i,k) = 0.0
            d_xtl(ixt,i,k) = 0.0
          enddo !do ixt=1,niso
#endif         
        rneb(i,k) = 0.0
        radliq(i,k) = 0.0
        frac_nucl(i,k) = 1. 
        frac_impa(i,k) = 1. 
     ENDDO !DO i = 1, klon
  ENDDO !DO k = 1, klev
  DO i = 1, klon
     rain(i) = 0.0
     snow(i) = 0.0
     zoliq(i)=0.
#ifdef ISO
         do ixt=1,ntraciso
            xtrain(ixt,i) = 0.0
            xtsnow(ixt,i) = 0.0
            zxtoliq(ixt,i)=0.
         enddo !do ixt=1,niso
#endif         
        ! temporaire:
!        write(*,*) 'ilp tmp 345: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)
     !     ENDDO
     !
     ! Initialiser le flux de precipitation a zero
     !
     !     DO i = 1, klon
     zrfl(i) = 0.0
#ifdef ISO
          if (essai_convergence) then
          else
            zrfln(i)=0.0
          endif
          do ixt=1,ntraciso
            zxtrfl(ixt,i)=0.0
            zxtrfln(ixt,i)=0.0
          enddo
#endif         
     zneb(i) = seuil_neb
  ENDDO !DO i = 1, klon
  !
  !
  !AA Pour plus de securite 

  zalpha_tr   = 0.
  zfrac_lessi = 0.

  !AA----------------------------------------------------------
  !
  ncoreczq=0
  ! Boucle verticale (du haut vers le bas)
  !
  !IM : klevm1
  !ym      klevm1=klev-1
  DO k = klev, 1, -1
     !
      
#ifdef ISO      
      ! cam debug!      
       ! verif en debut de boucle
#ifdef ISOVERIF
!       write(*,*) 'boucle en k: k=',k         
       do i=1,klon
        if (use_iso_eau.eq.1) then
        call iso_verif_egalite_choix(zxtrfl(iso_eau,i), &
     &           zrfl(i), &
     &           'il pleut 310',errmax,errmaxrel)
        endif !if (use_iso_eau.eq.1) then
        if (use_iso_HDO.eq.1) then  
         if (zrfl(i).gt.ridicule_rain) then
             call iso_verif_aberrant(zxtrfl(iso_HDO,i) &
     &           /zrfl(i),'il pleut 316')
         endif !if (zrfl(i).gt.ridicule_rain) then
        endif  !if (use_iso_HDO.eq.1) then  
#ifdef ISOTRAC   
        call iso_verif_traceur(zxtrfl(1,i), &
     &          'il pleut 365')   
        call iso_verif_traceur_pbidouille(zxtrfl(1,i), &
     &          'il pleut 388')
#endif         
       enddo !do i=1,klon
#endif
#ifdef ISOVERIFNAN   
!        write(*,*) 'ilp tmp 412: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)      
       do i=1,klon
         do ixt=1,ntraciso
           call iso_verif_noNaN(zxtrfl(ixt,i),'ilp 406')
           call iso_verif_noNaN(zxtoliq(ixt,i),'ilp 407')
         enddo !do ixt=1,ntraciso
       enddo !do i=1,klon
#endif
      ! end verif en debut de boucle
      ! end cam debug
#endif    
     !AA----------------------------------------------------------
     !
     DO i = 1, klon
        zt(i)=t(i,k)
        zq(i)=q(i,k)
#ifdef VERIFNEGATIF
        if (q(i,k).lt.0.0) then
               write(*,*) 'ilp 400: avant clmain 31janv: i,k,q=', &
     &           i,k,q(i,k)
               stop
        endif 
#endif
#ifdef ISO
         if (essai_convergence) then
         else            
           q(i,k)=max(0.0,q(i,k))
           zq(i)=max(0.0,zq(i))
         endif
#ifdef ISOTRAC
#ifdef ISOVERIF         
          call iso_verif_traceur(xt(1,i,k),'ilp 404')
!          write(*,*) 'ilp 402 temporaire: xt(1:ntraciso,i,k)=', &
!     &          xt(1:ntraciso,i,k)
#endif          
#endif          
         do ixt=1,ntraciso
           xt(ixt,i,k)=max(xt(ixt,i,k),0.0) 
         enddo !do ixt=1,niso
#ifdef ISOTRAC
#ifdef ISOVERIF            
          call iso_verif_traceur_pbidouille(xt(1,i,k),'ilp 407')
#endif             
#endif           
         do ixt=1,ntraciso
           zxt(ixt,i)=xt(ixt,i,k)
         enddo !do ixt=1,niso  

         ! cam verif
#ifdef ISOVERIF
             call iso_verif_positif(370.0-zt(i),'il pleut 425')
             call iso_verif_positif(zt(i)-100.0,'il pleut 426')
          if (use_iso_eau.eq.1) then        
            call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &                   'il pleut 289',errmax,errmaxrel)
          endif !if (use_iso_eau.eq.1) then
          if (use_iso_HDO.eq.1) then    
             if (zq(i).gt.ridicule) then
               call iso_verif_aberrant(zxt(iso_HDO,i)/zq(i), &
     &           'il pleut 337')
             endif !if (zq(i).gt.ridicule) then
          endif  !if (use_iso_HDO.eq.1) then  
#ifdef ISOTRAC
          call iso_verif_traceur(zxt(1,i),'ilp 431')
#endif           
#endif
         ! end cam verif
#endif            
     ENDDO
     !
     ! Calculer la varition de temp. de l'air du a la chaleur sensible
     ! transporter par la pluie.
     ! Il resterait a rajouter cet effet de la chaleur sensible sur les
     ! flux de surface, du a la diff. de temp. entre le 1er niveau et la
     ! surface.
     !
     IF(k.LE.klevm1) THEN         
        DO i = 1, klon
           !IM
           zmair=(paprs(i,k)-paprs(i,k+1))/RG
           zcpair=RCPD*(1.0+RVTMP2*zq(i))
           zcpeau=RCPD*RVTMP2
           zt(i) = ( (t(i,k+1)+d_t(i,k+1))*zrfl(i)*dtime*zcpeau &
                + zmair*zcpair*zt(i) ) &
                / (zmair*zcpair + zrfl(i)*dtime*zcpeau)
           !     C        WRITE (6,*) 'cppluie ', zt(i)-(t(i,k+1)+d_t(i,k+1))
        ENDDO
     ENDIF
     !
     !
     ! Calculer l'evaporation de la precipitation
     !



#ifdef ISOVERIF
!        write(*,*) 'ilp tmp 464'
           do i=1,klon
             call iso_verif_positif(370.0-zt(i),'il pleut 468')
             call iso_verif_positif(zt(i)-100.0,'il pleut 469')
           enddo
#endif    
!        write(*,*) 'ilp tmp 507: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)     
!
!
! Calculer l''evaporation de la precipitation
!
#ifdef ISO
      DO i = 1, klon
         zq_ancien(i)=zq(i)
         zqev_diag(i)=0.0
         zrfl_ancien(i)=zrfl(i)
         do ixt=1,ntraciso
           zxtrfl_ancien(ixt,i)=zxtrfl(ixt,i)
           zxt_ancien(ixt,i)=zxt(ixt,i)
         enddo 
      enddo   ! DO i = 1, klon  
#ifdef ISOVERIF
      if (use_iso_eau.eq.1) then
       DO i = 1, klon           
          call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &           'il pleut 426a',errmax,errmaxrel) 
          call iso_verif_egalite_choix(zxtrfl_ancien(iso_eau,i), &
     &            zrfl_ancien(i), &
     &           'il pleut 426b',errmax,errmaxrel)
        enddo
      endif
#ifdef ISOTRAC
       do i=1,klon      
            call iso_verif_traceur(zxt_ancien(1,i),'ilp 532')
            call iso_verif_traceur(zxtrfl_ancien(1,i), &
     &           'ilp 533')  
       enddo !do i=1,klon  
#endif      
#endif      
#endif

     IF (evap_prec) THEN
        DO i = 1, klon
           IF (zrfl(i) .GT.0.) THEN
              IF (thermcep) THEN
                 zdelta=MAX(0.,SIGN(1.,RTT-zt(i)))
                 zqs(i)= R2ES*FOEEW(zt(i),zdelta)/pplay(i,k)
                 zqs(i)=MIN(0.5,zqs(i))
                 zcor=1./(1.-RETV*zqs(i))
                 zqs(i)=zqs(i)*zcor
              ELSE
                 IF (zt(i) .LT. t_coup) THEN
                    zqs(i) = qsats(zt(i)) / pplay(i,k)
                 ELSE
                    zqs(i) = qsatl(zt(i)) / pplay(i,k)
                 ENDIF
              ENDIF
              zqev = MAX (0.0, (zqs(i)-zq(i))*zneb(i) )
              zqevt = coef_eva * (1.0-zq(i)/zqs(i)) * SQRT(zrfl(i)) &
                   * (paprs(i,k)-paprs(i,k+1))/pplay(i,k)*zt(i)*RD/RG
              zqevt = MAX(0.0,MIN(zqevt,zrfl(i))) &
                   * RG*dtime/(paprs(i,k)-paprs(i,k+1))
              zqev = MIN (zqev, zqevt)
              zrfln(i) = zrfl(i) - zqev*(paprs(i,k)-paprs(i,k+1)) &
                   /RG/dtime

#ifdef ISO
         zqev_diag(i)=zqev
#ifdef ISOVERIF         
         if (iso_verif_positif_nostop(zrfln(i), &
     &           'ilp 477: vrif bilan masse').eq.1) then
            if (zrfln(i).lt.-ridicule*1e3) then
                stop
            endif
         endif
!         if (i.eq.351) then
!             write(*,*) 'ilp 480, temp:'
!             write(*,*) 'zqev,zrfln(i),zrfl(i)=',zqev,zrfln(i),zrfl(i)
!             write(*,*) 'fac_ftmr=',RG*dtime/(paprs(i,k)-paprs(i,k+1))
!         endif
#endif  
#endif         

#ifdef ISO
         if (essai_convergence) then
         else
            zrfln(i)=max(0.0,zrfln(i))
         endif
#endif       
              ! pour la glace, on r-vapore toute la prcip dans la
              ! couche du dessous
              ! la glace venant de la couche du dessus est simplement
              ! dans la couche du dessous.

              IF (zt(i) .LT. t_coup.and.reevap_ice) zrfln(i)=0.

              zq(i) = zq(i) - (zrfln(i)-zrfl(i)) &
                   * (RG/(paprs(i,k)-paprs(i,k+1)))*dtime
              zt(i) = zt(i) + (zrfln(i)-zrfl(i)) &
                   * (RG/(paprs(i,k)-paprs(i,k+1)))*dtime &
                   * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
              zrfl(i) = zrfln(i)
           ENDIF
        ENDDO
#ifdef ISO
      ! * traitement de l'vaporation
      call iso_revap_fisrtilp(klon,klev,k, &
     &            zrfl_ancien,zrfl,zrfln,zt,zxt_ancien, &
     &            zxtrfl,zxtrfl_ancien,zxtrfln,zxt, &
     &            paprs,dtime, &
     &            zqs,zq_ancien,zqev_diag,zq)      
#endif

      ENDIF ! if evap_prec
     !
     ! Calculer Qs et L/Cp*dQs/dT:
#ifdef ISOVERIF
           do i=1,klon
             call iso_verif_positif(370.0-zt(i),'il pleut 938')
             call iso_verif_positif(zt(i)-100.0,'il pleut 939')
             do ixt=1,ntraciso
              call iso_verif_positif_choix(zxt(ixt,i),0.0,'ilp 614')
             enddo
           enddo
#ifdef ISOTRAC
!      write(*,*) 'ilp tmp 633'
           do i=1,klon
              call iso_verif_traceur(zxtrfl(1,i),'il pleut 632')
           enddo
!      write(*,*) 'ilp tmp 637,thermcep'
#endif           
#endif    
#ifdef ISOVERIFNAN
           do i=1,klon
            do ixt=1,ntraciso
             call iso_verif_noNaN(zxtrfl(ixt,i),'ilp 624')
            enddo !do ixt=1,ntraciso
           enddo ! do i=1,klon
!        write(*,*) 'ilp tmp 638: k,zoliq,zxtoliq(1,363)=',k,zoliq(363),zxtoliq(1,363)
#endif
     !
     IF (thermcep) THEN
        DO i = 1, klon
           zdelta = MAX(0.,SIGN(1.,RTT-zt(i)))
           zcvm5 = R5LES*RLVTT*(1.-zdelta) + R5IES*RLSTT*zdelta
           zcvm5 = zcvm5 /RCPD/(1.0+RVTMP2*zq(i))
           zqs(i) = R2ES*FOEEW(zt(i),zdelta)/pplay(i,k)
           zqs(i) = MIN(0.5,zqs(i))
           zcor = 1./(1.-RETV*zqs(i))
           zqs(i) = zqs(i)*zcor
           zdqs(i) = FOEDE(zt(i),zdelta,zcvm5,zqs(i),zcor)
        ENDDO !DO i = 1, klon
     ELSE !IF (thermcep) THEN
        DO i = 1, klon
           IF (zt(i).LT.t_coup) THEN
              zqs(i) = qsats(zt(i))/pplay(i,k)
              zdqs(i) = dqsats(zt(i),zqs(i))
           ELSE
              zqs(i) = qsatl(zt(i))/pplay(i,k)
              zdqs(i) = dqsatl(zt(i),zqs(i))
           ENDIF
        ENDDO !DO i = 1, klon
     ENDIF !IF (thermcep) THEN
     !
     ! Determiner la condensation partielle et calculer la quantite
     ! de l'eau condensee:
     !

     IF (cpartiel) THEN

        !        print*,'Dans partiel k=',k
        !
        !   Calcul de l'eau condensee et de la fraction nuageuse et de l'eau
        !   nuageuse a partir des PDF de Sandrine Bony.
        !   rneb  : fraction nuageuse
        !   zqn   : eau totale dans le nuage
        !   zcond : eau condensee moyenne dans la maille.
        !  on prend en compte le rchauffement qui diminue la partie
        ! condensee
        !
        !   Version avec les raqts

        if (ok_sursat_marine.eq.1) then
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Marine: c'est ici qu'intervient la sursaturation
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Calcul de zqs_star, seuil critique de sursaturation

        
        DO i = 1, klon

        IF (zt(i) .LT. t_coup) THEN
        CALL thresh_supersat(zt(i),RH_crit)
        zqs_star(i) = zqs(i)*RH_crit
        zpdf_beta(i)  = RH_crit
        ELSE
        RH_crit = 1.
        zqs_star(i) = zqs(i)
        zpdf_beta(i)  = RH_crit
        ENDIF

        if (zqs(i) .gt. zqs_star(i)) then
        write(*,*) 'ATTENTION zqs > zqs_star!!'
        write(*,*) 'zqs zqs_star RH_crit=', &
     & zqs(i), zqs_star(i), RH_crit
        STOP
        endif !if (zqs(i) .gt. zqs_star(i)) then

        ENDDO !DO i = 1, klon
        endif !if (sursat_marine.eq.1) then

#ifdef ISO          
#ifdef ISOVERIF
        do i=1,klon
          if (use_iso_eau.eq.1) then    
            if (iso_verif_egalite_choix_nostop(zxt(iso_eau,i),zq(i), &
     &            'il pleut 608',errmax,errmaxrel).eq.1) then
              write(*,*) 'i=',i
              stop
            endif
          endif !if (use_iso_eau.eq.1) then
#ifdef ISOTRAC
          call iso_verif_traceur(zxt(1,i),'il pleut 1106')
#endif          
        enddo !do i=1,klon        
#endif
#endif        

        if (iflag_pdf.eq.0) then

           do i=1,klon
              zdelq = min(ratqs(i,k),0.99) * zq(i)
              rneb(i,k) = (zq(i)+zdelq-zqs(i)) / (2.0*zdelq)
              zqn(i) = (zq(i)+zdelq+zqs(i))/2.0
           enddo !do i=1,klon

        if (ok_sursat_marine.eq.1) then
         DO i=1,klon
           IF (rneb(i,k) .LE. 0.0) THEN
              rhcl(i,k)=zq(i)/zqs(i)
           ELSE IF (rneb(i,k) .GE. 1.0) THEN
              rhcl(i,k)=1.0
           ELSE
              rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
           ENDIF
         ENDDO
        endif !if (ok_sursat_marine.eq.1) then
           
#ifdef ISO
            ! Hypothse simplificatrice mais peut-tre fausse: 
            ! eau nuageuse a mme composition que eau de la maille
            ! verif 
#ifdef ISOVERIF
            do i=1,klon
              if (use_iso_eau.eq.1) then
                call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &                  'il pleut 618',errmax,errmaxrel)
              endif !if (use_iso_eau.eq.1) then
           enddo !do i=1,klon   
#endif
            ! end verif
            do i=1,klon
            do ixt=1,ntraciso
              if (zq(i).gt.0) then
               zxtn(ixt,i)=zqn(i)*(zxt(ixt,i)/zq(i))
               zxtn(ixt,i)=max(0.0,zxtn(ixt,i))
              else
               zxtn(ixt,i)=0.
              endif                  
            enddo !do ixt=1,niso
            enddo !do i=1,klon

            ! verif 
#ifdef ISOVERIF
            do i=1,klon
              if (use_iso_eau.eq.1) then
               call iso_verif_egalite_choix(zxtn(iso_eau,i),zqn(i), &   
     &                'il pleut 625',errmax,errmaxrel)
              endif !if (use_iso_eau.eq.1) then
            enddo !do i=1,klon
#endif 
            ! end verif
            
#endif            
!           write(*,*) 'il pleut 641: fin des calculs de zqn et zxtn'           
!           write(*,*) 'iflag_pdf=',iflag_pdf
#ifdef ISOTRAC
           if (option_tmin.eq.1) then
               write(*,*) 'option_tmin pas encore prevue ici.'
               write(*,*) 'seulement prevue dans iflag_pdf=1'
               write(*,*) 'a programmer'
               stop
           endif !if (option_tmin.eq.1) then
#endif

        else !if (iflag_pdf.eq.0) then

#ifdef ISO             
             ! verif
#ifdef ISOVERIF
             do i=1,klon
             if (use_iso_eau.eq.1) then
             call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &          'il pleut 655',errmax,errmaxrel)
             endif !if (use_iso_eau) then
#ifdef ISOTRAC
             call iso_verif_traceur(zxt(1,i),'il pleut 1164')
#endif             
             enddo !do i=1,klon                   
!             write(*,*) 'ilp tmp 753'
#endif
             ! end verif
#endif     

           !   Version avec les nouvelles PDFs.
           do i=1,klon
              if(zq(i).lt.1.e-15) then
                 ncoreczq=ncoreczq+1
#ifdef ISO
                zq_ancien(i)=zq(i)
#endif                
                 zq(i)=1.e-15

#ifdef ISO
                ! on conserve le mme rapport pour les isotopes
                if (zq_ancien(i).gt.0.0) then
                   do ixt=1,ntraciso
                     zxt(ixt,i)=zxt(ixt,i)/zq_ancien(i)*zq(i)
                     zxt(ixt,i)=max(0.0,zxt(ixt,i))
                   enddo   
                else !if (zq_ancien(i).gt.0.0) then
                    do ixt=1,ntraciso
                     zxt(ixt,i)=0.0
                   enddo 
                endif !if (zq_ancien(i).gt.0.0) then

                ! verif
#ifdef ISOVERIF
                  if (use_iso_eau.eq.1) then
                    call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &                 'il pleut 654',errmax,errmaxrel)
                  endif !if (use_iso_eau.eq.1) then
#ifdef ISOTRAC
             call iso_verif_traceur(zxt(1,i),'il pleut 1207')
#endif                    
#endif
                ! end verif
#endif      
              endif !if(zq(i).lt.1.e-15) then
           enddo ! do i=1,klon

           if (iflag_cldcon>=5) then

              call cloudth(klon,klev,k,ztv, &
                   zq,zqta,fraca, &
                   qcloud,ctot,zpspsk,paprs,ztla,zthl, &
                   ratqs,zqs,t)

              do i=1,klon
                 rneb(i,k)=ctot(i,k)
                 zqn(i)=qcloud(i)

                if (ok_sursat_marine.eq.1) then
                ! on prend le rhcl du cas creneau 
                 
                  IF (rneb(i,k) .LE. 0.0) THEN
                        rhcl(i,k)=zq(i)/zqs(i)
                  ELSE IF (rneb(i,k) .GE. 1.0) THEN
                        rhcl(i,k)=1.0 
                  ELSE
                        rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
                  ENDIF
                endif !if (ok_sursat_marine.eq.1) then
           ! if (ok_sursat_marine.eq.1) then

              enddo !do i=1,klon

           endif !if (iflag_cldcon>=5) then

           if (iflag_cldcon <= 4) then
              lognormale = .true.
           elseif (iflag_cldcon >= 6) then
              ! lognormale en l'absence des thermiques
              lognormale = fraca(:,k) < 1e-10
           else
              ! Dans le cas iflag_cldcon=5, on prend systmatiquement la
              ! bi-gaussienne
              lognormale = .false.
           end if     

           do i=1,klon
              if (lognormale(i)) then
                 zpdf_sig(i)=ratqs(i,k)*zq(i)
                 zpdf_k(i)=-sqrt(log(1.+(zpdf_sig(i)/zq(i))**2))
                 
                 if (ok_sursat_marine.eq.1) then 
                   ys(i) = - (log(zqs(i)/zq(i)))/zpdf_k(i) - zpdf_k(i)/2.
                   yss(i) = ys(i) - log(zpdf_beta(i))/zpdf_k(i)

                   if (yss(i) .lt. ys(i)) then
                     write(*,*) 'attention yss < ys !!'
                     STOP
                   endif
                 endif !if (ok_sursat_marine.eq.1) then 

                 if ((ok_sursat_marine.eq.0).or. &
     &                 (zqs(i) .eq. zqs_star(i))) then
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Cas ou il n'y a pas sursaturation
! On conserve les formules d'origine 
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc    
          
                 zpdf_delta(i)=log(zq(i)/zqs(i))
                 zpdf_a(i)=zpdf_delta(i)/(zpdf_k(i)*sqrt(2.))
                 zpdf_b(i)=zpdf_k(i)/(2.*sqrt(2.))
                 zpdf_e1(i)=zpdf_a(i)-zpdf_b(i)
                 zpdf_e1(i)=sign(min(abs(zpdf_e1(i)),5.),zpdf_e1(i))
                 zpdf_e1(i)=1.-erf(zpdf_e1(i))
                 zpdf_e2(i)=zpdf_a(i)+zpdf_b(i)
                 zpdf_e2(i)=sign(min(abs(zpdf_e2(i)),5.),zpdf_e2(i))
                 zpdf_e2(i)=1.-erf(zpdf_e2(i))

               if (zpdf_e1(i).lt.1.e-10) then
                    rneb(i,k)=0.
                    zqn(i)=zqs(i)
               else !if (zpdf_e1(i).lt.1.e-10) then
                    rneb(i,k)=0.5*zpdf_e1(i)
                    zqn(i)=zq(i)*zpdf_e2(i)/zpdf_e1(i)
               endif !if (zpdf_e1(i).lt.1.e-10) then

               
!               if (ok_sursat_marine.eq.1) then
!                 rhcl(i,k) = &
!     & erf((zpdf_k(i)+yss(i))/sqrt(2.))-erf(zpdf_k(i)/sqrt(2.))
!                 rhcl(i,k) = 0.5*zq(i)*rhcl(i,k)
!
!                 rhcl(i,k) = rhcl(i,k)/zqs(i)
!
!                IF (rneb(i,k) .LE. 0.0) THEN
!                   rhcl(i,k)=zq(i)/zqs(i)
!                ELSE IF (rneb(i,k) .GE. 1.0) THEN
!                   rhcl(i,k)=1.0
!                ELSE
!                   rhcl(i,k)=rhcl(i,k)/(1.-rneb(i,k))
!                ENDIF
!              endif !if (ok_sursat_marine.eq.1) then

              else       ! sursaturation
                  ! on est dans ce cas soit si ok_sursat_marine.eq.1 et si on a
                  ! sursat

                  !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Cas ou il y a de la sursaturation
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

              pi = ACOS(-1.)

! Calcul de rneb orig
!cccccccccccccccccccc

          zpdf_delta_o(i) = log(zq(i)/zqs(i))
          zpdf_a_o(i) = zpdf_delta_o(i)/(zpdf_k(i)*sqrt(2.))
          zpdf_b_o(i) = zpdf_k(i)/(2.*sqrt(2.))
          zpdf_e1_o(i) = zpdf_a_o(i)-zpdf_b_o(i)
          zpdf_e1_o(i) = sign(min(abs(zpdf_e1_o(i)),5.),zpdf_e1_o(i))
          zpdf_e1_o(i)=1.-erf(zpdf_e1_o(i))

          if (zpdf_e1_o(i) .lt. 1.e-10) then
          rneb_o(i,k) = 0.
          else
          rneb_o(i,k) = 0.5*zpdf_e1_o(i)
          endif

! Calcul de rneb
!ccccccccccccccc

          if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          zpdf_e1(i) = 1.-erf(yss(i)/sqrt(2.))

          if (zpdf_e1(i) .lt. 1.e-10) then
          rneb(i,k) = exp(-yss(i)**2/2.)*(yss(i)-ys(i))/(2.*sqrt(2.*pi))
          else
          rneb(i,k) = 0.5*zpdf_e1(i) + &
     & exp(-yss(i)**2/2.)*(yss(i)-ys(i))/(2.*sqrt(2.*pi))
          endif

         if (rneb(i,k) .lt. 0.) then
         write(*,*) 'ys, yss, yss-ys, 0.5*erf(yss(i)/sqrt(2.))',&
     & ys(i), yss(i), yss(i)-ys(i), 0.5*erf(yss(i)/sqrt(2.))
         endif


          else !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

! DL de rneb pour beta proche de 1

          zpdf_e1(i) = 1.-erf(ys(i)/sqrt(2.))

          if (zpdf_e1(i) .lt. 1.e-10) then
          rneb(i,k) = exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1.)/(2.*zpdf_k(i)*sqrt(2.*pi))- &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**2/(4.*zpdf_k(i)*sqrt(2.*pi))+ &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**3* &
     & (1.+2*zpdf_k(i)**2-ys(i)**2)/(12.*zpdf_k(i)**3*sqrt(2.*pi))
          else
          rneb(i,k) = 0.5*zpdf_e1(i) + &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1.)/(2.*zpdf_k(i)*sqrt(2.*pi))- &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**2/(4.*zpdf_k(i)*sqrt(2.*pi))+ &
     & exp(-0.5*ys(i)**2)*(zpdf_beta(i)-1)**3* &
     & (1.+2*zpdf_k(i)**2-ys(i)**2)/(12.*zpdf_k(i)**3*sqrt(2.*pi))
          endif

          if (rneb(i,k) .lt. 0.) then
          write(*,*) 'rneb< 0 ds le cas DL'
          endif


          endif !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          if ( rneb(i,k) .lt. 0. .or.  rneb(i,k) .gt. 1.) then
          write(*,*) 'SURSAT:  rneb(i,k)< 0 or rneb(i,k)>1', &
     & 'rneb(i,k),ys(i),zpdf_beta(i),zpdf_k(i),pi', &
     & rneb(i,k),ys(i),zpdf_beta(i),zpdf_k(i),pi
          endif



! Flag presence de sursaturation
!ccccccccccccccccccccccccccccccc

         if (abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i)) .and. &
     & (rneb_o(i,k) - rneb(i,k) .gt. 1.e-5 .or. &
     & (rneb_o(i,k) .gt. 0.98 .and. rneb(i,k) .gt. 0.98))) then

         flag_ss(i,k) = 1.

         else

         flag_ss(i,k) = 0.

         if (rneb_o(i,k) - rneb(i,k) .lt. -1.e-5) then
         write(*,*) 'rneb_o < rneb, rneb_o, rneb, yss, ys,& 
     & 0.5*zpdf_e1_o, 0.5*zpdf_e1, exp',&
     & rneb_o(i,k),rneb(i,k),yss(i), ys(i), &
     & 0.5*zpdf_e1_o(i),0.5*zpdf_e1(i), &
     & exp(-yss(i)**2/2.)*(yss(i)-ys(i))/(2.*sqrt(2.*pi))
         endif

         endif

! Fraction de ciel clair sursature
!ccccccccccccccccccccccccccccccccc

        if (abs(zpdf_beta(i) -1.) .gt. epsilon(zpdf_beta(i)) .and. &
     & rneb_o(i,k) - rneb(i,k) .gt. 1.e-5) then
        clsk_ss(i,k) = rneb_o(i,k) - rneb(i,k)
        else
        clsk_ss(i,k) = 0.
        endif


! Nuage dans un environnement sursature
!ccccccccccccccccccccccccccccccccccccccc

         if (rneb(i,k) .gt. 0 .and. & 
     & abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then
         rneb_ss(i,k) = rneb(i,k)
         endif

! Autre detection de sursaturation
!ccccccccccccccccccccccccccccccccc

         if (rneb_o(i,k) .gt. 0 .and. &
     & abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then
         flagb_ss(i,k) = 1.
         else
         flagb_ss(i,k) = 0. 
         endif

! Calcul de zqn
!cccccccccccccc          

! Calcul de zqn
!cccccccccccccc

          if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          zpdf_e1(i) = 0.5*zq(i)*(1.-erf((yss(i)+zpdf_k(i))/sqrt(2.)))

          zqn(i) = zpdf_e1(i) + &
     & exp(-zpdf_k(i)**2/2.-yss(i)**2/2.-zpdf_k(i)*(ys(i)+yss(i)))* &
     & zq(i)* &
     & (exp(zpdf_k(i)*yss(i))+ &
     & exp(zpdf_k(i)*ys(i))*(zpdf_k(i)*ys(i)-zpdf_k(i)*yss(i)-1))/ &
     & (zpdf_k(i)**2*sqrt(2.*pi)*(yss(i)-ys(i)))

          else !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

! DL de zqn pour beta proche de 1

          zpdf_e1(i) = 0.5*zq(i)*(1.-erf((zpdf_k(i)+ys(i))/sqrt(2.)))

          zqn(i) = zpdf_e1(i)+ &
     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)/ &
     & (2.*zpdf_k(i)*sqrt(2.*pi))- &
     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**2/ &
     & (12.*zpdf_k(i)*sqrt(2.*pi))+ &
     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**3* &
     & (2.+zpdf_k(i)**2-2.*ys(i)**2)/(24.*zpdf_k(i)**3*sqrt(2.*pi))

          endif !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          if (rneb(i,k) .gt. 0. .and. rneb(i,k) .lt. 1.) then
          zqn(i) = zqn(i)/rneb(i,k)    ! a verifier  
          endif

! Calcul de rhcl
!ccccccccccccccc

!          if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

!          rhcl_p0 = &
!     & erf((zpdf_k(i)+yss(i))/sqrt(2.))-erf(zpdf_k(i)/sqrt(2.))

!          rhcl_p1 = &
!     & exp(-0.5*zpdf_k(i)**2-0.5*yss(i)**2-zpdf_k(i)*(ys(i)+yss(i)))

!          rhcl_p2 = exp(zpdf_k(i)*yss(i))+exp(zpdf_k(i)*ys(i))* &
!     & (-1.+zpdf_k(i)*ys(i)-zpdf_k(i)*yss(i))

!          rhcl_p = rhcl_p1*rhcl_p2*sqrt(2./pi)/ &
!     & ((ys(i)-yss(i))*zpdf_k(i)**2)

!          rhcl(i,k) = 0.5*zq(i)*(rhcl_p0+rhcl_p)

!          rhcl(i,k) = rhcl(i,k)/zqs(i)

!          else !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then!

! DL de rhcl pour beta proche de 1

!          rhcl(i,k) = &
!     & 0.5*zq(i)* &
!     & (erf((zpdf_k(i)+ys(i))/sqrt(2.))-erf(zpdf_k(i)/sqrt(2.)))- &
!     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)/ &
!     & (2.*zpdf_k(i)*sqrt(2.*pi))+ &
!     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**2/ &
!     & (12.*zpdf_k(i)*sqrt(2.*pi))- &
!     & exp(-0.5*(zpdf_k(i)+ys(i))**2)*zq(i)*(zpdf_beta(i)-1.)**3* &
!     & (2.+zpdf_k(i)**2-2.*ys(i)**2)/ &
!     & (24.*zpdf_k(i)**3*sqrt(2.*pi))

!          rhcl(i,k) = rhcl(i,k)/zqs(i)

!          endif !if ( abs(zpdf_beta(i)-1.) .gt. epsilon(zpdf_beta(i))) then

          
!           IF (rneb(i,k) .LE. 0.0) THEN
!              rhcl(i,k)=zq(i)/zqs(i)
!           ELSE IF (rneb(i,k) .GE. 1.0) THEN
!              rhcl(i,k)=1.0
!           ELSE
!              rhcl(i,k)=rhcl(i,k)/(1.-rneb(i,k)) 
!           ENDIF

          endif  ! sursaturation

          endif ! if (lognormale(i)) then
           enddo !do i=1,klon


        endif ! iflag_pdf

!        write(*,*) 'ilp tmp 1090: zzoliq,xtoliq(1,363)=',zoliq(363),zxtoliq(1,363)
        DO i=1,klon
           IF (rneb(i,k) .LE. 0.0) THEN
              zqn(i) = 0.0
              rneb(i,k) = 0.0
              zcond(i) = 0.0
           ELSE IF (rneb(i,k) .GE. 1.0) THEN
              zqn(i) = zq(i)
              rneb(i,k) = 1.0                  
              zcond(i) = MAX(0.0,zqn(i)-zqs(i))
           ELSE
              zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)
           ENDIF
        ENDDO !DO i=1,klon

        ! si pas sursat Marine, on calcule rchl  la fin.
        ! Sinon, a a dj t calcul avant, il ne faut pas l'craser
!        if (ok_sursat_marine.eq.0) then   
        ! 22 dec 2012: Marine envoie nouvelle version, maintenant on calcule
        ! rhcl tout  la fin
         DO i=1,klon
           IF (rneb(i,k) .LE. 0.0) THEN
                rhcl(i,k)=zq(i)/zqs(i)
           ELSE IF (rneb(i,k) .GE. 1.0) THEN
                rhcl(i,k)=1.0
           ELSE
                rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
           ENDIF
         ENDDO !DO i=1,klon
!        endif !if (ok_sursat_marine.eq.0) then

        !         do i=1,klon
        !            IF (rneb(i,k) .LE. 0.0) zqn(i) = 0.0
        !            IF (rneb(i,k) .GE. 1.0) zqn(i) = zq(i)
        !            rneb(i,k) = MAX(0.0,MIN(1.0,rneb(i,k)))
        !c           zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)/(1.+zdqs(i))
        !c  On ne divise pas par 1+zdqs pour forcer a avoir l'eau predite par
        !c  la convection.
        !c  ATTENTION !!! Il va falloir verifier tout ca.
        !            zcond(i) = MAX(0.0,zqn(i)-zqs(i))*rneb(i,k)
        !c           print*,'ZDQS ',zdqs(i)
        !c--Olivier
        !            rhcl(i,k)=(zqs(i)+zq(i)-zdelq)/2./zqs(i)
        !            IF (rneb(i,k) .LE. 0.0) rhcl(i,k)=zq(i)/zqs(i)
        !            IF (rneb(i,k) .GE. 1.0) rhcl(i,k)=1.0
        !c--fin

        if (ok_sursat_marine.eq.1) then
        DO i=1,klon
        if (zqn(i) .lt. 0) then
        write(*,*) 'ATTENTION zqn(i) < 0 '
        STOP
        endif
        if (rhcl(i,k) .lt. 0) then
        write(*,*) 'ATTENTION rhcl(i,k) < 0 '
        STOP
        endif
        ENDDO
        endif ! if (ok_sursat_marine.eq.1) then

        !           ENDDO
     ELSE  ! cpartiel
        DO i = 1, klon
           IF (zq(i).GT.zqs(i)) THEN
              rneb(i,k) = 1.0
           ELSE
              rneb(i,k) = 0.0
           ENDIF
           zcond(i) = MAX(0.0,zq(i)-zqs(i))/(1.+zdqs(i))
        ENDDO
     ENDIF




#ifdef ISO
        ! on calcule la compo iso de l'eau dans le nuage et en ciel
        ! clair: zxtn/zqn et zxtcs/zqcs

              ! Hypothsesimplificatrice mais peut-tre fausse: 
              ! eau nuageuse a mme composition que eau de la maille 

              ! ce calcul n'eat pas sens dpendre du type de PDF utilis     
         do i=1,klon  
#ifdef ISOTRAC
            if (option_tmin.eq.1) then
                ! c'est mieux si zqn>=0.0
            if (essai_convergence) then
            else
                zqn(i)=max(0.0,zqn(i))
            endif    

            if (rneb(i,k).lt.1.0) then
                zqcs(i)=(zq(i)-zqn(i)*rneb(i,k))/(1.0-rneb(i,k))
#ifdef ISOVERIF
!                write(*,*) 'rneb,zq,zqn,zqcs=',rneb(i,k),zq(i), &
!     &                zqn(i),zqcs(i)
                call iso_verif_positif(zqcs(i),'ipleut 853')
#endif                
            else
                zqcs(i)=0.0
            endif
            endif ! if (option_tmin.eq.1) then
#endif                  
#ifdef ISOVERIF
              if (use_iso_eau.eq.1) then  
                 call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &                   'il pleut 748',errmax,errmaxrel) 
              endif !if (use_iso_eau.eq.1) then
              do ixt=1,ntraciso
                call iso_verif_positif_choix(zxt(ixt,i),0.0,'ilp 857')
              enddo 
#ifdef ISOTRAC
              call iso_verif_traceur_pbidouille(zxt(1,i), &
     &           'il pleut 874')
#endif              
#endif
              if ((bidouille_anti_divergence).and. &
     &           (use_iso_eau.eq.1)) then
                zxt(iso_eau,i)= max(zq(i),0.0)  
!                if (zxt(iso_eau,i).gt.0.0) then
!                    do ixt=1,niso
!                        zxt(ixt,i)=zxt(ixt,i)*zq(i)/zxt(iso_eau,i)
!                    enddo
!                else !if (zxt(iso_eau,i).gt.0.0) then
!                    do ixt=1,niso
!                        zxt(ixt,i)=0.0
!                    enddo    
!                endif !if (zxt(iso_eau,i).gt.0.0) then
              endif !if ((bidouille_anti_divergence).and. 

              if (zq(i).gt.1e-15) then   
                ! 2 fev 2010: on modifie seuil de zq  
                ! avant, c'tait 0. Vrifier que a n'a pas de
                ! consquences trop graves pour les isotopes.
                do ixt=1,ntraciso             
                   zxtn(ixt,i)=zqn(i)*(zxt(ixt,i)/zq(i))  
                enddo !do ixt=1,niso         
#ifdef ISOTRAC
               if (option_tmin.eq.1) then
                 do ixt=1,ntraciso
                   zxtcs(ixt,i)=(zqcs(i))*(zxt(ixt,i)/zq(i))
                 enddo !do ixt=1,ntraciso
               endif ! if (option_tmin.eq.1) then
#endif                
              else !if (zq(i).gt.0.0) then 
                  ! problme: zq~0 et sans compo, mais zqn >0
                  ! on fait ce qu'on peut dans ce cas pathologique
                do ixt=1,ntraciso           
                   zxtn(ixt,i)=0.0              
                enddo !do ixt=1,niso 
#ifdef ISOTRAC
                if (option_tmin.eq.1) then
                 do ixt=1,ntraciso
                   zxtcs(ixt,i)=0.0
                 enddo !do ixt=1,niso    
                endif ! if (option_tmin.eq.1) then
#endif                     
                if ((bidouille_anti_divergence) &
     &                   .and.(use_iso_eau.eq.1)) then
!                   write(*,*) 'ilp tmp 848: warning: bidouille'
                   zxtn(iso_eau,i)=zqn(i)
#ifdef ISOTRAC
                   zxtn(index_trac(izone_poubelle,iso_eau),i)=zqn(i)  
                   if (option_tmin.eq.1) then                   
                     zxtcs(iso_eau,i)=zqcs(i)
                   endif ! if (option_tmin.eq.1) then
!                   write(*,*) 'zxtn(:,i)=',zxtn(:,i)
#endif                   
                endif !if ((bidouille_anti_divergence)
              endif  !if (zq(i).gt.0.0) then 

#ifdef ISOVERIF
             do ixt=1,niso     
                  call iso_verif_positif(zxtn(ixt,i),'il pleut 1062')  
             enddo !do ixt=1,niso
             if (use_iso_eau.eq.1) then
                   call iso_verif_egalite_choix(zxtn(iso_eau,i),zqn(i), &
     &                   'il pleut 932',errmax,errmaxrel)
             endif !if ((use_iso_eau.eq.1)) then
#ifdef ISOTRAC
!             write(*,*) 'zxt(:,i)=',zxt(:,i)   
!             write(*,*) 'zq,zqn=',zq(i) ,zqn(i)  
             call iso_verif_traceur_pbidouille(zxtn(1,i), &
     &           'il pleut 1277')
             if (option_tmin.eq.1) then
              call iso_verif_positif(zxtcs(ixt,i),'il pleut 895')  
              do ixt=1,niso     
               call iso_verif_positif(zxtcs(ixt,i),'il pleut 897')  
              enddo !do ixt=1,niso
              if (use_iso_eau.eq.1) then
               call iso_verif_egalite_choix(zxtcs(iso_eau,i), &
     &              zqcs(i),'il pleut 733',errmax,errmaxrel)
              endif !if ((use_iso_eau.eq.1)) then
             endif ! if (option_tmin.eq.1) then
#endif 
#endif                   
!                 write(*,*) 'il pleut 695: calcul de zxtn, i=',i
!                 write(*,*) 'zxtn(4,i)=',zxtn(4,i)
!                 write(*,*) 'zqn(i)=',zqn(i)
              do ixt=1,ntraciso    
               zxtn(ixt,i)=max(0.0,zxtn(ixt,i))          
              enddo !do ixt=1,niso
#ifdef ISOTRAC
              if (option_tmin.eq.1) then
              do ixt=1,ntraciso
               zxtcs(ixt,i)=max(0.0,zxtcs(ixt,i))
              enddo
              endif ! if (option_tmin.eq.1) then
#endif                            

#ifdef ISOVERIF
               if (use_iso_eau.eq.1) then
                  call iso_verif_egalite_choix(zxtn(iso_eau,i),zqn(i), &
     &                   'il pleut 746',errmax,errmaxrel) 
               endif !if (use_iso_eau.eq.1) then
               if (use_iso_HDO.eq.1) then
                if (zqn(i).gt.ridicule) then
                   call iso_verif_aberrant(zxtn(iso_HDO,i)/zqn(i), &
     &                   'il pleut 977') 
                endif  !if (zqn(i).gt.ridicule) then
               endif  !if (use_iso_HDO.eq.1) then  
#ifdef ISOTRAC               
             call iso_verif_traceur_pbidouille  &
     &                  (zxtn(1,i),'il pleut 1300')
             if (option_tmin.eq.1) then
               if (use_iso_eau.eq.1) then   
                   call iso_verif_egalite_choix(zxtcs(iso_eau,i), &
     &                   zqcs(i), &
     &                   'il pleut 971',errmax,errmaxrel)
               endif !if (use_iso_eau.eq.1) then
               call iso_verif_traceur_pbidouille &
     &                  (zxtcs(1,i),'il pleut 1300')
             
             endif ! if (option_tmin.eq.1) then
#endif                
#endif   
         enddo ! do i=1,klon
         
!        write(*,*) 'ilp tmp 1311: zoliq,zxtoliq(1,363)=',zoliq(363),zxtoliq(1,363)
#endif           

     !
     DO i = 1, klon
        zq(i) = zq(i) - zcond(i)
        !         zt(i) = zt(i) + zcond(i) * RLVTT/RCPD
        zt(i) = zt(i) + zcond(i) * RLVTT/RCPD/(1.0+RVTMP2*zq(i))
     ENDDO
     !
      ! isotopes lors de la condensation
#ifdef ISO          
#ifdef ISOVERIF
      do i=1,klon
        ! verif
        if (use_iso_eau.eq.1) then
          call iso_verif_egalite_choix(zxtn(iso_eau,i),zqn(i), &
     &           'il pleut 810',errmax,errmaxrel) 
        endif !if (use_iso_eau.eq.1) then 
#ifdef ISOTRAC
        call iso_verif_traceur(zxtn(1,i),'il pleut 1408')
!        write(*,*) 'fisrtilp 1421: appel condiso_liq_ice_vectall'
#endif
       enddo !do i=1,klon 
!       write(*,*) 'il pleut 961: avant condensation'
       ! end verif
#endif
      
      do i=1,klon  
!        call calcul_zfice(zt(i),zfice(i)) ! inliner pour optimisation:
        zfice(i)=1.0-(zt(i)-pxtice)/(pxtmelt-pxtice)
        zfice(i) = MIN(MAX(zfice(i),0.0),1.0)
       enddo  
        
        ! calcul de la composition du condensat
        ! hypothse simplificatrice (mais  garder en mmoire!): l'eau
        ! nuageuse a la mme composition que l'eau totale de la maille:
        ! c'est  dire zxtn=(zxt/zq)*zqn
        
        call condiso_liq_ice_vectall(zxtn,zqn,zcond,zt,zfice, &
     &            zxtice,zxtliq,klon)
#ifdef ISOTRAC
!#ifdef ISOVERIF
!        write(*,*) 'il pleut 980: condensation pour les traceurs'
!#endif        
        call condiso_liq_ice_vectall_trac(zxtn,zqn,zcond, &
     &            zt,zfice,zxtice,zxtliq,klon)
#ifdef ISOVERIF
        do i=1,klon             
             call iso_verif_traceur(zxtn(1,i),'il pleut 1428a')
             call iso_verif_traceur(zxtliq(1,i),'il pleut 1428b')
             call iso_verif_traceur(zxtice(1,i),'il pleut 1428c')
        enddo
#endif
#endif
        do i=1,klon
        do ixt=1,ntraciso                 
          zxtcond(ixt,i)=zxtice(ixt,i)+zxtliq(ixt,i)
          zxtcond(ixt,i)=max(zxtcond(ixt,i),0.0)
          zxtcond(ixt,i)=min(zxtcond(ixt,i),zxt(ixt,i))
        enddo !do ixt=1,niso 
        enddo !do i=1,klon

       do i=1,klon   
#ifdef DIAGISO       
        ! juste diag: fraction condense
        if (zq(i).gt.ridicule) then
          fcond_ls(i,k)=zcond(i)/zq(i)                   
        endif
        ! en juste diag  
#endif      
       enddo  !do i=1,klon       
        
!#ifdef ISOVERIFIDRIS
#ifdef ISOVERIFNAN
        do i=1,klon
          do ixt=1,ntraciso  
            call iso_verif_noNAN(zxtcond(ixt,i),'il pleut 638')
          enddo
        enddo !do i=1,klon 
#endif       
#ifdef ISOVERIF       
        if (use_iso_eau.eq.1) then
          do i=1,klon      
              call iso_verif_egalite_choix(zxtcond(iso_eau,i),zcond(i), &
     &                   'il pleut 645',errmax,errmaxrel)
          enddo ! do i=1,klon     
        endif
        if (use_iso_HDO.eq.1) then
           do i=1,klon  
              if (zcond(i).gt.ridicule) then
                 call iso_verif_aberrant(zxtcond(iso_HDO,i) &
     &                  /zcond(i), 'il pleut 637')                  
              endif  !if (zcond(i).gt.ridicule) then
              IF ((t(i,1)) .LT. RTT) THEN
                  call iso_verif_aberrant_choix(zxtcond(iso_hdo,i), &
     &           zcond(i),ridicule_rain,deltalim_snow,'il pleut 1276')
              endif
           enddo ! do i=1,klon  
        endif !if ((use_iso_eau.eq.1).and.(ixt.eq.iso_eau)) then
#ifdef ISOTRAC
        do i=1,klon
             if (iso_verif_traceur_nostop(zxtcond(1,i),'il pleut 1455') &
     &                   .eq.1) then
               write(*,*) 'zxtn(:,i)=',zxtn(:,i)
               write(*,*) 'zxt(:,i)=',zxt(:,i)
               write(*,*) 'zxtliq(:,i)=',zxtliq(:,i)
               write(*,*) 'zxtice(:,i)=',zxtice(:,i)
               stop
             endif
        enddo     
#endif         
#endif

       ! retranchage  zxt 
      do i=1,klon 
       do ixt=1,ntraciso
          zxt(ixt,i)=zxt(ixt,i)-zxtcond(ixt,i)
       enddo !do ixt=1,niso   
      enddo !do i=1,klon 

#ifdef ISOVERIF
      do i=1,klon
         do ixt=1,niso
          call iso_verif_positif(zxt(ixt,i),'il pleut 1088')
         enddo !do ixt=1,niso
#ifdef ISOTRAC  
         ! on spare pour mieux dbugger 8 fev 2010
         do ixt=1,ntraciso
          call iso_verif_positif(zxt(ixt,i),'il pleut 1093')
         enddo !do ixt=1,niso
#endif      
      enddo   !do i=1,klon
#endif

#ifdef ISOTRAC        
        if (option_tmin.ge.1) then
        do i=1,klon    
        ! colorier la vapeur rsiduelle selon temprature de
        ! condensation, et le condensat en un tag spcifique
        ! Attention: zqn,zxtn ne servent qu'au calcul de zcond,zxtcond.
        ! C'est en fait zxt qui se fait retranch
        ! correction le 31 mars 2010: c'est  qn qu'il faut retrancher
        ! le condensat, car la condensation LS est un processus sous
        ! maille -> ne tagguer que le rsidus de qn.
          if ((zcond(i).gt.0.0).and.(zq(i).gt.0.0)) then 

            if (rneb(i,k).gt.0.0) then
              zcondn(i)=zcond(i)/rneb(i,k)
              do ixt=1,ntraciso
                zxtcondn(ixt,i)=zxtcond(ixt,i)/rneb(i,k)
              enddo !do ixt=1,ntraciso
            else !if (rneb.eq.0.0)
              zcondn(i)=0.0
              do ixt=1,ntraciso
                zxtcondn(ixt,i)=0.0
              enddo !do ixt=1,ntraciso
#ifdef ISOVERIF
              call iso_verif_egalite(zcond(i),0.0,'ilp 1225')
#endif              
            endif !if (rneb.eq.0.0)
            
#ifdef ISOVERIF
          call iso_verif_egalite_choix(zxtcondn(iso_eau,i),zcondn(i), &
     &                   'il pleut 1244',errmax*10,errmaxrel*10)
          call iso_verif_traceur(zxtcondn(1,i),'cv3_routines 2302')
          zq_verif=rneb(i,k)*zqn(i)+(1.0-rneb(i,k))*zqcs(i)-zcond(i)
          call iso_verif_egalite_choix(zq(i),zq_verif, &
     &                   'il pleut 1235',errmax,errmaxrel)
          do ixt=1,niso
            zq_verif=rneb(i,k)*zxtn(ixt,i) &
     &           +(1.0-rneb(i,k))*zxtcs(ixt,i)-zxtcond(ixt,i)
            call iso_verif_egalite_choix(zxt(ixt,i),zq_verif, &
     &                   'il pleut 1235',errmax,errmaxrel)
          enddo
#endif            
          ! bidouille
          if (bidouille_anti_divergence) then
                zxtcondn(iso_eau,i)=zcondn(i)
#ifdef ISOVERIF                
                call iso_verif_traceur_pbidouille(zxtcondn(1,i), &
     &          'il pleut 1261')
#endif                
          endif

            if (option_traceurs.eq.17) then      
            call iso_recolorise_condensation(zqn(i),zcondn(i), &
     &           zxtn(1,i),zxtcondn(1,i),zt(i),1.0,zxtres, &
     &           seuil_tag_tmin_ls)
            else !if (option_traceurs.eq.17) then
                ! cas du tag 18 ou 19
                call iso_recolorise_condensation(zqn(i),zcondn(i), &
     &           zxtn(1,i),zxtcondn(1,i),zqs(i),1.0,zxtres, &
     &           seuil_tag_tmin_ls)
            endif !if (option_traceurs.eq.17) then
            do ixt=1+niso,ntraciso
               zxt(ixt,i)=rneb(i,k)*zxtres(ixt) &
     &           +(1.0-rneb(i,k))*zxtcs(ixt,i)
            enddo         
#ifdef ISOVERIF
         call iso_verif_traceur(zxt(1,i),'ilp 1025')
         do izone=izone_oce,izone_ddft
          if ((izone.eq.izone_oce).or.(izone.eq.izone_ddft)) then
           ! avec condensation, on ne peut que perdre des traceurs 
           ! part le tag rsuel et le condensat
           if (iso_verif_positif_choix_nostop( &
     &           zxt_ancien(index_trac(izone,iso_eau),i) &
     &          -zxt(index_trac(izone,iso_eau),i),1e-8,'ilp 1270') &
     &          .eq.1) then
            write(*,*) 'i,izone,rneb=',i,izone,rneb(i,k)
            write(*,*) 'zxt(:,i)=',zxt(:,i)
            write(*,*) 'zxt_ancien(:,i)=',zxt_ancien(:,i)
            write(*,*) 'zxtcs(:,i)=',zxtcs(:,i)
            write(*,*) 'zxtres(:)=',zxtres(:)
            write(*,*) 'zxtcond(:,i)=',zxtcond(:,i)
            write(*,*) 'zxtn(:,i)=',zxtn(:,i)
            write(*,*) 'zxtcondn(:,i)=',zxtcondn(:,i)
            stop
           endif !if (iso_verif_positif_nostop(
          endif !if ((izone.eq.izone_oce).or.(izone.eq.izone_ddft)) then
         enddo !do izone=izone_oce,izone_revap
#endif                     
          endif !if (cond.gt.0.0) then
        enddo !do i=1,klon

#ifdef ISOVERIF
        ! vrifier qu'on recolorise bien le condensat
        call iso_verif_positif(float(option_cond-1),'ilp 1310')
#endif        
        endif ! if (option_tmin.ge.1) then

        if (option_cond.eq.1) then
        ! colorier le condensat en un tag spcifique
        do i=1,klon        
          do ixt=1+niso,ntraciso
            if (index_zone(ixt).eq.izone_cond) then
              zxtcond(ixt,i)=zxtcond(index_iso(ixt),i)
            else
              zxtcond(ixt,i)=0.0
            endif
          enddo !do ixt=1,ntraciso
        enddo !do i=1,klon
#ifdef ISOVERIF
        do i=1,klon    
          call iso_verif_egalite_choix(zxtcond(iso_eau,i),zcond(i), &
     &                   'il pleut 1139',errmax,errmaxrel)
          call iso_verif_traceur(zxtcond(1,i),'cv3_routines 2302')
        enddo !do i=1,klon        
#endif         
      endif ! if (option_cond.ge.1) then
#endif        
     
      
#ifdef ISOVERIF
      do i=1,klon
         do ixt=1,niso
          call iso_verif_positif(zxt(ixt,i),'il pleut 778')
         enddo !do ixt=1,niso
#ifdef ISOTRAC  
         ! on spare pour mieux dbugger 8 fev 2010
         do ixt=1,ntraciso
          call iso_verif_positif(zxt(ixt,i),'il pleut 1135')
          call iso_verif_traceur(zxt(1,i),'il pleut 1485')
         enddo !do ixt=1,niso
#endif
         if (use_iso_eau.eq.1) then
            call iso_verif_egalite_choix(zxt(iso_eau,i),zq(i), &
     &                   'il pleut 644',errmax,errmaxrel)
         endif !if (use_iso_eau.eq.1) then
         if (use_iso_HDO.eq.1) then  
            if (zq(i).gt.ridicule) then
             call iso_verif_aberrant_encadre(zxt(iso_HDO,i)/zq(i), &
     &                   'il pleut 648') 
            endif !if (zq(i).gt.ridicule) then
         endif  ! if (use_iso_HDO.eq.1) then 
        enddo  ! i=1,klon
#ifdef ISOTRAC
        do i=1,klon
             call iso_verif_traceur(zxt(1,i),'il pleut 1485')
        enddo     
#endif         
#endif
#ifdef ISOVERIFNAN   
       do i=1,klon   
           do ixt=1,ntraciso
             call iso_verif_noNAN(zxtcond(ixt,i),'il pleut 1560')
             call iso_verif_noNAN(zxt(ixt,i),'il pleut 1561')
           enddo   ! ixt=1,niso
        enddo  !do i=1,klon         
#endif   
#endif      
     ! Partager l'eau condensee en precipitation et eau liquide nuageuse
     !
      DO i = 1, klon
        IF (rneb(i,k).GT.0.0) THEN
           zoliq(i) = zcond(i)
           zrho(i) = pplay(i,k) / zt(i) / RD
           zdz(i) = (paprs(i,k)-paprs(i,k+1)) / (zrho(i)*RG)
           zfice(i) = 1.0 - (zt(i)-ztglace) / (273.13-ztglace)
           zfice(i) = MIN(MAX(zfice(i),0.0),1.0)
           zfice(i) = zfice(i)**nexpo
           zneb(i) = MAX(rneb(i,k), seuil_neb)
           radliq(i,k) = zoliq(i)/REAL(ninter+1)
        ENDIF !IF (rneb(i,k).GT.0.0) THEN
      ENDDO !DO i = 1, klon
#ifdef ISO
      ! initilsation des quantits totales d'eau liquide et solide qui
      ! tombent
      do i=1,klon
         if (rneb(i,k).gt.0.0) then
            do ixt=1,ntraciso
               zxtoliq(ixt,i)=zxtcond(ixt,i)
            enddo
         endif ! if (rneb(i,k).gt.0.0)
       enddo
          
#ifdef ISOVERIFNAN
!       write(*,*) 'ilp tmp 1626: k,zoliq,zxtoliq(1,363)=',k,zoliq(363),zxtoliq(1,363)
       do i=1,klon   
         do ixt=1,ntraciso
           if (iso_verif_noNAN_nostop(zxtoliq(ixt,i), &
     &           'il pleut 1628').eq.1) then
             write(*,*) 'i,k,ixt=',i,k,ixt
             write(*,*) 'rneb(i,k)=',rneb(i,k)
             write(*,*) 'zxtcond(ixt,i)=',zxtcond(ixt,i)
             stop
           endif
         enddo   ! ixt=1,niso
        enddo  !do i=1,klon  
#endif                
#ifdef ISOVERIF
       do i=1,klon   
           ! cam verif
           if (use_iso_eau.eq.1) then
            call iso_verif_egalite_choix(zxtoliq(iso_eau,i),zoliq(i), &
     &            'il pleut 678',errmax,errmaxrel)
           endif !if (use_iso_eau.eq.1) then
           if (use_iso_HDO.eq.1) then 
            if (zoliq(i).gt.ridicule) then 
                call iso_verif_aberrant(zxtoliq(iso_HDO,i)/zoliq(i), &
     &           'il pleut 895')
             endif !if (zoliq(i).gt.ridicule) then
           endif  !if (use_iso_HDO.eq.1) then
          ! end cam verif
         enddo ! do i=1,klon
#endif
#endif
     !
     DO n = 1, ninter
        DO i = 1, klon
           IF (rneb(i,k).GT.0.0) THEN
              zrhol(i) = zrho(i) * zoliq(i) / zneb(i)

              IF (zneb(i).EQ.seuil_neb) THEN
                 ztot = 0.0
              ELSE
                 !  quantite d'eau a eliminer: zchau
                 !  meme chose pour la glace: zfroi
                 if (ptconv(i,k)) then
                    zcl   =cld_lc_con
                    zct   =1./cld_tau_con
                    zfroi    = dtime/REAL(ninter)/zdz(i)*zoliq(i) &
                         *fallvc(zrhol(i)) * zfice(i)
                 else
                    zcl   =cld_lc_lsc
                    zct   =1./cld_tau_lsc
                    zfroi    = dtime/REAL(ninter)/zdz(i)*zoliq(i) &
                         *fallvs(zrhol(i)) * zfice(i)
                 endif
                 zchau    = zct   *dtime/REAL(ninter) * zoliq(i) &
                      *(1.0-EXP(-(zoliq(i)/zneb(i)/zcl   )**2)) *(1.-zfice(i))
                 ztot    = zchau    + zfroi
                 ztot    = MAX(ztot   ,0.0)
              ENDIF
              ztot    = MIN(ztot,zoliq(i))
              zoliq(i) = MAX(zoliq(i)-ztot   , 0.0)
              radliq(i,k) = radliq(i,k) + zoliq(i)/REAL(ninter+1)
           ENDIF
        ENDDO
     ENDDO
     !


#ifdef ISO
      do i=1,klon
       IF (rneb(i,k).GT.0.0) THEN
        if (zcond(i).gt.ridicule**2) then
            ! le 21 dec: on change .gt.0 en .gt.ridicule**2 pour viter valeur
            ! ridiculement petites
           do ixt=1,ntraciso
             zxtoliq(ixt,i)=zoliq(i)*(zxtcond(ixt,i)/zcond(i))
           enddo !do ixt=1,niso           
        else !if (zcond(i).gt.0.0) then            
           if (zoliq(i).lt.ridicule) then
               do ixt=1,ntraciso
                zxtoliq(ixt,i)=0.0
               enddo
           else !if (zoliq(i).lt.ridicule) then
               ! modif 28 oct 2008
              do ixt=1,niso
                zxtoliq(ixt,i)=zoliq(i)*Rdefault(ixt)
              enddo !do ixt=1,niso
#ifdef ISOTRAC
              do ixt=niso+1,ntraciso
                if (index_zone(ixt).eq.izone_poubelle) then
                    zxtoliq(ixt,i)=zoliq(i)*Rdefault(ixt)
                else
                    zxtoliq(ixt,i)=0.0
                endif
              enddo  
#endif    
#ifdef ISOVERIF                   
              write(*,*) 'il pleut 1412: zcond(i)=',zcond(i)
              write(*,*) 'zoliq(i)=',zoliq(i)
              stop
#endif                   
            endif   !if (zoliq(i).lt.ridicule) then         
        endif !if (zcond(i).gt.0.0) then
       endif !IF (rneb(i,k).GT.0.0) THEN
      enddo ! do i=1,klon 
           
         ! cam verif
#ifdef ISOVERIFNAN
      do i=1,klon  
        IF (rneb(i,k).GT.0.0) THEN 
           do ixt=1,niso
             call iso_verif_noNAN(zxtoliq(ixt,i),'il pleut 718')
           ! end cam verif
           enddo ! do ixt=1,niso
          ! cam verif
        endif !IF (rneb(i,k).GT.0.0) THEN 
      enddo !do i=1,klon  
#endif
#ifdef ISOVERIF
      do i=1,klon  
        IF (rneb(i,k).GT.0.0) THEN
          if (use_iso_eau.eq.1) then
            call iso_verif_egalite_choix(zxtoliq(iso_eau,i),zoliq(i), &
     &           'il pleut 749',errmax,errmaxrel)
          endif !if (use_iso_eau.eq.1) then
          if (use_iso_HDO.eq.1) then
             if (zoliq(i).gt.ridicule) then
                call iso_verif_aberrant(zxtoliq(iso_HDO,i)/zoliq(i), &
     &           'il pleut 963')
             endif
          endif !if (use_iso_HDO.eq.1) then
        ! end cam verif
#ifdef ISOTRAC
!             write(*,*) 'ilp 1201 tmp: i=',i
!             write(*,*) 'zoliq(i)=',zoliq(i)
!             write(*,*) 'zxtoliq(i)=',zxtoliq(iso_eau:ntraciso:3,i)
!             write(*,*) 'zcond(i)=',zcond(i)
!             write(*,*) 'zxtcond(i)=',zxtcond(iso_eau:ntraciso:3,i)
             call iso_verif_traceur(zxtoliq(1,i),'il pleut 1634')
#endif        
         endif !IF (rneb(i,k).GT.0.0) THEN
       enddo ! do i=1,klon
#endif        
#endif      
                
     DO i = 1, klon
        IF (rneb(i,k).GT.0.0) THEN
           d_ql(i,k) = zoliq(i)
           zrfl(i) = zrfl(i)+ MAX(zcond(i)-zoliq(i),0.0) &
                * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)

         
#ifdef ISO   
#ifdef DIAGISO
        taux_cond_ls(i,k)=zcond(i)*(paprs(i,k)-paprs(i,k+1))/(RG*dtime)
        taux_precip_ls(i,k)=MAX(zcond(i)-zoliq(i),0.0) &
     &                    *(paprs(i,k)-paprs(i,k+1))/(RG*dtime)
#endif      
         do ixt=1,ntraciso
            d_xtl(ixt,i,k)=zxtoliq(ixt,i)
         enddo ! do ixt=1,niso   
            ! Rq: 28 oct 2008: si zoliq a la mme compo que zcond, alors
            ! on vite pb numriques si on prend directement zrfl avec
            ! la compo de zcond?
!          do ixt=1,niso  
!            zxtrfl(ixt,i)=zxtrfl(ixt,i) &
!     &           +max(zxtcond(ixt,i)-zxtoliq(ixt,i),0.0) &
!     &           * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)
!          enddo  
            if (zcond(i).gt.ridicule**2) then
                ! modif le 19 dec 2012 pour viter zcond pathologiquement petits
                do ixt=1,ntraciso
                  zxtrfl(ixt,i)=zxtrfl(ixt,i) &
     &                  +(zxtcond(ixt,i)/zcond(i)) &
     &                  * MAX(zcond(i)-zoliq(i),0.0) &
     &                  * (paprs(i,k)-paprs(i,k+1))/(RG*dtime)
                enddo !do ixt=1,niso
            else !if (zcond(i).gt.0.0) then
                if (zcond(i)-zoliq(i).gt.0.0) then
                    do ixt=1,niso
                        zxtrfl(ixt,i)=zxtrfl(ixt,i) &
     &                  +Rdefault(ixt) &
     &                  * MAX(zcond(i)-zoliq(i),0.0) &
     &                  * (paprs(i,k)-paprs(i,k+1))/(RG*dtime) 
                    enddo !do ixt=1,niso  
#ifdef ISOVERIF                   
                    write(*,*) 'il pleut 1023: zcond(i)=',zcond(i)
                    write(*,*) 'zcond(i),zoliq(i)=',zcond(i),zoliq(i)
                    stop
#endif        
#ifdef ISOTRAC
                   do ixt=niso+1,ntraciso
                      if (index_zone(ixt).eq.izone_poubelle) then
                        zxtrfl(ixt,i)=zxtrfl(ixt,i) &
     &                  +Rdefault(ixt) &
     &                  * MAX(zcond(i)-zoliq(i),0.0) &
     &                  * (paprs(i,k)-paprs(i,k+1))/(RG*dtime) 
                      endif
                    enddo !do ixt=1,niso
#endif                 
                else !if (zcond(i)-zoliq(i).gt.0.0) then
                    ! zrfl non modifi
                endif !if (zcond(i)-zoliq(i).gt.0.0) then
            endif !if (zcond(i).gt.0.0) then

         ! cam verif         
!#ifdef ISOVERIFIDRIS
#ifdef ISOVERIFNAN
            do ixt=1,ntraciso
             if ((iso_verif_noNAN_nostop(zxtrfl(ixt,i), &
     &                   'il pleut 772').eq.1).or. &
     &                   (iso_verif_noNAN_nostop(zxtoliq(ixt,i), &
     &                   'il pleut 772b').eq.1)) then
               write(*,*) 'zcond(i)=',zcond(i)
               write(*,*) 'zxtcond(ixt,i)=',zxtcond(ixt,i)
               write(*,*) 'zoliq(i)=',zoliq(i)
               write(*,*) 'Rdefault(ixt)=',Rdefault(ixt)
               write(*,*) 'paprs(i,k),paprs(i,k+1)=',paprs(i,k),paprs(i,k+1)
               stop
             endif
            enddo
#endif
#ifdef ISOVERIF
            if (use_iso_eau.eq.1) then
              call iso_verif_egalite_choix(d_xtl(iso_eau,i,k), &
     &           d_ql(i,k),'il pleut 771',errmax,errmaxrel)
              if (iso_verif_egalite_choix_nostop( &
     &           zxtrfl(iso_eau,i),zrfl(i), &
     &           'il pleut 771b',errmax,errmaxrel).eq.1)then
                 write(*,*) 'trace_cas(i)=',trace_cas(i)
                 write(*,*) 'zxtcond(iso_eau,i)',zxtcond(iso_eau,i)
                 write(*,*) 'zcond(i)',zcond(i)
                 write(*,*) 'zxtoliq(iso_eau,i)',zxtoliq(iso_eau,i)
                 write(*,*) 'zoliq(i)',zoliq(i)
                 stop
              endif
            endif !if (use_iso_eau.eq.1) then
            if (use_iso_hdo.eq.1) then
                IF ((t(i,1)) .LT. RTT) THEN
                call iso_verif_aberrant_choix(zxtrfl(iso_hdo,i), &
     &           zrfl(i),ridicule_rain,deltalim_snow,'il pleut 1458')
                endif
            endif !if (use_iso_hdo.eq.1) then
#ifdef ISOTRAC
             call iso_verif_traceur(zxtrfl(1,i),'il pleut 1729')
#endif            
#endif
         ! bidouille
         if ((bidouille_anti_divergence).and.(use_iso_eau.eq.1)) then 
               zxtrfl(iso_eau,i)= zrfl(i)   
!             if (zxtrfl(iso_eau,i).gt.0.0) then
!             do ixt=1,niso
!               zxtrfl(ixt,i)=zxtrfl(ixt,i)*zrfl(i)/zxtrfl(iso_eau,i)
!             enddo
!           endif !if (zxtrfl(iso_eau,i).gt.0.0) then
         endif ! if (bidouille_anti_divergence) then
         ! end bidouille
         ! end cam verif
#endif                     
        ENDIF !IF (rneb(i,k).GT.0.0) THEN

        IF (zt(i).LT.RTT) THEN
           psfl(i,k)=zrfl(i)
#ifdef ISO
         do ixt=1,ntraciso
           pxtsnowfl(ixt,i,k)=zxtrfl(ixt,i)
         enddo
#endif
        ELSE !IF (zt(i).LT.RTT) THEN
           prfl(i,k)=zrfl(i)
#ifdef ISO
         do ixt=1,ntraciso
           pxtrainfl(ixt,i,k)=zxtrfl(ixt,i)
         enddo         
#endif        
        ENDIF !IF (zt(i).LT.RTT) THEN
     ENDDO !DO i = 1, klon 
     !
#ifdef ISO
#ifdef ISOVERIF
      DO i = 1, klon
        if (use_iso_eau.eq.1) then
          call iso_verif_egalite_choix(pxtrainfl(iso_eau,i,k),prfl(i,k), & 
     &           'il pleut 804',errmax,errmaxrel)
          call iso_verif_egalite_choix(pxtsnowfl(iso_eau,i,k),psfl(i,k), &  
     &           'il pleut 805',errmax,errmaxrel) 
          call iso_verif_egalite_choix(xt(iso_eau,i,k),q(i,k), &  
     &           'il pleut 1612',errmax,errmaxrel) 
          if (iso_verif_positif_choix_nostop(pxtrainfl(iso_eau,i,k), &
     &           ridicule_rain,'ilp 1547').eq.1) then
            write(*,*) 'prfl(i,k)=',prfl(i,k)
            stop
          endif  
        endif !if (use_iso_eau.eq.1) then
        if (use_iso_HDO.eq.1) then
          if (prfl(i,k).gt.ridicule_rain) then
            call iso_verif_aberrant(pxtrainfl(iso_HDO,i,k)/prfl(i,k), &
     &            'il pleut 1020')
          endif !if (prfl(i,k).gt.ridicule_rain) then        
          if (psfl(i,k).gt.ridicule_rain) then
            call iso_verif_aberrant(pxtsnowfl(iso_HDO,i,k)/psfl(i,k), &
     &           'il pleut 1024')
          endif !if (psfl(i,k).gt.ridicule_rain) then
          if (zq(i).gt.ridicule) then
              call iso_verif_aberrant_encadre(zxt(iso_HDO,i)/zq(i), &
     &           'il pleut 1204')
          endif !if (zq(i).gt.ridicule) then          
         endif !if (use_iso_HDO.eq.1) then
#ifdef ISOTRAC
          call iso_verif_traceur(zxt(1,i),'il pleut 1792')
          call iso_verif_traceur(pxtsnowfl(1,i,k),'il pleut 1793')
          call iso_verif_traceur(pxtrainfl(1,i,k),'il pleut 1794')
#endif         
       ENDDO !DO i = 1, klon  
#endif
#endif         
     ! Calculer les tendances de q et de t:
     !
     DO i = 1, klon
        d_q(i,k) = zq(i) - q(i,k)
#ifdef ISO
         do ixt=1,ntraciso
             d_xt(ixt,i,k)=zxt(ixt,i)-xt(ixt,i,k)   
         enddo
         ! cam verif
#ifdef ISOVERIF
           if (use_iso_eau.eq.1) then
             if (iso_verif_egalite_choix_nostop(d_xt(iso_eau,i,k), &
     &            d_q(i,k),'il pleut 838',errmax,errmaxrel).eq.1) then
                write(*,*) 'i,k=',i,k
                write(*,*) 'zxt,zq=',zxt(iso_eau,i),zq(i)
                write(*,*) 'xt,q=',xt(iso_eau,i,k),q(i,k)
                stop
             endif
           endif !if (use_iso_eau.eq.1) then
           if (use_iso_HDO.eq.1) then
            if (abs(d_q(i,k)).gt.max(errmaxrel*q(i,k),errmax)) then
              if (iso_verif_aberrant_nostop( &
     &           d_xt(iso_HDO,i,k)/d_q(i,k),'il pleut 1229').eq.1) then
!                write(*,*) 'd_q(i,k)=',d_q(i,k)
!                write(*,*) 'q(i,k)=',q(i,k)
!                write(*,*) 'zq(i)=',zq(i)
!                write(*,*) 'deltaD_prec=',deltaD(xt(iso_HDO,i,k)/q(i,k))
!                write(*,*) 'deltaD=',deltaD(zxt(iso_HDO,i)/zq(i))
!                stop       
              endif !   if (iso_verif_aberrant_nostop(
            endif !if (zq(i).gt.ridicule) then          
           endif !if (use_iso_HDO.eq.1) then
#endif
         ! end cam verif
#endif           
        d_t(i,k) = zt(i) - t(i,k)
     ENDDO
     !
     !AA--------------- Calcul du lessivage stratiforme  -------------

     DO i = 1,klon
        !
        if(zcond(i).gt.zoliq(i)+1.e-10) then
         beta(i,k) = (zcond(i)-zoliq(i))/zcond(i)/dtime
        else
         beta(i,k) = 0.
        endif
        zprec_cond(i) = MAX(zcond(i)-zoliq(i),0.0) &
             * (paprs(i,k)-paprs(i,k+1))/RG
        IF (rneb(i,k).GT.0.0.and.zprec_cond(i).gt.0.) THEN
           !AA lessivage nucleation LMD5 dans la couche elle-meme
           if (t(i,k) .GE. ztglace) THEN
              zalpha_tr = a_tr_sca(3)
           else
              zalpha_tr = a_tr_sca(4)
           endif
           zfrac_lessi = 1. - EXP(zalpha_tr*zprec_cond(i)/zneb(i))
           pfrac_nucl(i,k)=pfrac_nucl(i,k)*(1.-zneb(i)*zfrac_lessi)
           frac_nucl(i,k)= 1.-zneb(i)*zfrac_lessi 
           !
           ! nucleation avec un facteur -1 au lieu de -0.5
           zfrac_lessi = 1. - EXP(-zprec_cond(i)/zneb(i))
           pfrac_1nucl(i,k)=pfrac_1nucl(i,k)*(1.-zneb(i)*zfrac_lessi)
        ENDIF
        !
     ENDDO      ! boucle sur i
     !
     !AA Lessivage par impaction dans les couches en-dessous
     DO kk = k-1, 1, -1
        DO i = 1, klon
           IF (rneb(i,k).GT.0.0.and.zprec_cond(i).gt.0.) THEN
              if (t(i,kk) .GE. ztglace) THEN
                 zalpha_tr = a_tr_sca(1)
              else
                 zalpha_tr = a_tr_sca(2)
              endif
              zfrac_lessi = 1. - EXP(zalpha_tr*zprec_cond(i)/zneb(i))
              pfrac_impa(i,kk)=pfrac_impa(i,kk)*(1.-zneb(i)*zfrac_lessi)
              frac_impa(i,kk)= 1.-zneb(i)*zfrac_lessi
           ENDIF
        ENDDO
     ENDDO
#ifdef ISO      
      ! verif en fin de boucle
#ifdef ISOVERIF
      if (use_iso_eau.eq.1) then
        do i=1,klon       
        call iso_verif_egalite_choix(zxtrfl(iso_eau,i) &
     &      ,zrfl(i), 'il pleut 1089',errmax,errmaxrel)
        call iso_verif_egalite_choix(d_xtl(iso_eau,i,k),d_ql(i,k), &
     &           'il pleut 1205',errmax,errmaxrel) 
#ifdef ISOTRAC
        call iso_verif_traceur(zxtrfl(1,i),'il pleut 1894')
        call iso_verif_traceur(d_xtl(1,i,k),'il pleut 1895')
#endif        
        enddo !do i=1,klon
      endif !  if (use_iso_eau.eq.1) then        
#endif
#endif      
      ! end verif en fin de boucle

     !
     !AA----------------------------------------------------------
     !                     FIN DE BOUCLE SUR K   
  end DO
  !
  !AA-----------------------------------------------------------
  !
  ! Pluie ou neige au sol selon la temperature de la 1ere couche
  !
  DO i = 1, klon
     IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
        snow(i) = zrfl(i)
        zlh_solid(i) = RLSTT-RLVTT
#ifdef ISO
        do ixt=1,ntraciso
          xtsnow(ixt,i) = zxtrfl(ixt,i)
        enddo !do ixt=1,ntraciso
#endif         
     ELSE !IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
        rain(i) = zrfl(i)
        zlh_solid(i) = 0.
#ifdef ISO
        do ixt=1,ntraciso
          xtrain(ixt,i) = zxtrfl(ixt,i)
        enddo !do ixt=1,ntraciso
#endif         
     ENDIF !IF ((t(i,1)+d_t(i,1)) .LT. RTT) THEN
#ifdef ISO
      ! cam verif 
#ifdef ISOVERIFNAN
       do ixt=1,ntraciso
        call iso_verif_noNAN(xtrain(ixt,i),'il pleut 927')
        call iso_verif_noNAN(xtsnow(ixt,i),'il pleut 927')
       enddo ! do ixt=1,niso
#endif         
#ifdef ISOVERIF
       if (use_iso_eau.eq.1) then
        call iso_verif_egalite_choix(xtrain(iso_eau,i),rain(i), &
     &           'il pleut 926a',errmax,errmaxrel)  
        if (snow(i)*dtime.gt.ridicule) then
           call iso_verif_egalite_choix(xtsnow(iso_eau,i),snow(i), &
     &            'il pleut 938a',errmax,errmaxrel)
        endif !if (snow(i)*dtime.gt.ridicule) then
       endif !if (use_iso_eau.eq.1) then
       if (use_iso_hdo.eq.1) then
           call iso_verif_aberrant_choix(xtrain(iso_hdo,i),rain(i), &
     &           ridicule_rain,deltalim,'il pleut 926b')  
           call iso_verif_aberrant_choix(xtsnow(iso_hdo,i),snow(i), &
     &            ridicule_rain,deltalim_snow,'il pleut 938b')
       endif !if (use_iso_eau.eq.1) then
#ifdef ISOTRAC
       call iso_verif_traceur(xtrain(1,i),'il pleut 1952')
       call iso_verif_traceur(xtsnow(1,i),'il pleut 1953')
#endif       
#endif
       ! end cam verif   
#endif       
  ENDDO
  !
  ! For energy conservation : when snow is present, the solification
  ! latent heat is considered.
  DO k = 1, klev
     DO i = 1, klon
        zcpair=RCPD*(1.0+RVTMP2*(q(i,k)+d_q(i,k)))
        zmair=(paprs(i,k)-paprs(i,k+1))/RG
        zm_solid = (prfl(i,k)-prfl(i,k+1)+psfl(i,k)-psfl(i,k+1))*dtime
        d_t(i,k) = d_t(i,k) + zlh_solid(i) *zm_solid / (zcpair*zmair)
     END DO
  END DO
  !

  if (ncoreczq>0) then
     WRITE(lunout,*)'WARNING : ZQ dans fisrtilp ',ncoreczq,' val < 1.e-15.'
  endif

#ifdef ISO
      ! verif en sortie de il pleut 
#ifdef ISOVERIF
       do k=1,klev
        do i=1,klon
         if (use_iso_eau.eq.1) then
           call iso_verif_egalite_choix(d_xtl(iso_eau,i,k),d_ql(i,k), &
     &            'il pleut 1289',errmax,errmaxrel)
         endif !if (use_iso_eau.eq.1) then
#ifdef ISOTRAC
         call iso_verif_traceur(d_xtl(1,i,k),'il pleut 1982')
#endif
        enddo !do i=1,klon
       enddo !do k=1,klev
#endif
      ! end verif
#endif       

END SUBROUTINE fisrtilp
