        subroutine iso_evap_sol_nu(qsol0,qevap,q10,Rsol0,R1,h,T,alphak,
     :    L, xtnu,Pveg)
      implicit none

      ! calcul de Rsol et Revap lors de l'vaporation de l'eau du sol
      ! par vap nue.

#include "wateriso2.h"
#include "iso_verif.h"
      ! inputs:
      real qsol0 ! eau du sol
      real qevap  ! eau perdue par le sol
      real Rsol0(niso) ! rapport iso initial dans sol
      real R1(niso) ! rapport iso dans couche 1, suppos constant
      real q10 ! humidit 1re couche, en mm
      real h ! humidit rel dans couche 1, suppose cosntante
      real T ! temprature
      real alphak(niso) ! coef cintique
      real L ! longueur de diffusion
      real Pveg ! just pour dbguage

      ! outputs!
      real xtnu(niso) !  flux iso dans l'vap
      
      ! locals
      real f ! fraction d'eau rsiduelle dans le sol   
      real interm(niso)        
      real betaprime(niso) ! beta de stewart75
      real gama(niso) ! le gama de Stewart75
      real zxtalphal(niso), zxtalphai(niso) ! coeffs frac
      integer ixt

      ! calcul de l'vap: ordre 1 (on prend l'vap en t0) ou bilan total
      ! (on prend l'vap tel que la 1re couche se mette  l'quilibre
      integer ordre1 ! 1: ordre 1: deltaDevap= si deltaDvap ne change pas
                     ! 2: deltaDvap change
      parameter (ordre1=2)

#ifdef ISOVERIF
      integer iso_verif_aberrant_nostop
      integer iso_verif_aberrant_choix_nostop
      integer iso_verif_egalite_choix_nostop
      real deltaD
      real xtnu2(niso)
#endif      
      
      ! ca ne marche que si dj de l'eau dans le sol au dpart
      if (qsol0.lt.ridicule_qsol) then
          do ixt=1,niso
             xtnu(ixt)=Rsol0(ixt)*qevap
          enddo
#ifdef ISOVERIF
            if (use_iso_HDO.eq.1) then
!                write(*,*) 'sol_nu 66: deltaDsol(iso_HDO)=',         
!     :           (Rsol(iso_HDO)/tnat(iso_HDO)-1)*1000
!                write(*,*) 'deltaDevap(iso_HDO)=',(Revap(iso_HDO)/
!     :           tnat(iso_HDO)-1)*1000
            endif !if (use_iso_HDO.eq.1) then
#endif
          return
      endif !if (qsol0.lt.ridicule_qsol) then

      ! vrif des rapports isotopiques en entre
#ifdef ISOVERIF
        ! provisoire,  enlever pour tests avec evap sol nu!
        if (P_veg.eq.1.0) then
         call iso_verif_egalite(Pveg,1.0,'iso_evap_sol_nu 64')
        endif
        if (use_iso_eau.eq.1) then
          call iso_verif_egalite_choix(Rsol0(iso_eau),1.0,
     :           'sol_nu 83',errmax,errmaxrel)
          call iso_verif_egalite_choix(R1(iso_eau),1.0,
     :           'sol_nu 56',errmax,errmaxrel)
        endif !if (use_iso_eau.eq.1) then
        if (use_iso_HDO.eq.1) then 
          if (qsol0.gt.ridicule_qsol*1e2) then   
             if (iso_verif_aberrant_nostop(Rsol0(iso_HDO)/faccond,
     :           'sol_nu 58').eq.1) then
               write(*,*) 'qsol0=',qsol0
               stop
             endif !if (iso_verif_aberrant_nostop
          endif !if (qsol0.gt.ridicule*1e2) then 
          if (h.gt.0.01) then
             call iso_verif_aberrant(R1(iso_HDO),'sol_nu 59')
          endif !if (h.gt.0.01) then
        endif  !if (use_iso_HDO.eq.1) then  
#endif

      ! calcul de la fraction rsiduelle de liq dans sol
      ! cas gnral: f=(L-qevap)/L
      ! cas si qevap>L: f=0
      ! cas si qsol0<L -> f=(qsol0-qevap)/qsol0
        f=max((min(L,qsol0)-qevap)/min(L,qsol0),0.0)
#ifdef ISOVERIF
        call iso_verif_positif(1.0-f,'iso_evap_sol_nu 68')
#endif        


      if (f.lt.ridicule) then
          ! il ne reste plus rien aprs l'vap
          ! -> evap sans frac  
          do ixt=1,niso
            xtnu(ixt)=qevap*Rsol0(ixt) 
          enddo

      else !if (f.lt.ridicule) then

      ! 2 e cas simple: h=1
      if (h.gt.0.99) then
#ifdef ISOVERIF
          write(*,*) 'sol_nu 102: h=',h
#endif                    
          do ixt=1,niso
            call fractcalk(ixt,T,zxtalphal(ixt),zxtalphai(ixt))
          enddo ! do ixt=1,niso
          if (qevap.gt.min(L,qsol0)) then
              ! vap trop rapide pour fractionner
             do ixt=1,niso
               xtnu(ixt)=qevap*Rsol0(ixt) 
             enddo ! do ixt=1,niso
          else if (qsol0.lt.L) then
              ! il ne reste plus beaucou d'eau, tout diffuse
             
             ! cas simple o q10>>qevap
             if (ordre1.eq.1) then
               do ixt=1,niso
                 xtnu(ixt)=qsol0*(Rsol0(ixt)-zxtalphal(ixt)*R1(ixt))
     :          +qevap*zxtalphal(ixt)*R1(ixt)
               enddo ! do ixt=1,niso
             else !if (ordre1.eq.1) then
               ! cas gnral
               do ixt=1,niso
               xtnu(ixt)=(qsol0*q10*(Rsol0(ixt)-zxtalphal(ixt)*R1(ixt))
     :           +qevap*(qsol0*Rsol0(ixt)+zxtalphal(ixt)*q10*R1(ixt)))
     :           /(q10+qevap+(qsol0-qevap)*zxtalphal(ixt))
               enddo ! do ixt=1,niso
             endif !if (ordre1.eq.1) then
             
          else !if (qevap.gt.min(L,qsol0)) then
              ! vaporation non totale et plus d'eau que dans couche de
              ! diffusion                                  
             ! cas simple o q10>>qevap
             if (ordre1.eq.1) then
               do ixt=1,niso
                xtnu(ixt)=L*(Rsol0(ixt)-zxtalphal(ixt)*R1(ixt))
     :          +qevap*zxtalphal(ixt)*R1(ixt)
               enddo ! do ixt=1,niso
              else !if (ordre1.eq.1) then
                ! cas gnral
                do ixt=1,niso
                  xtnu(ixt)=(L*q10*(Rsol0(ixt)-zxtalphal(ixt)*R1(ixt))
     :           +qevap*(L*Rsol0(ixt)+zxtalphal(ixt)*q10*R1(ixt)))
     :           /(q10+qevap+(L-qevap)*zxtalphal(ixt))
                enddo ! do ixt=1,niso
              endif !if (ordre1.eq.1) then
          endif !if (qevap.gt.min(L,qsol0)) then
          
          
#ifdef ISOVERIF
          do ixt=1,niso
            call iso_verif_noNAN(xtnu(ixt),
     &               'iso_evap_sol_nu 121')
          enddo
                if (use_iso_eau.eq.1) then
                 if (iso_verif_egalite_choix_nostop(xtnu(iso_eau),qevap,
     :                  'sol_nu 110',errmax,errmaxrel).eq.1) then
                   write(*,*) 'qevap=',qevap
                   write(*,*) 'qsol0=',qsol0
                   write(*,*) 'L=',L
                   write(*,*) 'Rsol0(iso_eau)=',Rsol0(iso_eau)
                   write(*,*) 'R1(iso_eau)=',R1(iso_eau)
                   write(*,*) 'q10=',q10
                   write(*,*) 'zxtalphal(iso_eau)=',zxtalphal(iso_eau)
                   stop
                 endif
                endif !if (use_iso_eau.eq.1) then
                if (use_iso_HDO.eq.1) then 
                  if (qevap.gt.ridicule_evap*1800) then
                    if (iso_verif_aberrant_nostop(xtnu(iso_HDO)/qevap,
     :                   'sol_nu 113').eq.1) then
                      write(*,*) 'qevap=',qevap 
                      write(*,*) 'qsol0=',qsol0
                      write(*,*) 'deltaD(R1)=',deltaD(R1((iso_HDO)))
                      write(*,*) 'deltaD(alpha*R1)=',deltaD
     :                   (zxtalphal(iso_HDO)*R1((iso_HDO)))
                      write(*,*) 'deltaD(Rsol0)=',
     :                   deltaD(Rsol0((iso_HDO)))
                      write(*,*) 'L=',L
                      write(*,*) 'Pveg=',Pveg
                      ! on ne plante que si ca donne lieu  des valeurs
                      ! aberrante de deltaD1
                      write(*,*) 'deltaD1new=',deltaD(( 
     &                   xtnu(iso_HDO)+q10*R1(iso_HDO)) 
     &                  /(q10+qevap))
                      call iso_verif_aberrant(
     :                   (xtnu(iso_hdo)+q10*R1(iso_hdo))
     :                   /(qevap+q10),
     :                  'sol_nu 1390')
                    endif !if (iso_verif_aberrant_nosto
                  endif !if (qevap.gt.ridicule_evap*1800) then
                  if (iso_verif_aberrant_choix_nostop(xtnu(iso_HDO),
     :             qevap,ridicule,1e5,'sol_nu 195').eq.1) then
                     write(*,*) 'h=',h
                     write(*,*) 'qsol0=',qsol0
                     write(*,*) 'deltaD(R1)=',deltaD(R1((iso_HDO)))
                     write(*,*) 'deltaD(alpha*R1)=',deltaD
     :                   (zxtalphal(iso_HDO)*R1((iso_HDO)))
                     write(*,*) 'deltaD(Rsol0)=',
     :                   deltaD(Rsol0((iso_HDO)))
                     write(*,*) 'deltaD1new=',deltaD(( 
     &                   xtnu(iso_HDO)+q10*R1(iso_HDO)) 
     &                  /(q10+qevap))
                     
                     if (ordre1.eq.1) then
                         ! l'ordre 2 aurait-il amlior?
                       do ixt=1,niso
                          xtnu2(ixt)=(L*q10*(Rsol0(ixt)
     :                    -zxtalphal(ixt)*R1(ixt))+qevap
     :                    *(L*Rsol0(ixt)+zxtalphal(ixt)*q10*R1(ixt)))
     :                    /(q10+qevap+(L-qevap)*zxtalphal(ixt))
                       enddo  
                       write(*,*) 'si 2e ordre:deltaDevap=',
     :                   deltaD(xtnu2(iso_hdo)/qevap)
                       write(*,*) 'si 2e ordre, deltaD1new=',
     &                   deltaD(( 
     &                   xtnu2(iso_HDO)+q10*R1(iso_HDO)) 
     &                  /(q10+qevap)) 
                     endif
                     call iso_verif_aberrant(( 
     &                   xtnu(iso_HDO)+q10*R1(iso_HDO)) 
     &                  /(q10+qevap),'sol_nu 224')
                  endif
                endif  !if (use_iso_HDO.eq.1) then  
                if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
                  if (qevap.gt.ridicule_evap) then
                    call iso_verif_aberrant_o17_nostop(xtnu(iso_O17)
     :               /qevap,xtnu(iso_O18)/qevap,
     :               'iso_evap_nu 238')
                  endif !if (qevap(i).gt.ridicule_evap) then
                endif !if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
#endif


      ! 3 e cas limite : f=1
      else !if (h.gt.0.99) then

      if (f.gt.0.95) then
#ifdef ISOVERIF          
!          write(*,*) 'sol_nu 139: f=',f
#endif          
          do ixt=1,niso
            call fractcalk(ixt,T,zxtalphal(ixt),zxtalphai(ixt))
            interm(ixt)=zxtalphal(ixt)*alphak(ixt)*(1-h)          
            betaprime(ixt)=(1.0-interm(ixt))/interm(ixt)  
            gama(ixt)=zxtalphal(ixt)*h/(1.0-interm(ixt))
!            Rsol(ixt)=(Rsol0(ixt)-gama(ixt)*R1(ixt))
!     :           *f**(betaprime(ixt))+gama(ixt)*R1(ixt)
!          Revap(ixt)=(1+betaprime(ixt))*Rsol0(ixt)
!     :           -betaprime(ixt)*gama(ixt)*R1(ixt) ! 1er ordre
!!     :          +(1-f)*(-Rsol0(ixt)*(1+betaprime(ixt))*0.5 ! 2e ordre
!!     :             +betaprime(ixt)*(0.5+gama(ixt)*R1(ixt))) ! 2e ordre 
          ! 2e ordre conserve mal la masse -> ne pas utiliser
           xtnu(ixt)= qevap*((Rsol0(ixt)/zxtalphal(ixt)-h*R1(ixt) )
     :       /alphak(ixt)/(1-h)) ! =Revap0 <=>1er ordre               
          enddo !do ixt=1,niso

#ifdef ISOVERIF
            do ixt=1,niso
                call iso_verif_noNAN(xtnu(ixt),
     &               'iso_evap_sol_nu 169')
            enddo
            if (use_iso_eau.eq.1) then
              call iso_verif_egalite_choix(xtnu(iso_eau),qevap,
     :           'sol_nu 151',errmax,errmaxrel)
            endif !if (use_iso_eau.eq.1) then
            if (use_iso_HDO.eq.1) then    
                if (qevap.gt.ridicule_evap*1800) then  
                  if (iso_verif_aberrant_nostop(xtnu(iso_HDO)/qevap,
     :           'sol_nu 154').eq.1) then
                        write(*,*) 'deltaDRsol0=',deltaD(Rsol0(iso_HDO))
                        write(*,*) 'deltaDR1=',deltaD(R1(iso_HDO))
                        write(*,*) 'deltaD gama*R1=',
     :                   deltaD(gama(iso_HDO)*R1(iso_HDO))
                        write(*,*) 'f=',f                        
                        write(*,*) 'qevap,L=',qevap,L
                        write(*,*) 'betaprime,h=',betaprime(iso_HDO),h
                        write(*,*) 'alphak,zxtalphal=',
     :                           alphak(iso_HDO),zxtalphal(iso_HDO)
                        ! on ne stppe que si deltaD1new devient
                        ! aberrant.
                        write(*,*) 'deltaD1new=',deltaD(( 
     &                   xtnu(iso_HDO)+q10*R1(iso_HDO)) 
     &                  /(q10+qevap))
                        call iso_verif_aberrant(
     :                   (xtnu(iso_hdo)+q10*R1(iso_hdo))
     :                   /(qevap+q10),
     :                  'sol_nu 282')
                   endif !if (iso_verif_aberrant_nostop
                endif !if (qevap.gt.ridicule_evap*1800) then
                call iso_verif_aberrant_choix(xtnu(iso_HDO),
     :             qevap,ridicule,1e5,'sol_nu 269')
            endif  !if (use_iso_HDO.eq.1) then 
           if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
                  if (qevap.gt.ridicule_evap) then
                    call iso_verif_aberrant_o17_nostop(xtnu(iso_O17)
     :               /qevap,xtnu(iso_O18)/qevap,
     :               'iso_evap_nu 307')
                  endif !if (qevap(i).gt.ridicule_evap) then
                endif !if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then 
#endif
      else !if (f.gt.0.90) then

      ! 4e cas simple: si h=0
      if (h.lt.0.01) then
#ifdef ISOVERIF          
        write(*,*) 'sol_nu 165: h=',h
#endif        
        do ixt=1,niso
          call fractcalk(ixt,T,zxtalphal(ixt),zxtalphai(ixt))
        enddo  
        do ixt=1,niso
          betaprime(ixt)=1.0/alphak(ixt)/zxtalphal(ixt)-1.0
          xtnu(ixt)=qevap*Rsol0(ixt)*(1-f**(1+betaprime(ixt)))/(1.0-f)  
        enddo !do ixt=1,niso
        
#ifdef ISOVERIF
          do ixt=1,niso            
            call iso_verif_noNAN(xtnu(ixt),
     &               'iso_evap_sol_nu 206')
          enddo
          if (use_iso_eau.eq.1) then
             call iso_verif_egalite_choix(xtnu(iso_eau),qevap,
     :           'sol_nu 168',errmax,errmaxrel)
          endif !if (use_iso_eau.eq.1) then
          if (use_iso_HDO.eq.1) then   
            if (qevap.gt.ridicule_qsol) then
              call iso_verif_aberrant(xtnu(iso_HDO)/qevap,
     :                   'sol_nu 171')
            endif !if (qevap.gt.ridicule_qsol) then
            call iso_verif_aberrant_choix(xtnu(iso_HDO),
     :             qevap,ridicule,1e5,'sol_nu 302')
          endif  !if (use_iso_HDO.eq.1) then  
          if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
               if (qevap.gt.ridicule_evap) then
                    call iso_verif_aberrant_o17_nostop(xtnu(iso_O17)
     :               /qevap,xtnu(iso_O18)/qevap,
     :               'iso_evap_nu 347')
               endif !if (qevap(i).gt.ridicule_evap) then
           endif !if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
#endif

      else !if (h.lt.0.01) then     

      ! cas gnral
#ifdef ISOVERIF      
!      write(*,*) 'sol_nu 182: cas gnral'
#endif      
      do ixt=1,niso
        call fractcalk(ixt,T,zxtalphal(ixt),zxtalphai(ixt))
        interm(ixt)=zxtalphal(ixt)*alphak(ixt)*(1.0-h)
        betaprime(ixt)=((1.0-interm(ixt))/interm(ixt))      
        gama(ixt)=zxtalphal(ixt)*h/(1.0-interm(ixt))
        xtnu(ixt)=qevap*(Rsol0(ixt)*(1.0-f**(1.0+betaprime(ixt)))
     :           -f*gama(ixt)*R1(ixt)*(1.0-f**betaprime(ixt)))/(1.0-f)
      enddo !do ixt=1,niso
      
#ifdef ISOVERIF
        do ixt=1,niso
!         write(*,*) 'qevap,Rsol0(ixt),f,betaprime(ixt)=',
!     :         qevap,Rsol0(ixt),f,betaprime(ixt)  
         call iso_verif_noNAN(xtnu(ixt),
     &               'iso_evap_sol_nu 234')
        enddo
        if (use_iso_eau.eq.1) then
          call iso_verif_egalite_choix(xtnu(iso_eau),qevap,      
     :           'sol_nu 185',errmax,errmaxrel)
        endif !if (use_iso_eau.eq.1) then
        if (use_iso_HDO.eq.1) then  
           if (qevap.gt.ridicule_evap*1800) then
             if (iso_verif_aberrant_nostop(xtnu(iso_HDO)/qevap,
     :           'sol_nu 189').eq.1) then
               write(*,*) 'deltaDRsol0=',deltaD(Rsol0(iso_HDO))
               write(*,*) 'deltaDR1=',deltaD(R1(iso_HDO))
               write(*,*) 'deltaD gama*R1=',
     :                   deltaD(gama(iso_HDO)*R1(iso_HDO))
               write(*,*) 'f=',f
               write(*,*) 'betaprime=',betaprime(iso_HDO)
               ! on ne stppe que si deltaD1new devient
               ! aberrant.
               write(*,*) 'deltaD1new=',deltaD(( 
     &                   xtnu(iso_HDO)+q10*R1(iso_HDO)) 
     &                  /(q10+qevap))
               call iso_verif_aberrant(
     :                   (xtnu(iso_hdo)+q10*R1(iso_hdo))
     :                   /(qevap+q10),
     :                  'sol_nu 321')
             endif !if (iso_verif_aberrant_nostop
           endif !if (qevap.gt.ridicule_evap*1800) then
           call iso_verif_aberrant_choix(xtnu(iso_HDO),
     :             qevap,ridicule,1e5,'sol_nu 354')
           if (qsol0-qevap.gt.ridicule_qsol*1e2) then
             if (iso_verif_aberrant_nostop((qsol0*Rsol0(iso_HDO)
     :           -xtnu(iso_HDO))/(qsol0-qevap)/faccond,
     :         'evap_sol_nu, reste sol aberrant 375').eq.1) then
               write(*,*) 'qsol0=',qsol0
               write(*,*) 'deltaDRsol0=',deltaD(Rsol0(iso_HDO))
               write(*,*) 'deltaD gama*R1=',
     :                   deltaD(gama(iso_HDO)*R1(iso_HDO))
               write(*,*) 'deltaDevap=',deltaD(xtnu(iso_HDO)/qevap)
               write(*,*) 'f,h=',f,h
               stop
             endif
           endif
         endif  !if (use_iso_HDO.eq.1) then  
         if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
             if (qevap.gt.ridicule_evap) then
               call iso_verif_aberrant_o17_nostop(xtnu(iso_O17)
     :               /qevap,xtnu(iso_O18)/qevap,
     :               'iso_evap_nu 419')
             endif !if (qevap(i).gt.ridicule_evap) then
          endif !if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
#endif
       endif !if (h.lt.0.01) then
       endif ! if f>0.9
       endif ! if h>0.99
       endif !if (f.lt.ridicule) then

#ifdef ISOVERIF
       do ixt=1,niso
         call iso_verif_noNAN(xtnu(ixt),
     &               'iso_evap_sol_nu 194')
       enddo
!       write(*,*) 'sol_nu tmp 252: xtnu(iso_eau),qevap=',
!     :           xtnu(iso_eau),qevap
       if (use_iso_eau.eq.1) then
          call iso_verif_egalite_choix(xtnu(iso_eau),qevap,      
     :           'sol_nu 244',errmax,errmaxrel)
      endif !if (use_iso_eau.eq.1) then 
      if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
          if (qevap.gt.ridicule_evap) then
             call iso_verif_aberrant_o17_nostop(xtnu(iso_O17)
     :           /qevap,xtnu(iso_O18)/qevap,
     :           'iso_evap_nu 443')
          endif !if (qevap(i).gt.ridicule_evap) then
       endif !if ((use_iso_o17.eq.1).and.(use_iso_o18.eq.1)) then
#endif       


      return
      end


