      subroutine fractcalk(kt, ptin, pxtfra, pfraice)
      implicit none

C-------------------------------------------------------------------------
C Calculation of the fractionation coefficient of water isotopes.
C
C March 2003
C Sandrine Bony (LMD/CNRS), after MPI code from Georg Hoffmann (LSCE)
C-------------------------------------------------------------------------

#include "wateriso2.h"
#include "iso_verif.h"

c -- inputs:
      integer kt    ! tracor number
      real ptin       ! temperature (K)

c -- outputs:
      real pxtfra   ! fractionation factor for vapor/liquid condensation
      real pfraice  ! fractionation factor for vapor/ice condensation

c -- local variables:
      real ZCELS, ZSATVAL
      parameter (ZCELS=273.15)
      real pt ! la temprature max(ptin,pxtmin)

      integer iso_verif_noNAN_nostop ! pour debugage


c-----------------------------------------------------------
C FRACTIONATION OVER WATER:
c-----------------------------------------------------------

       pt=max(ptin,pxtmin)

       if ((use_iso_O17.eq.1).and.(kt.eq.iso_O17)) then
         pxtfra=(EXP(talph1(kt)/(pt**2)+talph2(kt)/pt+talph3(kt)))
     :           **fac_coeff_eq17_liq
       else
           pxtfra=EXP(talph1(kt)/(pt**2)+talph2(kt)/pt+talph3(kt))
       endif

#ifdef ISOVERIF
       if (pt.gt.pxtice) then
           if (iso_verif_noNAN_nostop(pxtfra,'iso_fractcal 33')
     :            .eq.1) then
!             write(*,*) 'kt,pt=',kt,pt
!             write(*,*) 'talph1(kt),talph2(kt),talph3(kt)=',
!     :           talph1(kt),talph2(kt),talph3(kt)
           endif
       endif !if (pt.gt.pxtice) then
#endif       
       pxtfra=max(min(pxtfra,100.0),0.0)

c-----------------------------------------------------------
C FRACTIONATION OVER ICE
c-----------------------------------------------------------

       if ((use_iso_HTO.eq.1).and.(kt.eq.iso_HTO)) then
          pfraice=EXP(talps1(kt)/(pt**2)+talps2(kt)/pt)

       elseif ((use_iso_HDO.eq.1).and.(kt.eq.iso_HDO)) then
          pfraice=EXP(talps1(kt)/(pt**2)+talps2(kt))

       elseif ((use_iso_O18.eq.1).and.(kt.eq.iso_O18)) then
          pfraice=EXP(talps1(kt)/pt+talps2(kt))

       elseif ((use_iso_O17.eq.1).and.(kt.eq.iso_O17)) then
          pfraice=(EXP(talps1(kt)/pt+talps2(kt)))
     :           **fac_coeff_eq17_ice

       elseif ((use_iso_eau.eq.1).and.(kt.eq.iso_eau)) then
          pfraice=1.
       else
           write(*,*) 'iso_fractcal 47: non prvu: kt=',kt
       endif

#ifdef ISOVERIF       
       if (pt.lt.pxtmelt) then
           if (iso_verif_noNAN_nostop(pfraice,'iso_fractcal 55')
     :           .eq.1) then
!              write(*,*) 'kt,pt=',kt,pt
!              write(*,*) 'talps1(kt),talps2(kt)=',
!     :                   talps1(kt),talps2(kt)
           endif
       endif !if (pt.lt.pxtmelt) then
#endif       
       pfraice=max(min(pfraice,100.0),0.0)

c-----------------------------------------------------------
C EFFECTIVE FRACTIONATION OVER ICE if NECESSARY
c-----------------------------------------------------------

       if ((use_iso_eau.eq.1).and.(kt.eq.iso_eau)) then
         pfraice=1.
       else  !if ((use_iso_eau).and.(kt.eq.iso_eau)) then
         if (pt.lt.pxtmelt) then
           ZSATVAL=musi-lambda_sursat*(pt-ZCELS)
           pfraice=pfraice*(ZSATVAL/(1.+pfraice*(ZSATVAL-1.)
     :          *tdifrel(kt)))
         endif !if (pt.lt.pxtmelt) then
       endif !if ((kt.ne.iso_eau).or.(use_iso_eau.ne.1)) then

#ifdef ISOVERIF
         if (iso_verif_noNAN_nostop(pfraice,'iso_fractcal 73') 
     :           .eq.1) then 
!            write(*,*) 'kt,pt=',kt,pt
!            write(*,*) 'ZSATVAL,tdifrel(kt)=',ZSATVAL,tdifrel(kt)
         endif
         if ((use_iso_eau.eq.1).and.(kt.eq.iso_eau)) then  
             call iso_verif_egalite(pfraice,1.0,'iso_fractcal 63')
             call iso_verif_egalite(pxtfra,1.0,'iso_fractcal 67')
         endif !if ((use_iso_eau).and.(kt.eq.iso_eau)) then  
#endif
       pfraice=max(min(pfraice,100.0),0.0)

      return
      end

