#ifdef ISO
      SUBROUTINE iso_tritium(iim,jjm,
     :           klon,klev,
     :           rlat,rlon,rlatd,rlond,
     :           paprs,pplay,
     :           zphi,dtime,
     :           airephy,
     :           d_xt_prod_nucl,
     :           d_xt_cosmo,
     :           d_xt_decroiss,
     :           xt_seri)
      implicit none

c1506 #include "dimiso.h"    
#include "wateriso2.h"
#include "iso_verif.h"
#include "YOMCST.h"

! input
      integer iim,jjm ! nombre de couches en lat et lon
      integer klon,klev
      real rlat(klon), rlon(klon) ! Latitude et longitude en degre
      real rlatd(klon), rlond(klon) ! Latitude et longitude en radian
      real paprs(klon,klev+1) ! input-R-pression pour chaque inter-couche (en Pa)
      real zphi(klon,klev) ! input-R-geopotentiel de chaque couche (reference ocean, en m2/s2)
      real pplay(klon,klev) ! input-R-pression pour le mileu de chaque couche (en Pa)
      real dtime ! pas de temps en secondes
      real airephy(klon) ! aire d'une grille (m2)

! output
      real d_xt_prod_nucl(ntraciso,klon,klev) ! tritium provenant des essais nucleaires
      real d_xt_cosmo(ntraciso,klon,klev) ! production naturelle de tritium
      real d_xt_decroiss(ntraciso,klon,klev) ! decroissance radioactive
      real xt_seri(ntraciso,klon,klev) ! quantite d'isotopes de l'eau

! local
      integer iso_verif_noNAN_nostop ! pour debuggage
      integer iso_verif_positif_strict_nostop ! pour debuggage
      integer ixt,i,k,k_ref,kb,nlev_prod
      real pi
      real rlat_geo(klon) ! latitude geomagnetique de la grille (en radians, entre 0 et 90 degres)
      real glat ! latitude du pole geomagnetique
      real glon ! longitude du pole geomagnetique
      real lat_geo,qcos
      parameter (nlev_prod=34)
      real p_ref ! grille de pression de reference
      dimension p_ref(nlev_prod)
      real masse_tritium ! masse d'une molecule de HTO en kg
      real tau_decroissance_tritium ! periode radioactive du tritium (17.77 ans en secondes)
      data p_ref /
     :      100062.00, 97119.00, 94176.00, 91233.00,
     :      88290.00, 85347.00, 82404.00, 79461.00,
     :      76518.00, 73575.00, 70632.00, 67689.00,
     :      64746.00, 61803.00, 58860.00, 55917.00,
     :      52974.00, 50031.00, 47088.00, 44145.00,
     :      41202.00, 38259.00, 35316.00, 32373.00,
     :      29430.00, 26487.00, 23544.00, 20601.00,
     :      17658.00, 14715.00, 11772.00, 8829.00,
     :      5886.00, 2943.00 /

      integer j_1ere_bombe !  numero du premier essai nucleaire de la journee en cours (486 au total)
      integer nbombe ! pour savoir si c'est un jour de bombe et le nombre de bombes durant ce jour

#ifdef ISOVERIFNANIDRIS
      call iso_verif_noNaN_vect2D(xt_seri,
     :     'iso_tritium 66: debut iso_tritium',ntraciso,klon,klev)
#endif
     
! ---------------------------------------------------------------------     
! initialisation
! ---------------------------------------------------------------------
      pi=4.*atan(1.)
      masse_tritium=33.3388E-27
      tau_decroissance_tritium=560520955.6
      
      do ixt=1,ntraciso
      do i=1,klon
      do k=1,klev
      d_xt_cosmo(ixt,i,k)=0.
      d_xt_prod_nucl(ixt,i,k)=0.
      d_xt_decroiss(ixt,i,k)=0.         
      enddo
      enddo
      enddo

!#ifdef ISOVERIFIDRIS
!      do kb=1,nlev_prod
!      write(*,*) 'iso_tritium 103'
!      write(*,*) 'kb, p_ref', kb, p_ref(kb)
!      enddo
!#endif


! ----------------------------------------------------------------------------
! Production naturelle de tritium --> d_xt_cosmo
! ----------------------------------------------------------------------------

! On passe des coordonnees geographiques a la latitude geomagnetique

       glat = 78.5*pi/180.
       glon = -69.0*pi/180.
       
       do i=1,klon
          qcos=sin(glat)*sin(rlatd(i))
          qcos=qcos+cos(glat)*cos(rlatd(i))*cos(rlond(i)-glon)
          if ( qcos .lt. -1.) qcos = -1.
          if ( qcos .gt. 1.) qcos = 1.
          rlat_geo(i)=pi/2.-acos(qcos)
       enddo


! Pour chaque niveau de pression atmospherique, on implemente a chaque bande de latitude
! la production de tritium
       
       if (use_iso_HTO.eq.1) then ! Tritium
       ixt=iso_HTO

       do i = 1,klon
       do k = 1,klev

! Determination du niveau k_ref dans la grille de reference
       k_ref = 1
       do kb = 1,nlev_prod
          if (p_ref(kb) .gt. pplay(i,k)) k_ref=kb
       enddo

       lat_geo=(180./pi)*abs(rlat_geo(i)) ! latitude geomagnetique
       ! Pour le moment, la production d_xt_cosmo est exprime en at/g/s

       if ( k_ref .eq. 1 ) then   
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 8.9433E-7
                   
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then 
                   d_xt_cosmo(ixt,i,k) = 8.9432E-7
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.9247E-7
                   
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.4992E-7
                   
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.655E-7
                   
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.9815E-7
                   
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.4847E-7
                   
           else    
                  d_xt_cosmo(ixt,i,k) = 6.2824E-7
           
           endif       
       endif
       
       
       if ( k_ref .eq. 2 ) then    
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.736E-6
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.171E-6
                   
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1121E-6
                   
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.9709E-7
                   
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.0662E-7
                   
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.3986E-7
                   
           else    
                   d_xt_cosmo(ixt,i,k) = 8.1299E-7
           
           endif        
       endif
       
       
       if ( k_ref .eq. 3 ) then     
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.5402E-6
           
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.5365E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.4552E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.2989E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1775E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0879E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.0522E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 4 ) then     
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.0198E-6
           
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.0145E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.9024E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.6901E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.5273E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.4072E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.3599E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 5 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.6465E-6
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.6464E-6
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.6389E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.4846E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.1965E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.9785E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.8177E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.755E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 6 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 3.4646E-6
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.4645E-6
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.454E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.2415E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.851E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.5595E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.3444E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.2613E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 7 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 4.5316E-6
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.5315E-6
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.5166E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.2244E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.6958E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.3062E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.0191E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.909E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 8 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 5.9217E-6
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.9216E-6
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.9006E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.499E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.7842E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.2644E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.8815E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 3.736E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 9 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 7.7309E-6
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.7307E-6
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.701E-6
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.1498E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.1842E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.4915E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.9818E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 4.7894E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 10 ) then     
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.0082E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.004E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.2843E-6
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.9817E-6
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.0598E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.3824E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 6.1283E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 11 ) then     
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.3135E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.3076E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.204E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0285E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.0599E-6
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.1612E-6
           
           else    
                   d_xt_cosmo(ixt,i,k) = 7.8258E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 12 ) then     
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.7093E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.701E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.5592E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.3231E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1605E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0414E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 9.9722E-6
           
           endif    
       endif
       
       
       if ( k_ref .eq. 13 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.2217E-5
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.2216E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.21E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.0162E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.6989E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.4835E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.3261E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.2679E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 14 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.8816E-5
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.8815E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.8652E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.6002E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.1746E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.8898E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.6822E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.6056E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 15 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 3.7386E-5
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.7384E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.7157E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.3546E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.7847E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.4084E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.1349E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.0343E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 16 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 4.8393E-5
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.8392E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.8073E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.3156E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.5536E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.0578E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.6983E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.5663E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 17 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 6.2543E-5
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.2541E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.2097E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.541E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.5241E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.8722E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.4007E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 3.228E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 18 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 8.0696E-5
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.0693E-5
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.0074E-5
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.0993E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.7449E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.8897E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.273E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 4.0472E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 19 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.0393E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0392E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0306E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.0753E-5
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.2752E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.1561E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.3513E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 5.057E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 20 ) then     
           if (lat_geo.ge.60.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.3358E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.3238E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1573E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.1859E-5
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.7254E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.678E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 6.2953E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 21 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.7134E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.7133E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.6968E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.4718E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1561E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.661E-5
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.3017E-5
           
           else    
                   d_xt_cosmo(ixt,i,k) = 7.8054E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 22 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.1926E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.1925E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.1696E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.8664E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.45E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.2036E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0277E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 9.6352E-5
           
           endif    
       endif
       
       
       if ( k_ref .eq. 23 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.7986E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.7984E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.7669E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.3591E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.8117E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.4931E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.2663E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.1836E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 24 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 3.5619E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.5617E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.5183E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.9712E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.2536E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.8436E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.552E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.446E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 25 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 4.5186E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.5183E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.4587E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.7264E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.7894E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.2638E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.8906E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 1.7554E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 26 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 5.7102E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.7098E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.628E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.6503E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.4318E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.7618E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.2861E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.1146E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 27 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 7.1820E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.1815E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.0693E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.7675E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.1904E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.3416E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.7389E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.5228E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 28 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 8.9801E-4
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.9794E-4
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.8255E-4
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.0966E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.0671E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.0001E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.242E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 2.9724E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 29 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.1145E-3
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1144E-3
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0932E-3
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.6421E-4
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.0487E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.7209E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.7768E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 3.4441E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 30 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.3709E-3
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.3708E-3
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.3415E-3
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.0387E-3
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.1001E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.4689E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.3085E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 3.9032E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 31 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 1.6712E-3
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.671E-3
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.63E-3
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.2296E-3
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 8.1648E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.1899E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 4.787E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 4.2993E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 32 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.0296E-3
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.0293E-3
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.9704E-3
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.4366E-3
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 9.1906E-4
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 6.8278E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.1594E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 4.5743E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 33 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 2.4971E-3
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.4967E-3
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 2.4078E-3
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.6751E-3
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.017E-3
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.3425E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.3723E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 4.662E-4
           
           endif    
       endif
       
       
       if ( k_ref .eq. 34 ) then     
           if (lat_geo.ge.70.)                                  then
                   d_xt_cosmo(ixt,i,k) = 3.2169E-3
           
           else if ( (lat_geo.ge.60.0) .and. (lat_geo.lt.70.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.2161E-3
                   
           else if ( (lat_geo.ge.50.0) .and. (lat_geo.lt.60.0) ) then
                   d_xt_cosmo(ixt,i,k) = 3.0665E-3
           
           else if ( (lat_geo.ge.40.0) .and. (lat_geo.lt.50.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.9861E-3
           
           else if ( (lat_geo.ge.30.0) .and. (lat_geo.lt.40.0) ) then
                   d_xt_cosmo(ixt,i,k) = 1.1059E-3
           
           else if ( (lat_geo.ge.20.0) .and. (lat_geo.lt.30.0) ) then
                   d_xt_cosmo(ixt,i,k) = 7.6337E-4
           
           else if ( (lat_geo.ge.10.0) .and. (lat_geo.lt.20.0) ) then
                   d_xt_cosmo(ixt,i,k) = 5.2801E-4
           
           else    
                   d_xt_cosmo(ixt,i,k) = 4.4129E-4
           
           endif    
       endif
       
!#ifdef ISOVERIFIDRIS
!          if (k.eq.klev) then
!          write(*,*) 'iso_tritium 1096'
!          write(*,*) 'ixt,i,k,klev=',ixt,i,k,klev
!          write(*,*) 'rlat,rlatd=',rlat(i),rlatd(i)
!          write(*,*) 'rlon,rlond=',rlon(i),rlond(i)
!          write(*,*) 'rlat_geo(i), lat_geo=',rlat_geo(i),lat_geo
!          write(*,*) 'pplay(i,k)=',pplay(i,k)
!          write(*,*) 'k_ref=',k_ref
!          write(*,*) 'd_xt_cosmo(ixt,i,k)=',d_xt_cosmo(ixt,i,k)
!          endif
!#endif


#ifdef ISOVERIFIDRIS
          if (iso_verif_positif_strict_nostop(d_xt_cosmo(ixt,i,k),
     :      'iso_tritium 1110 : d_xt_cosmo negatif ou pas').eq.1) then
          write(*,*) 'i,k,klev=',i,k,klev
          write(*,*) 'rlat,rlatd=',rlat(i),rlatd(i)
          write(*,*) 'rlon,rlond=',rlon(i),rlond(i)
          write(*,*) 'rlat_geo(i), lat_geo=',rlat_geo(i),lat_geo
          write(*,*) 'pplay(i,k)=',pplay(i,k)
          write(*,*) 'kb,k_ref, p_ref=',kb,k_ref,p_ref(kb)
          stop
          endif
#endif
          
       enddo
       enddo
       endif
       
! Conversion de la production naturelle de tritium en kg(HTO)/kg(air)/s
! Facteur 1.3/0.7 : test augmentation/baisse de 30% de la production naturelle de tritium
      do ixt=1,ntraciso
      do i=1,klon
      do k=1,klev
         d_xt_cosmo(ixt,i,k)=d_xt_cosmo(ixt,i,k)*masse_tritium*1000.
#ifdef ISOVERIFIDRIS
         if ((use_iso_HTO.eq.1).and.(ixt.eq.iso_HTO)) then
            if (d_xt_cosmo(ixt,i,k).eq.0) then
               write(*,*) 'prod cosmo nulle iso_tritium 1134'
               write(*,*) 'ixt,i,k',ixt,i,k
               write(*,*) 'masse_tritium', masse_tritium
               stop
            endif
!            if ((k.eq.klev).and.(ixt.eq.iso_HTO)) then
!               write(*,*) 'iso_tritium 1140'
!               write(*,*) 'ixt,i,k,klev=',ixt,i,k,klev
!               write(*,*) 'pplay(i,k)=',pplay(i,k)
!               write(*,*) 'masse_tritium', masse_tritium
!               write(*,*) 'd_xt_cosmo(ixt,i,k)=',d_xt_cosmo(ixt,i,k)
!            endif
         endif
#endif
      enddo
      enddo
      enddo
  

#ifdef ISOVERIFNANIDRIS
      do ixt=1,ntraciso ! boucler sur tous les isotopes
          do i=1,klon ! boucler sur toutes les points horizontaux
          do k=1,klev ! boucler sur l'échelle vertical
            if (iso_verif_noNAN_nostop(d_xt_cosmo(ixt,i,k),
     :         'iso_tritium cosmo 1151').eq.1) then
          write(*,*) 'ixt,i,k,klev=',ixt,i,k,klev
          write(*,*) 'rlat,rlatd=',rlat(i),rlatd(i)
          write(*,*) 'rlon,rlond=',rlon(i),rlond(i)
          write(*,*) 'rlat_geo(i), lat_geo=',rlat_geo(i),lat_geo
          write(*,*) 'pplay(i,k)=',pplay(i,k)
          write(*,*) 'kb,k_ref, p_ref=',kb,k_ref,p_ref(kb)
          write(*,*) 'masse tritium=', masse_tritium
          stop
            endif
          enddo
          enddo
      enddo
#endif

#ifdef ISOVERIFNANIDRIS
      call iso_verif_noNaN_vect2D(xt_seri,
     :     'iso_tritium 1167: apres d_xt_cosmo',ntraciso,klon,klev)
#endif


! --------------------------------------------------------------------------------
! Production de tritium liee aux essais nucleaires --> d_xt_prod_nucl
! --------------------------------------------------------------------------------
       
      if (ok_prod_nucl_tritium) then ! production nucleaire de tritium = true 
          
          if (use_iso_HTO.eq.1) then ! Tritium
             ixt=iso_HTO 
             
             ! on verifie si la date dans la simulation est un jour d'essai nucleaire 
             CALL date_prod_nucl_HTO(nessai, day_nucl, month_nucl,
     :                               year_nucl, j_1ere_bombe, nbombe)
           write(*,*) 'iso_tritium 1183, apres call date_prod_nucl_HTO'
           write(*,*) 'j_1ere_bombe, nbombe', j_1ere_bombe, nbombe
 
             if (nbombe.ge.1) then ! si c'est un jour avec un ou plusieurs essais nucleaires
                 CALL lancer_bombes(iim, jjm, nbombe, j_1ere_bombe,
     :                              klon, klev, zphi,
     :                              rlat, rlon, paprs, airephy,
     :                              lat_nucl, lon_nucl,
     :                              zmin_nucl, zmax_nucl,
     :                              HTO_nucl, nessai,
     :                              d_xt_prod_nucl)
             endif ! if (nbombe.ge.1)
                 
          endif ! if tritium
      
      endif ! if ok_prod_nucl_tritium

#ifdef ISOVERIFIDRIS
      do ixt=1,ntraciso ! boucler sur tous les isotopes
      do i=1,klon ! boucler sur toutes les points horizontaux
      do k=1,klev ! boucler sur l'échelle verticle
         if ((.not.ok_prod_nucl_tritium).or.
     :       (use_iso_HTO.ne.1).or.(ixt.ne.iso_HTO).or.
     :      (nbombe.eq.0)) then
            if (d_xt_prod_nucl(ixt,i,k).ne.0.) then
            write(*,*) 'iso_tritium 1208 apres d_xt_prod_nucl'
            write(*,*) 'la prod nucleaire d isotopes devrait etre nulle'
            write(*,*) 'ixt, i, k', ixt, i, k
            write(*,*) 'd_xt_prod_nucl', d_xt_prod_nucl(ixt,i,k)
            stop
            endif
         endif
      enddo
      enddo
      enddo
#endif 

#ifdef ISOVERIFNANIDRIS
      do ixt=1,ntraciso ! boucler sur tous les isotopes
          do i=1,klon ! boucler sur toutes les points horizontaux
          do k=1,klev ! boucler sur l'échelle verticae
            if (iso_verif_noNAN_nostop(d_xt_prod_nucl(ixt,i,k),
     :         'iso_tritium prod nucl 1225').eq.1) then
          write(*,*) 'ixt,i,k,rlat(i)',ixt,i,k,rlat(i)
          stop
            endif
          enddo
          enddo
      enddo
#endif

#ifdef ISOVERIFNANIDRIS
      call iso_verif_noNaN_vect2D(xt_seri,
     :     'iso_tritium 1236: apres d_xt_prod_nucl',ntraciso,klon,klev)
#endif


! ------------------------------------------------------------------------
! Definition de la decroissance radioactive du tritium --> d_xt_decroiss
! ------------------------------------------------------------------------

        do ixt=1,ntraciso ! boucler sur tous les isotopes
           if ((use_iso_HTO.eq.1).and.(ixt.eq.iso_HTO)) then ! Tritium
           do i=1,klon ! boucler sur toutes les points horizontaux
           do k=1,klev ! boucler sur l'échelle verticale
           d_xt_decroiss(ixt,i,k)=-xt_seri(ixt,i,k)
     :     *1./tau_decroissance_tritium
           enddo
           enddo
           endif
        enddo ! fin de la boucle en ntraciso

#ifdef ISOVERIFNANIDRIS
      call iso_verif_noNaN_vect2D(xt_seri,
     :     'iso_tritium 1257: apres d_xt_decroiss',ntraciso,klon,klev)
#endif


! ----------------------------------------------------------------------
! concentration totale de tritium --> calcul de xt_seri
! ----------------------------------------------------------------------

      do ixt=1,ntraciso ! boucler sur tous les isotopes
         do i=1,klon ! boucler sur toutes les points horizontaux
         do k=1,klev ! boucler sur l'échelle verticale
            xt_seri(ixt,i,k)=xt_seri(ixt,i,k)
     :      +d_xt_cosmo(ixt,i,k)*dtime
     :      +d_xt_prod_nucl(ixt,i,k)*dtime
     :      +d_xt_decroiss(ixt,i,k)*dtime


!#ifdef ISOVERIFIDRIS
!      if ((ixt.eq.iso_HTO).and.(k.ge.(klev-2))) then !ok
!         write(*,*) 'iso_tritium 1284 - test concentration totale' 
!         write(*,*) 'ixt,i,k,klev=',ixt,i,k,klev
!         write(*,*) 'rlat,rlon=',rlat(i),rlon(i)
!         write(*,*) 'dtime=', dtime
!         write(*,*) 'd_xt_cosmo(ixt,i,k)=',d_xt_cosmo(ixt,i,k)
!         write(*,*) 'd_xt_prod_nucl(ixt,i,k)=',d_xt_prod_nucl(ixt,i,k)
!      endif
!#endif

         enddo
         enddo
      enddo

#ifdef ISOVERIFNANIDRIS
      call iso_verif_noNaN_vect2D(xt_seri,
     :     'iso_tritium 1289: fin de iso_tritium',ntraciso,klon,klev)
#endif


      return
      end

!===================================================================
!
!   End subroutine iso_tritium
!
!===================================================================



!===================================================================
!
! Subroutine chargement des tableaux de donnees pour production 
! nucleaire de tritium --> call dans iso_init.F
!
!===================================================================

      SUBROUTINE table_tritium_nucl(ntest,
     :                              day_essai,month_essai,year_essai,
     :                              lat_essai,lon_essai,
     :                              zmin_essai,zmax_essai,
     :                              HTO_essai)
      implicit none

#include "wateriso2.h"
#include "iso_verif.h"

!     Arguments
      integer ntest
      integer day_essai(ntest), month_essai(ntest), year_essai(ntest)
      real lat_essai(ntest), lon_essai(ntest)
      real  zmin_essai(ntest) ,zmax_essai(ntest)
      real HTO_essai(ntest)

!     local
      integer iessai


      if (ok_prod_nucl_tritium) then

      ! tableau pour day_nucl
      open(30, file='day_nucl.txt')
         do iessai=1,ntest
         read(30,*) day_essai(iessai)
         enddo
      close(30)

      ! tableau pour month_nucl
      open(31, file='month_nucl.txt')
         do iessai=1,ntest
         read(31,*) month_essai(iessai)
         enddo
      close(31)

      ! tableau pour year_nucl
      open(32, file='year_nucl.txt')
         do iessai=1,ntest
         read(32,*) year_essai(iessai)
         enddo
      close(32)

      ! tableau pour lat_nucl
      open(33, file='lat_nucl.txt')
         do iessai=1,ntest
         read(33,*) lat_essai(iessai)
         enddo
      close(33)

      ! tableau pour lon_nucl
      open(34, file='lon_nucl.txt')
         do iessai=1,ntest
         read(34,*) lon_essai(iessai)
         enddo
      close(34)

      ! tableau pour zmin_nucl
      open(35, file='zmin_nucl.txt')
         do iessai=1,ntest
         read(35,*) zmin_essai(iessai)
         enddo
      close(35)

      ! tableau pour zmax_nucl
      open(36, file='zmax_nucl.txt')
         do iessai=1,ntest
         read(36,*) zmax_essai(iessai)
         enddo
      close(36)

      ! tableau pour HTO_nucl
      open(37, file='HTO_nucl.txt')
         do iessai=1,ntest
         read(37,*) HTO_essai(iessai)
         enddo
      close(37)



      else 

      do iessai=1,ntest
         day_essai(iessai)   = 0.
         month_essai(iessai) = 0.
         year_essai(iessai)  = 0. 
         lat_essai(iessai)   = 0.
         lon_essai(iessai)   = 0.
         zmin_essai(iessai)  = 0.
         zmax_essai(iessai)  = 0.
         HTO_essai(iessai)   = 0.
      enddo

      endif ! if (ok_prod_nucl_tritium)

      return
      end

!===================================================================
!
! Subroutines production nucleaire utilisees par iso_tritium
!
!===================================================================

! Subroutines pour la production nucleaire de tritium :
! 1. Les donnees (temps, localisation, quantite de tritium injecte) sont
!    chargees prealablement dans la subroutine table_tritium_nucl qui
!    est appelee dans iso_init.F. Ces donnees sont mis dans le COMMON
!    de wateriso2
! 2. Determiner si le jour dans la simulation correspond a un jour d'un
!    essai nucleaire, connaitre la ligne correspondante dans le
!    fichier de forcage de production nucleaire, et savoir le nombres 
!    de bombes nbombe dans cette journee --> subroutine 
!    date_prod_nucl_HTO
! 3. Si oui (nbombe > 0), on utilise la subroutine lancer_bombes qui va
!    definir les variables de localisation et quantite de tritium
!    produit avec le bon nombre de bombes dans la journee et appeler les
!    deux subroutines suivantes
! 4. subroutine coord_prod_nucl_HTO --> pour la localisation de l'essai
!    nucleaire sur (klon,klev)
! 5. calcul de la production de tritium (kg) (a partir de P_HTO dans le tableau 
!    repertoriant tous les essais nucleaires) etalee uniformement sur la journee 
!    entre zmin et zmax --> subroutine calcul_prod_nucl_HTO


! --------------------------------------------------------------------------------
! date_prod_nucl_HTO
! --------------------------------------------------------------------------------
       SUBROUTINE date_prod_nucl_HTO(ntest, day_essai, month_essai,
     :                               year_essai, j_1ere_bombe, nbombe)

      use phys_cal_mod ! pour le calendrier
      implicit none

      ! Arguments
      integer ntest,nbombe,j_1ere_bombe ! pour un jour dans la simulation, on cherche le nombre de bombes nbombe de la journee et la ligne correspondant a la 1ere bombe de cette journee
      integer day_essai(ntest),month_essai(ntest),year_essai(ntest) ! date (jour, mois, annee) des essais nucleaires

      ! local
      integer j ! indices
 
      ! initialisation
      nbombe=0
      j_1ere_bombe=0
      write(*,*) 'iso_tritium 1456, subroutine date_prod_nucl_HTO'
      write(*,*) 'Date dans la simulation:',day_cur,mth_cur,year_cur

      do j=1,ntest ! il faut que le tableau d'entree soit dans l'ordre chronologique
         if (nbombe.eq.0) then
            if ((day_cur.eq.day_essai(j)).and.
     :          (mth_cur.eq.month_essai(j)).and.
     :          (year_cur.eq.year_essai(j))) then
            nbombe=1
            j_1ere_bombe=j
            endif
         else
            if ((day_cur.eq.day_essai(j)).and.
     :         (mth_cur.eq.month_essai(j)).and.
     :         (year_cur.eq.year_essai(j))) then
               nbombe=nbombe+1
            else
               exit
            endif
         endif
!#ifdef ISOVERIFIDRIS
!      write(*,*) 'controle subroutine date_prod_nucl_HTO' !ok
!      write(*,*) 'day_cur, mth_cur, year_cur',day_cur,mth_cur,year_cur
!      write(*,*) 'j, nessai', j, ntest
!      write(*,*) 'nbombe,  j_1ere_bombe', nbombe, j_1ere_bombe
!      write(*,*) 'day_nucl(j), month_nucl(j), year_nucl(j)',
!     :            day_essai(j), month_essai(j), year_essai(j)
!#endif
      enddo
             

      return
      end



! --------------------------------------------------------------------------------
! lancer_bombes
! --------------------------------------------------------------------------------

      SUBROUTINE lancer_bombes(iim, jjm, nbombe, j_1ere_bombe, 
     :                         klon, klev, zphi,
     :                         rlat, rlon, paprs, airephy,
     :                         lat_essai, lon_essai,
     :                         zmin_essai, zmax_essai,
     :                         HTO_essai, ntest,
     :                         prod_nucl_HTO)
      implicit none

#include "wateriso2.h"
#include "iso_verif.h"

!     Arguments
      integer iim, jjm, nbombe, j_1ere_bombe
      integer klon, klev
      integer ntest
      real zphi(klon,klev)
      real rlat(klon), rlon(klon), paprs(klon,klev+1), airephy(klon)
      real lat_essai(ntest), lon_essai(ntest) ! latitude et longitude des essais nucleaires
      real zmin_essai(ntest), zmax_essai(ntest) ! altitudes min et max des nbombes champignons atomiques
      real HTO_essai(ntest) ! production de HTO en kg par les essais nucleaires
      real prod_nucl_HTO(ntraciso,klon,klev) ! calcul de la production de tritium (kg/kg d'air) liee aux essais nucleaire de la journee --> d_xt_prod_nucl(ntraciso,klon,klev)

!     local
      integer ibombe,jessai
      real lat_HTO, lon_HTO ! latitude et longitude de l'essai nucleaire jessai
      real zmin_HTO, zmax_HTO ! altitudes min et max du champignon atomique jessai
      real P_HTO ! production de HTO en kg de l'essai nucleaire jessai
      integer coord_HTO 
      integer kmin_HTO, kmax_HTO ! coordonnees lat, lon, zmin et zmax de l'essai nucleaire jessai

      do ibombe=1,nbombe
         jessai=j_1ere_bombe+ibombe-1
         lat_HTO = lat_essai(jessai)
         lon_HTO = lon_essai(jessai)
         zmin_HTO = zmin_essai(jessai)
         zmax_HTO = zmax_essai(jessai)
         P_HTO = HTO_essai(jessai)
!#ifdef ISOVERIFIDRIS !ok
!      write(*,*) 'controle subroutine lancer_bombes'
!      write(*,*) 'ibombe, nbombe', ibombe, nbombe
!      write(*,*) 'jessai, j_1ere_bombe', jessai, j_1ere_bombe
!      write(*,*) 'lat_HTO, lon_HTO', lat_HTO, lon_HTO
!      write(*,*) 'zmin_HTO, zmax_HTO', zmin_HTO, zmax_HTO
!      write(*,*) 'P_HTO', P_HTO 
!#endif
         
        CALL coord_prod_nucl_HTO(iim, jjm,
     :                            klon, klev, zphi,
     :                            rlat, rlon,
     :                            lat_HTO, lon_HTO,
     :                            zmin_HTO, zmax_HTO,
     :                            coord_HTO, 
     :                            kmin_HTO,kmax_HTO)
            
         if (coord_HTO.gt.0) then ! quand on trouve les coordonnees de l'essai nucleaire dans la simulation
       write(*,*) 'iso_tritium 1552 dans subroutine lancer_bombes'
       write(*,*) 'Apres call coord_prod_nucl_HTO pour coord_HTO>0'
       write(*,*) 'ibombe, nbombe', ibombe, nbombe
       write(*,*) 'coord_HTO',coord_HTO
       write(*,*) 'rlat(coord_HTO), rlon(coord_HTO)', 
     :             rlat(coord_HTO), rlon(coord_HTO)
       write(*,*) 'kmin_HTO, kmax_HTO', kmin_HTO, kmax_HTO

         CALL calcul_prod_nucl_HTO(P_HTO,coord_HTO,
     :                             kmin_HTO,kmax_HTO,
     :                             klon, klev,
     :                             paprs,airephy,
     :                             prod_nucl_HTO)
         endif

      enddo

      return
      end
! --------------------------------------------------------------------------------
! coord_prod_nucl_HTO
! --------------------------------------------------------------------------------
      SUBROUTINE coord_prod_nucl_HTO(iim, jjm,
     :                               klon, klev, zphi,
     :                               rlat, rlon,
     :                               lat_jessai, lon_jessai, 
     :                               zmin_jessai, zmax_jessai,
     :                               coord_jessai,
     :                               kmin_jessai,kmax_jessai)
      implicit none

#include "YOMCST.h"

!     Arguments
      integer klon, klev ! indices grilles horizontales et verticales
      integer iim, jjm ! nombre de mailles en longitude et latitude 
      real zphi(klon,klev) ! input-R-geopotentiel de chaque couche (reference ocean, en m2/s2)
      real rlat(klon), rlon(klon) ! latitude et longitude en degres
      real lat_jessai, lon_jessai ! latitude et longitude de l'essai nucleaire jessai
      real zmin_jessai, zmax_jessai ! altitudes min et max du champignon atomique jessai
      integer coord_jessai, kmin_jessai, kmax_jessai ! coordonnees lat, lon, zmin et zmax en sortie de la subroutine de l'essai nucleaire jessai

!     local
      integer i,k ! pour le boucles
      real dlat, dlon ! pas en latitude et longitude
      real alt(klon,klev) ! altitude a chaque niveau

!     initialisation
      kmin_jessai=0
      kmax_jessai=0
      coord_jessai=0

      dlat=180./jjm
      dlon=(2.*180.)/iim
      
       do i=1,klon
         do k=1,klev
           alt(i,k)=zphi(i,k)/RG
         enddo
       enddo

!#ifdef ISOVERIFIDRIS !ok
!       do i=1,klon
!       do k=1,klev
!          write(*,*) 'i,k,RG',i,k,RG
!          write(*,*) 'zphi', zphi(i,k)
!          write(*,*) 'alt', alt(i,k)
!       enddo
!       enddo
!#endif
      
      do i=1,klon
         if (((rlat(i)-dlat/2.).le.lat_jessai).and.
     :       ((rlat(i)+dlat/2.).gt.lat_jessai).and.
     :       ((rlon(i)-dlon/2.).le.lon_jessai).and.
     :       ((rlon(i)+dlon/2.).gt.lon_jessai)) then
                coord_jessai=i
                if (alt(i,1).ge.zmin_jessai)    kmin_jessai = 1 ! si base du champignon est plus bas que le niveau 1 dans le modele
                if (alt(i,1).ge.zmax_jessai)    kmax_jessai = 2 ! si le haut du champignon est plus bas que le niveau 1 du modele
                if (alt(i,klev).lt.zmin_jessai) kmin_jessai = klev  ! si base du champignon est plus haut que le niveau klev dans le modele
                if (alt(i,klev).lt.zmax_jessai) kmax_jessai = klev ! si le haut du champignon est plus haut que le niveau klev du modele
                do k=1,klev-1
                    if ((alt(i,k).lt.zmin_jessai).and.
     :                  (alt(i,k+1).ge.zmin_jessai)) then
                         kmin_jessai = max(1,k) ! si base du champignon est entre le niveau k et k+1
                    endif

                    if ((alt(i,k).lt.zmax_jessai).and.
     :                  (alt(i,k+1).ge.zmax_jessai)) then
                         kmax_jessai = min(k+1,klev) ! si le haut du champignon est entre le niveau k et k+1
                    endif
                enddo ! boucle klev
            exit ! on arrete la boucle le long de klon quand on a trouve les bonnes coordonnees
         endif ! trouve les bonnes coordonnees sur klon
      enddo ! boucle klon

#ifdef ISOVERIFIDRIS
      if (kmin_jessai.gt.kmax_jessai) then ! on plante si kmin>=kmax pour k<klev
         write(*,*) 'Pb subroutine coord_prod_nucl_HTO (kmin>kmax)'
         write(*,*) 'coord_jessai', coord_jessai
         write(*,*) 'lon_nucl, lat_nucl', lon_jessai, lat_jessai
         write(*,*) 'zmin_nucl, zmax_nucl', zmin_jessai, zmax_jessai
         write(*,*) 'kmin,kmax',kmin_jessai,kmax_jessai
         stop
       endif

      if ((kmin_jessai.eq.klev).and.(kmax_jessai.ne.klev)) then ! on plante si on n'a pas kmax=klev quand kmin=klev
         write(*,*) 'Pb subroutine coord_prod_nucl_HTO (kmin=klev)'
         write(*,*) 'coord_jessai', coord_jessai
         write(*,*) 'lon_nucl, lat_nucl', lon_jessai, lat_jessai
         write(*,*) 'zmin_nucl, zmax_nucl', zmin_jessai, zmax_jessai
         write(*,*) 'kmin,kmax',kmin_jessai,kmax_jessai
         stop
      endif
#endif

!#ifdef ISOVERIFIDRIS 
!      write(*,*) 'controle subroutine coord_prod_nucl_HTO' !ok
!      write(*,*) 'iim,jjm,dlat,dlon',iim,jjm,dlat,dlon
!      write(*,*) 'indice coord_HTO', coord_jessai
!      write(*,*) 'rlat, lat_HTO',rlat(coord_jessai),lat_jessai
!      write(*,*) 'rlon, lon_HTO',rlon(coord_jessai),lon_jessai
!      write(*,*) 'kmin_HTO, kmax_HTO', kmin_jessai, kmax_jessai
!      write(*,*) 'alt_min(k), alt_min(k+1), zmin_HTO', 
!     :            alt(coord_jessai,kmin_jessai),
!     :            alt(coord_jessai,kmin_jessai+1),
!     :            zmin_jessai
!      write(*,*) 'alt_max(k), alt_max(k+1), zmax_HTO',
!     :            alt(coord_jessai,kmax_jessai),
!     :            alt(coord_jessai,kmax_jessai+1),
!     :            zmax_jessai
!#endif
      
      return
      end




! --------------------------------------------------------------------------------
! calcul_prod_nucl_HTO
! --------------------------------------------------------------------------------
      SUBROUTINE calcul_prod_nucl_HTO(P_jessai,coord_jessai,
     :                                kmin_jessai,kmax_jessai,
     :                                klon, klev, 
     :                                paprs,airephy,
     :                                prod_nucl)
      implicit none

#include "wateriso2.h"
#include "iso_verif.h"
#include "YOMCST.h"

!     Arguments
      real P_jessai ! production de HTO en kg par l'essai nucleaire jessai
      integer coord_jessai ! indice de coordonnee lat(coord_jessai) et lon(coord_jessai) de l'essai nucleaire jessai
      integer kmin_jessai, kmax_jessai ! indice de hauteur (ou pression) min et max du champignon atomique jessai
      integer klon, klev
      real paprs(klon,klev+1)  ! input-R-pression pour chaque inter-couche (en Pa) 
      real airephy(klon) ! aire d'une grille (m2)
      real prod_nucl(ntraciso,klon,klev) ! calcul de la production de HTO en kg par kg d'air sur la journee entre zmin et zmax --> d_xt_prod_nucl

!     local
      integer ixt,i,j,k ! pour les boucles
      real day_sec ! 1 jour en secondes
      real prod_nucl_tmp(klon,klev) ! calcul de la production de HTO en kg/kg d'air pour l'essai nucleaire jessai

      day_sec = 86400.
      ixt=iso_HTO
      j = coord_jessai
      do i=1,klon
      do k=1,klev
         prod_nucl_tmp(i,k)=0.
      enddo
      enddo
!#ifdef ISOVERIFIDRIS
!      write(*,*) 'ixt,j,coord_HTO',ixt,j,coord_jessai !ok
!      write(*,*) 'kmin_HTO, kmax_HTO', kmin_jessai, kmax_jessai
!      write(*,*) 'day_sec, P_HTO', day_sec, P_jessai
!#endif

      if (kmin_jessai.lt.klev) then ! si kmin < klev, normalement kmin < kmax
         do k=kmin_jessai,kmax_jessai
              prod_nucl_tmp(j,k) = 
     :          (P_jessai/day_sec)/( 
     :          (paprs(j,kmin_jessai)-paprs(j,kmax_jessai))
     :          /RG*airephy(j)       )
              prod_nucl(ixt,j,k)=prod_nucl(ixt,j,k)+prod_nucl_tmp(j,k)
#ifdef ISOVERIFIDRIS
       if (kmin_jessai.ge.kmax_jessai) then
       write(*,*) 'Pb subroutine calcul_prod_nucl_HTO (k<klev)'
       write(*,*) 'kmin_HTO devrait etre plus petit que kmax_HTO'
       write(*,*) 'kmin_HTO,kmax_HTO',kmin_jessai,kmax_jessai
       write(*,*) 'ixt,i,k',ixt,j,k
       write(*,*) 'coord_HTO', coord_jessai
       stop
       endif

       if ((prod_nucl_tmp(j,k).le.0.).or.
     :      (prod_nucl(ixt,j,k).le.0.)) then
       write(*,*) 'Pb subroutine calcul_prod_nucl_HTO (k<klev)'
       write(*,*) 'prod_nucl_tmp(i,k) ou d_xt_prod_nucl 
     :             devraient etre positifs'
       write(*,*) 'ixt,i,k',ixt,j,k
       write(*,*) 'RG, airephy(i)', RG, airephy(j)
       write(*,*) 'coord_HTO,kmin_HTO,kmax_HTO',
     :               coord_jessai,kmin_jessai,kmax_jessai
       write(*,*) 'P_HTO, day_sec', P_jessai, day_sec
       write(*,*) 'paprs(coord_HTO,kmin),paprs(coord_HTO,kmax)',
     :               paprs(j,kmin_jessai), paprs(j,kmax_jessai)
       write(*,*) 'prod_nucl_tmp(i,k)', prod_nucl_tmp(j,k)
       write(*,*) 'd_xt_prod_nucl(ixt,i,k)',prod_nucl(ixt,j,k)
       stop
       endif
#endif
         enddo
      else  ! si kmin = klev (et donc kmax = klev si tout va bien)
         do k=kmin_jessai,kmax_jessai
              prod_nucl_tmp(j,k) =
     :          (P_jessai/day_sec)/(
     :          (paprs(j,kmin_jessai)-paprs(j,kmax_jessai+1))
     :          /RG*airephy(j)       )
              prod_nucl(ixt,j,k)=prod_nucl(ixt,j,k)+prod_nucl_tmp(j,k)
#ifdef ISOVERIFIDRIS
       if ((kmin_jessai.ne.kmax_jessai).and.
     :     (kmin_jessai.ne.klev)) then
       write(*,*) 'Pb subroutine calcul_prod_nucl_HTO (k=klev)'
       write(*,*) 'kmin_HTO et kmax_HTO devraient etre egaux a klev'
       write(*,*) 'kmin_HTO,kmax_HTO',kmin_jessai,kmax_jessai
       write(*,*) 'ixt,i,k',ixt,j,k
       write(*,*) 'coord_HTO', coord_jessai
       stop
       endif

       if ((prod_nucl_tmp(j,k).le.0.).or.
     :     (prod_nucl(ixt,j,k).le.0.)) then
       write(*,*) 'Pb subroutine calcul_prod_nucl_HTO (k=klev)'
       write(*,*) 'prod_nucl_tmp(i,k) ou d_xt_prod_nucl
     :             devraient etre positifs'
       write(*,*) 'ixt,i,k',ixt,j,k
       write(*,*) 'RG, airephy(i)', RG, airephy(j)
       write(*,*) 'coord_HTO,kmin_HTO,kmax_HTO',
     :               coord_jessai,kmin_jessai,kmax_jessai
       write(*,*) 'P_HTO, day_sec', P_jessai, day_sec
       write(*,*) 'paprs(coord_HTO,kmin),paprs(coord_HTO,kmax+1)',
     :               paprs(j,kmin_jessai), paprs(j,kmax_jessai+1)
       write(*,*) 'prod_nucl_tmp(i,k)', prod_nucl_tmp(j,k)
       write(*,*) 'd_xt_prod_nucl(ixt,i,k)',prod_nucl(ixt,j,k)
       stop
       endif
#endif
         enddo
      endif

!#ifdef ISOVERIFIDRIS
!      do i=1,klon
!      do k=1,klev
!         write(*,*) 'controle calcul_prod_nucl_HTO' !ok
!         write(*,*) 'call if ok_prod_nucl_tritium, iso_HTO, nbombe>=1'
!         write(*,*) 'klon, klev', klon, klev
!         write(*,*) 'ixt, i, k', ixt, i, k
!         write(*,*) 'RG, airephy(i)', RG, airephy(i)
!         write(*,*) 'coord_HTO,kmin_HTO,kmax_HTO',
!     :               coord_jessai,kmin_jessai,kmax_jessai
!         write(*,*) 'P_HTO, day_sec', P_jessai, day_sec
!         write(*,*) 'paprs(coord_HTO,kmin),paprs(coord_HTO,kmax)',
!     :               paprs(coord_jessai,kmin_jessai), 
!     :               paprs(coord_jessai,kmax_jessai)
!         write(*,*) 'd_xt_prod_nucl(ixt,i,k)', prod_nucl(ixt,i,k)
!      enddo
!      enddo
!#endif

      return
      end


!===================================================================
!
!   End subroutines utilisees par iso_tritium
!
!===================================================================

#endif
