!
MODULE surf_landice_mod
  
  IMPLICIT NONE

CONTAINS
!
!****************************************************************************************
!
  SUBROUTINE surf_landice(itime, dtime, knon, knindex, &
       swnet, lwnet, tsurf, p1lay, &
       cdragh, cdragm, precip_rain, precip_snow, temp_air, spechum, &
       AcoefH, AcoefQ, BcoefH, BcoefQ, &
       AcoefU, AcoefV, BcoefU, BcoefV, &
       ps, u1, v1, rugoro, pctsrf, &
       snow, qsurf, qsol, agesno, &
       tsoil, z0_new, alb1, alb2, evap, fluxsens, fluxlat, &
       tsurf_new, dflux_s, dflux_l, &
       flux_u1, flux_v1 &
#ifdef ISO
         &      ,xtprecip_rain, xtprecip_snow,xtspechum,Rland_ice, &
         &      xtsnow,xtsol,xtevap &
#ifdef ISOTRAC
         &      ,bassin_map &
#endif         
#endif               
           &    )

    USE dimphy
    USE surface_data,     ONLY : type_ocean, calice, calsno
#ifdef ISO    
    USE fonte_neige_mod,  ONLY : fonte_neige, run_off_lic, &
    & fonte_neige_export_xtrun_off_lic_0
#else  
    USE fonte_neige_mod,  ONLY : fonte_neige, run_off_lic
#endif    
    USE cpl_mod,          ONLY : cpl_send_landice_fields
    USE calcul_fluxs_mod
    USE phys_output_var_mod

    INCLUDE "indicesol.h"
    INCLUDE "dimsoil.h"
    INCLUDE "YOMCST.h"
    INCLUDE "clesphys.h"

! Input variables 
!****************************************************************************************
    INTEGER, INTENT(IN)                           :: itime, knon
    INTEGER, DIMENSION(klon), INTENT(in)          :: knindex
    REAL, INTENT(in)                              :: dtime
    REAL, DIMENSION(klon), INTENT(IN)             :: swnet ! net shortwave radiance
    REAL, DIMENSION(klon), INTENT(IN)             :: lwnet ! net longwave radiance
    REAL, DIMENSION(klon), INTENT(IN)             :: tsurf
    REAL, DIMENSION(klon), INTENT(IN)             :: p1lay
    REAL, DIMENSION(klon), INTENT(IN)             :: cdragh, cdragm
    REAL, DIMENSION(klon), INTENT(IN)             :: precip_rain, precip_snow
    REAL, DIMENSION(klon), INTENT(IN)             :: temp_air, spechum
    REAL, DIMENSION(klon), INTENT(IN)             :: AcoefH, AcoefQ
    REAL, DIMENSION(klon), INTENT(IN)             :: BcoefH, BcoefQ
    REAL, DIMENSION(klon), INTENT(IN)             :: AcoefU, AcoefV, BcoefU, BcoefV
    REAL, DIMENSION(klon), INTENT(IN)             :: ps
    REAL, DIMENSION(klon), INTENT(IN)             :: u1, v1
    REAL, DIMENSION(klon), INTENT(IN)             :: rugoro
    REAL, DIMENSION(klon,nbsrf), INTENT(IN)       :: pctsrf
#ifdef ISO
#include "wateriso2F90.h"
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)        :: xtprecip_rain, xtprecip_snow 
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)        :: xtspechum 
#ifdef ISOTRAC
    integer, DIMENSION(klon), INTENT(IN)        ::bassin_map
#endif       
#endif    

! In/Output variables
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(INOUT)          :: snow, qsol
    REAL, DIMENSION(klon), INTENT(INOUT)          :: agesno
    REAL, DIMENSION(klon, nsoilmx), INTENT(INOUT) :: tsoil
#ifdef ISO
    REAL, DIMENSION(niso,klon), INTENT(INOUT)        :: xtsnow, xtsol
    REAL, DIMENSION(niso,klon), INTENT(INOUT)        :: Rland_ice
#endif  

! Output variables
!****************************************************************************************
    REAL, DIMENSION(klon), INTENT(OUT)            :: qsurf
    REAL, DIMENSION(klon), INTENT(OUT)            :: z0_new
    REAL, DIMENSION(klon), INTENT(OUT)            :: alb1  ! new albedo in visible SW interval
    REAL, DIMENSION(klon), INTENT(OUT)            :: alb2  ! new albedo in near IR interval
    REAL, DIMENSION(klon), INTENT(OUT)            :: evap, fluxsens, fluxlat
    REAL, DIMENSION(klon), INTENT(OUT)            :: tsurf_new
    REAL, DIMENSION(klon), INTENT(OUT)            :: dflux_s, dflux_l      
    REAL, DIMENSION(klon), INTENT(OUT)            :: flux_u1, flux_v1
#ifdef ISO
    REAL, DIMENSION(ntraciso,klon), INTENT(IN)        :: xtevap  
    real, DIMENSION(niso,klon) :: xtrun_off_lic_0_diag ! est une variable globale de
!    fonte_neige
#endif 

! Local variables
!****************************************************************************************
    REAL, DIMENSION(klon)    :: soilcap, soilflux
    REAL, DIMENSION(klon)    :: cal, beta, dif_grnd
    REAL, DIMENSION(klon)    :: zfra, alb_neig
    REAL, DIMENSION(klon)    :: radsol
    REAL, DIMENSION(klon)    :: u0, v0, u1_lay, v1_lay
    INTEGER                  :: i,j
#ifdef ISO       
      real, parameter :: t_coup = 273.15
      real, dimension(klon) :: fqfonte_diag
      real, dimension(klon) ::  snow_evap_diag 
      real, dimension(klon) ::  fqcalving_diag 
      real max_eau_sol_diag  
      real, dimension(klon) ::  runoff_diag   
      integer ixt
      REAL, DIMENSION(niso,klon) :: xtsnow_prec,xtsol_prec
      REAL, DIMENSION(klon) :: snow_prec,qsol_prec
      real, DIMENSION(klon) :: run_off_lic_0_diag
#ifdef ISOVERIF
#include "iso_verif.h"
#endif
#endif 
!#ifdef ISO     

#ifdef ISO
#ifdef ISOVERIF
!     write(*,*) 'surf_land_ice 1499'   
     do i=1,knon
        if (use_iso_eau.eq.1) then
             call iso_verif_egalite_choix(xtsnow(iso_eau,i),snow(i), &
    &            'surf_land_ice 126',errmax,errmaxrel)
        endif !if (use_iso_eau.eq.1) then      
      enddo !do i=1,knon  
#endif
#endif

! End definition
!****************************************************************************************
!
! Initialize output variables
    alb2(:) = 999999.
    alb1(:) = 999999.

!****************************************************************************************
! Calculate total absorbed radiance at surface
!
!****************************************************************************************
    radsol(:) = 0.0
    radsol(1:knon) = swnet(1:knon) + lwnet(1:knon)

!****************************************************************************************
! Soil calculations
! 
!****************************************************************************************
    IF (soil_model) THEN 
       CALL soil(dtime, is_lic, knon, snow, tsurf, tsoil, soilcap, soilflux)
       cal(1:knon) = RCPD / soilcap(1:knon)
       radsol(1:knon)  = radsol(1:knon) + soilflux(1:knon)
    ELSE 
       cal = RCPD * calice
       WHERE (snow > 0.0) cal = RCPD * calsno
    ENDIF


!****************************************************************************************
! Calulate fluxes
!
!****************************************************************************************
    beta(:) = 1.0
    dif_grnd(:) = 0.0

! Suppose zero surface speed
    u0(:)=0.0
    v0(:)=0.0
    u1_lay(:) = u1(:) - u0(:)
    v1_lay(:) = v1(:) - v0(:)

    CALL calcul_fluxs(knon, is_lic, dtime, &
         tsurf, p1lay, cal, beta, cdragh, ps, &
         precip_rain, precip_snow, snow, qsurf,  &
         radsol, dif_grnd, temp_air, spechum, u1_lay, v1_lay, &
         AcoefH, AcoefQ, BcoefH, BcoefQ, &
         tsurf_new, evap, fluxlat, fluxsens, dflux_s, dflux_l)

         ! Rq: snow pas modifie par calcul_fluxs
#ifdef ISO
   ! verif
#ifdef ISOVERIF
     write(*,*) 'surf_land_ice 1499'   
     do i=1,knon
       if (use_iso_eau.eq.1) then
           if (snow(i).gt.ridicule) then
             call iso_verif_egalite_choix(xtsnow(iso_eau,i),snow(i), &
    &            'surf_land_ice 1151',errmax,errmaxrel)
            endif !if ((snow(i).gt.ridicule)) then
        endif !if (use_iso_eau.eq.1) then      
      enddo !do i=1,knon  
#endif
!#ifdef ISOVERIF

    do i=1,knon
      snow_prec(i)=snow(i)
      do ixt=1,niso
      xtsnow_prec(ixt,i)=xtsnow(ixt,i)
      enddo !do ixt=1,niso
      ! initialisation:
      fqfonte_diag(i)=0.0
      snow_evap_diag(i)=0.0
    enddo !do i=1,knon
#endif         
!#ifdef ISO

    CALL calcul_flux_wind(knon, dtime, &
         u0, v0, u1, v1, cdragm, &
         AcoefU, AcoefV, BcoefU, BcoefV, &
         p1lay, temp_air, &
         flux_u1, flux_v1)

!****************************************************************************************
! Calculate snow height, age, run-off,..
!    
!****************************************************************************************
    CALL fonte_neige( knon, is_lic, knindex, dtime, &
         tsurf, precip_rain, precip_snow, &
         snow, qsol, tsurf_new, evap &
#ifdef ISO    
     & ,fqfonte_diag,snow_evap_diag,fqcalving_diag   &
     & ,max_eau_sol_diag,runoff_diag,run_off_lic_0_diag   &
#endif
     &   )


#ifdef ISO
#ifdef ISOVERIF
      do i=1,knon  
       if (use_iso_eau.eq.1) then  
          call iso_verif_egalite_choix(Rland_ice(iso_eau,i),1.0, &
     &         'surf_landice_mod 217',errmax,errmaxrel)
       endif !if (use_iso_eau.eq.1) then   
      enddo !do i=1,knon  
#endif
!#ifdef ISOVERIF

       call calcul_iso_surf_lic_vectall(klon,knon, &
     &           evap,snow_evap_diag,Tsurf_new,snow, &
     &           fqfonte_diag,dtime,t_coup, &
     &           precip_snow,xtprecip_snow, snow_prec,xtsnow_prec, &
     &           xtspechum,spechum,ps,Rland_ice,run_off_lic_0_diag, &
     &           xtevap,xtsnow,xtrun_off_lic_0_diag,fqcalving_diag,knindex &
#ifdef ISOTRAC
     &           ,bassin_map &
#endif        
     &  ) 

        call fonte_neige_export_xtrun_off_lic_0(knon,xtrun_off_lic_0_diag)


#ifdef ISOVERIF
        if (use_iso_eau.eq.1) then  
         do i=1,knon
          j = knindex(i)
          if ((j.eq.291).or.(j.eq.231).or.(j.eq.418).or. &
     &                 (j.eq.38).or.(j.eq.60)) then
           write(*,*) 'surf_landice 260 tmp: i,j,klon,knon,', &
     &           'run_off_lic_0,xt=',i,j,klon,knon,      &
     &           run_off_lic_0_diag(j),xtrun_off_lic_0_diag(iso_eau,j)
          endif
         enddo !do i=1,knon
        endif
#endif
!#ifdef ISOVERIF
#endif
!#ifdef ISO

!****************************************************************************************
! Calculate albedo
!
!****************************************************************************************
    CALL albsno(klon,knon,dtime,agesno(:),alb_neig(:), precip_snow(:))  
    WHERE (snow(1 : knon) .LT. 0.0001) agesno(1 : knon) = 0.
    zfra(1:knon) = MAX(0.0,MIN(1.0,snow(1:knon)/(snow(1:knon)+10.0)))
    alb1(1:knon) = alb_neig(1:knon)*zfra(1:knon) + &
         0.6 * (1.0-zfra(1:knon))
!
!IM: plusieurs choix/tests sur l'albedo des "glaciers continentaux"
!       alb1(1 : knon)  = 0.6 !IM cf FH/GK 
!       alb1(1 : knon)  = 0.82
!       alb1(1 : knon)  = 0.77 !211003 Ksta0.77
!       alb1(1 : knon)  = 0.8 !KstaTER0.8 & LMD_ARMIP5
!IM: KstaTER0.77 & LMD_ARMIP6    

! Attantion: alb1 and alb2 are the same!
    alb1(1:knon)  = 0.77
    alb2(1:knon)  = alb1(1:knon)


!****************************************************************************************
! Rugosity
!
!****************************************************************************************
    z0_new(:) = MAX(1.E-3,rugoro(:))

!****************************************************************************************
! Send run-off on land-ice to coupler if coupled ocean.
! run_off_lic has been calculated in fonte_neige
!
!****************************************************************************************
    IF (type_ocean=='couple') THEN
       CALL cpl_send_landice_fields(itime, knon, knindex, run_off_lic)
    ENDIF
  
!****************************************************************************************
       snow_o=0.
       zfra_o = 0.
       DO j = 1, knon
           i = knindex(j)
           snow_o(i) = snow(j) 
           zfra_o(i) = zfra(j)
       ENDDO

!****************************************************************************************
       snow_o=0.
       zfra_o = 0.
       DO j = 1, knon
           i = knindex(j)
           snow_o(i) = snow(j)
           zfra_o(i) = zfra(j)
       ENDDO

#ifdef ISO
#ifdef ISOVERIF
        DO i = 1, knon
         if (use_iso_eau.eq.1) then  
          call iso_verif_egalite_choix(Rland_ice(iso_eau,i),1.0, &
    &         'surf_landice_mod 294',errmax,errmaxrel)
             call iso_verif_egalite_choix(xtsnow(iso_eau,i),snow(i), &
    &            'surf_land_ice 311',errmax,errmaxrel)
         endif !if (use_iso_eau.eq.1) then
        enddo !DO j = 1, knon
#endif        
#endif   

  END SUBROUTINE surf_landice
!
!****************************************************************************************
!
END MODULE surf_landice_mod



